
def factory( name='epsd_monitor',filename='',nx=20.0,nE=20.0,xmin=0.0,xmax=0.0,ymin=0.0,ymax=0.0,xwidth=0.0,yheight=0.0,Emin=0,Emax=0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from EPSD_monitorboostpython import EPSD_monitor as _factory_mcvine
    return component( _factory_mcvine( name,filename,nx,nE,xmin,xmax,ymin,ymax,xwidth,yheight,Emin,Emax ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Kim Lefmann"""
info.copyright = info_copyright
info_declare = """
  double *PSD_N_00, *PSD_p_00, *PSD_p2_00;
"""
info.declare = info_declare
info_finalize = """
  free(PSD_N_00); free(PSD_p_00); free(PSD_p2_00);
"""
info.finalize = info_finalize
info_full_description = """A monitor measuring neutron intensity vs. position, x, and neutron energy, E"""
info.full_description = info_full_description
info_initialize = """
    PSD_N_00 = (double *)malloc(nx*nE*sizeof(double));
    PSD_p_00 = (double *)malloc(nx*nE*sizeof(double));
    PSD_p2_00 = (double *)malloc(nx*nE*sizeof(double));

    int i,j;

    if (xwidth  > 0) { xmax = xwidth/2;  xmin = -xmax; }
    if (yheight > 0) { ymax = yheight/2; ymin = -ymax; }

    if ((xmin >= xmax) || (ymin >= ymax)) {
            printf("EPSD_monitor: %s: Null detection area !\n"
                   "ERROR         (xwidth,yheight,xmin,xmax,ymin,ymax). Exiting",
           NAME_CURRENT_COMP);
      exit(0);
    }

    for (i=0; i<nx; i++)
     for (j=0; j<nE; j++)
     {
      PSD_N(i,j) = 0;
      PSD_p(i,j) = 0;
      PSD_p2(i,j) = 0;
     }
  """
info.initialize = info_initialize
item0 = Parameter()
item0_default = """epsd_monitor"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = """"""
item1.default = item1_default
item1_description = """Name of file in which to store the detector image (text)"""
item1.description = item1_description
item1_name = """filename"""
item1.name = item1_name
item1_type = """char *"""
item1.type = item1_type
item2 = Parameter()
item2_default = 20.0
item2.default = item2_default
item2_description = """Number of pixel columns in scattering plane (1)"""
item2.description = item2_description
item2_name = """nx"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 20.0
item3.default = item3_default
item3_description = """Number of energy bins (1)"""
item3.description = item3_description
item3_name = """nE"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0.0
item4.default = item4_default
item4_description = """Lower x bound of detector opening (m)"""
item4.description = item4_description
item4_name = """xmin"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0.0
item5.default = item5_default
item5_description = """Upper x bound of detector opening (m)"""
item5.description = item5_description
item5_name = """xmax"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0.0
item6.default = item6_default
item6_description = """Lower y bound of detector opening (m)"""
item6.description = item6_description
item6_name = """ymin"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0.0
item7.default = item7_default
item7_description = """Upper y bound of detector opening (m)"""
item7.description = item7_description
item7_name = """ymax"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0.0
item8.default = item8_default
item8_description = """Width/diameter of detector (x). Overrides xmin,xmax. (m)"""
item8.description = item8_description
item8_name = """xwidth"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0.0
item9.default = item9_default
item9_description = """Height of detector (y). Overrides ymin,ymax. (m)"""
item9.description = item9_description
item9_name = """yheight"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 0
item10.default = item10_default
item10_description = """Lower bound of energy (meV)"""
item10.description = item10_description
item10_name = """Emin"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = 0
item11.default = item11_default
item11_description = """Upper bound of energy (meV)"""
item11.description = item11_description
item11_name = """Emax"""
item11.name = item11_name
item11_type = """double"""
item11.type = item11_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11 ]
info.input_parameters = info_input_parameters
info_name = """EPSD_monitor"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """Array of neutron counts"""
item0.description = item0_description
item0_name = """PSD_N"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """Array of neutron weight counts"""
item1.description = item1_description
item1_name = """PSD_p"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """Array of second moments"""
item2.description = item2_description
item2_name = """PSD_p2"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
info_output_parameters = [ item0, item1, item2 ]
info.output_parameters = info_output_parameters
info_save = """
    DETECTOR_OUT_2D("EPSD monitor", "Position [m]", "Energy [meV]",
        xmin, xmax, Emin, Emax, nx, nE,
        PSD_N_00,PSD_p_00,PSD_p2_00,
        filename);
"""
info.save = info_save
item0 = """
// ----------  header for SHARE ----------
inline double & element_2Darr(double *array2D, int dim_y, int i, int j) 
  {
    return *(array2D+j+i*dim_y);
  }
#define PSD_N( i,  j)  element_2Darr(PSD_N_00, nE, i, j)
#define PSD_p( i,  j)  element_2Darr(PSD_p_00, nE, i, j)
#define PSD_p2( i,  j)  element_2Darr(PSD_p2_00, nE, i, j)  
"""
item1 = """// ----------  implementation for SHARE ----------
"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """A monitor measuring neutron intensity vs. position, x, and neutron energy, E"""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
    int i,j;
    double E,v;

    PROP_Z0;
    v=vx*vx+vy*vy+vz*vz;
    E=VS2E*(vx*vx+vy*vy+vz*vz);

    if (x>xmin && x<xmax && y>ymin && y<ymax && E>Emin && E<Emax)
    {
      i = floor((x - xmin)*nx/(xmax - xmin));
      j = floor((E - Emin)*nE/(Emax - Emin));
      PSD_N(i,j)++;
      PSD_p(i,j) += p;
      PSD_p2(i,j) += p*p;
    }
  """
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
