
def factory( name='e_monitor',nchan=20,filename='IE.dat',xmin=0.0,xmax=0.0,ymin=0.0,ymax=0.0,xwidth=0.2,yheight=0.2,Emin=10.0,Emax=100.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from E_monitorboostpython import E_monitor as _factory_mcvine
    return component( _factory_mcvine( name,nchan,filename,xmin,xmax,ymin,ymax,xwidth,yheight,Emin,Emax ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Kristian Nielsen and Kim Lefmann"""
info.copyright = info_copyright
info_declare = """
    double *E_N;
    double *E_p, *E_p2;
    double S_p, S_pE, S_pE2;
  """
info.declare = info_declare
info_finalize = """
  delete [] E_N;
  delete [] E_p;
  delete [] E_p2;
  printf("E_monitor: finalized.\n");
"""
info.finalize = info_finalize
info_full_description = """A square single monitor that measures the energy of the incoming neutrons."""
info.full_description = info_full_description
info_initialize = """
    int i;

    if (nchan <= 0) {
      printf("E_monitor: number of channel must be positive.\n" );
      exit(0);
    }
    E_N = new double[ nchan ];
    E_p = new double[ nchan ];
    E_p2 = new double[ nchan ];

    if (xwidth  > 0) { xmax = xwidth/2;  xmin = -xmax; }
    if (yheight > 0) { ymax = yheight/2; ymin = -ymax; }

    if ((xmin >= xmax) || (ymin >= ymax)) {
            printf("E_monitor: %s: Null detection area !\n"
                   "ERROR      (xwidth,yheight,xmin,xmax,ymin,ymax). Exiting",
           NAME_CURRENT_COMP);
      exit(0);
    }

    printf("E_monitor: initialized. E window=%g,%g, x window=%g,%g, y window = %g, %g\n",
	   Emin, Emax, xmin, xmax, ymin, ymax);

    for (i=0; i<nchan; i++)
    {
      E_N[i] = 0;
      E_p[i] = 0;
      E_p2[i] = 0;
    }
    S_p = S_pE = S_pE2 = 0;
  """
info.initialize = info_initialize
item0 = Parameter()
item0_default = """e_monitor"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 20
item1.default = item1_default
item1_description = """Number of energy channels (1)"""
item1.description = item1_description
item1_name = """nchan"""
item1.name = item1_name
item1_type = """int"""
item1.type = item1_type
item2 = Parameter()
item2_default = """IE.dat"""
item2.default = item2_default
item2_description = """Name of file in which to store the detector image (text)"""
item2.description = item2_description
item2_name = """filename"""
item2.name = item2_name
item2_type = """char *"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0.0
item3.default = item3_default
item3_description = """Lower x bound of detector opening (m)"""
item3.description = item3_description
item3_name = """xmin"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0.0
item4.default = item4_default
item4_description = """Upper x bound of detector opening (m)"""
item4.description = item4_description
item4_name = """xmax"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0.0
item5.default = item5_default
item5_description = """Lower y bound of detector opening (m)"""
item5.description = item5_description
item5_name = """ymin"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0.0
item6.default = item6_default
item6_description = """Upper y bound of detector opening (m)"""
item6.description = item6_description
item6_name = """ymax"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0.2
item7.default = item7_default
item7_description = """Width/diameter of detector (x). Overrides xmin,xmax. (m)"""
item7.description = item7_description
item7_name = """xwidth"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0.2
item8.default = item8_default
item8_description = """Height of detector (y). Overrides ymin,ymax. (m)"""
item8.description = item8_description
item8_name = """yheight"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 10.0
item9.default = item9_default
item9_description = """Minimum energy to detect (meV)"""
item9.description = item9_description
item9_name = """Emin"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 100.0
item10.default = item10_default
item10_description = """Maximum energy to detect (meV)"""
item10.description = item10_description
item10_name = """Emax"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10 ]
info.input_parameters = info_input_parameters
info_name = """E_monitor"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """Array of neutron counts"""
item0.description = item0_description
item0_name = """E_N"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """Array of neutron weight counts"""
item1.description = item1_description
item1_name = """E_p"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """Array of second moments"""
item2.description = item2_description
item2_name = """E_p2"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """Sum of neutron weight counts"""
item3.description = item3_description
item3_name = """S_p"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """Sum of weighted energies"""
item4.description = item4_description
item4_name = """S_pE"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0
item5.default = item5_default
item5_description = """Sum of weighted energy squared"""
item5.description = item5_description
item5_name = """S_pE2"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
info_output_parameters = [ item0, item1, item2, item3, item4, item5 ]
info.output_parameters = info_output_parameters
info_save = """
    DETECTOR_OUT_1D(
        "Energy monitor",
        "Energy [meV]",
        "Intensity",
        "E", Emin, Emax, nchan,
        &E_N[0],&E_p[0],&E_p2[0],
        filename);
    if (S_p) printf("<E> : %g meV , E-width : %g meV \n",
     S_pE/S_p,sqrt(S_pE2/S_p - S_pE*S_pE/(S_p*S_p)) );

    printf("E_monitor: saved data to file '%s'.\n", filename);
  """
info.save = info_save
item0 = """"""
item1 = """"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Energy-sensitive monitor."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
    int i;
    double E;

    //printf("E_monitor: Trace: %g,%g,%g, %g,%g,%g\n", x,y,z,vx,vy,vz);
    PROP_Z0;
    //printf("E_monitor: Propagated to z=0: %g,%g,%g, %g,%g,%g\n", x,y,z,vx,vy,vz);

    //printf("E_monitor: Trace: E window=%g,%g, x window=%g,%g, y window = %g, %g\n",
    //Emin, Emax, xmin, xmax, ymin, ymax);
    if (x>xmin && x<xmax && y>ymin && y<ymax)
    {
      E = VS2E*(vx*vx + vy*vy + vz*vz);
      //printf("E_monitor: E=%g\n", E);
      S_p += p;
      S_pE += p*E;
      S_pE2 += p*E*E;
      
      i = floor((E-Emin)*nchan/(Emax-Emin));
      // printf("E=%g, Emin=%g, Emax=%g, nchan=%d, i=%d\n", E, Emin, Emax, nchan, i);
      if(i >= 0 && i < nchan)
      {
        E_N[i]++;
        E_p[i] += p;
        E_p2[i] += p*p;
        SCATTER;
      }
    }
  """
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
