
def factory( name='iqe_monitor',Ei=60.0,Qmin=0.0,Qmax=10.0,Emin=-45.0,Emax=45.0,nQ=100,nE=90,max_angle_in_plane=120.0,min_angle_in_plane=0.0,max_angle_out_of_plane=30.0,min_angle_out_of_plane=-30.0,filename='iqe_monitor.dat' ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from IQE_monitorboostpython import IQE_monitor as _factory_mcvine
    return component( _factory_mcvine( name,Ei,Qmin,Qmax,Emin,Emax,nQ,nE,max_angle_in_plane,min_angle_in_plane,max_angle_out_of_plane,min_angle_out_of_plane,filename ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Jiao Lin@caltech"""
info.copyright = info_copyright
info_declare = """

    double *IQE_N;
    double *IQE_p;
    double *IQE_p2;

  """
info.declare = info_declare
info_finalize = """
    delete [] IQE_N;
    delete [] IQE_p;
    delete [] IQE_p2;
  """
info.finalize = info_finalize
info_full_description = """a fake monitor recording I(scalar Q,E)"""
info.full_description = info_full_description
info_initialize = """
    int i;
    int n;

    n = nQ*nE;
    IQE_N = new double[n];
    IQE_p = new double[n];
    IQE_p2 = new double[n];

    for (i=0; i<n; i++)
    {
      IQE_N[i] = 0;
      IQE_p[i] = 0;
      IQE_p2[i] = 0;
    }

  """
info.initialize = info_initialize
item0 = Parameter()
item0_default = """iqe_monitor"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 60.0
item1.default = item1_default
item1_description = """incident energy of neutron(meV)"""
item1.description = item1_description
item1_name = """Ei"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0.0
item2.default = item2_default
item2_description = """lower Q bound(AA-1)"""
item2.description = item2_description
item2_name = """Qmin"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 10.0
item3.default = item3_default
item3_description = """upper Q bound(AA-1)"""
item3.description = item3_description
item3_name = """Qmax"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = -45.0
item4.default = item4_default
item4_description = """lower E bound(meV)"""
item4.description = item4_description
item4_name = """Emin"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 45.0
item5.default = item5_default
item5_description = """upper E bound(meV)"""
item5.description = item5_description
item5_name = """Emax"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 100
item6.default = item6_default
item6_description = """number of Q bins"""
item6.description = item6_description
item6_name = """nQ"""
item6.name = item6_name
item6_type = """int"""
item6.type = item6_type
item7 = Parameter()
item7_default = 90
item7.default = item7_default
item7_description = """number of E bins"""
item7.description = item7_description
item7_name = """nE"""
item7.name = item7_name
item7_type = """int"""
item7.type = item7_type
item8 = Parameter()
item8_default = 120.0
item8.default = item8_default
item8_description = """maximum in-plane scattering angle allowed. plane means the horizontal plane define centered at the sample position"""
item8.description = item8_description
item8_name = """max_angle_in_plane"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0.0
item9.default = item9_default
item9_description = """minimum in-plane scattering angle allowed. plane means the horizontal plane define centered at the sample position"""
item9.description = item9_description
item9_name = """min_angle_in_plane"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 30.0
item10.default = item10_default
item10_description = """maximum out-of-plane scattering angle allowed. plane means the horizontal plane define centered at the sample position"""
item10.description = item10_description
item10_name = """max_angle_out_of_plane"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = -30.0
item11.default = item11_default
item11_description = """minimum out-of-plane scattering angle allowed. plane means the horizontal plane define centered at the sample position"""
item11.description = item11_description
item11_name = """min_angle_out_of_plane"""
item11.name = item11_name
item11_type = """double"""
item11.type = item11_type
item12 = Parameter()
item12_default = """iqe_monitor.dat"""
item12.default = item12_default
item12_description = """Name of file in which to store the detector image (text)"""
item12.description = item12_description
item12_name = """filename"""
item12.name = item12_name
item12_type = """char *"""
item12.type = item12_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11, item12 ]
info.input_parameters = info_input_parameters
info_name = """IQE_monitor"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """IQE_N"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """"""
item1.description = item1_description
item1_name = """IQE_p"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """"""
item2.description = item2_description
item2_name = """IQE_p2"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
info_output_parameters = [ item0, item1, item2 ]
info.output_parameters = info_output_parameters
info_save = """
    DETECTOR_OUT_2D(
        "I(Q,E) monitor",
	"Q [AA-1]",
        "E [meV]",
        Qmin, Qmax,
        Emin, Emax,
        nQ, nE,
        IQE_N,IQE_p,IQE_p2,
        filename);

  """
info.save = info_save
item0 = """"""
item1 = """"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """a fake monitor recording I(scalar Q,E)"""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
    int ii,jj, index1;
    double vi=sqrt(Ei)*SE2V;
    double Q, E;
    double angle_in_plane, angle_out_of_plane, theta;

    angle_in_plane = atan2( vx,vz )/PI*180.;
    if (vy!=0) {
      theta = atan( sqrt(vx*vx+vz*vz)/vy )/PI*180.;
      if (vy>0) angle_out_of_plane = 90.-theta;
      else angle_out_of_plane = 90. + theta;
    }
    else angle_out_of_plane = 0.;

//  printf("vx,vy,vz=%g,%g,%g\n", vx,vy,vz);
//  printf("angle_in_paln, angle_out_of_plane=%g,%g\n", angle_in_plane, angle_out_of_plane);

    if ( min_angle_in_plane < angle_in_plane 
	 && angle_in_plane < max_angle_in_plane 
	 && min_angle_out_of_plane < angle_out_of_plane
	 && angle_out_of_plane < max_angle_out_of_plane ) {

      // determine Q and E
      Q = sqrt( vx*vx + vy*vy + (vz-vi)*(vz-vi) )*V2Q;
      E = Ei - (vx*vx+vy*vy+vz*vz) * VS2E;
      // find out the bin numbers
      // bin number of Q
      ii = -1;
      if (Q>=Qmin && Q<Qmax) {
	//ii=(int) floor( (Q-Qmin)/(Qmax-Qmin)*nQ );
	ii=(int) floor( (Q-Qmin)/(Qmax-Qmin)*nQ );
      }
      // bin number of E
      jj = -1;
      if (E>=Emin && E<Emax) {
	jj=(int) floor( (E-Emin)/(Emax-Emin)*nE );
      }

      
      
      if (ii!=-1 && jj!=-1) {

	index1=ii*nE+jj;
	IQE_N[index1]++;
	IQE_p[index1]+=p;
	IQE_p2[index1]+=p*p;
	//printf("%d,%d,%d,%d,%d\n",i,j,k,ii,jj);
	//printf("%d,%d\n", index, index1);

      }  

    }

    SCATTER;


  """
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
