
def factory( name='monitor',xmin=0.0,xmax=0.0,ymin=0.0,ymax=0.0,xwidth=0.0,yheight=0.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from Monitorboostpython import Monitor as _factory_mcvine
    return component( _factory_mcvine( name,xmin,xmax,ymin,ymax,xwidth,yheight ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Kim Lefmann"""
info.copyright = info_copyright
info_declare = """
    double Nsum;
    double psum, p2sum;
  """
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """Sums neutrons (0th, 1st, and 2nd moment of p) flying through
the rectangular monitor opening. May also be used as detector."""
info.full_description = info_full_description
info_initialize = """
    psum = 0;
    p2sum = 0;
    Nsum = 0;

    if (xwidth  > 0) { xmax = xwidth/2;  xmin = -xmax; }
    if (yheight > 0) { ymax = yheight/2; ymin = -ymax; }

    if ((xmin >= xmax) || (ymin >= ymax)) {
            printf("Monitor: %s: Null detection area !\n"
                   "ERROR    (xwidth,yheight,xmin,xmax,ymin,ymax). Exiting",
           NAME_CURRENT_COMP);
      exit(0);
    }
  """
info.initialize = info_initialize
item0 = Parameter()
item0_default = """monitor"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0.0
item1.default = item1_default
item1_description = """Lower x bound of opening [m]"""
item1.description = item1_description
item1_name = """xmin"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0.0
item2.default = item2_default
item2_description = """Upper x bound of opening [m]"""
item2.description = item2_description
item2_name = """xmax"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0.0
item3.default = item3_default
item3_description = """Lower y bound of opening [m]"""
item3.description = item3_description
item3_name = """ymin"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0.0
item4.default = item4_default
item4_description = """Upper y bound of opening [m]"""
item4.description = item4_description
item4_name = """ymax"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0.0
item5.default = item5_default
item5_description = """Width/diameter of detector (x). Overrides xmin,xmax. (m)"""
item5.description = item5_description
item5_name = """xwidth"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0.0
item6.default = item6_default
item6_description = """Height of detector (y). Overrides ymin,ymax. (m)"""
item6.description = item6_description
item6_name = """yheight"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6 ]
info.input_parameters = info_input_parameters
info_name = """Monitor"""
info.name = info_name
info_output_parameters = [  ]
info.output_parameters = info_output_parameters
info_save = """
//    DETECTOR_OUT_0D("Single monitor " NAME_CURRENT_COMP, Nsum, psum, p2sum);
  """
info.save = info_save
item0 = """"""
item1 = """"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Simple single detector/monitor."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
    PROP_Z0;
    if (x>xmin && x<xmax && y>ymin && y<ymax)
    {
      Nsum++;
      psum += p;
      p2sum += p*p;
      SCATTER;
    }
  """
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
