
def factory( name='monitor_nd',options='',filename='',user1=0,user2=0,username1='',username2='',xwidth=0.1,yheight=0.1,zthick=0.0,xmin=0.0,xmax=0.0,ymin=0.0,ymax=0.0,zmin=0.0,zmax=0.0,bins=0.0,min=-1e+40,max=1e+40 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from Monitor_nDboostpython import Monitor_nD as _factory_mcvine
    return component( _factory_mcvine( name,options,filename,user1,user2,username1,username2,xwidth,yheight,zthick,xmin,xmax,ymin,ymax,zmin,zmax,bins,min,max ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """<a href="mailto:farhi@ill.fr">Emmanuel Farhi</a>"""
info.copyright = info_copyright
info_declare = """
  MonitornD_Defines_type DEFS;
  MonitornD_Variables_type Vars;
"""
info.declare = info_declare
info_finalize = """
  
  Monitor_nD_Finally(&DEFS, &Vars);
"""
info.finalize = info_finalize
info_full_description = """This component is a general Monitor that can output 0/1/2D signals
It can produce many 1D signals (one for any variable specified in
option list), or a single 2D output (two variables correlation).
Also, an additional 'list' of neutron events can be produced.
By default, monitor is square (in x/y plane). A disk shape is also possible
The 'cylinder' and 'banana' option will change that for a banana shape
The 'sphere' option simulates spherical detector. The 'box' is a box.
The cylinder, sphere and banana should be centered on the scattering point.
The monitored flux may be per monitor unit area, and weighted by
a lambda/lambda(2200m/s) factor to obtain standard integrated capture flux.
In normal configuration, the Monitor_nD measures the current parameters
of the neutron that is beeing detected. But a PreMonitor_nD component can
be used in order to study correlations between a neutron being detected in
a Monitor_nD place, and given parameters that are monitored elsewhere
(at <b>PreMonitor_nD</b>).
The monitor can also act as a 3He gas detector, taking into account the
detection efficiency.

The 'bins' and 'limits' modifiers are to be used after each variable,
and 'auto','log' and 'abs' come before it. (eg: auto abs log hdiv bins=10
limits=[-5 5]) When placed after all variables,  these two latter modifiers
apply to the signal (e.g. intensity). Unknown keywords are ignored.

In the case of multiple components at the same position, the 'parallel'
keyword must be used in each instance instead of defining a GROUP.

<b>Possible options are</b>
Variables to record:
kx ky kz k wavevector [Angs-1] (    usually axis are
vx vy vz v            [m/s]         x=horz., y=vert., z=on axis)
x y z                 [m]      Distance, Position
kxy vxy xy radius     [m]      Radial wavevector, velocity and position
t time                [s]      Time of Flight
energy omega          [meV]
lambda wavelength     [Angs]
p intensity flux      [n/s] or [n/cm^2/s]
ncounts               [1]
sx sy sz              [1]      Spin
vdiv ydiv dy          [deg]    vertical divergence (y)
hdiv divergence xdiv  [deg]    horizontal divergence (x)
angle                 [deg]    divergence from <z> direction
theta longitude       [deg]    longitude (x/z) [for sphere and cylinder]
phi   lattitude       [deg]    lattitude (y/z) [for sphere and cylinder]

user user1            will monitor the [Mon_Name]_Vars.UserVariable{1|2}
user2                 to be assigned in an other component (see below)

<b>Other options keywords are:</b>
abs                       Will monitor the abs of the following variable
or of the signal (if used after all variables)
auto                      Automatically set detector limits for one/all
all  {limits|bins|auto}   To set all limits or bins values or auto mode
binary {float|double}     with 'source' option, saves in compact files
bins=[bins=20]            Number of bins in the detector along dimension
borders                   To also count off-limits neutrons
(X < min or X > max)
capture                   weight by lambda/lambda(2200m/s) capture flux
exclusive                 absorb neutrons out of monitor limits
file=string               Detector image file name. default is component
name, plus date and variable extension.
incoming                  Monitor incoming beam in non flat det
intermediate=[5 in %]     Save ALL results every n% steps in simulation
limits=[min max]          Lower/Upper limits for axes
(see up for the variable unit)
list=[counts=1000] or all For a long file of neutron characteristics
with [counts] or all events
log                       Will monitor the log of the following variable
or of the signal (if used after all variables)
min=[min_value]           Same as limits, but only sets the min or max
max=[max_value]
multiple                  Create multiple independant 1D monitors files
no or not                 Revert next option
outgoing                  Monitor outgoing beam (default)
parallel                  Use this option when the next component is at
the same position (parallel components)
per cm2                   Intensity will be per cm^2
per steradian             Intensity will be per steradian (requires auto)
premonitor                Will monitor neutron parameters stored
previously with <b>PreMonitor_nD</b>.
signal=[var]              Will monitor [var] instead of usual intensity
slit or absorb            Absorb neutrons that are out detector
source                    The monitor will save neutron states
unactivate                To unactivate detector (0D detector)
verbose                   To display additional informations
3He_pressure=[3 in bars]  The 3He gas pressure in detector.
3He_pressure=0 is perfect detector (default)

Detector shape options (specified as xwidth,yheight,zthick or x/y/z/min/max)
box                       Box of size xwidth, yheight, zthick.
cylinder                  To get a cylindrical monitor
(diameter is xwidth, height is yheight).
banana                    Same as cylinder, without top/bottom,
on restricted angular area; use theta
variable with limits to define arc.
(diameter is xwidth, height is yheight).
disk                      Disk flat xy monitor. radius is xwidth.
sphere                    To get a spherical monitor (e.g. a 4PI)
(diameter is xwidth).
square                    Square flat xy monitor (xwidth, yheight).
previous                  The monitor uses PREVIOUS component as detector
surface.

<b>EXAMPLES:</b>
MyMon = Monitor_nD(name, 
xwidth = 0.1, yheight = 0.1, zthick = 0,
options = "intensity per cm2 angle,limits=[-5 5] bins=10,with
borders, file = mon1");
will monitor neutron angle from [z] axis, between -5
and 5 degrees, in 10 bins, into "mon1.A" output 1D file
options = "sphere theta phi outgoing"  for a sphere PSD detector (out
beam)  and saves into file "MyMon_[Date_ID].th_ph"
options = "banana, theta limits=[10,130], bins=120, y" a theta/height
banana detector
options = "angle radius all auto"   is a 2D monitor with automatic limits
options = "list=1000 kx ky kz energy" records 1000 neutron event in a file
options = "multiple kx ky kz, auto abs log t, and list all neutrons"
makes 4 output 1D files and produces a complete list for all neutrons
and monitor log(abs(tof)) within automatic limits (for t)


<b>How to monitor any instrument/component variable into a Monitor_nD</b>
Suppose you want to monitor a variable 'age' which you assign somwhere in
the instrument:
COMPONENT MyMonitor = Monitor_nD(name, 
xwidth = 0.1, yheight = 0.1,
user1=age, username1="Age of the Captain [years]",
options="user1, auto")
AT ...

See also the example in <a href="PreMonitor_nD.html">PreMonitor_nD</a> to
monitor neutron parameters cross-correlations."""
info.full_description = info_full_description
info_initialize = """
  char tmp[256];
  strcpy(Vars.compcurname, NAME_CURRENT_COMP);
  if (options != NULL)
    strncpy(Vars.option, options, 1024);
  else {
    strcpy(Vars.option, "x y");
    printf("Monitor_nD: %s has no option specified. Setting to PSD ('x y') monitor.\n", NAME_CURRENT_COMP);
  }
  //  Vars.compcurpos = POS_A_CURRENT_COMP;
  Coords o = {0,0,0};
  Vars.compcurpos = o;

  if (strstr(Vars.option, "source"))
    strcat(Vars.option, " list, x y z vx vy vz t sx sy sz ");

  if (bins) { sprintf(tmp, " all bins=%ld ", (long)bins); strcat(Vars.option, tmp); }
  if (min > -FLT_MAX && max < FLT_MAX) { sprintf(tmp, " all limits=[%g %g]", min, max); strcat(Vars.option, tmp); }
  else if (min > -FLT_MAX) { sprintf(tmp, " all min=%g", min); strcat(Vars.option, tmp); }
  else if (max <  FLT_MAX) { sprintf(tmp, " all max=%g", max); strcat(Vars.option, tmp); }

  strncpy(Vars.UserName1, username1 && strlen(username1) ? username1 : "", 32);
  strncpy(Vars.UserName2, username2 && strlen(username2) ? username2 : "", 32);

  Monitor_nD_Init(&DEFS, &Vars, xwidth, yheight, zthick, xmin,xmax,ymin,ymax,zmin,zmax);

  if (filename != NULL) strncpy(Vars.Mon_File, filename, 128);

  
  if ( ((Vars.Flag_Multiple && Vars.Coord_Number > 1) || Vars.Flag_List) && strchr(Vars.Mon_File,'.') )
  { char *XY; XY = strrchr(Vars.Mon_File,'.'); *XY='_'; }
"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """monitor_nd"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = """"""
item1.default = item1_default
item1_description = """[str]  String that specifies the configuration of the monitor"""
item1.description = item1_description
item1_name = """options"""
item1.name = item1_name
item1_type = """char *"""
item1.type = item1_type
item2 = Parameter()
item2_default = """"""
item2.default = item2_default
item2_description = """[str] Output file name (overrides file=XX option)."""
item2.description = item2_description
item2_name = """filename"""
item2.name = item2_name
item2_type = """char *"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """[variable] Variable assigned to User1"""
item3.description = item3_description
item3_name = """user1"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """[variable] Variable assigned to User2"""
item4.description = item4_description
item4_name = """user2"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = """"""
item5.default = item5_default
item5_description = """[str] Name assigned to User1"""
item5.description = item5_description
item5_name = """username1"""
item5.name = item5_name
item5_type = """char *"""
item5.type = item5_type
item6 = Parameter()
item6_default = """"""
item6.default = item6_default
item6_description = """[str] Name assigned to User2"""
item6.description = item6_description
item6_name = """username2"""
item6.name = item6_name
item6_type = """char *"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0.1
item7.default = item7_default
item7_description = """[m]  Width/diameter of detector (x). Default is 10 cm."""
item7.description = item7_description
item7_name = """xwidth"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0.1
item8.default = item8_default
item8_description = """[m]  Height of detector (y). Default is 10 cm."""
item8.description = item8_description
item8_name = """yheight"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0.0
item9.default = item9_default
item9_description = """[m]  Thichness of detector (z)."""
item9.description = item9_description
item9_name = """zthick"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 0.0
item10.default = item10_default
item10_description = """[m]    Lower x bound of opening"""
item10.description = item10_description
item10_name = """xmin"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = 0.0
item11.default = item11_default
item11_description = """[m]    Upper x bound of opening"""
item11.description = item11_description
item11_name = """xmax"""
item11.name = item11_name
item11_type = """double"""
item11.type = item11_type
item12 = Parameter()
item12_default = 0.0
item12.default = item12_default
item12_description = """[m]    Lower y bound of opening"""
item12.description = item12_description
item12_name = """ymin"""
item12.name = item12_name
item12_type = """double"""
item12.type = item12_type
item13 = Parameter()
item13_default = 0.0
item13.default = item13_default
item13_description = """[m]    Upper y bound of opening"""
item13.description = item13_description
item13_name = """ymax"""
item13.name = item13_name
item13_type = """double"""
item13.type = item13_type
item14 = Parameter()
item14_default = 0.0
item14.default = item14_default
item14_description = """[m]    Lower z bound of opening"""
item14.description = item14_description
item14_name = """zmin"""
item14.name = item14_name
item14_type = """double"""
item14.type = item14_type
item15 = Parameter()
item15_default = 0.0
item15.default = item15_default
item15_description = """[m]    Upper z bound of opening"""
item15.description = item15_description
item15_name = """zmax"""
item15.name = item15_name
item15_type = """double"""
item15.type = item15_type
item16 = Parameter()
item16_default = 0.0
item16.default = item16_default
item16_description = """[1]    Number of bins to force for all variables."""
item16.description = item16_description
item16_name = """bins"""
item16.name = item16_name
item16_type = """double"""
item16.type = item16_type
item17 = Parameter()
item17_default = -1e+40
item17.default = item17_default
item17_description = """[u]    Minimum range value to force for all variables"""
item17.description = item17_description
item17_name = """min"""
item17.name = item17_name
item17_type = """double"""
item17.type = item17_type
item18 = Parameter()
item18_default = 1e+40
item18.default = item18_default
item18_description = """[u]    Maximum range value to force for all variables"""
item18.description = item18_description
item18_name = """max"""
item18.name = item18_name
item18_type = """double"""
item18.type = item18_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11, item12, item13, item14, item15, item16, item17, item18 ]
info.input_parameters = info_input_parameters
info_name = """Monitor_nD"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """structure containing Monitor_nD Defines [struct]"""
item0.description = item0_description
item0_name = """DEFS"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """structure containing Monitor_nD variables [struct]"""
item1.description = item1_description
item1_name = """Vars"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
info_output_parameters = [ item0, item1 ]
info.output_parameters = info_output_parameters
info_save = """
  
  Monitor_nD_Save(&DEFS, &Vars);
"""
info.save = info_save
item0 = """
// ----------- added by Alex Dementsov --------
// ----------  header for SHARE ----------
// ----------- added by Alex Dementsov --------

#include "mcstas2/share/monitor_nd-lib.h"

// ----------- added by Alex Dementsov --------"""
item1 = """// ----------  implementation for SHARE ----------
// ----------- added by Alex Dementsov --------
#include <string.h>

"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """(Intensity or signal vs. [something] and vs. [something] ...)"""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
  double  XY=0;
  double  t0 = 0;
  double  t1 = 0;
  double  pp;
  int     intersect   = 0;
  char    Flag_Restore = 0;

  if (user1 != FLT_MAX) Vars.UserVariable1 = user1;
  if (user2 != FLT_MAX) Vars.UserVariable2 = user2;

  

  if (abs(Vars.Flag_Shape) == DEFS.SHAPE_SQUARE) 
  {
    PROP_Z0;
    intersect = (x>=Vars.mxmin && x<=Vars.mxmax && y>=Vars.mymin && y<=Vars.mymax);
  }
  else if (abs(Vars.Flag_Shape) == DEFS.SHAPE_DISK)   
  {
    PROP_Z0;
    intersect = ((x*x + y*y) <= Vars.Sphere_Radius*Vars.Sphere_Radius);
  }
  else if (abs(Vars.Flag_Shape) == DEFS.SHAPE_SPHERE) 
  {
    intersect = sphere_intersect(&t0, &t1, x, y, z, vx, vy, vz, Vars.Sphere_Radius);
  
  }
  else if ((abs(Vars.Flag_Shape) == DEFS.SHAPE_CYLIND) || (abs(Vars.Flag_Shape) == DEFS.SHAPE_BANANA)) 
  {
    intersect = cylinder_intersect(&t0, &t1, x, y, z, vx, vy, vz, Vars.Sphere_Radius, Vars.Cylinder_Height);
    if ((abs(Vars.Flag_Shape) == DEFS.SHAPE_BANANA) && (intersect != 1)) intersect = 0; 
  }
  else if (abs(Vars.Flag_Shape) == DEFS.SHAPE_BOX) 
  {
    intersect = box_intersect(&t0, &t1, x, y, z, vx, vy, vz, fabs(Vars.mxmax-Vars.mxmin), fabs(Vars.mymax-Vars.mymin), fabs(Vars.mzmax-Vars.mzmin));
  }
  else if (abs(Vars.Flag_Shape) == DEFS.SHAPE_PREVIOUS) 
  { intersect = 1; }

  if (intersect)
  {
    if ((abs(Vars.Flag_Shape) == DEFS.SHAPE_SPHERE) || (abs(Vars.Flag_Shape) == DEFS.SHAPE_CYLIND) || (abs(Vars.Flag_Shape) == DEFS.SHAPE_BOX) || (abs(Vars.Flag_Shape) == DEFS.SHAPE_BANANA))
    {
      if (t0 < 0 && t1 > 0)
        t0 = t;  
      if (t1 < 0 && t0 > 0) 
        t1 = t;
      
      if ((Vars.Flag_Shape < 0) && (t1 > 0))
        PROP_DT(t1); 
      else
        PROP_DT(t0); 
    }

    
    if (Vars.Flag_UsePreMonitor != 1)
    {
      Vars.cp = p;
      Vars.cx = x;
      Vars.cvx = vx;
      Vars.cy = y;
      Vars.cvy = vy;
      Vars.cz = z;
      Vars.cvz = vz;
      // MCVINE:
      // spin vector is not yet supported
      
      Vars.ct = t;
    }

    if ((Vars.He3_pressure > 0) && (t1 != t0) && ((abs(Vars.Flag_Shape) == DEFS.SHAPE_SPHERE) || (abs(Vars.Flag_Shape) == DEFS.SHAPE_CYLIND) || (abs(Vars.Flag_Shape) == DEFS.SHAPE_BOX)))
    {
      XY = exp(-7.417*Vars.He3_pressure*fabs(t1-t0)*2*PI*K2V);
      
      Vars.cp *= 1-XY;
      
      p *= XY;
    }

    if (Vars.Flag_capture)
    {
      XY = sqrt(Vars.cvx*Vars.cvx+Vars.cvy*Vars.cvy+Vars.cvz*Vars.cvz);
      XY *= V2K;
      if (XY != 0) XY = 2*PI/XY; 
      Vars.cp *= XY/1.7985;
    }

    pp = Monitor_nD_Trace(&DEFS, &Vars);
    if (pp==0.0)
    { ABSORB;
    }
    else
    { Vars.Nsum++;
      Vars.psum += pp;
      Vars.p2sum += pp*pp;
      SCATTER;
    }

    
    // MCVINE
    // intermediate results are handled by mcvine
    
    if (Vars.Flag_parallel) 
      Flag_Restore = 1;
  } 
  else {
    if (Vars.Flag_Absorb && !Vars.Flag_parallel) ABSORB;
    else Flag_Restore = 1;  
  }

  if (Flag_Restore)
  {
    // MCVINE
    // spin vector is not yet supported
    // RESTORE_NEUTRON(INDEX_CURRENT_COMP, x, y, z, vx, vy, vz, t, sx, sy, sz, p);
  }
"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
