
def factory( name='psd_tew_monitor',nxchan=20.0,nychan=20.0,nbchan=20.0,type='time',filename='',format='table',xwidth=0.0,yheight=0.0,bmin=0.0,bmax=0.0,deltab=0.0,restore_neutron=0.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from PSD_TEW_monitorboostpython import PSD_TEW_monitor as _factory_mcvine
    return component( _factory_mcvine( name,nxchan,nychan,nbchan,type,filename,format,xwidth,yheight,bmin,bmax,deltab,restore_neutron ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """F. Gallmeier"""
info.copyright = info_copyright
info_declare = """
    double *TOF_N_00, *TOF_p_00, *TOF_p2_00;
    double b_min, b_max, delta_b;
    double x_min, x_max, delta_x;
    double y_min, y_max, delta_y;
  """
info.declare = info_declare
info_finalize = """
  free(TOF_N_00); free(TOF_p_00); free(TOF_p2_00);
"""
info.finalize = info_finalize
info_full_description = """"""
info.full_description = info_full_description
info_initialize = """
    int i, j, k;

    TOF_N_00    = (double *)malloc(nxchan*nychan*nbchan*sizeof(double));
    TOF_p_00    = (double *)malloc(nxchan*nychan*nbchan*sizeof(double));
    TOF_p2_00   = (double *)malloc(nxchan*nychan*nbchan*sizeof(double));

    if(!(strcmp(type,"time")==0) && !(strcmp(type,"energy")==0) && !(strcmp(type,"wavelength")==0)) {
       printf("PSD_TEW: error in type keyword\n stop\n");
       exit(0);
    }
    if(!(strcmp(format,"table")==0) && !(strcmp(format,"McStas")==0)) {
       printf("PSD_TEW: error in format keyword\n");
       printf("PSD_TEW: data are not recorded!");
       exit(0);
    }
    if ((xwidth<=0) || (yheight<0)) {
       printf("PSD TOFEL: %s: Null detection area !\n" 
              "ERROR        (xwidth,yheight). Exiting",
              NAME_CURRENT_COMP);
       exit(0);
    }

    if ((bmin >= bmax) && (bmax > 0)) {
       printf("PSD tew: %s:  bmin larger bmax !\n" 
              "ERROR        (bmin,bmax). Exiting",
              NAME_CURRENT_COMP);
        exit(0);
    }



    x_min=-xwidth/2;       
    x_max=-x_min;
    delta_x=xwidth/nxchan;


    y_min=-yheight/2;
    y_max=-y_min;
    delta_y=yheight/nychan;


    if (bmax!=0) {
        b_max=bmax;
        b_min=bmin;
        delta_b=(b_max-b_min)/nbchan;
    }
    else {
        delta_b=deltab;
        b_min=0;
        b_max=nbchan*delta_b+b_min;
    }

    for (i=0; i<nxchan; i++) {
	for (j=0; j<nychan; j++) {
	    for (k=0; k<nbchan; k++) {
                TOF_N(i,j,k) = 0;
                TOF_p(i,j,k) = 0;
                TOF_p2(i,j,k) = 0;
	    }
	}
    }
    printf("PSD_TEW:\n");
    printf("x_min= %10.5f, x_max= %10.5f  delta_x= %10.5f\n", x_min, x_max, delta_x);
    printf("y_min= %10.5f, y_max= %10.5f  delta_y= %10.5f\n", y_min, y_max, delta_y);
    printf("b_min= %10.5f, b_max= %10.5f  delta_b= %10.5f\n", b_min, b_max, delta_b);
  """
info.initialize = info_initialize
item0 = Parameter()
item0_default = """psd_tew_monitor"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 20.0
item1.default = item1_default
item1_description = """Number of x bins (1)"""
item1.description = item1_description
item1_name = """nxchan"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 20.0
item2.default = item2_default
item2_description = """Number of x bins (1)"""
item2.description = item2_description
item2_name = """nychan"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 20.0
item3.default = item3_default
item3_description = """Number of tow bins (1)"""
item3.description = item3_description
item3_name = """nbchan"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = """time"""
item4.default = item4_default
item4_description = """detector type "time"/"energy"/"wavelength" (string)"""
item4.description = item4_description
item4_name = """type"""
item4.name = item4_name
item4_type = """char *"""
item4.type = item4_type
item5 = Parameter()
item5_default = """"""
item5.default = item5_default
item5_description = """Name of file in which to store the detector image (string)"""
item5.description = item5_description
item5_name = """filename"""
item5.name = item5_name
item5_type = """char *"""
item5.type = item5_type
item6 = Parameter()
item6_default = """table"""
item6.default = item6_default
item6_description = """"table"- binned values on ascii file; "detector_out"- McStas format ascii"""
item6.description = item6_description
item6_name = """format"""
item6.name = item6_name
item6_type = """char *"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0.0
item7.default = item7_default
item7_description = """Width/diameter of detector (x). Overrides xmin,xmax. (m)"""
item7.description = item7_description
item7_name = """xwidth"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0.0
item8.default = item8_default
item8_description = """Height of detector (y). Overrides ymin,ymax. (m)"""
item8.description = item8_description
item8_name = """yheight"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0.0
item9.default = item9_default
item9_description = """Lower time/energy/wavelength limit (ms/meV/AA)"""
item9.description = item9_description
item9_name = """bmin"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 0.0
item10.default = item10_default
item10_description = """Upper time/energy/wavelength limit (ms/meV/AA)"""
item10.description = item10_description
item10_name = """bmax"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = 0.0
item11.default = item11_default
item11_description = """time/energy/wavelength bin width"""
item11.description = item11_description
item11_name = """deltab"""
item11.name = item11_name
item11_type = """double"""
item11.type = item11_type
item12 = Parameter()
item12_default = 0.0
item12.default = item12_default
item12_description = """If set >0, the monitor does not influence the neutron state"""
item12.description = item12_description
item12_name = """restore_neutron"""
item12.name = item12_name
item12_type = """double"""
item12.type = item12_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11, item12 ]
info.input_parameters = info_input_parameters
info_name = """PSD_TEW_monitor"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """TOF_N_00"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """"""
item1.description = item1_description
item1_name = """TOF_p_00"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """"""
item2.description = item2_description
item2_name = """TOF_p2_00"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """"""
item3.description = item3_description
item3_name = """b_min"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """"""
item4.description = item4_description
item4_name = """b_max"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0
item5.default = item5_default
item5_description = """"""
item5.description = item5_description
item5_name = """delta_b"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0
item6.default = item6_default
item6_description = """"""
item6.description = item6_description
item6_name = """x_min"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0
item7.default = item7_default
item7_description = """"""
item7.description = item7_description
item7_name = """x_max"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0
item8.default = item8_default
item8_description = """"""
item8.description = item8_description
item8_name = """delta_x"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0
item9.default = item9_default
item9_description = """"""
item9.description = item9_description
item9_name = """y_min"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 0
item10.default = item10_default
item10_description = """"""
item10.description = item10_description
item10_name = """y_max"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = 0
item11.default = item11_default
item11_description = """"""
item11.description = item11_description
item11_name = """delta_y"""
item11.name = item11_name
item11_type = """double"""
item11.type = item11_type
info_output_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11 ]
info.output_parameters = info_output_parameters
info_save = """
    char third_ordinate[15];
    char b_units[11];
    double b_multiplier;
    if(strcmp(type,"time")==0) {
       b_multiplier=0.1;
    }
    else if(strcmp(type,"energy")==0) {
       b_multiplier=0.1;
    }
    else if(strcmp(type,"wavelength")==0) {
       b_multiplier=1e-4;
    }

    if(strcmp(format,"McStas")==0) {
       if(strcmp(type,"time")==0) {
          DETECTOR_OUT_3D(
            "Time-of-flight 2D monitor",
            "x-coordinate [m]",
            "y-coordinate [m]",
            "time [ms]",
            "x", "y", type, 
             x_min, x_max, y_min, y_max, b_min, b_max, 
             nxchan, nychan, nbchan,
             TOF_N_00,TOF_p_00,TOF_p2_00,
             filename);
       }
       else if(strcmp(type,"energy")==0) {
          DETECTOR_OUT_3D(
            "Time-of-flight 2D monitor",
            "x-coordinate [m]",
            "y-coordinate [m]",
            "energy [meV]",
            "x", "y", type, 
             x_min, x_max, y_min, y_max, b_min, b_max, 
             nxchan, nychan, nbchan,
             TOF_N_00,TOF_p_00,TOF_p2_00,
             filename);
       }
       else if(strcmp(type,"wavelength")==0) {
          DETECTOR_OUT_3D(
            "Time-of-flight 2D monitor",
            "x-coordinate [m]",
            "y-coordinate [m]",
            "wavelength [AA]",
            "x", "y", type, 
             x_min, x_max, y_min, y_max, b_min, b_max, 
             nxchan, nychan, nbchan,
             TOF_N_00, TOF_p_00, TOF_p2_00,
             filename);
       }
    }
    else if(strcmp(format,"table")==0) {
       int i, j, k, N;
       double I, ERR, stdv, one_over_counts;
       FILE *fp;
       one_over_counts = 1.0/mcget_run_num();
       N=0;
       I=0;
       ERR=0;
       for(i=0; i<nxchan; i++){
          for(j=0; j<nychan; j++){
             for(k=0;k<nbchan;k++) {
                N += TOF_N(i,j,k);
                I += TOF_p(i,j,k);
                ERR += TOF_p2(i,j,k);
             }
          }
       }
       if(I>0) ERR=sqrt(ERR/I/I-one_over_counts);
       printf("Detector: %s_I=%12.5e %s_ERR=%12.5e %s_N=%d %s%s%s\n",
               NAME_CURRENT_COMP, I, 
               NAME_CURRENT_COMP, I*ERR,
               NAME_CURRENT_COMP, N,
               "\"",filename,"\"");

       fp=fopen(filename,"w");
       fprintf(fp,"# TOF 2D monitor output\n");
       fprintf(fp,"# \n");    
       fprintf(fp,"#     counts=%10d\n", N);
       fprintf(fp,"#     total intensity=%10.4e +/- %10.4e neutron counts\n", I, I*ERR);
       fprintf(fp,"# \n");    
       if(strcmp(type,"time")==0) {
          fprintf(fp,"#                         intensity +- stdv [n/cm^2/s] \n");
          fprintf(fp,"# x [m]       y [m]       time [ms]\n ");
       }
       else if(strcmp(type,"energy")==0) {
          fprintf(fp,"#                         intensity +- stdv [n/cm^2/eV] \n");
          fprintf(fp,"# x [m]       y [m]       energy [meV]\n ");
       }
       else if(strcmp(type,"wavelength")==0) {
          fprintf(fp,"#                         intensity +- stdv [n/cm^2/AA] \n");
          fprintf(fp,"# x [m]       y [m]       wavelength [AA]\n ");
       }
       fprintf(fp,"#                       ");    
       for(k=0;k<=nbchan;k++) {
	  fprintf(fp,"  %10.4f         ",b_min+k*delta_b);
       }
       fprintf(fp,"\n");

       fprintf(fp,"#\n");
       for(i=0; i<nxchan; i++){
          for(j=0; j<nychan; j++){
             fprintf(fp,"  %10.4f  %10.4f", x_min+delta_x*(i+0.5), y_min+delta_y*(j+0.5));
             for(k=0;k<nbchan;k++) {
                stdv =0.0;
                if(TOF_p2(i,j,k)>0) {
                   stdv = TOF_p2(i,j,k)/TOF_p(i,j,k)/TOF_p(i,j,k)-one_over_counts;
                }
                if(stdv>1){stdv=1.0;}
                else if(stdv<0){stdv=0.0;}
                else{stdv=sqrt(stdv);}
                fprintf(fp,"  %10.4e  %7.5f",TOF_p(i,j,k)/delta_x/delta_y/delta_b*b_multiplier,stdv);
             }
             fprintf(fp,"\n");
          }	  
          fprintf(fp,"\n"); 
       }    
       fclose(fp);
    }
    else {
       printf("PSD_TEW: error in format keyword\n");
       printf("PSD_TEW: data are not recorded!");
       exit(0);
    }
  """
info.save = info_save
item0 = """
// ----------- added by Alex Dementsov --------
// ----------  header for SHARE ----------
// ----------- added by Alex Dementsov --------
inline double & element_3Darr(double *array3D, int dim_y, int dim_z, int i, int j, int k)
  {
    return *(array3D + i*dim_y*dim_z + j*dim_z + k );
  }
#define TOF_N(i,j,k)  element_3Darr(TOF_N_00, nychan, nbchan, i, j, k)
#define TOF_p(i,j,k)  element_3Darr(TOF_p_00, nychan, nbchan, i, j, k)
#define TOF_p2(i,j,k)  element_3Darr(TOF_p2_00, nychan, nbchan, i, j, k)

// ----------- added by Alex Dementsov --------"""
item1 = """// ----------  implementation for SHARE ----------
// ----------- added by Alex Dementsov --------
#include <string.h>

"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Rectangular PSD monitor bined in x, y, and time or energy or wavelength ."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
    int i, j, k;
    double t0 = 0;
    double t1 = 0;
    double bb = 0;


    if(strcmp(type,"time")==0) {
       bb = t*1e3;
    }
    else if(strcmp(type,"energy")==0) {
       bb = VS2E*(vx*vx + vy*vy + vz*vz);
    }
    else if(strcmp(type,"wavelength")==0) {
       bb = (2*PI/V2K)/sqrt(vx*vx + vy*vy + vz*vz);
    }
    if(bb>=b_min && bb<=b_max && box_intersect(&t0,&t1,x,y,z,vx,vy,vz,x_max-x_min,y_max-y_min,0.0001))
    {
       PROP_DT(t0);
       if (p<0) return;

    
       i = floor((x-x_min)/delta_x);              
       if(i >= nxchan) {i = nxchan - 1;}
       if(i < 0) {i=0;}

       j = floor((y-y_min)/delta_y);              
       if(j >= nychan) {j = nychan - 1;}
       if(j < 0) {j=0;}

         
       k = floor((bb-b_min)/delta_b);            
       if(k >= nbchan) {k = nbchan - 1;}
       if(k < 0) {k=0;}

       TOF_N(i,j,k)++;
       TOF_p(i,j,k) += p;
       TOF_p2(i,j,k) += p*p;
       SCATTER;

       //if (restore_neutron) RESTORE_NEUTRON(INDEX_CURRENT_COMP, x, y, z, vx, vy, vz, t, sx, sy, sz, p);
    }
  """
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
