
def factory( name='psd_monitor',nx=90.0,ny=90.0,filename='',xmin=0.0,xmax=0.0,ymin=0.0,ymax=0.0,xwidth=0.0,yheight=0.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from PSD_monitorboostpython import PSD_monitor as _factory_mcvine
    return component( _factory_mcvine( name,nx,ny,filename,xmin,xmax,ymin,ymax,xwidth,yheight ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Kim Lefmann"""
info.copyright = info_copyright
info_declare = """
    double *PSD_N_00, *PSD_p_00, *PSD_p2_00;
  """
info.declare = info_declare
info_finalize = """
  free(PSD_N_00); free(PSD_p_00); free(PSD_p2_00);
"""
info.finalize = info_finalize
info_full_description = """An (n times m) pixel PSD monitor. This component may also be used as a beam
detector."""
info.full_description = info_full_description
info_initialize = """
    int i,j;

    PSD_N_00 = (double *)malloc(nx*ny*sizeof(double));
    PSD_p_00 = (double *)malloc(nx*ny*sizeof(double));
    PSD_p2_00 = (double *)malloc(nx*ny*sizeof(double));

    if (xwidth  > 0) { xmax = xwidth/2;  xmin = -xmax; }
    if (yheight > 0) { ymax = yheight/2; ymin = -ymax; }

    if ((xmin >= xmax) || (ymin >= ymax)) {
            printf("PSD_monitor: %s: Null detection area !\n"
                   "ERROR        (xwidth,yheight,xmin,xmax,ymin,ymax). Exiting",
           NAME_CURRENT_COMP);
      exit(0);
    }

    for (i=0; i<nx; i++)
     for (j=0; j<ny; j++)
     {
      PSD_N(i,j) = 0;
      PSD_p(i,j) = 0;
      PSD_p2(i,j) = 0;
     }
  """
info.initialize = info_initialize
item0 = Parameter()
item0_default = """psd_monitor"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 90.0
item1.default = item1_default
item1_description = """Number of pixel columns (1)"""
item1.description = item1_description
item1_name = """nx"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 90.0
item2.default = item2_default
item2_description = """Number of pixel rows (1)"""
item2.description = item2_description
item2_name = """ny"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = """"""
item3.default = item3_default
item3_description = """Name of file in which to store the detector image (text)"""
item3.description = item3_description
item3_name = """filename"""
item3.name = item3_name
item3_type = """char *"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0.0
item4.default = item4_default
item4_description = """Lower x bound of detector opening (m)"""
item4.description = item4_description
item4_name = """xmin"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0.0
item5.default = item5_default
item5_description = """Upper x bound of detector opening (m)"""
item5.description = item5_description
item5_name = """xmax"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0.0
item6.default = item6_default
item6_description = """Lower y bound of detector opening (m)"""
item6.description = item6_description
item6_name = """ymin"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0.0
item7.default = item7_default
item7_description = """Upper y bound of detector opening (m)"""
item7.description = item7_description
item7_name = """ymax"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0.0
item8.default = item8_default
item8_description = """Width/diameter of detector (x). Overrides xmin,xmax. (m)"""
item8.description = item8_description
item8_name = """xwidth"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0.0
item9.default = item9_default
item9_description = """Height of detector (y). Overrides ymin,ymax. (m)"""
item9.description = item9_description
item9_name = """yheight"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.input_parameters = info_input_parameters
info_name = """PSD_monitor"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """PSD_N_00"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """"""
item1.description = item1_description
item1_name = """PSD_p_00"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """"""
item2.description = item2_description
item2_name = """PSD_p2_00"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
info_output_parameters = [ item0, item1, item2 ]
info.output_parameters = info_output_parameters
info_save = """
    DETECTOR_OUT_2D(
        "PSD monitor",
        "X position [cm]",
        "Y position [cm]",
        xmin*100.0, xmax*100.0, ymin*100.0, ymax*100.0,
        nx, ny,
        PSD_N_00,PSD_p_00,PSD_p2_00,
        filename);
  """
info.save = info_save
item0 = """
// ----------- added by Jiao Lin --------
// ----------  header for SHARE ----------
// ----------- added by Jiao Lin --------
inline double & element_2Darr(double *array2D, int dim_y, int i, int j) 
  {
    return *(array2D+j+i*dim_y);
  }
#define PSD_N( i,  j)  element_2Darr(PSD_N_00, ny, i, j)
#define PSD_p( i,  j)  element_2Darr(PSD_p_00, ny, i, j)
#define PSD_p2( i,  j)  element_2Darr(PSD_p2_00, ny, i, j)  

// ----------- added by Jiao Lin --------"""
item1 = """// ----------  implementation for SHARE ----------
// ----------- added by Jiao Lin --------
"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Position-sensitive monitor."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
    int i,j;

    PROP_Z0;
    if (x>xmin && x<xmax && y>ymin && y<ymax)
    {
      i = floor((x - xmin)*nx/(xmax - xmin));
      j = floor((y - ymin)*ny/(ymax - ymin));
      PSD_N(i,j)++;
      PSD_p(i,j) += p;
      PSD_p2(i,j) += p*p;
      SCATTER;
    }
  """
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
