
def factory( name='psd_monitor_4pi',nx=90.0,ny=90.0,filename='',radius=0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from PSD_monitor_4PIboostpython import PSD_monitor_4PI as _factory_mcvine
    return component( _factory_mcvine( name,nx,ny,filename,radius ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Kim Lefmann and Kristian Nielsen"""
info.copyright = info_copyright
info_declare = """
  double *PSD_N_00, *PSD_p_00, *PSD_p2_00;
  double save_x, save_y, save_z, save_vx, save_vy, save_vz, save_t;
"""
info.declare = info_declare
info_finalize = """
  free(PSD_N_00); free(PSD_p_00); free(PSD_p2_00);
"""
info.finalize = info_finalize
info_full_description = """An (n times m) pixel spherical PSD monitor using a cylindrical projection.
Mostly for test and debugging purposes."""
info.full_description = info_full_description
info_initialize = """
  int i,j;

  PSD_N_00 = (double *)malloc(nx*ny*sizeof(double));
  PSD_p_00 = (double *)malloc(nx*ny*sizeof(double));
  PSD_p2_00 = (double *)malloc(nx*ny*sizeof(double));

  for (i=0; i<nx; i++)
    for (j=0; j<ny; j++)
    {
      PSD_N(i,j) = 0;
      PSD_p(i,j) = 0;
      PSD_p2(i,j) = 0;
    }
"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """psd_monitor_4pi"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 90.0
item1.default = item1_default
item1_description = """Number of pixel columns (1)"""
item1.description = item1_description
item1_name = """nx"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 90.0
item2.default = item2_default
item2_description = """Number of pixel rows (1)"""
item2.description = item2_description
item2_name = """ny"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = """"""
item3.default = item3_default
item3_description = """Name of file in which to store the detector image (text)"""
item3.description = item3_description
item3_name = """filename"""
item3.name = item3_name
item3_type = """char *"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """Radius of detector (m)"""
item4.description = item4_description
item4_name = """radius"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
info_input_parameters = [ item0, item1, item2, item3, item4 ]
info.input_parameters = info_input_parameters
info_name = """PSD_monitor_4PI"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """PSD_N_00"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """"""
item1.description = item1_description
item1_name = """PSD_p_00"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """"""
item2.description = item2_description
item2_name = """PSD_p2_00"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
info_output_parameters = [ item0, item1, item2 ]
info.output_parameters = info_output_parameters
info_save = """
  DETECTOR_OUT_2D(
    "4PI PSD monitor",
    "Longitude [deg]",
    "Lattitude [deg]",
    -180, 180, -90, 90,
    nx, ny,
    PSD_N_00,PSD_p_00,PSD_p2_00,
    filename);
"""
info.save = info_save
item0 = """
// ----------- added by Jiao Lin --------
// ----------  header for SHARE ----------
// ----------- added by Jiao Lin --------
inline double & element_2Darr(double *array2D, int dim_y, int i, int j) 
  {
    return *(array2D+j+i*dim_y);
  }
#define PSD_N( i,  j)  element_2Darr(PSD_N_00, ny, i, j)
#define PSD_p( i,  j)  element_2Darr(PSD_p_00, ny, i, j)
#define PSD_p2( i,  j)  element_2Darr(PSD_p2_00, ny, i, j)  

// ----------- added by Jiao Lin --------"""
item1 = """// ----------  implementation for SHARE ----------
// ----------- added by Jiao Lin --------
"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Spherical position-sensitive detector."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
// #define DEBUG

  double t0, t1, phi, theta;
  int i,j;

#ifdef DEBUG
  std::cout 
      << "r=(" 
      << x << ","
      << y << ","
      << z << "), "
      << "v=("
      << vx << ","
      << vy << ","
      << vz << ")"
      << std::endl
      ;
#endif

  if(sphere_intersect(&t0, &t1, x, y, z, vx, vy, vz, radius) && t1 > 0)
  {
    save_vx = vx; save_vy = vy; save_vz = vz;
    save_x = x; save_y = y; save_z = z;
    save_t = t;

    if(t0 < 0)
      t0 = t1;
    
    PROP_DT(t0);
    phi = atan2(x,z);
    i = floor(nx*(phi/(2*PI)+0.5));
    if(i >= nx)
      i = nx-1;                      
    else if(i < 0)
      i = 0;
    theta=asin(y/radius);
    j = floor(ny*(theta+PI/2)/PI+0.5);
    if(j >= ny)
      j = ny-1;                      
    else if(j < 0)
      j = 0;

#ifdef DEBUG
    std::cout 
      << "propagated r=(" 
      << x << ","
      << y << ","
      << z << "), "
      << "theta=" << theta << ", "
      << "phi=" << phi 
      << std::endl;
    std::cout
      << "i=" << i <<", "
      << "j=" << j 
      << std::endl;
#endif
    PSD_N(i,j)++;
    PSD_p(i,j) += p;
    PSD_p2(i,j) += p*p;
    SCATTER;

    vx = save_vx; vy = save_vy; vz = save_vz;
    x = save_x; y = save_y; z = save_z;
    t = save_t;
  }
"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
