
def factory( name='psdcyl_monitor',nr=0,ny=0,filename='',height=0,radius=0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from PSDcyl_monitorboostpython import PSDcyl_monitor as _factory_mcvine
    return component( _factory_mcvine( name,nr,ny,filename,height,radius ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Kim Lefmann"""
info.copyright = info_copyright
info_declare = """
  double *PSD_N_00, *PSD_p_00, *PSD_p2_00;
"""
info.declare = info_declare
info_finalize = """
  free(PSD_N_00); free(PSD_p_00); free(PSD_p2_00);
"""
info.finalize = info_finalize
info_full_description = """An (n times m) pixel PSD monitor with cylinder shape,
vertical axis, centered at (0,0,0)."""
info.full_description = info_full_description
info_initialize = """
  int i,j;

  PSD_N_00 = (double *)malloc(nr*ny*sizeof(double));
  PSD_p_00 = (double *)malloc(nr*ny*sizeof(double));
  PSD_p2_00 = (double *)malloc(nr*ny*sizeof(double));

  for (i=0; i<nr; i++)
    for (j=0; j<ny; j++)
    {
      PSD_N(i,j) = 0;
      PSD_p(i,j) = 0;
      PSD_p2(i,j) = 0;
    }
"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """psdcyl_monitor"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """Number of pixel (radial) columns (1)"""
item1.description = item1_description
item1_name = """nr"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """Number of pixel rows (1)"""
item2.description = item2_description
item2_name = """ny"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = """"""
item3.default = item3_default
item3_description = """Name of file in which to store the detector image (text)"""
item3.description = item3_description
item3_name = """filename"""
item3.name = item3_name
item3_type = """char *"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """Height of detector (m)"""
item4.description = item4_description
item4_name = """height"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0
item5.default = item5_default
item5_description = """Radius of detector (m)"""
item5.description = item5_description
item5_name = """radius"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5 ]
info.input_parameters = info_input_parameters
info_name = """PSDcyl_monitor"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """PSD_N_00"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """"""
item1.description = item1_description
item1_name = """PSD_p_00"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """"""
item2.description = item2_description
item2_name = """PSD_p2_00"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
info_output_parameters = [ item0, item1, item2 ]
info.output_parameters = info_output_parameters
info_save = """
    DETECTOR_OUT_2D(
        "PSD cylindrical monitor",
        "radial position [deg]",
        "height [m]",
        -180, 180, -height/2.0, height/2.0,
        nr, ny,
        PSD_N_00,PSD_p_00,PSD_p2_00,
        filename);
  """
info.save = info_save
item0 = """
// ----------  header for SHARE ----------
inline double & element_2Darr(double *array2D, int dim_y, int i, int j) 
  {
    return *(array2D+j+i*dim_y);
  }
#define PSD_N( i,  j)  element_2Darr(PSD_N_00, ny, i, j)
#define PSD_p( i,  j)  element_2Darr(PSD_p_00, ny, i, j)
#define PSD_p2( i,  j)  element_2Darr(PSD_p2_00, ny, i, j)  
"""
item1 = """// ----------  implementation for SHARE ----------
"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """the axis vertical. The monitor covers the whole cylinder (360 degrees)."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
    int i,j;
    double t0,t1,phi;

    if(cylinder_intersect(&t0, &t1, x, y, z, vx, vy, vz, radius, height))
     {
      if(t0>0)
       {
         printf("FATAL ERROR: Did not come from within cylinder.\n");
         exit(1);
       };
      if(t1<0)
       {
         printf("FATAL ERROR: Did not hit cylinder from inside.\n");
         exit(1);
       }
      else
       {
        PROP_DT(t1);
                     
        if (fabs(y)<(height- 1e-10))
         {
          phi=atan2(x,z);
          i=(int)floor((phi/(2*PI))*nr);
          j=(int)floor((y/height)*ny);
          PSD_N(i, j)++;
          PSD_p(i, j) += p;
          PSD_p2(i, j) += p*p;
         }
        else
          ABSORB;  
       }
     }
    else
     {
      ABSORB;
     }
  """
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
