
def factory( name='toflambda_monitor',nlam=0,nt=0,filename='',t_0=0,t_1=0,xmin=0.0,xmax=0.0,ymin=0.0,ymax=0.0,xwidth=0.0,yheight=0.0,lambda_0=0,lambda_1=0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from TOFLambda_monitorboostpython import TOFLambda_monitor as _factory_mcvine
    return component( _factory_mcvine( name,nlam,nt,filename,t_0,t_1,xmin,xmax,ymin,ymax,xwidth,yheight,lambda_0,lambda_1 ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """KL"""
info.copyright = info_copyright
info_declare = """
  double *TOFL_N_00, *TOFL_p_00, *TOFL_p2_00;
  double tt_0, tt_1;
"""
info.declare = info_declare
info_finalize = """
  free(TOFL_N_00); free(TOFL_p_00); free(TOFL_p2_00);
"""
info.finalize = info_finalize
info_full_description = """2D detector for intensity as a function of both time-of-flight
and wavelength."""
info.full_description = info_full_description
info_initialize = """
    int i,j;

    TOFL_N_00 = (double *)malloc(nt*nlam*sizeof(double));
    TOFL_p_00 = (double *)malloc(nt*nlam*sizeof(double));
    TOFL_p2_00 = (double *)malloc(nt*nlam*sizeof(double));

    if (xwidth  > 0) { xmax = xwidth/2;  xmin = -xmax; }
    if (yheight > 0) { ymax = yheight/2; ymin = -ymax; }

    if ((xmin >= xmax) || (ymin >= ymax)) {
            printf("TOFLambda_monitor: %s: Null detection area !\n"
                   "ERROR              (xwidth,yheight,xmin,xmax,ymin,ymax). Exiting",
           NAME_CURRENT_COMP);
      exit(0);
    }

    tt_0 = t_0*1e-6;
    tt_1 = t_1*1e-6;
    for (i=0; i<nlam; i++)
     for (j=0; j<nt; j++)
     {
      TOFL_N(j, i) = 0;
      TOFL_p(j, i) = 0;
      TOFL_p2(j, i) = 0;
     }
  """
info.initialize = info_initialize
item0 = Parameter()
item0_default = """toflambda_monitor"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """Number of bins in wavelength (1)"""
item1.description = item1_description
item1_name = """nlam"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """Number of bins in TOF (1)"""
item2.description = item2_description
item2_name = """nt"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = """"""
item3.default = item3_default
item3_description = """Name of file in which to store the detector image (string)"""
item3.description = item3_description
item3_name = """filename"""
item3.name = item3_name
item3_type = """char *"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """Minimum time (us)"""
item4.description = item4_description
item4_name = """t_0"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0
item5.default = item5_default
item5_description = """Maximum time (us)"""
item5.description = item5_description
item5_name = """t_1"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0.0
item6.default = item6_default
item6_description = """Lower x bound of detector opening (m)"""
item6.description = item6_description
item6_name = """xmin"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0.0
item7.default = item7_default
item7_description = """Upper x bound of detector opening (m)"""
item7.description = item7_description
item7_name = """xmax"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0.0
item8.default = item8_default
item8_description = """Lower y bound of detector opening (m)"""
item8.description = item8_description
item8_name = """ymin"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0.0
item9.default = item9_default
item9_description = """Upper y bound of detector opening (m)"""
item9.description = item9_description
item9_name = """ymax"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 0.0
item10.default = item10_default
item10_description = """Width/diameter of detector (x). Overrides xmin,xmax. (m)"""
item10.description = item10_description
item10_name = """xwidth"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = 0.0
item11.default = item11_default
item11_description = """Height of detector (y). Overrides ymin,ymax. (m)"""
item11.description = item11_description
item11_name = """yheight"""
item11.name = item11_name
item11_type = """double"""
item11.type = item11_type
item12 = Parameter()
item12_default = 0
item12.default = item12_default
item12_description = """Minimum wavelength detected (AA)"""
item12.description = item12_description
item12_name = """lambda_0"""
item12.name = item12_name
item12_type = """double"""
item12.type = item12_type
item13 = Parameter()
item13_default = 0
item13.default = item13_default
item13_description = """Maximum wavelength detected (AA)"""
item13.description = item13_description
item13_name = """lambda_1"""
item13.name = item13_name
item13_type = """double"""
item13.type = item13_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11, item12, item13 ]
info.input_parameters = info_input_parameters
info_name = """TOFLambda_monitor"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """tt_0"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """"""
item1.description = item1_description
item1_name = """tt_1"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """"""
item2.description = item2_description
item2_name = """TOFL_N_00"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """"""
item3.description = item3_description
item3_name = """TOFL_p_00"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """"""
item4.description = item4_description
item4_name = """TOFL_p2_00"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
info_output_parameters = [ item0, item1, item2, item3, item4 ]
info.output_parameters = info_output_parameters
info_save = """
    DETECTOR_OUT_2D(
        "TOF-wavelength monitor",
        "Time-of-flight [\\gms]", "Wavelength [AA]",
        t_0, t_1, lambda_0, lambda_1,
        nt, nlam,
        TOFL_N_00, TOFL_p_00, TOFL_p2_00,
        filename);
  """
info.save = info_save
item0 = """
// ----------  header for SHARE ----------
inline double & element_2Darr(double *array2D, int dim_y, int i, int j) 
  {
    return *(array2D+j+i*dim_y);
  }
#define TOFL_N( i,  j)  element_2Darr(TOFL_N_00, nlam, i, j)
#define TOFL_p( i,  j)  element_2Darr(TOFL_p_00, nlam, i, j)
#define TOFL_p2( i,  j)  element_2Darr(TOFL_p2_00, nlam, i, j)  
"""
item1 = """// ----------  implementation for SHARE ----------
"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Time-of-flight/wavelength monitor."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
    int i,j;
    double div;
    double lambda;

    PROP_Z0;
    lambda = (2*PI/V2K)/sqrt(vx*vx + vy*vy + vz*vz);
    if (x>xmin && x<xmax && y>ymin && y<ymax &&
        lambda > lambda_0 && lambda < lambda_1)
    {
      if (t < tt_1 && t > tt_0)
      {
        i = floor((lambda - lambda_0)*nlam/(lambda_1 - lambda_0));
        j = floor((t-tt_0)*nt/(tt_1-tt_0));
        TOFL_N(j, i)++;
        TOFL_p(j, i) += p;
        TOFL_p2(j, i) += p*p;
      }
    }
  """
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
