
def factory( name='tof_cylpsd_monitor',nt=0,filename='',nphi=0,radius=0,height=0,t_0=0,t_1=0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from TOF_cylPSD_monitorboostpython import TOF_cylPSD_monitor as _factory_mcvine
    return component( _factory_mcvine( name,nt,filename,nphi,radius,height,t_0,t_1 ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Kim Lefmann"""
info.copyright = info_copyright
info_declare = """
  double *TOF_N_00, *TOF_p_00, *TOF_p2_00;
  double binphi, tt_0, tt_1;
"""
info.declare = info_declare
info_finalize = """
  free(TOF_N_00); free(TOF_p_00); free(TOF_p2_00);
"""
info.finalize = info_finalize
info_full_description = """Cylindrical (2pi) PSD Time-of-flight monitor."""
info.full_description = info_full_description
info_initialize = """
    int i,j;

    TOF_N_00 = (double *)malloc(nt*nphi*sizeof(double));
    TOF_p_00 = (double *)malloc(nt*nphi*sizeof(double));
    TOF_p2_00 = (double *)malloc(nt*nphi*sizeof(double));

    tt_0=t_0*1e-6;
    tt_1=t_1*1e-6;
    for (i=0; i<nt; i++)
      for (j=0; j<nphi; j++)
      {
        TOF_N(i, j) = 0;
        TOF_p(i, j) = 0;
        TOF_p2(i, j) = 0;
      }
    binphi=360.0/(double)nphi;

  """
info.initialize = info_initialize
item0 = Parameter()
item0_default = """tof_cylpsd_monitor"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """Number of time bins (1)"""
item1.description = item1_description
item1_name = """nt"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = """"""
item2.default = item2_default
item2_description = """Name of file in which to store the detector image (text)"""
item2.description = item2_description
item2_name = """filename"""
item2.name = item2_name
item2_type = """char *"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """Number of angular bins (deg)"""
item3.description = item3_description
item3_name = """nphi"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """Cylinder radius (m)"""
item4.description = item4_description
item4_name = """radius"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0
item5.default = item5_default
item5_description = """Cylinder height (m)"""
item5.description = item5_description
item5_name = """height"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0
item6.default = item6_default
item6_description = """Beginning of time window (mu-s)"""
item6.description = item6_description
item6_name = """t_0"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0
item7.default = item7_default
item7_description = """End of time window (mu-s)"""
item7.description = item7_description
item7_name = """t_1"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7 ]
info.input_parameters = info_input_parameters
info_name = """TOF_cylPSD_monitor"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """TOF_N_00"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """"""
item1.description = item1_description
item1_name = """TOF_p_00"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """"""
item2.description = item2_description
item2_name = """TOF_p2_00"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
info_output_parameters = [ item0, item1, item2 ]
info.output_parameters = info_output_parameters
info_save = """
    DETECTOR_OUT_2D(
        "Cylindrical Time-of-flight PSD monitor",
        "Time-of-flight [\\gms]",
        "Angle [deg]",
        t_0, t_1, -180, 180,
        nt, nphi,
        TOF_N_00, TOF_p_00, TOF_p2_00,
        filename);
  """
info.save = info_save
item0 = """
// ----------  header for SHARE ----------
inline double & element_2Darr(double *array2D, int dim_y, int i, int j) 
  {
    return *(array2D+j+i*dim_y);
  }
#define TOF_N( i,  j)  element_2Darr(TOF_N_00, nphi, i, j)
#define TOF_p( i,  j)  element_2Darr(TOF_p_00, nphi, i, j)
#define TOF_p2( i,  j)  element_2Darr(TOF_p2_00, nphi, i, j)  
"""
item1 = """// ----------  implementation for SHARE ----------
"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Cylindrical (2pi) PSD Time-of-flight monitor."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
    int i,j;
    double cyl_t0,cyl_t1,dt,phi;

    if(!cylinder_intersect(&cyl_t0, &cyl_t1, x,y,z,vx,vy,vz, radius, height))
     
      ABSORB;
    if(cyl_t0>0)  
      ABSORB;
    dt=cyl_t1;
    PROP_DT(dt);
    if(y>=height/2 || y<= -height/2)
      ABSORB;  

    i = floor((t-tt_0)*nt/(tt_1-tt_0));              

    if (i < 0 || i >= nt)        
    {
    }
    else
     {
      phi = atan2(x,z);
      j = floor((double)nphi/2.0 + RAD2DEG*phi/(double)binphi);
      TOF_N(i, j)++;
      TOF_p(i, j) += p;
      TOF_p2(i, j) += p*p;
     }
  """
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
