
def factory( name='tof_monitor',nchan=20,filename='',xmin=0.0,xmax=0.0,ymin=0.0,ymax=0.0,xwidth=0.0,yheight=0.0,t0=0.0,t1=0.0,dt=1.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from TOF_monitorboostpython import TOF_monitor as _factory_mcvine
    return component( _factory_mcvine( name,nchan,filename,xmin,xmax,ymin,ymax,xwidth,yheight,t0,t1,dt ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """KN, M. Hagen"""
info.copyright = info_copyright
info_declare = """
    double *TOF_N;
    double *TOF_p;
    double *TOF_p2;
    double t_min, t_max, delta_t;
  """
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """Rectangular Time-of-flight monitor."""
info.full_description = info_full_description
info_initialize = """
    int i;

    TOF_N = new double[nchan];
    TOF_p = new double[nchan];
    TOF_p2 = new double[nchan];

    if (xwidth  > 0) { xmax = xwidth/2;  xmin = -xmax; }
    if (yheight > 0) { ymax = yheight/2; ymin = -ymax; }

    if ((xmin >= xmax) || (ymin >= ymax)) {
            printf("TOF_monitor: %s: Null detection area !\n"
                   "ERROR        (xwidth,yheight,xmin,xmax,ymin,ymax). Exiting",
           NAME_CURRENT_COMP);
      exit(0);
    }

    for (i=0; i<nchan; i++)
    {
      TOF_N[i] = 0;
      TOF_p[i] = 0;
      TOF_p2[i] = 0;
    }
    if (t1!=0)
    {
      t_max=t1;
      t_min=t0;
      delta_t=(t_max-t_min)/nchan;
    }
    else
    {
     delta_t=dt;
     t_min=0;
     t_max=nchan*dt+t0;
    }
  """
info.initialize = info_initialize
item0 = Parameter()
item0_default = """tof_monitor"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 20
item1.default = item1_default
item1_description = """Number of time bins (1)"""
item1.description = item1_description
item1_name = """nchan"""
item1.name = item1_name
item1_type = """int"""
item1.type = item1_type
item2 = Parameter()
item2_default = """"""
item2.default = item2_default
item2_description = """Name of file in which to store the detector image (text)"""
item2.description = item2_description
item2_name = """filename"""
item2.name = item2_name
item2_type = """char *"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0.0
item3.default = item3_default
item3_description = """Lower x bound of detector opening (m)"""
item3.description = item3_description
item3_name = """xmin"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0.0
item4.default = item4_default
item4_description = """Upper x bound of detector opening (m)"""
item4.description = item4_description
item4_name = """xmax"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0.0
item5.default = item5_default
item5_description = """Lower y bound of detector opening (m)"""
item5.description = item5_description
item5_name = """ymin"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0.0
item6.default = item6_default
item6_description = """Upper y bound of detector opening (m)"""
item6.description = item6_description
item6_name = """ymax"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0.0
item7.default = item7_default
item7_description = """Width/diameter of detector (x). Overrides xmin,xmax. (m)"""
item7.description = item7_description
item7_name = """xwidth"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0.0
item8.default = item8_default
item8_description = """Height of detector (y). Overrides ymin,ymax. (m)"""
item8.description = item8_description
item8_name = """yheight"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0.0
item9.default = item9_default
item9_description = """Lower time limit (mu-s)"""
item9.description = item9_description
item9_name = """t0"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 0.0
item10.default = item10_default
item10_description = """Upper time limit (mu-s)"""
item10.description = item10_description
item10_name = """t1"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = 1.0
item11.default = item11_default
item11_description = """Length of each time bin (mu-s)"""
item11.description = item11_description
item11_name = """dt"""
item11.name = item11_name
item11_type = """double"""
item11.type = item11_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11 ]
info.input_parameters = info_input_parameters
info_name = """TOF_monitor"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """Array of neutron counts"""
item0.description = item0_description
item0_name = """TOF_N"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """Array of neutron weight counts"""
item1.description = item1_description
item1_name = """TOF_p"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """Array of second moments"""
item2.description = item2_description
item2_name = """TOF_p2"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """"""
item3.description = item3_description
item3_name = """t_min"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """"""
item4.description = item4_description
item4_name = """t_max"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0
item5.default = item5_default
item5_description = """"""
item5.description = item5_description
item5_name = """delta_t"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
info_output_parameters = [ item0, item1, item2, item3, item4, item5 ]
info.output_parameters = info_output_parameters
info_save = """
    DETECTOR_OUT_1D(
        "Time-of-flight monitor",
        "Time-of-flight [\\gms]",
        "Intensity",
        "t", t_min, t_max, nchan,
        &TOF_N[0],&TOF_p[0],&TOF_p2[0],
        filename);
  """
info.save = info_save
item0 = """"""
item1 = """"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Rectangular Time-of-flight monitor."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
    int i;

    PROP_Z0;
    if (x>xmin && x<xmax && y>ymin && y<ymax)
    {
      i = floor((1E6*t-t_min)/delta_t);              
      if(i >= nchan) i = nchan - 1;
      if(i < 0) i=0;

      TOF_N[i]++;
      TOF_p[i] += p;
      TOF_p2[i] += p*p;
      SCATTER;
    }
  """
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
