
def factory( name='tof_monitor2',nchan=100,xmin=-0.1,xmax=0.1,ymin=-0.1,ymax=0.1,tmin=0.0,tmax=5e-05,filename='tof.dat' ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from TOF_monitor2boostpython import TOF_monitor2 as _factory_mcvine
    return component( _factory_mcvine( name,nchan,xmin,xmax,ymin,ymax,tmin,tmax,filename ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """KN, M. Hagan"""
info.copyright = info_copyright
info_declare = """
    double *TOF_N;
    double *TOF_p;
    double *TOF_p2;
  """
info.declare = info_declare
info_finalize = """
    free(TOF_N); free(TOF_p); free(TOF_p2);
  """
info.finalize = info_finalize
info_full_description = """Rectangular Time-of-flight monitor."""
info.full_description = info_full_description
info_initialize = """
    int i;

    TOF_N = (double *)malloc(nchan*sizeof(double));
    TOF_p = (double *)malloc(nchan*sizeof(double));
    TOF_p2 = (double *)malloc(nchan*sizeof(double));

    for (i=0; i<nchan; i++)
    {
      TOF_N[i] = 0;
      TOF_p[i] = 0;
      TOF_p2[i] = 0;
    }
  """
info.initialize = info_initialize
item0 = Parameter()
item0_default = """tof_monitor2"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 100
item1.default = item1_default
item1_description = """number of time bins (1)"""
item1.description = item1_description
item1_name = """nchan"""
item1.name = item1_name
item1_type = """int"""
item1.type = item1_type
item2 = Parameter()
item2_default = -0.1
item2.default = item2_default
item2_description = """Lower x bound of detector opening (m)"""
item2.description = item2_description
item2_name = """xmin"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0.1
item3.default = item3_default
item3_description = """Upper x bound of detector opening (m)"""
item3.description = item3_description
item3_name = """xmax"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = -0.1
item4.default = item4_default
item4_description = """Lower y bound of detector opening (m)"""
item4.description = item4_description
item4_name = """ymin"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0.1
item5.default = item5_default
item5_description = """Upper y bound of detector opening (m)"""
item5.description = item5_description
item5_name = """ymax"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0.0
item6.default = item6_default
item6_description = """lower t bound of detector (s)"""
item6.description = item6_description
item6_name = """tmin"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 5e-05
item7.default = item7_default
item7_description = """upper t bound of detector (s)"""
item7.description = item7_description
item7_name = """tmax"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = """tof.dat"""
item8.default = item8_default
item8_description = """Name of file in which to store the detector image (text)"""
item8.description = item8_description
item8_name = """filename"""
item8.name = item8_name
item8_type = """char *"""
item8.type = item8_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8 ]
info.input_parameters = info_input_parameters
info_name = """TOF_monitor2"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """Array of neutron counts"""
item0.description = item0_description
item0_name = """TOF_N"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """Array of neutron weight counts"""
item1.description = item1_description
item1_name = """TOF_p"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """Array of second moments"""
item2.description = item2_description
item2_name = """TOF_p2"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
info_output_parameters = [ item0, item1, item2 ]
info.output_parameters = info_output_parameters
info_save = """
    double dt;
     dt=(tmax-tmin)/nchan;
    DETECTOR_OUT_1D(
        "Time-of-flight monitor",
        "Time-of-flight [\\gms]",
        "Intensity",
        "t", tmin*1.0e6, (tmin+nchan*(double)dt)*1.0e6, nchan,
        &TOF_N[0],&TOF_p[0],&TOF_p2[0],
        filename);
  """
info.save = info_save
item0 = """"""
item1 = """"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Rectangular Time-of-flight monitor."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
    int i;
     double dt;
     dt=(tmax-tmin)/nchan;
    PROP_Z0;
//    if (x>xmin && x<xmax && y>ymin && y<ymax && t>=tmin && t<=tmax)
    if (x>xmin && x<xmax && y>ymin && y<ymax )
    {
    
      i = floor((t-tmin)/dt);              
      if(i >= nchan) i = nchan - 1;
      if(i < 0)
      {
        //printf("FATAL ERROR: negative time-of-flight.\n");
        //exit(1);
	i = 0;
      }
      TOF_N[i]++;
      TOF_p[i] += p;
      TOF_p2[i] += p*p;
      SCATTER;
    }
  """
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
