
def factory( name='toflog_mon',tlow=0,thigh=0,ndec=0,filename='',xmin=0.0,xmax=0.0,ymin=0.0,ymax=0.0,xwidth=0.0,yheight=0.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from TOFlog_monboostpython import TOFlog_mon as _factory_mcvine
    return component( _factory_mcvine( name,tlow,thigh,ndec,filename,xmin,xmax,ymin,ymax,xwidth,yheight ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Kim Lefmann"""
info.copyright = info_copyright
info_declare = """
    double *TOF_N;
    double *TOF_p;
    double *TOF_p2;
    int nchan;
  """
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """A rectangular time-of-flight monitor with logarithmic time binning.
(The neutron intensity is NOT given logarithmically)"""
info.full_description = info_full_description
info_initialize = """
    TOF_N = (double *)malloc(LARGENUMBER*sizeof(double));
    TOF_p = (double *)malloc(LARGENUMBER*sizeof(double));
    TOF_p2 = (double *)malloc(LARGENUMBER*sizeof(double));

    int i;
    if (xwidth  > 0) { xmax = xwidth/2;  xmin = -xmax; }
    if (yheight > 0) { ymax = yheight/2; ymin = -ymax; }

    if ((xmin >= xmax) || (ymin >= ymax)) {
            printf("TOFlog_mon: %s: Null detection area !\n"
                   "ERROR       (xwidth,yheight,xmin,xmax,ymin,ymax). Exiting",
           NAME_CURRENT_COMP);
      exit(0);
    }

    nchan=(int)ceil(ndec*log(thigh/tlow)/log(10.0));
    if (nchan>LARGENUMBER)
      printf("FATAL ERROR, too many time channels \n");

    for (i=0; i<nchan; i++)
    {
      TOF_N[i] = 0;
      TOF_p[i] = 0;
      TOF_p2[i] = 0;
    }
  """
info.initialize = info_initialize
item0 = Parameter()
item0_default = """toflog_mon"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """Lower bound for time bins (mus)"""
item1.description = item1_description
item1_name = """tlow"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """"""
item2.description = item2_description
item2_name = """thigh"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """Number of time bins per decade (1)"""
item3.description = item3_description
item3_name = """ndec"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = """"""
item4.default = item4_default
item4_description = """Name of file in which to store the detector image (text)"""
item4.description = item4_description
item4_name = """filename"""
item4.name = item4_name
item4_type = """char *"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0.0
item5.default = item5_default
item5_description = """Lower x bound of detector opening (m)"""
item5.description = item5_description
item5_name = """xmin"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0.0
item6.default = item6_default
item6_description = """Upper x bound of detector opening (m)"""
item6.description = item6_description
item6_name = """xmax"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0.0
item7.default = item7_default
item7_description = """Lower y bound of detector opening (m)"""
item7.description = item7_description
item7_name = """ymin"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0.0
item8.default = item8_default
item8_description = """Upper y bound of detector opening (m)"""
item8.description = item8_description
item8_name = """ymax"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0.0
item9.default = item9_default
item9_description = """Width/diameter of detector (x). Overrides xmin,xmax. (m)"""
item9.description = item9_description
item9_name = """xwidth"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 0.0
item10.default = item10_default
item10_description = """Height of detector (y). Overrides ymin,ymax. (m)"""
item10.description = item10_description
item10_name = """yheight"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10 ]
info.input_parameters = info_input_parameters
info_name = """TOFlog_mon"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """Array of neutron counts"""
item0.description = item0_description
item0_name = """TOF_N"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """Array of neutron weight counts"""
item1.description = item1_description
item1_name = """TOF_p"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """Array of second moments"""
item2.description = item2_description
item2_name = """TOF_p2"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
info_output_parameters = [ item0, item1, item2 ]
info.output_parameters = info_output_parameters
info_save = """
    DETECTOR_OUT_1D(
        "Time-of-flight monitor",
        "Log(Time-of-flight [\\gms])",
        "Intensity",
        "t", log(tlow)/log(10.0), log(thigh)/log(10.0), nchan,
        &TOF_N[0],&TOF_p[0],&TOF_p2[0],
        filename);

  """
info.save = info_save
item0 = """
// ----------  header for SHARE ----------
#define LARGENUMBER 10000"""
item1 = """// ----------  implementation for SHARE ----------

"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Rectangular Time-of-flight monitor with logarithmic time binning."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
    int i;

    PROP_Z0;
    if (x>xmin && x<xmax && y>ymin && y<ymax)
    {
      i = (int)floor(ndec*log(1E6*t/tlow)/log(10.0));         
      if(i >= nchan)
        i = nchan - 1;
      if(i < 0)
        i = 0;
      TOF_N[i]++;
      TOF_p[i] += p;
      TOF_p2[i] += p*p;
    }
  """
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
