
def factory( name='bender',w=0,h=0,r=0,Win=0.04,k=1.0,d=0.001,R0a=0.99,Qca=0.021,alphaa=6.07,ma=2.0,Wa=0.003,R0i=0.99,Qci=0.021,alphai=6.07,mi=2.0,Wi=0.003,R0s=0.99,Qcs=0.021,alphas=6.07,ms=2.0,Ws=0.003,l=0.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from Benderboostpython import Bender as _factory_mcvine
    return component( _factory_mcvine( name,w,h,r,Win,k,d,R0a,Qca,alphaa,ma,Wa,R0i,Qci,alphai,mi,Wi,R0s,Qcs,alphas,ms,Ws,l ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Philipp Bernhardt"""
info.copyright = info_copyright
info_declare = """
      double bk, mWin;
 """
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """Models a curved neutron guide with cylindrical walls.

Bender radius, entrance width and height are necessary input data. To define
the bender, you may either enter the deviation angle 'Win' or the length 'l'.
The bender may consist of 'k' vertical channels, separated by partitioning walls
of thickness 'd'. Three different reflectivity profiles can be given: for outer
walls, for inner walls and for the top and bottom walls. The partitioning walls
have the same coating as the exterior walls.

The entrance lies in the X-Y plane, centered on the Z axis. The neutrons will
also leave the bender in the X-Y plane at the z-value l=r*Win, i.e. they are
centred on (0,0,r*Win); they have an (average) flight direction along the z-axis.
Therefore, the following component is adjacent, if positioned AT (0,0,r*Win)
<b>without</b> rotation.
So, seen from outside, it behaves like a straight guide along the Z axis. As a
consequence, it is shown straight in 'mcdisplay'.
This behaviour results from a co-ordinate transformation inside the component.
It is done to facilitate its use. Neither rotation nor shift along the x-axis
need to be calculated; a new arm is not necessary. Internally, the bender is
bent to the negative X axis;

Bender(name, w=0.05,h=0.12,r=250,Win=0.04,
R0a=0.99,Qca=0.021,alphaa=6.07,ma=2,Wa=0.003,
R0i=0.99,Qci=0.021,alphai=6.07,mi=2,Wi=0.003,
R0s=0.99,Qcs=0.021,alphas=6.07,ms=2,Ws=0.003)

%BUGS
Some users have reported potentially strange behaviours with this component.
This component does not work with gravitation on."""
info.full_description = info_full_description
info_initialize = """
      if (r <0)
      { fprintf(stderr,"Bender: error: %s: to bend in the other direction\n", NAME_CURRENT_COMP);
        fprintf(stderr,"        rotate comp on z-axis by 180 deg.\n"); exit(-1); }

      if (k*d > w)
      { fprintf(stderr,"Bender: error: %s has (k*d > w).\n", NAME_CURRENT_COMP);
        exit(-1); }
      if (w*h*r*Win*k == 0)
      { fprintf(stderr,"Bender: error: %s has one of w,h,r,Win,k null.\n", NAME_CURRENT_COMP);
        exit(-1); }
      
      mWin = Win;
      if (l!= 0 && r != 0) mWin = (double)l/(double)r;
      bk=(w+d)/k;

 """
info.initialize = info_initialize
item0 = Parameter()
item0_default = """bender"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """(m)          Width at the bender entry and exit"""
item1.description = item1_description
item1_name = """w"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """(m)          Height at the bender entry and exit"""
item2.description = item2_description
item2_name = """h"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """(m)          Radius of the bender"""
item3.description = item3_description
item3_name = """r"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0.04
item4.default = item4_default
item4_description = """(rad)        Angle of the deflection of the whole bender"""
item4.description = item4_description
item4_name = """Win"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 1.0
item5.default = item5_default
item5_description = """(1)          Number of channels inside the bender"""
item5.description = item5_description
item5_name = """k"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0.001
item6.default = item6_default
item6_description = """(m)          Thickness of one blade separating the channels"""
item6.description = item6_description
item6_name = """d"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0.99
item7.default = item7_default
item7_description = """(1)          Low-angle reflectivity at the <b>outer</b> side of the bender"""
item7.description = item7_description
item7_name = """R0a"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0.021
item8.default = item8_default
item8_description = """(AA-1)       Critical scattering vector"""
item8.description = item8_description
item8_name = """Qca"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 6.07
item9.default = item9_default
item9_description = """(AA)         Slope of reflectivity"""
item9.description = item9_description
item9_name = """alphaa"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 2.0
item10.default = item10_default
item10_description = """(1)          m-value of material"""
item10.description = item10_description
item10_name = """ma"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = 0.003
item11.default = item11_default
item11_description = """(AA-1)       Width of supermirror cut-off"""
item11.description = item11_description
item11_name = """Wa"""
item11.name = item11_name
item11_type = """double"""
item11.type = item11_type
item12 = Parameter()
item12_default = 0.99
item12.default = item12_default
item12_description = """(1)          Low-angle reflectivity at the <b>inner</b> side of the bender"""
item12.description = item12_description
item12_name = """R0i"""
item12.name = item12_name
item12_type = """double"""
item12.type = item12_type
item13 = Parameter()
item13_default = 0.021
item13.default = item13_default
item13_description = """(AA-1)       Critical scattering vector"""
item13.description = item13_description
item13_name = """Qci"""
item13.name = item13_name
item13_type = """double"""
item13.type = item13_type
item14 = Parameter()
item14_default = 6.07
item14.default = item14_default
item14_description = """(AA)         Slope of reflectivity"""
item14.description = item14_description
item14_name = """alphai"""
item14.name = item14_name
item14_type = """double"""
item14.type = item14_type
item15 = Parameter()
item15_default = 2.0
item15.default = item15_default
item15_description = """(1)          m-value of material"""
item15.description = item15_description
item15_name = """mi"""
item15.name = item15_name
item15_type = """double"""
item15.type = item15_type
item16 = Parameter()
item16_default = 0.003
item16.default = item16_default
item16_description = """(AA-1)       Width of supermirror cut-off"""
item16.description = item16_description
item16_name = """Wi"""
item16.name = item16_name
item16_type = """double"""
item16.type = item16_type
item17 = Parameter()
item17_default = 0.99
item17.default = item17_default
item17_description = """(1)          Low-angle reflectivity at the <b>top and bottom</b> side of the bender"""
item17.description = item17_description
item17_name = """R0s"""
item17.name = item17_name
item17_type = """double"""
item17.type = item17_type
item18 = Parameter()
item18_default = 0.021
item18.default = item18_default
item18_description = """(AA-1)       Critical scattering vector"""
item18.description = item18_description
item18_name = """Qcs"""
item18.name = item18_name
item18_type = """double"""
item18.type = item18_type
item19 = Parameter()
item19_default = 6.07
item19.default = item19_default
item19_description = """(AA)         Slope of reflectivity"""
item19.description = item19_description
item19_name = """alphas"""
item19.name = item19_name
item19_type = """double"""
item19.type = item19_type
item20 = Parameter()
item20_default = 2.0
item20.default = item20_default
item20_description = """(1)          m-value of material"""
item20.description = item20_description
item20_name = """ms"""
item20.name = item20_name
item20_type = """double"""
item20.type = item20_type
item21 = Parameter()
item21_default = 0.003
item21.default = item21_default
item21_description = """(AA-1)       Width of supermirror cut-off"""
item21.description = item21_description
item21_name = """Ws"""
item21.name = item21_name
item21_type = """double"""
item21.type = item21_type
item22 = Parameter()
item22_default = 0.0
item22.default = item22_default
item22_description = """(m)          length of bender l=r*Win"""
item22.description = item22_description
item22_name = """l"""
item22.name = item22_name
item22_type = """double"""
item22.type = item22_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11, item12, item13, item14, item15, item16, item17, item18, item19, item20, item21, item22 ]
info.input_parameters = info_input_parameters
info_name = """Bender"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """(m)          Width of 1 channel + 1 separating blade"""
item0.description = item0_description
item0_name = """bk"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """(rad)        Angle of the deflection of the whole bender"""
item1.description = item1_description
item1_name = """mWin"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
info_output_parameters = [ item0, item1 ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """
// ----------- added by Jiao Lin --------
// ----------  header for SHARE ----------
// ----------- added by Jiao Lin --------

#ifndef BENDER_DECLARE
#define BENDER_DECLARE
double bd_sgn(double x);
#endif


// ----------- added by Jiao Lin --------"""
item1 = """// ----------  implementation for SHARE ----------
// ----------- added by Jiao Lin --------

double bd_sgn(double x)
{
  if (x>=0)
     return 1.0;
  else
     return -1.0;
}

 """
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Models a curved neutron guide."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
    int i,num,numa,numi;
    double dru,ab,dab,R,Q,arg,arga,argi,Ta,vpl;
    double einmWin,ausmWin,zykmWin,aeumWin,innmWin,ref,innref,aeuref;
    double einzei,auszei,zykzei;

    
    PROP_Z0;
    if ((fabs(x)<w/2) && (fabs(y)<h/2))
    {
      

      
      dru=floor((w/2-x)/bk)*bk;
      ab=w/2.0-x-dru;

      
      R=r-dru;

      
      if (ab<bk-d)
      {
        
        vpl=sqrt(vx*vx+vz*vz);

        
        einmWin=atan(vx/vz);

        
        dab=R-cos(einmWin)*(R-ab);

        
        aeumWin=acos((R-dab)/R);

        
        arga=0.0;
        Q=2.0*V2K*vpl*sin(aeumWin);
        if (Q<=Qca)
           aeuref=R0a;
        else {
           arga=(Q-ma*Qca)/Wa;
           aeuref=0.5*R0a*(1.0-tanh(arga))*(1.0-alphaa*(Q-Qca)); }

        
        innmWin=0.0;
        innref=1.0;
        argi=0.0;
        if (dab>bk-d)
        {
           
           innmWin=acos((R-dab)/(R-bk+d));
           Q=2.0*V2K*vpl*sin(innmWin);
           if (Q<=Qci)
              innref=R0i;
           else {
              argi=(Q-mi*Qci)/Wi;
              innref=0.5*R0i*(1.0-tanh(argi))*(1.0-alphai*(Q-Qci)); }
        }

        
        zykmWin=2.0*(aeumWin-innmWin);
        ausmWin=fmod(mWin+einmWin+aeumWin-innmWin
          *(1.0+bd_sgn(einmWin)),zykmWin)-zykmWin/2.0;
        ausmWin+=innmWin*bd_sgn(ausmWin);

        
        numa=(mWin+einmWin+aeumWin-innmWin*(1.0+bd_sgn(einmWin)))/zykmWin;

        
        numi=numa;
        if (ausmWin*einmWin<0)
        {
           if (ausmWin-einmWin>0)
              numi++;
           else
              numi--;
        }

        
        if (((numa>0) && (arga>10.0)) || ((numi>0) && (argi>10.0)))
           ABSORB;

        
        for (i=1;i<=numa;i++)
            p*=aeuref;
        for (i=1;i<=numi;i++)
            p*=innref;

        
        Ta=(2*numa*(tan(aeumWin)-tan(innmWin))
          +tan(ausmWin)-tan(einmWin)
          -tan(innmWin)*(bd_sgn(ausmWin)-bd_sgn(einmWin)))
          *(R-dab)/vpl;
        t+=Ta;

        
        ab=R-(R-dab)/cos(ausmWin);

        
        x=w/2.0-ab-dru;
        z=r*mWin;
        vx=sin(ausmWin)*vpl;
        vz=cos(ausmWin)*vpl;

        

        if (vy!=0.0)
        {
          
          Q=2.0*V2K*fabs(vy);
          if (Q<=Qcs) {
             ref=R0s; arg=0; }
          else {
             arg=(Q-ms*Qcs)/Ws;
             ref=0.5*R0s*(1.0-tanh(arg))*(1.0-alphas*(Q-Qcs)); }

          
          einzei=h/2.0/fabs(vy)+y/vy;
          zykzei=h/fabs(vy);
          num=(Ta+einzei)/zykzei;

          
          auszei=fmod(Ta+einzei,zykzei);

          
          if ((num>0) && (arg>10.0))
             ABSORB;

          
          for (i=1;i<=num;i++) {
               p*=ref;
               vy*=-1.0; }

          
          y=auszei*vy-vy*h/fabs(vy)/2.0;
        } 
        SCATTER;
      } 
      else
        ABSORB; 
    }
    else 
      ABSORB; 

 """
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
