
def factory( name='collimator_linear',xmin=0,xmax=0,ymin=0,ymax=0,len=0,divergence=40.0,transmission=1.0,divergenceV=0.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from Collimator_linearboostpython import Collimator_linear as _factory_mcvine
    return component( _factory_mcvine( name,xmin,xmax,ymin,ymax,len,divergence,transmission,divergenceV ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Kristian Nielsen"""
info.copyright = info_copyright
info_declare = """
  double slope, slopeV;
"""
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """Soller collimator with rectangular opening and specified length. The
transmission function is an average and does not utilize knowledge of the
actual neutron trajectory. A zero divergence disables collimation (then the
component works as a double slit)."""
info.full_description = info_full_description
info_initialize = """
  slope = tan(MIN2RAD*divergence);
  slopeV= tan(MIN2RAD*divergenceV);
"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """collimator_linear"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """(m)              Lower x bound on slits"""
item1.description = item1_description
item1_name = """xmin"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """(m)              Upper x bound on slits"""
item2.description = item2_description
item2_name = """xmax"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """(m)              Lower y bound on slits"""
item3.description = item3_description
item3_name = """ymin"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """(m)              Upper y bound on slits"""
item4.description = item4_description
item4_name = """ymax"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0
item5.default = item5_default
item5_description = """(m)              Distance between slits"""
item5.description = item5_description
item5_name = """len"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 40.0
item6.default = item6_default
item6_description = """(minutes of arc) Divergence horizontal angle (calculated as atan(d/len),"""
item6.description = item6_description
item6_name = """divergence"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 1.0
item7.default = item7_default
item7_description = """(1)              Transmission of Soller (0<=t<=1)"""
item7.description = item7_description
item7_name = """transmission"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0.0
item8.default = item8_default
item8_description = """(minutes of arc) Divergence vertical angle"""
item8.description = item8_description
item8_name = """divergenceV"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8 ]
info.input_parameters = info_input_parameters
info_name = """Collimator_linear"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """slope"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """"""
item1.description = item1_description
item1_name = """slopeV"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
info_output_parameters = [ item0, item1 ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """"""
item1 = """"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """A simple analytical Soller collimator (with triangular transmission)."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
    double phi, dt;

    PROP_Z0;
    if (x<xmin || x>xmax || y<ymin || y>ymax)
      ABSORB;
    dt = len/vz;
    PROP_DT(dt);
    if (x<xmin || x>xmax || y<ymin || y>ymax)
      ABSORB;

    if(slope > 0.0)
    {
      phi = fabs(vx/vz);
      if (phi > slope)
        ABSORB;
      else
        p *= transmission*(1.0 - phi/slope);
      SCATTER;
    }
    if (slopeV > 0) {
      phi = fabs(vy/vz);
      if (phi > slopeV)
        ABSORB;
      else
        p *= transmission*(1.0 - phi/slopeV);
      SCATTER;
    }
"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
