
def factory( name='diskchopper',theta_0=0,R=0,h=0.0,omega=0,n=3.0,j=0.0,theta_1=0.0,t_0=0.0,IsFirst=0.0,n_pulse=1.0,abs_out=1.0,phi_0=0.0,w=0.0,wc=0.0,compat=0.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from DiskChopperboostpython import DiskChopper as _factory_mcvine
    return component( _factory_mcvine( name,theta_0,R,h,omega,n,j,theta_1,t_0,IsFirst,n_pulse,abs_out,phi_0,w,wc,compat ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Peter Willendrup"""
info.copyright = info_copyright
info_declare = """
  double Tg,To,Tc,delta_y,height;
 """
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """Models a disc chopper with n identical slits, which are symmetrically disposed
on the disc. At time t=0, the centre of the first slit opening will be situated at the
vertical axis.

Possibly, the slits can have central absorbing inserts ('beamstops'). 

For more complicated gemometries, see component manual example of DiskChopper GROUPing.

If the chopper is the 1st chopper of the instrument, it sets t time with phase
- Only relevant for when using continuous source modules.

Example values: theta_0=10 h=0.2 R=0.5 omega=2500 n=3 t_0=0 n_pulse=1 j=0 wc=0"""
info.full_description = info_full_description
info_initialize = """
      
      if (h == 0) {
	height=R;
      } else {
	height=h;
      }
      
      if (w) {
	theta_0=2*RAD2DEG*asin((w/2)/R);
      }
      if (wc) {
	theta_1=2*RAD2DEG*asin((wc/2)/R);
      }
      if (compat) {
	delta_y = R;
      } else {
	delta_y = R-height/2;
      }
      
      if (n<=0 || theta_0 < 0 || R <=0)
      { fprintf(stderr,"DiskChopper: %s: n, theta_0 and R must be > 0\n", NAME_CURRENT_COMP);
        exit(-1); }
      if (n*theta_0 >= 360)
      { fprintf(stderr,"DiskChopper: %s: n * theta_0 exceeds 2PI\n", NAME_CURRENT_COMP);
        exit(-1); }
      if (theta_1 >= theta_0)
      { fprintf(stderr,"DiskChopper: %s: theta_1 must me lower than theta_0\n", NAME_CURRENT_COMP);
        exit(-1); }
      if (h && h>R) {
	fprintf(stderr,"DiskChopper: %s: h must be < R\n", NAME_CURRENT_COMP);
        exit(-1); }
      if (IsFirst && n_pulse <=0)
      { fprintf(stderr,"DiskChopper: %s: wrong First chopper pulse number (n_pulse)\n", NAME_CURRENT_COMP);
        exit(-1); }
      if (!omega) {
	fprintf(stderr,"DiskChopper: %s WARNING: chopper frequency is 0!\n", NAME_CURRENT_COMP);
	omega = 1e-15; 
      }
      if (!(abs_out==1)) {
	fprintf(stderr,"DiskChopper: %s WARNING: chopper will NOT absorb neutrons outside radius R\n", NAME_CURRENT_COMP);
	abs_out=0;
      }
      if (IsFirst && theta_1) {
	fprintf(stderr,"DiskChopper: %s WARNING: Options IsFirst and beamstops not compatible. Beamstops removed.\n", NAME_CURRENT_COMP);
	theta_1=0;
      }
      
      theta_0*=DEG2RAD;
      theta_1*=DEG2RAD;

      
      
      t_0 *=omega/fabs(omega);
      if (phi_0) {
	if (t_0) {
	  fprintf(stderr,"DiskChopper: %s WARNING: t_0 AND phi_0 specified. Using phi_0 setting\n", NAME_CURRENT_COMP);  
	}
	phi_0*=DEG2RAD;
	
	t_0=omega*phi_0/(omega*omega);
      } else {
	phi_0=t_0*omega;
      } 
      
      
      Tg=2*PI/fabs(omega)/n;
      
      
      To=theta_0/fabs(omega);
      
      
      Tc=theta_1/fabs(omega);
 """
info.initialize = info_initialize
item0 = Parameter()
item0_default = """diskchopper"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """(deg)    Angular width of the slits."""
item1.description = item1_description
item1_name = """theta_0"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """(m)      Radius of the disc"""
item2.description = item2_description
item2_name = """R"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0.0
item3.default = item3_default
item3_description = """(m)      Slit height (if = 0, equal to R). Auto centering of beam at h/2."""
item3.description = item3_description
item3_name = """h"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """(rad/s)  Angular frequency of the Chopper"""
item4.description = item4_description
item4_name = """omega"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 3.0
item5.default = item5_default
item5_description = """(1)      Number of slits"""
item5.description = item5_description
item5_name = """n"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0.0
item6.default = item6_default
item6_description = """"""
item6.description = item6_description
item6_name = """j"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0.0
item7.default = item7_default
item7_description = """(deg)    Angular width of optional beamstop in chopper windows"""
item7.description = item7_description
item7_name = """theta_1"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0.0
item8.default = item8_default
item8_description = """(s)      Time 'delay'."""
item8.description = item8_description
item8_name = """t_0"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0.0
item9.default = item9_default
item9_description = """(0/1)    Set it to 1 for the first chopper position in a cw source"""
item9.description = item9_description
item9_name = """IsFirst"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 1.0
item10.default = item10_default
item10_description = """(1)      Number of pulses (Only if IsFirst)"""
item10.description = item10_description
item10_name = """n_pulse"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = 1.0
item11.default = item11_default
item11_description = """(0/1)    Absorb neutrons hitting outside of chopper radius?"""
item11.description = item11_description
item11_name = """abs_out"""
item11.name = item11_name
item11_type = """double"""
item11.type = item11_type
item12 = Parameter()
item12_default = 0.0
item12.default = item12_default
item12_description = """(deg)    Angular 'delay' (suppresses t_0)"""
item12.description = item12_description
item12_name = """phi_0"""
item12.name = item12_name
item12_type = """double"""
item12.type = item12_type
item13 = Parameter()
item13_default = 0.0
item13.default = item13_default
item13_description = """(m)      'width' of slits for compatibility with Chopper.comp"""
item13.description = item13_description
item13_name = """w"""
item13.name = item13_name
item13_type = """double"""
item13.type = item13_type
item14 = Parameter()
item14_default = 0.0
item14.default = item14_default
item14_description = """(m)      'width' of beamstops for compatibility with Chopper.comp"""
item14.description = item14_description
item14_name = """wc"""
item14.name = item14_name
item14_type = """double"""
item14.type = item14_type
item15 = Parameter()
item15_default = 0.0
item15.default = item15_default
item15_description = """(1)      Chopper placement compatible with original Chopper.comp"""
item15.description = item15_description
item15_name = """compat"""
item15.name = item15_name
item15_type = """double"""
item15.type = item15_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11, item12, item13, item14, item15 ]
info.input_parameters = info_input_parameters
info_name = """DiskChopper"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """Tg"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """"""
item1.description = item1_description
item1_name = """T"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """"""
item2.description = item2_description
item2_name = """To"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """"""
item3.description = item3_description
item3_name = """Tc"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """"""
item4.description = item4_description
item4_name = """delta_y"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0
item5.default = item5_default
item5_description = """"""
item5.description = item5_description
item5_name = """height"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
info_output_parameters = [ item0, item1, item2, item3, item4, item5 ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """"""
item1 = """"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Kaspar Hewitt Klenoe (jan 2006), adjustments by Rob Bewey (march 2006)"""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
    double toff;
    double yprime;

    yprime = y+delta_y;
    PROP_Z0;
    
    
    if ((x*x+yprime*yprime)>R*R && abs_out==1) {
      ABSORB;
    }
    
    if ((x*x+yprime*yprime)<(R-height)*(R-height)) {
      ABSORB;
    }
    
    
    if (IsFirst)
      {
	t=atan2(x,yprime)/omega + To*randpm1()/2.0 - t_0 + j*randnorm() + floor(n_pulse*rand01())*Tg;
	p*=n*theta_0/2/PI;
      }
    else 
      {
	toff=fabs(t-atan2(x,yprime)/omega - t_0 + j*randnorm());
	
	if (fmod(toff+Tc/2.0,Tg)<Tc) ABSORB;
	
	if (fmod(toff+To/2.0,Tg)>To) ABSORB; 
      }
    SCATTER;
    
 """
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
