
def factory( name='fermichopper',phase=0.0,radius=0.04,nu=100.0,ymin=0.0,ymax=0.0,w=0.00022475,Nslit=200.0,R0=0.0,Qc=0.02176,alpha=2.33,m=0.0,W=0.002,length=0.012,eff=0.95,zero_time=0.0,width=0.0,verbose=0.0,height=0.08,rad=0.0,slit=0.0,alpham=0.0,Wi=0.0,dist=0.0,Vi=0.0,curvature=0.0,time=0.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from FermiChopperboostpython import FermiChopper as _factory_mcvine
    return component( _factory_mcvine( name,phase,radius,nu,ymin,ymax,w,Nslit,R0,Qc,alpha,m,W,length,eff,zero_time,width,verbose,height,rad,slit,alpham,Wi,dist,Vi,curvature,time ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """M. Poehlmann, C. Carbogno, H. Schober, E. Farhi"""
info.copyright = info_copyright
info_declare = """
     struct FermiChopper_struct FCVars;
"""
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """Models a fermi chopper with optional supermirror coated blades
supermirror facilities may be disabled by setting m = 0, R0=0
Slit packages are straight. Chopper slits are separated by an infinitely
thin absorbing material. The effective transmission (resulting from fraction
of the transparent material and its transmission) may be specified.
The chopper slit package width may be specified through the total width 'width'
of the full package or the width 'w' of each single slit. The other parameter
is calculated by: width = Nslit*w.
The chopper needs a default velocity to calculate the initial value for an
iterative calculation of TOF values. This can be specified by its velocity,
energy, wavelength or wavevector.

%VALIDATION
Apr 2005: extensive external test, most problems solved (cf. 'Bugs')
Validated by: K. Lieutenant

limitations:
no absorbing blade width used

%BUGS
- overestimates peak width for long wavelengths
- may not give the right pulse position, shape and width for slit widths below 0.1 mm"""
info.full_description = info_full_description
info_initialize = """


  FCVars.omega    = 2*PI*nu;
  if (!phase && time) {
     FCVars.ph0= fmod(-time*nu*360,360)*DEG2RAD;
  } else FCVars.ph0      = phase*DEG2RAD;
  FCVars.sum_t=FCVars.sum_v=FCVars.sum_N=FCVars.sum_N_pass=0;

  
  if (rad)    radius=rad;
  if (slit)   length=slit;
  if (alpham) alpha =alpham;
  if (Wi)     W     =Wi;
  if (dist)
    printf("FermiChopper: %s: Parameter 'dist' is not valid anymore.\n"
           "WARNING       Use 'phase' instead. Ignoring.\n", NAME_CURRENT_COMP);
  
  if (Nslit < 1) Nslit=1;
  if (height>0) { ymax = height/2; ymin=-ymax; }
  if (ymax-ymin <= 0) exit(printf("FermiChopper: %s: FATAL: unrealistic cylinder height =%g [m]\n", NAME_CURRENT_COMP, ymax-ymin));
  height = fabs(ymax-ymin);
  ymax= height/2;
  ymin=-ymax;

  if (m <= 0) { m=0; R0=0; }
  if (radius <= 0) {
    printf("FermiChopper: %s: FATAL: unrealistic cylinder radius radius=%g [m]\n", NAME_CURRENT_COMP, radius);
    exit(-1);
  }
  if (width > 0 && width < radius*2 && Nslit > 0) {
    w = width/Nslit;
  }
  if (w <= 0) {
    printf("FermiChopper: %s: FATAL: Slits in the package have unrealistic width w=%g [m]\n", NAME_CURRENT_COMP, w);
    exit(-1);
  }
  if (Nslit*w > radius*2) {
    Nslit = floor(radius/w);
    printf("FermiChopper: %s: Too many slits to fit in the cylinder\n"
           "    Adjusting Nslit=%f\n", NAME_CURRENT_COMP, Nslit);
  }
  if (length > radius*2) {
    length = 2*sqrt(radius*radius - Nslit*w*Nslit*w/4);
    printf("FermiChopper: %s: Slit package is longer than the whole\n"
           "    chopper cylinder. Adjusting length=%g [m]\n", NAME_CURRENT_COMP, length);
  }
  if (Nslit == 1) {
    length = 2*sqrt(radius*radius - Nslit*w*Nslit*w/4);
    printf("FermiChopper: %s: No slit package (Nslit=1). Adjusting length=%g [m]\n",
      NAME_CURRENT_COMP, length);
  }

  if (eff <= 0 || eff > 1) {
    eff = 0.95;
    printf("FermiChopper: %s: Efficiency is unrealistic\n"
           "    Adjusting eff=%f\n", NAME_CURRENT_COMP, eff);
  }
  if (Qc <= 0) { Qc = 0.02176; m = 0; R0 = 0; }
  if (W <= 0) W=1e-6;

  if (curvature) {
    FCVars.C_slit = curvature;
    if (1 < fabs(radius*curvature))
      exit(printf("FermiChopper: %s: Slit curvature is unrealistic\n",
           NAME_CURRENT_COMP));
  }
  FCVars.L_slit = length;
  if (verbose && nu)
    printf("FermiChopper: %s: frequency=%g [Hz] %g [rpm], time frame=%g [s] phase=%g [deg]\n"
      , NAME_CURRENT_COMP, nu, nu*60, 2/nu, FCVars.ph0*RAD2DEG);

  FCVars.absorb_alreadyinside    = 0;
  FCVars.absorb_topbottom        = 0;
  FCVars.absorb_cylentrance      = 0;
  FCVars.absorb_sideentrance     = 0;
  FCVars.absorb_notreachentrance = 0;
  FCVars.absorb_packentrance     = 0;
  FCVars.absorb_slitcoating      = 0;
  FCVars.warn_notreachslitwall = 0;
  FCVars.absorb_exitslitpack     = 0;
  FCVars.absorb_maxiterations    = 0;
  FCVars.absorb_wrongdirection   = 0;
  FCVars.absorb_nocontrol        = 0;
  FCVars.absorb_cylexit          = 0;
  FCVars.warn_notreachslitoutput=0;

"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """fermichopper"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0.0
item1.default = item1_default
item1_description = """(deg)  chopper phase at t=0"""
item1.description = item1_description
item1_name = """phase"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0.04
item2.default = item2_default
item2_description = """(m)   chopper cylinder radius"""
item2.description = item2_description
item2_name = """radius"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 100.0
item3.default = item3_default
item3_description = """(Hz)  chopper frequency"""
item3.description = item3_description
item3_name = """nu"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0.0
item4.default = item4_default
item4_description = """(m)   lower y bound of cylinder"""
item4.description = item4_description
item4_name = """ymin"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0.0
item5.default = item5_default
item5_description = """(m)   upper y bound of cylinder"""
item5.description = item5_description
item5_name = """ymax"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0.00022475
item6.default = item6_default
item6_description = """(m)   width of one chopper slit"""
item6.description = item6_description
item6_name = """w"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 200.0
item7.default = item7_default
item7_description = """(1)   number of chopper slits"""
item7.description = item7_description
item7_name = """Nslit"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0.0
item8.default = item8_default
item8_description = """(1)    low-angle reflectivity"""
item8.description = item8_description
item8_name = """R0"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0.02176
item9.default = item9_default
item9_description = """(AA-1) critical scattering vector"""
item9.description = item9_description
item9_name = """Qc"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 2.33
item10.default = item10_default
item10_description = """(AA)   slope of reflectivity"""
item10.description = item10_description
item10_name = """alpha"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = 0.0
item11.default = item11_default
item11_description = """(1)    m-value of material. Zero means completely absorbing."""
item11.description = item11_description
item11_name = """m"""
item11.name = item11_name
item11_type = """double"""
item11.type = item11_type
item12 = Parameter()
item12_default = 0.002
item12.default = item12_default
item12_description = """(AA-1) width of supermirror cut-off"""
item12.description = item12_description
item12_name = """W"""
item12.name = item12_name
item12_type = """double"""
item12.type = item12_type
item13 = Parameter()
item13_default = 0.012
item13.default = item13_default
item13_description = """(m)   channel length of the Fermi chopper"""
item13.description = item13_description
item13_name = """length"""
item13.name = item13_name
item13_type = """double"""
item13.type = item13_type
item14 = Parameter()
item14_default = 0.95
item14.default = item14_default
item14_description = """(1)   efficiency = transmission x fraction of transparent material"""
item14.description = item14_description
item14_name = """eff"""
item14.name = item14_name
item14_type = """double"""
item14.type = item14_type
item15 = Parameter()
item15_default = 0.0
item15.default = item15_default
item15_description = """(1)  set time to zero"""
item15.description = item15_description
item15_name = """zero_time"""
item15.name = item15_name
item15_type = """double"""
item15.type = item15_type
item16 = Parameter()
item16_default = 0.0
item16.default = item16_default
item16_description = """(m)   optional total width of slit package"""
item16.description = item16_description
item16_name = """width"""
item16.name = item16_name
item16_type = """double"""
item16.type = item16_type
item17 = Parameter()
item17_default = 0.0
item17.default = item17_default
item17_description = """(1)   set to 1 or 2 gives debugging information"""
item17.description = item17_description
item17_name = """verbose"""
item17.name = item17_name
item17_type = """double"""
item17.type = item17_type
item18 = Parameter()
item18_default = 0.08
item18.default = item18_default
item18_description = """(m)   height of slit package"""
item18.description = item18_description
item18_name = """height"""
item18.name = item18_name
item18_type = """double"""
item18.type = item18_type
item19 = Parameter()
item19_default = 0.0
item19.default = item19_default
item19_description = """"""
item19.description = item19_description
item19_name = """rad"""
item19.name = item19_name
item19_type = """double"""
item19.type = item19_type
item20 = Parameter()
item20_default = 0.0
item20.default = item20_default
item20_description = """"""
item20.description = item20_description
item20_name = """slit"""
item20.name = item20_name
item20_type = """double"""
item20.type = item20_type
item21 = Parameter()
item21_default = 0.0
item21.default = item21_default
item21_description = """"""
item21.description = item21_description
item21_name = """alpham"""
item21.name = item21_name
item21_type = """double"""
item21.type = item21_type
item22 = Parameter()
item22_default = 0.0
item22.default = item22_default
item22_description = """"""
item22.description = item22_description
item22_name = """Wi"""
item22.name = item22_name
item22_type = """double"""
item22.type = item22_type
item23 = Parameter()
item23_default = 0.0
item23.default = item23_default
item23_description = """"""
item23.description = item23_description
item23_name = """dist"""
item23.name = item23_name
item23_type = """double"""
item23.type = item23_type
item24 = Parameter()
item24_default = 0.0
item24.default = item24_default
item24_description = """"""
item24.description = item24_description
item24_name = """Vi"""
item24.name = item24_name
item24_type = """double"""
item24.type = item24_type
item25 = Parameter()
item25_default = 0.0
item25.default = item25_default
item25_description = """(m-1) Curvature of slits (1/radius of curvature)."""
item25.description = item25_description
item25_name = """curvature"""
item25.name = item25_name
item25_type = """double"""
item25.type = item25_type
item26 = Parameter()
item26_default = 0.0
item26.default = item26_default
item26_description = """(s)  sets phase so that transmision is centered on 'time'"""
item26.description = item26_description
item26_name = """time"""
item26.name = item26_name
item26_type = """double"""
item26.type = item26_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11, item12, item13, item14, item15, item16, item17, item18, item19, item20, item21, item22, item23, item24, item25, item26 ]
info.input_parameters = info_input_parameters
info_name = """FermiChopper"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """(-)     structure"""
item0.description = item0_description
item0_name = """FCVars"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
info_output_parameters = [ item0 ]
info.output_parameters = info_output_parameters
info_save = """
  double mean_k, mean_v, mean_t, mean_w=0, mean_L=0;

  if (FCVars.sum_N) {
    mean_v = FCVars.sum_v/FCVars.sum_N;
    mean_t = FCVars.sum_t/FCVars.sum_N;
    mean_k = V2K*mean_v;
    if (mean_k) mean_L = 2*PI/mean_k;
    mean_w = VS2E*mean_v*mean_v;
    
    double div, mean_phase;
    div        = atan(w/length);
    mean_phase = fmod(mean_t*nu*360, 360);
    if (mean_phase > 180) mean_phase -= 360;

    if (!FCVars.sum_N_pass)
    printf("FermiChopper: %s: No neutron can pass the chopper.\n", NAME_CURRENT_COMP);
    if (!FCVars.sum_N_pass || verbose)
    printf("FermiChopper: %s\n"
           "              Mean velocity     v     = %g [m/s]\n"
           "              Mean wavelength   lambda= %g [Angs]\n"
           "              Mean energy       omega = %g [meV]\n"
           "              Mean arrival time t     = %g [s]\n"
           "              Mean phase              = %g [deg] (negate to optimize transmission)\n"
           "              Opening time      dt    = %g [s]\n"
           "              Intensity reaching FC   = %g [n/s]\n"
           "              Intensity passing  FC   = %g [n/s]\n"
           , NAME_CURRENT_COMP,
           mean_v, mean_L, mean_w, mean_t, mean_phase,
           (nu ? div/PI/nu : 1),
           FCVars.sum_N,
           FCVars.sum_N_pass);
    if (!FCVars.sum_N_pass || verbose) {
      printf("FermiChopper: %s: Lost events anaylsis\n"
             "              Already inside:            %i\n"
             "              By Top/Bottom of cylinder: %i\n"
             "              At cylinder entrance:      %i\n"
             "              Hit cyl. entrance sides:   %i\n"
             "              Can't prop. to slit pack:  %i\n"
             "              At slit pack entrance:     %i\n"
             "              On absorbing slit coating: %i\n"
             "Warning:      Can not reach slit wall:   %i\n"
             "              Exiting slit pack:         %i\n"
             "              Too many iterations:       %i\n"
             "              Prop. in wrong direction : %i\n"
             "              Mad neutron (no control):  %i\n"
             "              At cylinder exit:          %i\n"
             "Warning:      Can not reach slit output: %i\n"
      , NAME_CURRENT_COMP,
      FCVars.absorb_alreadyinside,
      FCVars.absorb_topbottom,
      FCVars.absorb_cylentrance,
      FCVars.absorb_sideentrance,
      FCVars.absorb_notreachentrance,
      FCVars.absorb_packentrance,
      FCVars.absorb_slitcoating,
      FCVars.warn_notreachslitwall,
      FCVars.absorb_exitslitpack,
      FCVars.absorb_maxiterations,
      FCVars.absorb_wrongdirection,
      FCVars.absorb_nocontrol,
      FCVars.absorb_cylexit,
      FCVars.warn_notreachslitoutput);
    }

  }
"""
info.save = info_save
item0 = """
// ----------  header for SHARE ----------
#ifndef FermiChopper_TimeAccuracy
#define FermiChopper_TimeAccuracy 1e-9
#define FermiChopper_MAXITER      10

  
  struct FermiChopper_struct {

    double omega;
    double ph0;  
    double C_slit;  
    double L_slit;
    double sum_t;
    double sum_v;
    double sum_N;
    double sum_N_pass;
    
    long absorb_alreadyinside;
    long absorb_topbottom;
    long absorb_cylentrance;
    long absorb_sideentrance;
    long absorb_notreachentrance;
    long absorb_packentrance;
    long absorb_slitcoating;
    long warn_notreachslitwall;
    long absorb_exitslitpack;
    long absorb_maxiterations;
    long absorb_wrongdirection;
    long absorb_nocontrol;
    long absorb_cylexit;
    long warn_notreachslitoutput;
  } ;


double FC_zrot(double X, double Z, double T, struct FermiChopper_struct FCs);


double FC_xrot(double X, double Z, double T, struct FermiChopper_struct FCs);


double FC_xzrot_dt(double x, double z, double vx, double vz,
                   double t, double dt, char type, struct FermiChopper_struct FCs); 


#ifndef SIGN
#define SIGN(a,b) ((b >= 0 ? fabs(a) : -fabs(a)))
#endif
double FC_xzridder(double x, double z, double vx, double vz,
                   double t, double dt,
                   char type, double d, struct FermiChopper_struct FCs) ;

#endif"""
item1 = """// ----------  implementation for SHARE ----------


double FC_zrot(double X, double Z, double T, struct FermiChopper_struct FCs){
  double omega=FCs.omega;
  double ph0   =FCs.ph0;

  return( Z*cos(omega*T+ph0)-X*sin(omega*T+ph0) );
}



double FC_xrot(double X, double Z, double T, struct FermiChopper_struct FCs){
  double omega=FCs.omega;
  double ph0   =FCs.ph0;
  double C_slit=FCs.C_slit;
  double ret, tmp;

  ret = X*cos(omega*T+ph0)+Z*sin(omega*T+ph0);

  if (C_slit) {
    tmp  = fabs(FC_zrot(X, Z, T, FCs));
    if (tmp < FCs.L_slit/2) {
      tmp  = (FCs.L_slit/2 - tmp)*C_slit;
      ret += (1-sqrt(1-tmp*tmp))/C_slit;
    }
  }
  return( ret );
}


double FC_xzrot_dt(double x, double z, double vx, double vz,
                   double t, double dt, char type, struct FermiChopper_struct FCs) {
  if (dt) 
    return( (type == 'x' ? FC_xrot(x+vx*dt, z+vz*dt, t+dt, FCs)
                         : FC_zrot(x+vx*dt, z+vz*dt, t+dt, FCs)) );
  else    
    return( (type == 'x' ? FC_xrot(x,z,t,FCs)
                         : FC_zrot(x,z,t,FCs)) );
}


double FC_xzridder(double x, double z, double vx, double vz,
                   double t, double dt,
                   char type, double d, struct FermiChopper_struct FCs) {
  int j;
  float fl, fh, fm, fnew;
  float dth, dtl, dtm, dtnew, ans;
  float s, dtacc=FermiChopper_TimeAccuracy;
  dtl=0; dth=dt;
  fl = FC_xzrot_dt(x,z,vx,vz, t,dtl, type, FCs) - d;
  fh = FC_xzrot_dt(x,z,vx,vz, t,dth, type, FCs) - d;

  if ((fl > 0 && fh < 0) || (fl < 0 && fh > 0)) { 
    ans=-FLT_MAX;
    for (j=1; j<=FermiChopper_MAXITER; j++) {
      dtm  =0.5*(dtl+dth); 
      fm   =FC_xzrot_dt(x,z,vx,vz, t,dtm,   type, FCs) - d;
      s    =sqrt(fm*fm-fl*fh);
      if (!s) return(ans);
      dtnew=dtm+(dtm-dtl)*((fl >= fh ? fm:-fm)/s); 
      if (fabs(dtnew-ans) <= dtacc) return(ans);
      ans  =dtnew;
      fnew =FC_xzrot_dt(x,z,vx,vz, t,dtnew, type, FCs) - d;
      if (!fnew) return(ans);
      
      if (SIGN(fm, fnew) != fm) {
        dtl=dtm; fl=fm;
        dth=ans; fh=fnew;
      } else if (SIGN(fl, fnew) != fl) {
        dth=ans; fh=fnew;
      } else if (SIGN(fh, fnew) != fh) {
        dtl=ans; fl=fnew;
      } else return(-1); 
      if (fabs(dth-dtl) <= dtacc) return(ans);
    } 
    return(-2); 
  } 
  else {
    if (!fl) return(dtl);
    if (!fh) return(dth);
    return(-3); 
  }
} 

"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Fermi Chopper with rotating frame."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """

  

  
  double slit_input; 

  
  double xp1, zp1, xp2, zp2, vxp1, vxp2, xp3, vzp1;

  
  double distance_W;
  double n1,n2,n3;

  
  double q;

  
  int loopcounter=0;   

  
  double t3;      
  double dt;      
  double t1,t2;   


  if (zero_time == 1 && nu)
    t -= floor( (t+1/(4*nu))*(2*nu) )/(2*nu);


  if (cylinder_intersect (&t1, &t2, x, y, z, vx, vy, vz, radius, height))
  {
    if (t1 <= 0) { FCVars.absorb_alreadyinside++; ABSORB; }    

    dt=t2-t1;     

    PROP_DT(t1);  
    SCATTER;

    
    if (y >= 0.99*ymax || y <= 0.99*ymin ) { FCVars.absorb_topbottom++; ABSORB; }

    vxp1 = sqrt(vx*vx+vy*vy+vz*vz);
    FCVars.sum_v += p*vxp1;
    FCVars.sum_t += p*t;
    FCVars.sum_N += p;

    if (zero_time > 1 && FCVars.sum_N) { 
      double mean_t, mean_phase;
      mean_t     = FCVars.sum_t/FCVars.sum_N;
      mean_phase = fmod(mean_t*nu*2*PI, 2*PI);
      FCVars.ph0 = -mean_phase;
    }

    
    xp1 = FC_xrot(x,z, t,FCVars); 
    if (fabs(xp1) >= Nslit*w/2) { FCVars.absorb_cylentrance++; ABSORB; }



    
    zp1  = FC_zrot(x,z, t, FCVars);

    
    slit_input = (zp1 > 0 ? length/2 : -length/2);

    
    if (fabs(zp1) < length/2) { FCVars.absorb_sideentrance++; ABSORB; }

    
    t3 = FC_xzridder(x,z,vx,vz, t,dt, 'z', slit_input, FCVars);

    if( (t3 < 0)||(t3 > dt) ) {
      if (verbose > 1 && FCVars.absorb_notreachentrance < FermiChopper_MAXITER) {
        printf("FermiChopper: %s: Can not reach entrance of slits\n"
               "              dt=%g. Absorb on chopper aperture.\n",
        NAME_CURRENT_COMP, t3);
        if (t3 == -3)
            printf("          No sign change to determine intersection (FC_xzidder:1)\n");
        else if (t3 == -2)
            printf("          Max iterations reached in (FC_xzidder:1)\n");
        else if (t3 < 0)
            printf("          Error when solving intersection (FC_xzidder:1)\n");
      }
      FCVars.absorb_notreachentrance++;
      ABSORB; 
    }

    
    PROP_DT(t3);
    dt -= t3; 

    
    xp1 = FC_xrot(x,z, t, FCVars);
    if (fabs(xp1) >= Nslit*w/2) { FCVars.absorb_packentrance++; ABSORB; }
    
    
 //   if (mcdotrace) {
 //     zp1 = FC_zrot(x,z, t, FCVars);
 //     
 //     xp2 = x; zp2 = z; x = xp1; z=zp1; SCATTER; x=xp2; z=zp2;
 //   } else SCATTER;
    SCATTER;
    

    
    t3 = FC_xzridder(x,z,vx,vz, t,dt, 'z', -slit_input, FCVars);

    if((t3 < 0)||(t3 > dt)) {
      if (verbose > 1 && FCVars.warn_notreachslitoutput < FermiChopper_MAXITER) {
        printf("FermiChopper: %s: Can not reach exit of slits\n"
               "              dt=%g. Ignore.\n",
          NAME_CURRENT_COMP, t3);
        if (t3 == -3)
          printf("              No sign change to determine intersection (FC_xzidder:2)\n");
        else if (t3 == -2)
          printf("              Max iterations reached in (FC_xzidder:2)\n");
        else if (t3 < 0)
          printf("              Error when solving intersection (FC_xzidder:2)\n");
      }
      FCVars.warn_notreachslitoutput++;
      
    } else dt = t3; 



    
    n1 = floor(xp1/w);



    for (loopcounter=0; loopcounter<=FermiChopper_MAXITER; loopcounter++) {

      
      xp1 = FC_xrot    (x,z,          t,   FCVars);          
      xp2 = FC_xzrot_dt(x,z,vx,vz,    t,   dt, 'x', FCVars); 

      
      n2 = floor(xp2/w);

      vxp1= FC_xrot    (vx+z*FCVars.omega,vz-x*FCVars.omega,
                                      t,   FCVars);          

      vxp2= FC_xrot    (vx+(z+vz*dt)*FCVars.omega,vz-(x+vx*dt)*FCVars.omega,
                                      t+dt,FCVars);          

      
      t3 = (vxp1 - vxp2 ? ((xp2 - xp1) + (t*vxp1 - (t+dt)*vxp2))/(vxp1 - vxp2)
                        : -1);

//      printf("loopcounter=%i: [in: xp1=%g n1=%g] dt=%g [out: xp2=%g n2=%g]  ", loopcounter, xp1, n1, dt, xp2, n2);

      
      if((t3 < 0)||(t3 > dt)) t3=dt*0.5; else t3 -= t;

      
      xp3 = FC_xzrot_dt(x,z,vx,vz, t, t3, 'x', FCVars); 

      
      n3 = floor(xp3/w);

//      printf("[mid: xp3=%g n3=%g] t3=%g\n", xp3, n3, t3);

      
      if ( (n2!=n1) || (n3!=n1) ) {

        if (m == 0 || R0 == 0) { FCVars.absorb_slitcoating++; ABSORB; }

        
        if (n3!=n1) { 
          n2=n3; dt = t3; }

        if (n2 > n1) { 
          distance_W = (n1+1)*w;
        } else {       
          distance_W =  n1*w;
        }

        
        t3 = FC_xzridder(x,z,vx,vz, t,dt, 'x', distance_W, FCVars);

        if((t3 < 0)||(t3 > dt)) {
          if (verbose > 1 && FCVars.warn_notreachslitwall < FermiChopper_MAXITER) {
            printf("FermiChopper: %s: Can not reach slit wall (iteration %i)\n"
                  "              dt=%g.\n",
              NAME_CURRENT_COMP, dt, loopcounter);
            if (t3 == -3)
              printf("        No sign change to determine intersection (FC_xzidder:3)\n");
            else if (t3 == -2)
              printf("        Max iterations reached in (FC_xzidder:3)\n");
            else if (t3 < 0)
              printf("        Error when solving intersection (FC_xzidder:3)\n");
          }
          
          FCVars.warn_notreachslitwall++;
          t3 = dt/2;
        }
        zp1 = FC_xzrot_dt(x,z,vx,vz, t,t3, 'z', FCVars);

        
        if (fabs(zp1) > length/2) break;

        
        PROP_DT(t3); dt -= t3;

        
        vxp1 = FC_xrot(vx,vz, t, FCVars);
        vzp1 = FC_zrot(vx,vz, t, FCVars);

        q    = 2*V2Q*(fabs(vxp1));

        if(q > Qc)
        {
          double arg = (q-m*Qc)/W;
          if(arg < 10)
            p *= .5*(1-tanh(arg))*(1-alpha*(q-Qc));
          else
            { FCVars.absorb_slitcoating++; ABSORB; } 
        }
        p *= R0;
        
     //   if (mcdotrace) {
     //     xp1 = FC_xrot(x,z, t,FCVars);
     //     zp1 = FC_zrot(x,z, t,FCVars);
     //     
     //     xp2 = x; zp2 = z; x = xp1; z=zp1; SCATTER; x=xp2; z=zp2;
     //   } else SCATTER;
        SCATTER;
         

        
        vxp1 *= -1;
        
        vx = FC_xrot( vxp1,-vzp1, t,FCVars);
        vz = FC_zrot(-vxp1, vzp1, t,FCVars);

        n1 = n3;

      } 
      else { 
        break;
      }
    } 

    if (loopcounter >= FermiChopper_MAXITER) {
      if (verbose > 1 && FCVars.absorb_maxiterations < FermiChopper_MAXITER)
      printf("FermiChopper: %s:Max iterations %i reached inside slit. Absorb.\n",
        NAME_CURRENT_COMP, FermiChopper_MAXITER);
      FCVars.absorb_maxiterations++;
      ABSORB;
    }


    

    
    if (!cylinder_intersect (&t1, &t2, x, y, z, vx, vy, vz, radius, height))
      { FCVars.absorb_exitslitpack++;  ABSORB; } 

    if (t1 > 0) {
      if (verbose > 1 && FCVars.absorb_wrongdirection < FermiChopper_MAXITER)
      printf("FermiChopper: %s: Neutrons are leaving chopper\n"
             "              in the wrong direction. Absorb.\n", NAME_CURRENT_COMP);
      FCVars.absorb_wrongdirection++;
      ABSORB;
    }

    if (t2 <= 0 && FCVars.absorb_nocontrol < FermiChopper_MAXITER) {
      if (verbose > 1)
      printf("FermiChopper: %s: Neutrons are leaving chopper\n"
             "              without any control. Absorb.\n", NAME_CURRENT_COMP);
      FCVars.absorb_nocontrol++;
      ABSORB;
    }

    
    PROP_DT(t2);
    SCATTER;

    
    if (fabs(y) >= 0.99*height/2) { FCVars.absorb_topbottom++; ABSORB; }


    
    xp1 = FC_xrot(x,z, t,FCVars);
    if (fabs(xp1) >= Nslit*w/2) { FCVars.absorb_cylexit++; ABSORB; }

    
    p = p*eff;          //finite cross section + transmission

    FCVars.sum_N_pass += p;

  } 

"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
