
def factory( name='filter_gen',file='',options='',xmin=0,xmax=0,ymin=0,ymax=0,thickness=1.0,scaling=1.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from Filter_genboostpython import Filter_gen as _factory_mcvine
    return component( _factory_mcvine( name,file,options,xmin,xmax,ymin,ymax,thickness,scaling ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """EF (from Flux_adapter)"""
info.copyright = info_copyright
info_declare = """
  char   Mode_Table;
  char   Type_Table;
  t_Table pTable;
"""
info.declare = info_declare
info_finalize = """
  Table_Free(&pTable);
"""
info.finalize = info_finalize
info_full_description = """This component changes the neutron flux (weight) in order to match
a reference table in a file.
Typically you may set the neutron flux (source-like), or multiply it
using a transmission table (filter-like).
The component may be placed after a source, in order to e.g.
simulate a real source from a reference table, or used as a filter (BeO)
, or as a window (Al). The behaviour of the component is
specified using the 'options' parameter, or from the file itself (see below)
If the thickness for the transmission data file D was t0, and a different
thickness t1 would be required, then the resulting transmission is:
D^(t1/t0).
You may use the 'thickness' and 'scaling' parameter for that purpose.

<b>File format:</b>
This file may be of any 2-columns free format (k[Angs-1],p), (omega[meV],p)
and (lambda[Angs],p) where p is the weight. The type of the file may be
written explicitely in the file, as a comment, or using the 'options'
parameter.
Non mumerical content in file is treated as comment (e.g. lines starting
with '#' character).
A table rebinning and linear interpolation are performed.

EXAMPLE : in order to simulate a PG filter, using the lib/data/HOPG.trm file
Filter_gen(name, xmin = -0.1, xmax = 0.1,
ymin = -0.1, ymax = 0.1,
file="HOPG.trm")

in this file, the comment line
# wavevector multiply
sets the behaviour of the component. One may as well have used
options="wavevector multiply"
in the component instance parameters."""
info.full_description = info_full_description
info_initialize = """
  char verbose = 0;

  Mode_Table = FLUX_ADAPT_MULT;
  Type_Table = UNKNOWN_TABLE;

  FilterGen_Mode(options, &Mode_Table, &Type_Table, &verbose);
  if (file != NULL)
  {
    if (Table_Read(&pTable, file, 1) <= 0) 
      exit(fprintf(stderr,"Filter_gen: %s: can not read file %s\n", NAME_CURRENT_COMP, file));

    Table_Rebin(&pTable);         
    if (pTable.rows < 2 || !pTable.step_x) Table_Free(&pTable);
    if (pTable.data)
    {
      FilterGen_Mode(pTable.header, &Mode_Table, &Type_Table, &verbose);
      if (verbose)
      {
        Table_Info(pTable);
        printf("Filter_gen : %s : Filter data [", NAME_CURRENT_COMP);
        if (Type_Table == ENERGY_TABLE) printf("Energy");
        if (Type_Table == WAVEVECTOR_TABLE) printf("Wavevector");
        if (Type_Table == WAVELENGTH_TABLE) printf("Wavelength");
        if (Type_Table == UNKNOWN_TABLE) printf("UNKNOWN (not used)");
        printf(", Flux] in ");
        if (Mode_Table == FLUX_ADAPT_MULT) printf("multiply");
        else if (Mode_Table == FLUX_ADAPT_ADD) printf("add");
        else printf("set");
        printf(" mode\n");
      }
    }

  } else pTable.data = NULL;
"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """filter_gen"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = """"""
item1.default = item1_default
item1_description = """[str] name of the file to look at (first two columns data)"""
item1.description = item1_description
item1_name = """file"""
item1.name = item1_name
item1_type = """char *"""
item1.type = item1_type
item2 = Parameter()
item2_default = """"""
item2.default = item2_default
item2_description = """[str] string that can contain"""
item2.description = item2_description
item2_name = """options"""
item2.name = item2_name
item2_type = """char *"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """[m] dimension of filter"""
item3.description = item3_description
item3_name = """xmin"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """[m] dimension of filter"""
item4.description = item4_description
item4_name = """xmax"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0
item5.default = item5_default
item5_description = """[m] dimension of filter"""
item5.description = item5_description
item5_name = """ymin"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0
item6.default = item6_default
item6_description = """[m] dimension of filter"""
item6.description = item6_description
item6_name = """ymax"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 1.0
item7.default = item7_default
item7_description = """[1] relative thickness. D = D^(thickness)."""
item7.description = item7_description
item7_name = """thickness"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 1.0
item8.default = item8_default
item8_description = """[1] scaling factor. D = D*scaling."""
item8.description = item8_description
item8_name = """scaling"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8 ]
info.input_parameters = info_input_parameters
info_name = """Filter_gen"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """pTable"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """"""
item1.description = item1_description
item1_name = """Mode_Table"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """"""
item2.description = item2_description
item2_name = """Type_Table"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
info_output_parameters = [ item0, item1, item2 ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """
// ----------- added by Jiao Lin --------
// ----------  header for SHARE ----------
// ----------- added by Jiao Lin --------

#ifndef FILTER_GEN
#define FILTER_GEN "1.0"
#define UNKNOWN_TABLE    0
#define ENERGY_TABLE     1
#define WAVEVECTOR_TABLE 2
#define WAVELENGTH_TABLE 3
#define FLUX_ADAPT_SET   0
#define FLUX_ADAPT_MULT  1
#define FLUX_ADAPT_ADD   2
#include "mcstas2/share/read_table-lib.h"
#endif



// ----------- added by Jiao Lin --------"""
item1 = """// ----------  implementation for SHARE ----------
// ----------- added by Jiao Lin --------

char FilterGen_Mode(char *str, char *Mode, char *Type, char *verbose)
{
  long i;
  char *c;
  if (!str || !strlen(str)) return(0);
  c = (char *)malloc(strlen(str));
  for (i=0; i<strlen(str); i++) c[i] = tolower(str[i]);
  
  if (strstr(str," k ") || strstr(str," q ") || strstr(str,"wavevector"))
    *Type = WAVEVECTOR_TABLE;
  if (strstr(str,"omega") || strstr(str," e ") || strstr(str,"energy"))
    *Type = ENERGY_TABLE;
  if (strstr(str,"lambda") || strstr(str,"wavelength") || strstr(str," L "))
    *Type = WAVELENGTH_TABLE;
  if (strstr(str,"set")) *Mode  = FLUX_ADAPT_SET;
  if (strstr(str,"add")) *Mode  = FLUX_ADAPT_ADD;
  if (strstr(str,"multiply")) *Mode  = FLUX_ADAPT_MULT;
  if (strstr(str,"verbose")) *verbose = 1;

  return(*Mode);
}

"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """an external data file."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
  double v2, K, L, E, X, new_p;

  PROP_Z0;
  if (Type_Table && (x>xmin && x<xmax && y>ymin && y<ymax))
  {
    v2 = (vx*vx + vy*vy + vz*vz);
    K = V2K*sqrt(v2);        
    L = (2*PI/K);        
    E = VS2E*v2;        
    if (Type_Table == ENERGY_TABLE)     X=E;
    if (Type_Table == WAVEVECTOR_TABLE) X=K;
    if (Type_Table == WAVELENGTH_TABLE) X=L;
    
    if (pTable.data != NULL)
    {
      double y1, y2, x1;
      long   Index;
      Index = floor((X - pTable.min_x)/pTable.step_x);
      y1 = Table_Index(pTable, Index,   1); 
      x1 = Table_Index(pTable, Index,   0); 
      y2 = Table_Index(pTable, Index+1, 1); 
      new_p = scaling*(y1+(X - x1)*(y2-y1)/pTable.step_x); 
      if (thickness != 1) new_p = pow(new_p, thickness);
    }
    else new_p = 1;

    if (Mode_Table == FLUX_ADAPT_MULT) p *= new_p;
    else p = new_p;
    SCATTER;
  }
  else
    if (Type_Table) ABSORB;
"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
