
def factory( name='guide',w1=0,h1=0,w2=0,h2=0,l=0,R0=0.99,Qc=0.0219,alpha=6.07,m=2.0,W=0.003,reflect='' ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from Guideboostpython import Guide as _factory_mcvine
    return component( _factory_mcvine( name,w1,h1,w2,h2,l,R0,Qc,alpha,m,W,reflect ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Kristian Nielsen"""
info.copyright = info_copyright
info_declare = """
  t_Table pTable;
"""
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """Models a rectangular guide tube centered on the Z axis. The entrance lies
in the X-Y plane.
For details on the geometry calculation see the description in the McStas
reference manual.
The reflectivity profile may either use an analytical mode (see Component
Manual) or a 2-columns reflectivity free text file with format
[q(Angs-1) R(0-1)].

%VALIDATION
May 2005: extensive internal test, no bugs found
Validated by: K. Lieutenant

%BUGS
This component does not work with gravitation on. Use component Guide_gravity then.

Example values: m=4 Qc=0.0219 W=1/300 alpha=6.49 R0=1"""
info.full_description = info_full_description
info_initialize = """


  if (reflect && strlen(reflect)) {
    if (Table_Read(&pTable, reflect, 1) <= 0) 
      exit(fprintf(stderr,"Guide: %s: can not read file %s\n", NAME_CURRENT_COMP, reflect));
  } else {
    if (W < 0 || R0 < 0 || Qc < 0 || m < 0)
    { fprintf(stderr,"Guide: %s: W R0 Qc must be >0.\n", NAME_CURRENT_COMP);
      exit(-1); }
    if (m < 1 && m != 0) fprintf(stderr,"WARNING: Guide: %s: m < 1 behaves as if m=1.\n",
      NAME_CURRENT_COMP);
  }
"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """guide"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """(m)    Width at the guide entry"""
item1.description = item1_description
item1_name = """w1"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """(m)    Height at the guide entry"""
item2.description = item2_description
item2_name = """h1"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """(m)    Width at the guide exit"""
item3.description = item3_description
item3_name = """w2"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """(m)    Height at the guide exit"""
item4.description = item4_description
item4_name = """h2"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0
item5.default = item5_default
item5_description = """(m)    length of guide"""
item5.description = item5_description
item5_name = """l"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0.99
item6.default = item6_default
item6_description = """(1)    Low-angle reflectivity"""
item6.description = item6_description
item6_name = """R0"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0.0219
item7.default = item7_default
item7_description = """(AA-1) Critical scattering vector"""
item7.description = item7_description
item7_name = """Qc"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 6.07
item8.default = item8_default
item8_description = """(AA)   Slope of reflectivity"""
item8.description = item8_description
item8_name = """alpha"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 2.0
item9.default = item9_default
item9_description = """(1)    m-value of material. Zero means completely absorbing."""
item9.description = item9_description
item9_name = """m"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 0.003
item10.default = item10_default
item10_description = """(AA-1) Width of supermirror cut-off"""
item10.description = item10_description
item10_name = """W"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = """"""
item11.default = item11_default
item11_description = """(str)  Reflectivity file name. Format [q(Angs-1) R(0-1)]"""
item11.description = item11_description
item11_name = """reflect"""
item11.name = item11_name
item11_type = """char *"""
item11.type = item11_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11 ]
info.input_parameters = info_input_parameters
info_name = """Guide"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """pTable"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
info_output_parameters = [ item0 ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """
// ----------- added by Jiao Lin --------
// ----------  header for SHARE ----------
// ----------- added by Jiao Lin --------
#include "mcstas2/share/read_table-lib.h"

// ----------- added by Jiao Lin --------"""
item1 = """// ----------  implementation for SHARE ----------
// ----------- added by Jiao Lin --------
"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Neutron guide."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
  double t1,t2;                                 
  double av,ah,bv,bh,cv1,cv2,ch1,ch2,d;         
  double weight;                                
  double vdotn_v1,vdotn_v2,vdotn_h1,vdotn_h2;   
  int i;                                        
  double q;                                     
  double nlen2;                                 

  
  double ww = .5*(w2 - w1), hh = .5*(h2 - h1);
  double whalf = .5*w1, hhalf = .5*h1;

  
  PROP_Z0;
  
  SCATTER;
  if(x <= -whalf || x >= whalf || y <= -hhalf || y >= hhalf)
    ABSORB;
  for(;;)
  {
    
    av = l*vx; bv = ww*vz;
    ah = l*vy; bh = hh*vz;
    vdotn_v1 = bv + av;         
    vdotn_v2 = bv - av;         
    vdotn_h1 = bh + ah;         
    vdotn_h2 = bh - ah;         
    
    cv1 = -whalf*l - z*ww; cv2 = x*l;
    ch1 = -hhalf*l - z*hh; ch2 = y*l;
    
    t1 = (l - z)/vz;
    i = 0;
    if(vdotn_v1 < 0 && (t2 = (cv1 - cv2)/vdotn_v1) < t1)
    {
      t1 = t2;
      i = 1;
    }
    if(vdotn_v2 < 0 && (t2 = (cv1 + cv2)/vdotn_v2) < t1)
    {
      t1 = t2;
      i = 2;
    }
    if(vdotn_h1 < 0 && (t2 = (ch1 - ch2)/vdotn_h1) < t1)
    {
      t1 = t2;
      i = 3;
    }
    if(vdotn_h2 < 0 && (t2 = (ch1 + ch2)/vdotn_h2) < t1)
    {
      t1 = t2;
      i = 4;
    }
    if(i == 0)
      break;                    
    PROP_DT(t1);
    switch(i)
    {
      case 1:                   
        nlen2 = l*l + ww*ww;
        q = V2Q*(-2)*vdotn_v1/sqrt(nlen2);
        d = 2*vdotn_v1/nlen2;
        vx = vx - d*l;
        vz = vz - d*ww;
        break;
      case 2:                   
        nlen2 = l*l + ww*ww;
        q = V2Q*(-2)*vdotn_v2/sqrt(nlen2);
        d = 2*vdotn_v2/nlen2;
        vx = vx + d*l;
        vz = vz - d*ww;
        break;
      case 3:                   
        nlen2 = l*l + hh*hh;
        q = V2Q*(-2)*vdotn_h1/sqrt(nlen2);
        d = 2*vdotn_h1/nlen2;
        vy = vy - d*l;
        vz = vz - d*hh;
        break;
      case 4:                   
        nlen2 = l*l + hh*hh;
        q = V2Q*(-2)*vdotn_h2/sqrt(nlen2);
        d = 2*vdotn_h2/nlen2;
        vy = vy + d*l;
        vz = vz - d*hh;
        break;
    }
    
    weight = 1.0; 
    if(m == 0)
      ABSORB;
    if (reflect && strlen(reflect))
      weight = Table_Value(pTable, q, 1);
    else if(q > Qc)
    {
      double arg = (q-m*Qc)/W;
      if(arg < 10)
        weight = .5*(1-tanh(arg))*(1-alpha*(q-Qc));
      else
        ABSORB;                               
      weight *= R0;
    } else { 
      weight *= R0;
    }
    p *= weight;
    SCATTER;
  }
"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
