
def factory( name='guide_channeled',w1=0,h1=0,w2=0,h2=0,l=0,R0=0.99,Qcx=0.021,Qcy=0.021,alphax=6.07,alphay=6.07,W=0.003,k=1.0,d=0.0005,mx=1.0,my=1.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from Guide_channeledboostpython import Guide_channeled as _factory_mcvine
    return component( _factory_mcvine( name,w1,h1,w2,h2,l,R0,Qcx,Qcy,alphax,alphay,W,k,d,mx,my ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Kristian Nielsen"""
info.copyright = info_copyright
info_declare = """
  double w1c;
  double w2c;
  double ww, hh;
  double whalf, hhalf;
  double lwhalf, lhhalf;
"""
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """Models a rectangular guide tube centered on the Z axis. The entrance lies
in the X-Y plane.
The guide may be tapered, and may have vertical subdivisions (used for
bender devices).

%BUGS
This component does not work with gravitation on. Use Guide_gravity.
This component does not work in multichannel focusing geometry."""
info.full_description = info_full_description
info_initialize = """
  if (k <= 0 || W <=0)
  { fprintf(stderr,"Guide_channeled: %s: k abd W must be positive\n", NAME_CURRENT_COMP);
    exit(-1); }
  w1c = (w1 + d)/(double)k;
  w2c = (w2 + d)/(double)k;
  ww = .5*(w2c - w1c);
  hh = .5*(h2 - h1);
  whalf = .5*(w1c - d);
  hhalf = .5*h1;
  lwhalf = l*whalf;
  lhhalf = l*hhalf;

  if ((k > 1) && (w1 != w2))
  {
    fprintf(stderr,"WARNING: Guide_channeled: %s:"
    "This component does not work with multichannel focusing guide\n"
    "Use Guide_gravity for that.\n", NAME_CURRENT_COMP);
    exit(-1);
  }


"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """guide_channeled"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """(m)    Width at the guide entry"""
item1.description = item1_description
item1_name = """w1"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """(m)    Height at the guide entry"""
item2.description = item2_description
item2_name = """h1"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """(m)    Width at the guide exit"""
item3.description = item3_description
item3_name = """w2"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """(m)    Height at the guide exit"""
item4.description = item4_description
item4_name = """h2"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0
item5.default = item5_default
item5_description = """(m)    Length of guide"""
item5.description = item5_description
item5_name = """l"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0.99
item6.default = item6_default
item6_description = """(1)    Low-angle reflectivity"""
item6.description = item6_description
item6_name = """R0"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0.021
item7.default = item7_default
item7_description = """(AA-1) Critical scattering vector for left and right vertical"""
item7.description = item7_description
item7_name = """Qcx"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0.021
item8.default = item8_default
item8_description = """(AA-1) Critical scattering vector for top and bottom mirrors"""
item8.description = item8_description
item8_name = """Qcy"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 6.07
item9.default = item9_default
item9_description = """(AA)   Slope of reflectivity for left and right vertical"""
item9.description = item9_description
item9_name = """alphax"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 6.07
item10.default = item10_default
item10_description = """(AA)   Slope of reflectivity for top and bottom mirrors"""
item10.description = item10_description
item10_name = """alphay"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = 0.003
item11.default = item11_default
item11_description = """(AA-1) Width of supermirror cut-off for all mirrors"""
item11.description = item11_description
item11_name = """W"""
item11.name = item11_name
item11_type = """double"""
item11.type = item11_type
item12 = Parameter()
item12_default = 1.0
item12.default = item12_default
item12_description = """(1)    Number of channels in the guide (>= 1)"""
item12.description = item12_description
item12_name = """k"""
item12.name = item12_name
item12_type = """double"""
item12.type = item12_type
item13 = Parameter()
item13_default = 0.0005
item13.default = item13_default
item13_description = """(m)    Thickness of subdividing walls"""
item13.description = item13_description
item13_name = """d"""
item13.name = item13_name
item13_type = """double"""
item13.type = item13_type
item14 = Parameter()
item14_default = 1.0
item14.default = item14_default
item14_description = """(1)    m-value of material for left and right vertical mirrors"""
item14.description = item14_description
item14_name = """mx"""
item14.name = item14_name
item14_type = """double"""
item14.type = item14_type
item15 = Parameter()
item15_default = 1.0
item15.default = item15_default
item15_description = """(1)    m-value of material for top and bottom mirrors. Zero"""
item15.description = item15_description
item15_name = """my"""
item15.name = item15_name
item15_type = """double"""
item15.type = item15_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11, item12, item13, item14, item15 ]
info.input_parameters = info_input_parameters
info_name = """Guide_channeled"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """w1c"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """"""
item1.description = item1_description
item1_name = """w2c"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """"""
item2.description = item2_description
item2_name = """ww"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """"""
item3.description = item3_description
item3_name = """hh"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """"""
item4.description = item4_description
item4_name = """whalf"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0
item5.default = item5_default
item5_description = """"""
item5.description = item5_description
item5_name = """hhalf"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0
item6.default = item6_default
item6_description = """"""
item6.description = item6_description
item6_name = """lwhalf"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0
item7.default = item7_default
item7_description = """"""
item7.description = item7_description
item7_name = """lhhalf"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
info_output_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7 ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """"""
item1 = """"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Neutron guide with channels (bender section)."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
  double t1,t2;                                 
  double av,ah,bv,bh,cv1,cv2,ch1,ch2,dd;        
  double vdotn_v1,vdotn_v2,vdotn_h1,vdotn_h2;   
  int i;                                        
  double q;                                     
  double vlen2,nlen2;                           
  double edge;
  double hadj;                                  

  
  PROP_Z0;
  
  SCATTER;
  if(x <= w1/-2.0 || x >= w1/2.0 || y <= -hhalf || y >= hhalf)
    ABSORB;
  
  x += w1/2.0;
  edge = floor(x/w1c)*w1c;
  if(x - edge > w1c - d)
  {
    x -= w1/2.0; 
    ABSORB;
  }
  x -= (edge + (w1c - d)/2.0);
  hadj = edge + (w1c - d)/2.0 - w1/2.0;
  for(;;)
  {
    
    av = l*vx; bv = ww*vz;
    ah = l*vy; bh = hh*vz;
    vdotn_v1 = bv + av;         
    vdotn_v2 = bv - av;         
    vdotn_h1 = bh + ah;         
    vdotn_h2 = bh - ah;         
    
    cv1 = -whalf*l - z*ww; cv2 = x*l;
    ch1 = -hhalf*l - z*hh; ch2 = y*l;
    
    t1 = (l - z)/vz;
    i = 0;
    if(vdotn_v1 < 0 && (t2 = (cv1 - cv2)/vdotn_v1) < t1)
    {
      t1 = t2;
      i = 1;
    }
    if(vdotn_v2 < 0 && (t2 = (cv1 + cv2)/vdotn_v2) < t1)
    {
      t1 = t2;
      i = 2;
    }
    if(vdotn_h1 < 0 && (t2 = (ch1 - ch2)/vdotn_h1) < t1)
    {
      t1 = t2;
      i = 3;
    }
    if(vdotn_h2 < 0 && (t2 = (ch1 + ch2)/vdotn_h2) < t1)
    {
      t1 = t2;
      i = 4;
    }
    if(i == 0)
      break;                    
    PROP_DT(t1);
    switch(i)
    {
      case 1:                   
        nlen2 = l*l + ww*ww;
        q = V2Q*(-2)*vdotn_v1/sqrt(nlen2);
        dd = 2*vdotn_v1/nlen2;
        vx = vx - dd*l;
        vz = vz - dd*ww;
        break;
      case 2:                   
        nlen2 = l*l + ww*ww;
        q = V2Q*(-2)*vdotn_v2/sqrt(nlen2);
        dd = 2*vdotn_v2/nlen2;
        vx = vx + dd*l;
        vz = vz - dd*ww;
        break;
      case 3:                   
        nlen2 = l*l + hh*hh;
        q = V2Q*(-2)*vdotn_h1/sqrt(nlen2);
        dd = 2*vdotn_h1/nlen2;
        vy = vy - dd*l;
        vz = vz - dd*hh;
        break;
      case 4:                   
        nlen2 = l*l + hh*hh;
        q = V2Q*(-2)*vdotn_h2/sqrt(nlen2);
        dd = 2*vdotn_h2/nlen2;
        vy = vy + dd*l;
        vz = vz - dd*hh;
        break;
    }
    
    if((i <= 2 && mx == 0) || (i > 2 && my == 0))
    {
      x += hadj; 
      ABSORB;
    }
    if((i <= 2 && q > Qcx) || (i > 2 && q > Qcy))
    {
      if (i <= 2)
      {
        double arg = (q - mx*Qcx)/W;
        if(arg < 10)
          p *= .5*(1-tanh(arg))*(1-alphax*(q-Qcx));
        else
        {
          x += hadj; 
          ABSORB;                               
        }
      } else {
        double arg = (q - my*Qcy)/W;
        if(arg < 10)
          p *= .5*(1-tanh(arg))*(1-alphay*(q-Qcy));
        else
        {
          x += hadj; 
          ABSORB;                               
        }
      }
    }
    p *= R0;
    x += hadj; SCATTER; x -= hadj;
  }
  x += hadj; 
"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
