
def factory( name='guide_gravity',reflect='',w1=0,h1=0,w2=0.0,h2=0.0,l=0,R0=0.99,Qc=0.0219,alpha=6.07,m=1.0,W=0.003,k=1.0,d=0.0005,mleft=-1.0,mright=-1.0,mtop=-1.0,mbottom=-1.0,kh=1.0,G=0.0,wavy=0.0,wavy_z=0.0,wavy_tb=0.0,wavy_lr=0.0,chamfers=0.0,chamfers_z=0.0,chamfers_lr=0.0,chamfers_tb=0.0,nelements=1.0,nu=0.0,phase=0.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from Guide_gravityboostpython import Guide_gravity as _factory_mcvine
    return component( _factory_mcvine( name,reflect,w1,h1,w2,h2,l,R0,Qc,alpha,m,W,k,d,mleft,mright,mtop,mbottom,kh,G,wavy,wavy_z,wavy_tb,wavy_lr,chamfers,chamfers_z,chamfers_lr,chamfers_tb,nelements,nu,phase ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """<a href="mailto:farhi@ill.fr">Emmanuel Farhi</a>"""
info.copyright = info_copyright
info_declare = """
  Gravity_guide_Vars_type GVars;
  t_Table pTable;
"""
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """Models a rectangular straight guide tube centered on the Z axis, with
gravitation handling. The entrance lies in the X-Y plane.
The guide can be channeled (k,d,kh parameters). The guide coating
specifications may be entered via different ways (global, or for
each wall m-value).

Waviness (random) may be specified either globally or for each mirror type.
Side chamfers (due to substrate processing) may be specified the same way.
In order to model a realistic straight guide assembly, a long guide of
length 'l' may be splitted into 'nelements' between which chamfers/gaps are
positioned.

The reflectivity may be specified either using an analytical mode (see
Component manual) or as a text file in free format, with 1st
column as q[Angs-1] and 2nd column as the reflectivity R in [0-1].
For details on the geometry calculation see the description in the McStas
component manual.

There is a special rotating mode in order to approximate a Fermi Chopper
behaviour, in the case the neutron trajectory is nearly linear inside the
chopper slits, i.e. the neutrons are fast w/r/ to the chopper speed.
Slits are straight, but may be super-mirror coated. In this case, the
component is NOT centered, but located at its entry window.

%VALIDATION
May 2005: extensive internal test, all problems solved
Validated by: K. Lieutenant"""
info.full_description = info_full_description
info_initialize = """
  double Gx=0, Gy=-GRAVITY, Gz=0;
  Coords mcLocG;
  int i;

  if (reflect && strlen(reflect)) {
    if (Table_Read(&pTable, reflect, 1) <= 0) 
      exit(fprintf(stderr,"Guide_gravity: %s: can not read file %s\n", NAME_CURRENT_COMP, reflect));
  } else {
    if (W < 0 || R0 < 0 || Qc < 0)
    { fprintf(stderr,"Guide_gravity: %s: W R0 Qc must be >0.\n", NAME_CURRENT_COMP);
      exit(-1); }
    if (m < 1 && m != 0) fprintf(stderr,"WARNING: Guide_gravity: %s: m < 1 behaves as if m=1.\n",
      NAME_CURRENT_COMP);
  }

  if (k <= 0 || kh <= 0)
  { fprintf(stderr,"Guide_gravity: %s: k kh must be >0.\n", NAME_CURRENT_COMP);
    exit(-1); }

  if (!w1 || !h1)
  { fprintf(stderr,"Guide_gravity: %s: input widow is closed (w1=h1=0).\n", NAME_CURRENT_COMP);
    exit(-1); }

  if (!w2) w2=w1;
  if (!h2) h2=h1;

  if (mcgravitation) G=-GRAVITY;
  mcLocG = rot_apply(ROT_A_CURRENT_COMP, coords_set(0,G,0));
  coords_get(mcLocG, &Gx, &Gy, &Gz);

  strcpy(GVars.compcurname, NAME_CURRENT_COMP);

  if (l > 0 && nelements > 0) {

    Gravity_guide_Init(&GVars,
      w1, h1, w2, h2, l, R0,
      Qc, alpha, m, W, k, d,
      Gx, Gy, Gz, mleft, mright, mtop,
      mbottom, kh, wavy_lr, wavy_tb, wavy_z, wavy,
      chamfers_z, chamfers_lr, chamfers_tb, chamfers,nu,phase);
    if (!G) for (i=0; i<5; GVars.A[i++] = 0);
    if (GVars.fc_freq != 0 || GVars.fc_phase != 0)
      printf("Guide_gravity: %s: Fermi Chopper mode: frequency=%g [Hz] phase=%g [deg]\n",
        NAME_CURRENT_COMP, GVars.fc_freq, GVars.fc_phase);
  } else printf("Guide_gravity: %s: unactivated (l=0 or nelements=0)\n", NAME_CURRENT_COMP);

"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """guide_gravity"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = """"""
item1.default = item1_default
item1_description = """(str) Reflectivity file name. Format [q(Angs-1) R(0-1)]"""
item1.description = item1_description
item1_name = """reflect"""
item1.name = item1_name
item1_type = """char *"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """(m)    Width at the guide entry"""
item2.description = item2_description
item2_name = """w1"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """(m)    Height at the guide entry"""
item3.description = item3_description
item3_name = """h1"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0.0
item4.default = item4_default
item4_description = """(m)    Width at the guide exit. If 0, use w1."""
item4.description = item4_description
item4_name = """w2"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0.0
item5.default = item5_default
item5_description = """(m)    Height at the guide exit. If 0, use h1."""
item5.description = item5_description
item5_name = """h2"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0
item6.default = item6_default
item6_description = """(m)    length of guide"""
item6.description = item6_description
item6_name = """l"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0.99
item7.default = item7_default
item7_description = """(1)    Low-angle reflectivity"""
item7.description = item7_description
item7_name = """R0"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0.0219
item8.default = item8_default
item8_description = """(AA-1) Critical scattering vector"""
item8.description = item8_description
item8_name = """Qc"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 6.07
item9.default = item9_default
item9_description = """(AA)   Slope of reflectivity"""
item9.description = item9_description
item9_name = """alpha"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 1.0
item10.default = item10_default
item10_description = """(1)    m-value of material. Zero means completely absorbing."""
item10.description = item10_description
item10_name = """m"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = 0.003
item11.default = item11_default
item11_description = """(AA-1) Width of supermirror cut-off"""
item11.description = item11_description
item11_name = """W"""
item11.name = item11_name
item11_type = """double"""
item11.type = item11_type
item12 = Parameter()
item12_default = 1.0
item12.default = item12_default
item12_description = """(1)    Number of vertical channels in the guide (>= 1)"""
item12.description = item12_description
item12_name = """k"""
item12.name = item12_name
item12_type = """double"""
item12.type = item12_type
item13 = Parameter()
item13_default = 0.0005
item13.default = item13_default
item13_description = """(m)    Thickness of subdividing walls"""
item13.description = item13_description
item13_name = """d"""
item13.name = item13_name
item13_type = """double"""
item13.type = item13_type
item14 = Parameter()
item14_default = -1.0
item14.default = item14_default
item14_description = """(1)    m-value of material for left.   vert. mirror"""
item14.description = item14_description
item14_name = """mleft"""
item14.name = item14_name
item14_type = """double"""
item14.type = item14_type
item15 = Parameter()
item15_default = -1.0
item15.default = item15_default
item15_description = """(1)    m-value of material for right.  vert. mirror"""
item15.description = item15_description
item15_name = """mright"""
item15.name = item15_name
item15_type = """double"""
item15.type = item15_type
item16 = Parameter()
item16_default = -1.0
item16.default = item16_default
item16_description = """(1)    m-value of material for top.    horz. mirror"""
item16.description = item16_description
item16_name = """mtop"""
item16.name = item16_name
item16_type = """double"""
item16.type = item16_type
item17 = Parameter()
item17_default = -1.0
item17.default = item17_default
item17_description = """(1)    m-value of material for bottom. horz. mirror"""
item17.description = item17_description
item17_name = """mbottom"""
item17.name = item17_name
item17_type = """double"""
item17.type = item17_type
item18 = Parameter()
item18_default = 1.0
item18.default = item18_default
item18_description = """(1)    Number of horizontal channels in the guide (>= 1)."""
item18.description = item18_description
item18_name = """kh"""
item18.name = item18_name
item18_type = """double"""
item18.type = item18_type
item19 = Parameter()
item19_default = 0.0
item19.default = item19_default
item19_description = """(m/s2) Gravitation norm. 0 value disables G effects."""
item19.description = item19_description
item19_name = """G"""
item19.name = item19_name
item19_type = """double"""
item19.type = item19_type
item20 = Parameter()
item20_default = 0.0
item20.default = item20_default
item20_description = """(deg)  Global guide waviness"""
item20.description = item20_description
item20_name = """wavy"""
item20.name = item20_name
item20_type = """double"""
item20.type = item20_type
item21 = Parameter()
item21_default = 0.0
item21.default = item21_default
item21_description = """(deg)  Partial waviness along propagation axis"""
item21.description = item21_description
item21_name = """wavy_z"""
item21.name = item21_name
item21_type = """double"""
item21.type = item21_type
item22 = Parameter()
item22_default = 0.0
item22.default = item22_default
item22_description = """(deg)  Partial waviness in transverse direction for top/bottom mirrors"""
item22.description = item22_description
item22_name = """wavy_tb"""
item22.name = item22_name
item22_type = """double"""
item22.type = item22_type
item23 = Parameter()
item23_default = 0.0
item23.default = item23_default
item23_description = """(deg)  Partial waviness in transverse direction for left/right mirrors"""
item23.description = item23_description
item23_name = """wavy_lr"""
item23.name = item23_name
item23_type = """double"""
item23.type = item23_type
item24 = Parameter()
item24_default = 0.0
item24.default = item24_default
item24_description = """(m)    Global chamfers specifications (in/out/mirror sides)."""
item24.description = item24_description
item24_name = """chamfers"""
item24.name = item24_name
item24_type = """double"""
item24.type = item24_type
item25 = Parameter()
item25_default = 0.0
item25.default = item25_default
item25_description = """(m) Input and output chamfers"""
item25.description = item25_description
item25_name = """chamfers_z"""
item25.name = item25_name
item25_type = """double"""
item25.type = item25_type
item26 = Parameter()
item26_default = 0.0
item26.default = item26_default
item26_description = """(m) Chamfers on left/right mirror sides"""
item26.description = item26_description
item26_name = """chamfers_lr"""
item26.name = item26_name
item26_type = """double"""
item26.type = item26_type
item27 = Parameter()
item27_default = 0.0
item27.default = item27_default
item27_description = """(m) Chamfers on top/bottom mirror sides"""
item27.description = item27_description
item27_name = """chamfers_tb"""
item27.name = item27_name
item27_type = """double"""
item27.type = item27_type
item28 = Parameter()
item28_default = 1.0
item28.default = item28_default
item28_description = """(1) Number of sections in the guide (length l/nelements)."""
item28.description = item28_description
item28_name = """nelements"""
item28.name = item28_name
item28_type = """double"""
item28.type = item28_type
item29 = Parameter()
item29_default = 0.0
item29.default = item29_default
item29_description = """(Hz) Rotation frequency (round/s) for Fermi Chopper approximation"""
item29.description = item29_description
item29_name = """nu"""
item29.name = item29_name
item29_type = """double"""
item29.type = item29_type
item30 = Parameter()
item30_default = 0.0
item30.default = item30_default
item30_description = """(deg) Phase shift for the Fermi Chopper approximation"""
item30.description = item30_description
item30_name = """phase"""
item30.name = item30_name
item30_type = """double"""
item30.type = item30_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11, item12, item13, item14, item15, item16, item17, item18, item19, item20, item21, item22, item23, item24, item25, item26, item27, item28, item29, item30 ]
info.input_parameters = info_input_parameters
info_name = """Guide_gravity"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """GVars"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """"""
item1.description = item1_description
item1_name = """pTable"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
info_output_parameters = [ item0, item1 ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """

// ----------- added by Alex Dementsov --------
// ----------  header for SHARE ----------
// ----------- added by Alex Dementsov --------

#ifndef Gravity_guide_Version
#define Gravity_guide_Version "$Revision: 1.31 $"



#ifndef GUIDE_GRAVITY_DEFS
#define GUIDE_GRAVITY_DEFS

#include "mcstas2/share/read_table-lib.h"



  typedef struct Gravity_guide_Vars
  {
    double gx;
    double gy;
    double gz;
    double nx[6], ny[6], nz[6];
    double wx[6], wy[6], wz[6];
    double A[6], norm_n2[6], norm_n[6];
    long   N_reflection[7];
    double w1c, h1c;
    double w2c, h2c;
    double M[5];
    double nzC[5], norm_n2xy[5], Axy[5];
    double wav_lr, wav_tb, wav_z;
    double chamfer_z, chamfer_lr, chamfer_tb;
    char   compcurname[256];
    double fc_freq, fc_phase;
  } Gravity_guide_Vars_type;


  void Gravity_guide_Init(Gravity_guide_Vars_type *aVars,
    MCNUM a_w1, MCNUM a_h1, MCNUM a_w2, MCNUM a_h2, MCNUM a_l, MCNUM a_R0,
    MCNUM a_Qc, MCNUM a_alpha, MCNUM a_m, MCNUM a_W, MCNUM a_k, MCNUM a_d,
    MCNUM a_Gx, MCNUM a_Gy, MCNUM a_Gz,
    MCNUM a_mleft, MCNUM a_mright, MCNUM a_mtop, MCNUM a_mbottom, MCNUM a_kh,
    MCNUM a_wavy_lr, MCNUM a_wavy_tb, MCNUM a_wavy_z, MCNUM a_wavy,
    MCNUM a_chamfers_z, MCNUM a_chamfers_lr, MCNUM a_chamfers_tb, MCNUM a_chamfers,
    MCNUM a_nu, MCNUM a_phase);


  int Gravity_guide_Trace(double *dt,
        Gravity_guide_Vars_type *aVars,
        double cx, double cy, double cz,
        double cvx, double cvy, double cvz,
        double cxnum, double cxk, double cynum, double cyk,
        double *cnx, double *cny,double *cnz);

#endif
#endif

#include "mcstas2/share/read_table-lib.h"


// ----------- added by Alex Dementsov --------"""
item1 = """// ----------  implementation for SHARE ----------
// ----------- added by Alex Dementsov --------

  void Gravity_guide_Init(Gravity_guide_Vars_type *aVars,
    MCNUM a_w1, MCNUM a_h1, MCNUM a_w2, MCNUM a_h2, MCNUM a_l, MCNUM a_R0,
    MCNUM a_Qc, MCNUM a_alpha, MCNUM a_m, MCNUM a_W, MCNUM a_k, MCNUM a_d,
    MCNUM a_Gx, MCNUM a_Gy, MCNUM a_Gz,
    MCNUM a_mleft, MCNUM a_mright, MCNUM a_mtop, MCNUM a_mbottom, MCNUM a_kh,
    MCNUM a_wavy_lr, MCNUM a_wavy_tb, MCNUM a_wavy_z, MCNUM a_wavy,
    MCNUM a_chamfers_z, MCNUM a_chamfers_lr, MCNUM a_chamfers_tb, MCNUM a_chamfers,
    MCNUM a_nu, MCNUM a_phase)
  {
    int i;

    for (i=0; i<7; aVars->N_reflection[i++] = 0);
    for (i=0; i<5; aVars->M[i++] = 0);

    aVars->gx = a_Gx; 
    aVars->gy = a_Gy;
    aVars->gz = a_Gz;

    if (a_k <= 0 || a_kh <= 0) { fprintf(stderr,"%s: Fatal: no channel in this guide (kh or k=0).\n", aVars->compcurname); exit(-1); }
    if (a_d < 0) { fprintf(stderr,"%s: Fatal: subdividing walls have negative thickness in this guide (d<0).\n", aVars->compcurname); exit(-1); }
    aVars->w1c = (a_w1 - (a_k-1) *a_d)/(double)a_k;
    aVars->w2c = (a_w2 - (a_k-1) *a_d)/(double)a_k;
    aVars->h1c = (a_h1 - (a_kh-1)*a_d)/(double)a_kh;
    aVars->h2c = (a_h2 - (a_kh-1)*a_d)/(double)a_kh;

    for (i=0; i <= 4;   aVars->M[i++]=a_m);
    if (a_mleft   >= 0) aVars->M[1] =a_mleft  ;
    if (a_mright  >= 0) aVars->M[2] =a_mright ;
    if (a_mtop    >= 0) aVars->M[3] =a_mtop   ;
    if (a_mbottom >= 0) aVars->M[4] =a_mbottom;
    
    aVars->nx[1] =  a_l; aVars->ny[1] =  0;   aVars->nz[1] =  0.5*(aVars->w2c-aVars->w1c);  
    aVars->nx[2] = -a_l; aVars->ny[2] =  0;   aVars->nz[2] = -aVars->nz[1];             
    aVars->nx[3] =  0;   aVars->ny[3] =  a_l; aVars->nz[3] =  0.5*(aVars->h2c-aVars->h1c);  
    aVars->nx[4] =  0;   aVars->ny[4] = -a_l; aVars->nz[4] = -aVars->nz[3];             
    aVars->nx[5] =  0;   aVars->ny[5] =  0;   aVars->nz[5] =  a_l;                      
    aVars->nx[0] =  0;   aVars->ny[0] =  0;   aVars->nz[0] = -a_l;                      
    
    aVars->wx[1] = +(aVars->w1c)/2; aVars->wy[1] =  0;              aVars->wz[1] = 0;   
    aVars->wx[2] = -(aVars->w1c)/2; aVars->wy[2] =  0;              aVars->wz[2] = 0;   
    aVars->wx[3] =  0;              aVars->wy[3] = +(aVars->h1c)/2; aVars->wz[3] = 0;   
    aVars->wx[4] =  0;              aVars->wy[4] = -(aVars->h1c)/2; aVars->wz[4] = 0;   
    aVars->wx[5] =  0;              aVars->wy[5] =  0;              aVars->wz[5] = a_l; 
    aVars->wx[0] =  0;              aVars->wy[0] =  0;              aVars->wz[0] = 0;   

    for (i=0; i <= 5; i++)
    {
      aVars->A[i] = scalar_prod(aVars->nx[i], aVars->ny[i], aVars->nz[i], aVars->gx, aVars->gy, aVars->gz)/2;
      aVars->norm_n2[i] = aVars->nx[i]*aVars->nx[i] + aVars->ny[i]*aVars->ny[i] + aVars->nz[i]*aVars->nz[i];
      if (aVars->norm_n2[i] <= 0)
        { fprintf(stderr,"%s: Fatal: normal vector norm %i is null/negative ! check guide dimensions.\n", aVars->compcurname, i); exit(-1); } 
      else
        aVars->norm_n[i] = sqrt(aVars->norm_n2[i]);
    }
    
    for (i=1; i <= 4; i++)
    { 
      aVars->nzC[i]      =  aVars->nz[i]; 
      aVars->norm_n2xy[i]=  aVars->nx[i]*aVars->nx[i] + aVars->ny[i]*aVars->ny[i];
      aVars->Axy[i]      = (aVars->nx[i]*aVars->gx    + aVars->ny[i]*aVars->gy)/2;
    }
    
    if (a_wavy && (!a_wavy_tb && !a_wavy_lr && !a_wavy_z))
    { aVars->wav_tb=aVars->wav_lr=aVars->wav_z=a_wavy; }
    else
    { aVars->wav_tb=a_wavy_tb; aVars->wav_lr=a_wavy_lr; aVars->wav_z=a_wavy_z; }
    aVars->wav_tb *= DEG2RAD/(sqrt(8*log(2)));   
    aVars->wav_lr *= DEG2RAD/(sqrt(8*log(2)));
    aVars->wav_z  *= DEG2RAD/(sqrt(8*log(2)));
    
    if (a_chamfers && (!a_chamfers_z && !a_chamfers_lr && !a_chamfers_tb))
    { aVars->chamfer_z=aVars->chamfer_lr=aVars->chamfer_tb=a_chamfers; }
    else
    {
      aVars->chamfer_z=a_chamfers_z;
      aVars->chamfer_lr=a_chamfers_lr;
      aVars->chamfer_tb=a_chamfers_tb;
    }

    aVars->fc_freq  = a_nu;
    aVars->fc_phase = a_phase;
  }

  int Gravity_guide_Trace(double *dt,
        Gravity_guide_Vars_type *aVars,
        double cx, double cy, double cz,
        double cvx, double cvy, double cvz,
        double cxnum, double cxk, double cynum, double cyk,
        double *cnx, double *cny,double *cnz)
  {
    double B, C;
    int    ret=0;
    int    side=0;
    double n1;
    double dt0, dt_min=0;
    int    i;
    double loc_num, loc_k;
    int    i_slope=3;

    
    
    
    B = aVars->nz[5]*cvz; C = aVars->nz[5]*(cz - aVars->wz[5]);
    ret = solve_2nd_order(&dt0, aVars->A[5], B, C);
    if (ret && dt0>1e-10) { dt_min = dt0; side=5; }

    loc_num = cynum; loc_k = cyk;
    for (i=4; i>0; i--)
    {
      if (i == 2) { i_slope=1; loc_num = cxnum; loc_k = cxk; }

      if (aVars->nzC[i_slope] != 0) {
        n1 = loc_k - 2*(loc_num);  
        loc_num++; 
        aVars->nz[i]= aVars->nzC[i]*n1;
        aVars->A[i] = aVars->Axy[i] + aVars->nz[i]*aVars->gz/2;
      }
      if (i < 3)
      {      B = aVars->nx[i]*cvx + aVars->nz[i]*cvz; C = aVars->nx[i]*(cx-aVars->wx[i]) + aVars->nz[i]*cz; }
      else { B = aVars->ny[i]*cvy + aVars->nz[i]*cvz; C = aVars->ny[i]*(cy-aVars->wy[i]) + aVars->nz[i]*cz; }
      ret = solve_2nd_order(&dt0, aVars->A[i], B, C);
      if (ret && dt0>1e-10 && (dt0<dt_min || !dt_min))
      { dt_min = dt0; side=i;
        if (aVars->nzC[i] != 0)
        { aVars->norm_n2[i] = aVars->norm_n2xy[i] + aVars->nz[i]*aVars->nz[i];
          aVars->norm_n[i]  = sqrt(aVars->norm_n2[i]); }
      }
     }

    *dt = dt_min;
    
    if (side > 0 && side < 5 && (aVars->wav_z || aVars->wav_lr || aVars->wav_tb))
    {
      double nt_x, nt_y, nt_z;  
      double nn_x, nn_y, nn_z;  
      double phi;
      
      vec_prod(nt_x,nt_y,nt_z, aVars->nx[side],aVars->ny[side],aVars->nz[side], 0,0,1);
      
      if (aVars->wav_z) {
        phi = aVars->wav_z;
        rotate(nn_x,nn_y,nn_z, aVars->nx[side],aVars->ny[side],aVars->nz[side], aVars->wav_z*randnorm(), nt_x,nt_y,nt_z);
      } else { nn_x=aVars->nx[side]; nn_y=aVars->ny[side]; nn_z=aVars->nz[side]; }
      
      phi = (side <=2) ? aVars->wav_lr : aVars->wav_tb;
      if (phi) {
        rotate(nt_x,nt_y,nt_z, nn_x,nn_y,nn_z, phi*randnorm(), 0,0,1);
      } else { nt_x=nn_x; nt_y=nn_y; nt_z=nn_z; }
      *cnx=nt_x; *cny=nt_y; *cnz=nt_z;
    } else
    { *cnx=aVars->nx[side]; *cny=aVars->ny[side]; *cnz=aVars->nz[side]; }
    return (side);
  }


"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Waviness may be specified, as well as side chamfers (on substrate)."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
  if (l > 0 && nelements > 0) {
    double B, C, dt;
    int    ret, bounces = 0;
    double this_width, this_height;
    double angle=0;

    if (GVars.fc_freq != 0 || GVars.fc_phase != 0) { 
      
      Coords   X = coords_set(x,y,z-l/2);  
      Rotation R;
      angle=fmod(360*GVars.fc_freq*t+GVars.fc_phase, 360); 
      
      if (angle > 90 && angle < 270) angle -= 180;
      angle *= DEG2RAD;
      rot_set_rotation(R, 0, -angle, 0); 
      
      Coords   RX = rot_apply(R, X);
      coords_get(RX, &x, &y, &z);
      z = z+l/2;
      
      X  = coords_set(vx,vy,vz);
      RX = rot_apply(R, X);
      coords_get(RX, &vx, &vy, &vz);
    }

    
    
    
    B = -l*vz; C = -l*z;

    ret = solve_2nd_order(&dt, GVars.A[0], B, C);
    if (ret==0) ABSORB;

    if (dt>0.0) PROP_GRAV_DT(dt, GVars.gx, GVars.gy, GVars.gz); else if (angle) ABSORB;
    GVars.N_reflection[6]++;

    this_width  = w1;
    this_height = h1;

  
    if (fabs(x) > this_width/2 || fabs(y) > this_height/2)
      ABSORB;
    else
    {
      double w_edge, w_adj; 
      double h_edge, h_adj; 
      double w_chnum,h_chnum; 

      
      x += w1/2.0;
      w_chnum = floor(x/(GVars.w1c+d));  
      w_edge  = w_chnum*(GVars.w1c+d);
      if(x - w_edge > GVars.w1c)
      {
        x -= w1/2.0; 
        ABSORB;
      }
      w_adj = w_edge + (GVars.w1c)/2.0;
      x -= w_adj; w_adj -=  w1/2.0;

      
      y += h1/2.0;
      h_chnum = floor(y/(GVars.h1c+d));  
      h_edge  = h_chnum*(GVars.h1c+d);
      if(y - h_edge > GVars.h1c)
      {
        y -= h1/2.0; 
        ABSORB;
      }
      h_adj = h_edge + (GVars.h1c)/2.0;
      y -= h_adj; h_adj -=  h1/2.0;


      
      
      for(;;)
      {
        
        double q, nx,ny,nz;
        double this_length;
        int side=0;

        bounces++;
        
        side = Gravity_guide_Trace(&dt, &GVars, x, y, z,
            vx, vy, vz, w_chnum, k, h_chnum, kh,
            &nx, &ny, &nz);

        
        
        if (side == 0 || dt <= 0)
          { fprintf(stderr,"%s: warning: neutron has entered guide, but can not exit !\n", GVars.compcurname);
            x += w_adj; y += h_adj; ABSORB; } 

        
        PROP_GRAV_DT(dt, GVars.gx, GVars.gy, GVars.gz);

        
        if (side == 5) 
          { GVars.N_reflection[side]++; x += w_adj; y += h_adj; SCATTER; x -= w_adj; y -= h_adj; break; }
        
        if(GVars.M[side] == 0 || Qc == 0 || R0 == 0)  
          { x += w_adj; y += h_adj; ABSORB; }
        
        this_width = w1+(w2-w1)*z/l;
        this_height= h1+(h2-h1)*z/l;
        this_length= fmod(z, l/nelements);
        
        if (GVars.chamfer_z && (this_length<GVars.chamfer_z || this_length>l/nelements-GVars.chamfer_z))
        { x += w_adj; y += h_adj; ABSORB; }
        
        if (GVars.chamfer_lr && (side==1 || side==2) && (fabs(y+h_adj)>this_height/2-GVars.chamfer_lr))
        { x += w_adj; y += h_adj; ABSORB; }
        if (GVars.chamfer_tb && (side==3 || side==4) && (fabs(x+w_adj)>this_width/2- GVars.chamfer_tb))
        { x += w_adj; y += h_adj; ABSORB; }
        
        GVars.N_reflection[side]++; 
        
        B = scalar_prod(vx,vy,vz,nx,ny,nz);
        dt = 2*B/GVars.norm_n2[side]; 
        vx -= nx*dt;
        vy -= ny*dt;
        vz -= nz*dt;

        
        
        q = 2*V2Q*fabs(B)/GVars.norm_n[side];

        if (reflect && strlen(reflect))
          B=Table_Value(pTable, q, 1);
        else {
          B = R0;
          if(q > Qc)
          {
            double arg;
            if (W>0) arg = (q-GVars.M[side]*Qc)/W;
            else arg = (q-GVars.M[side]*Qc)*10000; 

            if(arg < 10) B *= .5*(1-tanh(arg))*(1-alpha*(q-Qc));
            else { x += w_adj; y += h_adj; ABSORB; }; 
          }
        }
        if (B < 0) B=0;
        else if (B > 1) B=1;
        p *= B;
        x += w_adj; y += h_adj; SCATTER; x -= w_adj; y -= h_adj;
        GVars.N_reflection[0]++;
        
        if (bounces > 1000) ABSORB;
      } 
      x += w_adj; y += h_adj; 
    }

    if (GVars.fc_freq != 0 || GVars.fc_phase != 0) { 
      
      Coords   X = coords_set(x,y,z-l/2);  
      Rotation R;
      rot_set_rotation(R, 0, angle, 0); 
      
      Coords   RX = rot_apply(R, X);
      coords_get(RX, &x, &y, &z);
      z = z+l/2;
      
      X  = coords_set(vx,vy,vz);
      RX = rot_apply(R, X);
      coords_get(RX, &vx, &vy, &vz);
    }

  } 
"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
