
def factory( name='guide_wavy',w1=0,h1=0,w2=0,h2=0,l=0,R0=0,Qc=0,alpha1=0,m1=0,W1=0,alpha2=0,m2=0,W2=0,alpha3=0,m3=0,W3=0,alpha4=0,m4=0,W4=0,wavy_z=0,wavy_xy=0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from Guide_wavyboostpython import Guide_wavy as _factory_mcvine
    return component( _factory_mcvine( name,w1,h1,w2,h2,l,R0,Qc,alpha1,m1,W1,alpha2,m2,W2,alpha3,m3,W3,alpha4,m4,W4,wavy_z,wavy_xy ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Kim Lefmann"""
info.copyright = info_copyright
info_declare = """
 double whalf,hhalf, lwhalf, lhhalf, norm_nv, norm_nh, f_h, f_v, eta_z, eta_xy;
"""
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """Models a rectangular guide tube centered on the Z axis. The entrance lies
in the X-Y plane.
For details on the geometry calculation see the description in the McStas
reference manual.

%BUGS
This component does not work with gravitation on. Use component Guide_gravity then."""
info.full_description = info_full_description
info_initialize = """
  f_h = 0.5*(w2 - w1), f_v = 0.5*(h2 - h1);
  whalf = 0.5*w1, hhalf = 0.5*h1;
  lwhalf = l*whalf, lhhalf = l*hhalf;
  norm_nv = sqrt(l*l+f_v*f_v);
  norm_nh = sqrt(l*l+f_h*f_h);
  eta_z = wavy_z/(sqrt(8*log(2)));   
  eta_xy = wavy_xy/(sqrt(8*log(2)));


"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """guide_wavy"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """(m)    Width at the guide entry"""
item1.description = item1_description
item1_name = """w1"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """(m)    Height at the guide entry"""
item2.description = item2_description
item2_name = """h1"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """(m)    Width at the guide exit"""
item3.description = item3_description
item3_name = """w2"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """(m)    Height at the guide exit"""
item4.description = item4_description
item4_name = """h2"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0
item5.default = item5_default
item5_description = """(m)    length of guide"""
item5.description = item5_description
item5_name = """l"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0
item6.default = item6_default
item6_description = """(1)    Low-angle reflectivity"""
item6.description = item6_description
item6_name = """R0"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0
item7.default = item7_default
item7_description = """(AA-1) Critical scattering vector"""
item7.description = item7_description
item7_name = """Qc"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0
item8.default = item8_default
item8_description = """(AA)   Slope of reflectivity"""
item8.description = item8_description
item8_name = """alpha1"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0
item9.default = item9_default
item9_description = """(1)    m-value of material, left"""
item9.description = item9_description
item9_name = """m1"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 0
item10.default = item10_default
item10_description = """(AA-1) Width of supermirror cut-off"""
item10.description = item10_description
item10_name = """W1"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = 0
item11.default = item11_default
item11_description = """(AA)   Slope of reflectivity"""
item11.description = item11_description
item11_name = """alpha2"""
item11.name = item11_name
item11_type = """double"""
item11.type = item11_type
item12 = Parameter()
item12_default = 0
item12.default = item12_default
item12_description = """(1)    m-value of material, right."""
item12.description = item12_description
item12_name = """m2"""
item12.name = item12_name
item12_type = """double"""
item12.type = item12_type
item13 = Parameter()
item13_default = 0
item13.default = item13_default
item13_description = """(AA-1) Width of supermirror cut-off"""
item13.description = item13_description
item13_name = """W2"""
item13.name = item13_name
item13_type = """double"""
item13.type = item13_type
item14 = Parameter()
item14_default = 0
item14.default = item14_default
item14_description = """(AA)   Slope of reflectivity"""
item14.description = item14_description
item14_name = """alpha3"""
item14.name = item14_name
item14_type = """double"""
item14.type = item14_type
item15 = Parameter()
item15_default = 0
item15.default = item15_default
item15_description = """(1)    m-value of material, top."""
item15.description = item15_description
item15_name = """m3"""
item15.name = item15_name
item15_type = """double"""
item15.type = item15_type
item16 = Parameter()
item16_default = 0
item16.default = item16_default
item16_description = """(AA-1) Width of supermirror cut-off"""
item16.description = item16_description
item16_name = """W3"""
item16.name = item16_name
item16_type = """double"""
item16.type = item16_type
item17 = Parameter()
item17_default = 0
item17.default = item17_default
item17_description = """(AA)   Slope of reflectivity"""
item17.description = item17_description
item17_name = """alpha4"""
item17.name = item17_name
item17_type = """double"""
item17.type = item17_type
item18 = Parameter()
item18_default = 0
item18.default = item18_default
item18_description = """(1)    m-value of material, bottom."""
item18.description = item18_description
item18_name = """m4"""
item18.name = item18_name
item18_type = """double"""
item18.type = item18_type
item19 = Parameter()
item19_default = 0
item19.default = item19_default
item19_description = """(AA-1) Width of supermirror cut-off"""
item19.description = item19_description
item19_name = """W4"""
item19.name = item19_name
item19_type = """double"""
item19.type = item19_type
item20 = Parameter()
item20_default = 0
item20.default = item20_default
item20_description = """(deg)  Waviness in the z-(flight-)direction"""
item20.description = item20_description
item20_name = """wavy_z"""
item20.name = item20_name
item20_type = """double"""
item20.type = item20_type
item21 = Parameter()
item21_default = 0
item21.default = item21_default
item21_description = """(deg)  Waviness in the transverse direction"""
item21.description = item21_description
item21_name = """wavy_xy"""
item21.name = item21_name
item21_type = """double"""
item21.type = item21_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11, item12, item13, item14, item15, item16, item17, item18, item19, item20, item21 ]
info.input_parameters = info_input_parameters
info_name = """Guide_wavy"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """whalf"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """"""
item1.description = item1_description
item1_name = """hhalf"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """"""
item2.description = item2_description
item2_name = """lwhalf"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """"""
item3.description = item3_description
item3_name = """lhhalf"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """"""
item4.description = item4_description
item4_name = """norm_nv"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0
item5.default = item5_default
item5_description = """"""
item5.description = item5_description
item5_name = """norm_nh"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0
item6.default = item6_default
item6_description = """"""
item6.description = item6_description
item6_name = """f_h"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0
item7.default = item7_default
item7_description = """"""
item7.description = item7_description
item7_name = """f_v"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0
item8.default = item8_default
item8_description = """"""
item8.description = item8_description
item8_name = """eta_z"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0
item9.default = item9_default
item9_description = """"""
item9.description = item9_description
item9_name = """eta_xy"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
info_output_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """"""
item1 = """"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Neutron guide with gaussian waviness."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
  double t_min,t_tmp;                           
  double av,ah,bv,bh,cv1,cv2,ch1,ch2,d;         
  double vdotn_v1,vdotn_v2,vdotn_h1,vdotn_h2;   
  double vdotn;
  double d_xy, d_z;                             
  double m, alpha, W;
  int i;                                        
  double q;                                     
  double dvx, dvy, dvz;                         
  double vlen2,nlen2, norm_n2;                  
  double nperp,pz,nxy;                          
  double R;                                     

  
  PROP_Z0;
  
  SCATTER;
  if(x <= -whalf || x >= whalf || y <= -hhalf || y >= hhalf)
    ABSORB;
  for(;;)
  {
    
    av = -l*vx ; bv = f_h*vz;
    ah = -l*vy ; bh = f_v*vz;
    vdotn_v1 = bv + av;         
    vdotn_v2 = bv - av;         
    vdotn_h1 = bh + ah;         
    vdotn_h2 = bh - ah;         
    
    cv1 = -whalf*l - z*f_h; cv2 = x*l;
    ch1 = -hhalf*l - z*f_v; ch2 = y*l;
    
    t_min = (l - z)/vz;
  
    i = 0;
    if(vdotn_v1 < 0 && (t_tmp = (cv1 + cv2)/vdotn_v1) < t_min)
    {
      t_min = t_tmp;
      i = 1;

    }
    if(vdotn_v2 < 0 && (t_tmp = (cv1 - cv2)/vdotn_v2) < t_min)
    {
      t_min = t_tmp;
      i = 2;

    }
    if(vdotn_h1 < 0 && (t_tmp = (ch1 + ch2)/vdotn_h1) < t_min)
    {
      t_min = t_tmp;
      i = 3;

    }
    if(vdotn_h2 < 0 && (t_tmp = (ch1 - ch2)/vdotn_h2) < t_min)
    {
      t_min = t_tmp;
      i = 4;

    }
    if(i == 0)
      break;                    
    PROP_DT(t_min);



    d_z=DEG2RAD*eta_z*randnorm();
    d_xy=DEG2RAD*eta_xy*randnorm();



    switch(i)
    {
      case 1:                   
        m=m1; W=W1; alpha=alpha1;
        norm_n2 =  (-l+d_z*f_h)*(-l+d_z*f_h)+(d_xy*l)*(d_xy*l)
                  +(f_h-d_z*l)*(f_h-d_z*l); 
        vdotn = (vx*(-l+f_h*d_z)+ vy*(l*d_xy)+ vz*(f_h-l*d_z) );
        q = 2 * V2Q * fabs(vdotn) / sqrt(norm_n2);
        dvx = -2*(-l+f_h*d_z)*vdotn/norm_n2;
        dvy = -2*(l*d_xy)*vdotn/norm_n2;
        dvz = -2*(f_h-l*d_z)*vdotn/norm_n2;
        break;
      case 2:                   
        m=m2; W=W2; alpha=alpha2;
        norm_n2 =  (l+d_z*f_h)*(l+d_z*f_h)+(-d_xy*l)*(-d_xy*l)
                  +(f_h+d_z*l)*(f_h+d_z*l); 
        vdotn = (vx*(l+f_h*d_z)+ vy*(-l*d_xy)+ vz*(f_h+l*d_z) );
        q = 2 * V2Q * fabs(vdotn) / sqrt(norm_n2);
        dvx = -2*(l+f_h*d_z)*vdotn/norm_n2;
        dvy = -2*(-l*d_xy)*vdotn/norm_n2;
        dvz = -2*(f_h+l*d_z)*vdotn/norm_n2;
        break;
      case 3:                   
        m=m3; W=W3; alpha=alpha3;
        norm_n2 =  (d_xy*l)*(d_xy*l)+(-l+d_z*f_v)*(-l+d_z*f_v)
                  +(f_v-d_z*l)*(f_v-d_z*l); 
        vdotn = (vx*(l*d_xy)+ vy*(-l+f_v*d_z)+ vz*(f_v-l*d_z) );
        q = 2 * V2Q * fabs(vdotn) / sqrt(norm_n2);
        dvx = -2*(l*d_xy)*vdotn/norm_n2;
        dvy = -2*(-l+f_v*d_z)*vdotn/norm_n2;
        dvz = -2*(f_v-l*d_z)*vdotn/norm_n2;
        break;
      case 4:                   
        m=m4; W=W4; alpha=alpha4;
        norm_n2 =  (-d_xy*l)*(-d_xy*l)+(l+d_z*f_v)*(l+d_z*f_v)
                  +(f_v+d_z*l)*(f_v+d_z*l); 
        vdotn = (vx*(-l*d_xy)+ vy*(l+f_v*d_z)+ vz*(f_v+l*d_z) );
        q = 2 * V2Q * fabs(vdotn) / sqrt(norm_n2);
        dvx = -2*(-l*d_xy)*vdotn/norm_n2;
        dvy = -2*(l+f_v*d_z)*vdotn/norm_n2;
        dvz = -2*(f_v+l*d_z)*vdotn/norm_n2;
        break;
      default:
        printf("Fatal error: No guide wall hit");
        exit(1);
    }
    
    if(m == 0)
      ABSORB;
    if(q > Qc)
    {
      double arg = (q-m*Qc)/W;
      if(arg < 10)
      {
        R = .5*(1-tanh(arg))*(1-alpha*(q-Qc));
        p *= R;
        
      }
      else
        ABSORB;                               
    }
    p *= R0;               
    vx += dvx;
    vy += dvy;
    vz += dvz;
    SCATTER;
  }
"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
