
def factory( name='mirror',reflect='',xlength=0,yheight=0,R0=0.99,Qc=0.021,alpha=6.07,m=2.0,W=0.003,center=0.0,transmit=0.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from Mirrorboostpython import Mirror as _factory_mcvine
    return component( _factory_mcvine( name,reflect,xlength,yheight,R0,Qc,alpha,m,W,center,transmit ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Kristian Nielsen"""
info.copyright = info_copyright
info_declare = """
  t_Table pTable;
"""
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """Single mirror plate (used to build guides in compound components).
The component Mirror models a single rectangular neutron mirror plate.
It can be used as a sample component or to e.g. assemble a complete neutron
guide by putting multiple mirror components at appropriate locations and
orientations in the instrument definition, much like a real guide is build
from individual mirrors.
Additionally, it may be centered in order to be used as a sample or
monochromator plate, else it starts from AT position.
Reflectivity is either defined by an analytical model (see Component Manual)
or from a two-columns file with q [Angs-1] as first and R [0-1] as second.
In the local coordinate system, the mirror lies in the first quadrant of the
x-y plane, with one corner at (0, 0, 0). Plane is thus perpendicular to the
incoming beam, and should usually be rotated."""
info.full_description = info_full_description
info_initialize = """
  if (reflect && strlen(reflect)) {
    if (Table_Read(&pTable, reflect, 1) <= 0) 
      exit(fprintf(stderr,"Mirror: %s: can not read file %s\n", NAME_CURRENT_COMP, reflect));
  }
"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """mirror"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = """"""
item1.default = item1_default
item1_description = """(str)  Name of relfectivity file. Format [q(Angs-1) R(0-1)]"""
item1.description = item1_description
item1_name = """reflect"""
item1.name = item1_name
item1_type = """char *"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """(m)    length of mirror plate"""
item2.description = item2_description
item2_name = """xlength"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """(m)    height of mirror plate"""
item3.description = item3_description
item3_name = """yheight"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0.99
item4.default = item4_default
item4_description = """(1)    Low-angle reflectivity"""
item4.description = item4_description
item4_name = """R0"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0.021
item5.default = item5_default
item5_description = """(AA-1) Critical scattering vector"""
item5.description = item5_description
item5_name = """Qc"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 6.07
item6.default = item6_default
item6_description = """(AA)   Slope of reflectivity"""
item6.description = item6_description
item6_name = """alpha"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 2.0
item7.default = item7_default
item7_description = """(1)    m-value of material. Zero means completely absorbing."""
item7.description = item7_description
item7_name = """m"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0.003
item8.default = item8_default
item8_description = """(AA-1) Width of supermirror cut-off"""
item8.description = item8_description
item8_name = """W"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0.0
item9.default = item9_default
item9_description = """(1)    if true (1), the Mirror is centered arount AT position."""
item9.description = item9_description
item9_name = """center"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 0.0
item10.default = item10_default
item10_description = """(1)    When true, non reflected neutrons are transmitted through"""
item10.description = item10_description
item10_name = """transmit"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10 ]
info.input_parameters = info_input_parameters
info_name = """Mirror"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """pTable"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
info_output_parameters = [ item0 ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """
// ----------- added by Jiao Lin --------
// ----------  header for SHARE ----------
// ----------- added by Jiao Lin --------
#include "mcstas2/share/read_table-lib.h"

// ----------- added by Jiao Lin --------"""
item1 = """// ----------  implementation for SHARE ----------
// ----------- added by Jiao Lin --------
"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Single mirror plate."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
  double dt, q, B;
  char intersect=0;

  
  if(vz != 0.0 && (dt = -z/vz) >= 0)
  {
    double old_x = x, old_y = y;
    x += vx*dt;
    y += vy*dt;
    
    intersect = (center == 0 ?
      x >= 0 && x <= xlength && y >= 0 && y <= yheight :
      x>= -xlength/2 && x<=xlength/2 && y >= -yheight/2 && y <= yheight/2);
    if (intersect) {
      z = 0;
      t += dt;
      q = fabs(2*vz*V2Q);
      vz = -vz;
      
      if(m == 0)
        ABSORB;
      if (reflect && strlen(reflect))
         B=Table_Value(pTable, q, 1);
      else {
          B = R0;
          if(q > Qc)
          {
            double arg = (q-m*Qc)/W;
            if(arg < 10)
              B *= .5*(1-tanh(arg))*(1-alpha*(q-Qc));
            else  B=0;
          }
      }
      if (B < 0) B=0;
      else if (B > 1) B=1;
      
      if (!transmit) {
        if (!B) ABSORB;
        p *= B;
        SCATTER;
      } else {
        
        if (B == 0 || rand01() >= B) { vz = -vz; }
        SCATTER;
      }
    }

    if (!intersect) {
      
      x = old_x;
      y = old_y;
    }
  }
"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
