
def factory( name='monochromator_curved',reflect='',transmit='',zwidth=0.01,yheight=0.01,gap=0.0005,NH=11.0,NV=11.0,mosaich=30.0,mosaicv=30.0,r0=0.7,t0=1.0,Q=1.8734,RV=0.0,RH=0.0,DM=0.0,mosaic=0.0,width=0.0,height=0.0,verbose=0.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from Monochromator_curvedboostpython import Monochromator_curved as _factory_mcvine
    return component( _factory_mcvine( name,reflect,transmit,zwidth,yheight,gap,NH,NV,mosaich,mosaicv,r0,t0,Q,RV,RH,DM,mosaic,width,height,verbose ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Emmanuel Farhi, Kim, Lefmann, Peter Link"""
info.copyright = info_copyright
info_declare = """
  double mos_rms_y; 
  double mos_rms_z;
  double mos_rms_max;
  double mono_Q;
  double SlabWidth, SlabHeight;
  t_Table rTable, tTable;
  double row,col;
"""
info.declare = info_declare
info_finalize = """
  Table_Free(&rTable);
  Table_Free(&tTable);
"""
info.finalize = info_finalize
info_full_description = """escription
Double bent infinitely thin mosaic crystal, useful as a monochromator or
analyzer. which uses a small-mosaicity approximation and taking into account
higher  order scattering. The mosaic is anisotropic gaussian, with different
FWHMs in the Y and Z directions. The scattering vector is perpendicular to the
surface. For an unrotated monochromator component, the crystal plane lies in
the y-z plane (ie. parallel to the beam). The component works in reflection, but
also transmits the non-diffracted beam. Reflectivity and transmission files may
be used. The slabs are positioned in the vertical plane (not on a
cylinder/sphere), and are rotated according to the curvature radius.
When curvatures are set to 0, the monochromator is flat.
The curvatures approximation for parallel beam focusing to distance L, with
monochromator rotation angle A1 are:
RV = 2*L*sin(DEG2RAD*A1);
RH = 2*L/sin(DEG2RAD*A1);

When you rotate the component by A1 = asin(Q/2/Ki)*RAD2DEG, do not forget to
rotate the following components by A2=2*A1 (for 1st order) !

Example values for lattice parameters
PG       002 DM=3.355 AA (Highly Oriented Pyrolytic Graphite)
PG       004 DM=1.607 AA
Heusler  111 DM=3.362 AA (Cu2MnAl)
CoFe         DM=1.771 AA (Co0.92Fe0.08)
Ge       311 DM=1.714 AA
Si       111 DM=3.135 AA
Cu       111 DM=2.087 AA
Cu       002 DM=1.807 AA
Cu       220 DM=1.278 AA
Cu       111 DM=2.095 AA"""
info.full_description = info_full_description
info_initialize = """
  if (mosaic != 0) {
    mos_rms_y = MIN2RAD*mosaic/sqrt(8*log(2));
    mos_rms_z = mos_rms_y; }
  else {
    mos_rms_y = MIN2RAD*mosaich/sqrt(8*log(2));
    mos_rms_z = MIN2RAD*mosaicv/sqrt(8*log(2)); }
  mos_rms_max = mos_rms_y > mos_rms_z ? mos_rms_y : mos_rms_z;

  mono_Q = Q;
  if (DM != 0) mono_Q = 2*PI/DM;

  if (mono_Q <= 0) { fprintf(stderr,"Monochromator_curved: %s: Error scattering vector Q = 0\n", NAME_CURRENT_COMP); exit(-1); }
  if (r0 <  0) { fprintf(stderr,"Monochromator_curved: %s: Error reflectivity r0 is negative\n", NAME_CURRENT_COMP); exit(-1); }
  if (r0 == 0) { fprintf(stderr,"Monochromator_curved: %s: Reflectivity r0 is null. Ingnoring component.\n", NAME_CURRENT_COMP); }
  if (NH*NV == 0) { fprintf(stderr,"Monochromator_curved: %s: no slabs ??? (NH or NV=0)\n", NAME_CURRENT_COMP); exit(-1); }


  if (verbose && r0)
  {
    printf("Monochromator_curved: component %s Q=%.3g Angs-1 (DM=%.4g Angs)\n", NAME_CURRENT_COMP, mono_Q, 2*PI/mono_Q);
    if (NH*NV == 1) printf("            flat.\n");
    else
    { if (NH > 1)
      { printf("            horizontal: %i blades", (int)NH);
        if (RH != 0) printf(" focusing with RH=%.3g [m]", RH);
        printf("\n");
      }
      if (NV > 1)
      { printf("            vertical:   %i blades", (int)NV);
        if (RV != 0) printf(" focusing with RV=%.3g [m]", RV);
        printf("\n");
      }
    }
  }

  if (reflect != NULL && r0)
  {
    if (verbose) fprintf(stdout, "Monochromator_curved : %s : Reflectivity data (k, R)\n", NAME_CURRENT_COMP);
    Table_Read(&rTable, reflect, 1); 
    Table_Rebin(&rTable);         
    if (rTable.rows < 2) Table_Free(&rTable);
    if (verbose) Table_Info(rTable);
  } else rTable.data = NULL;

  if (transmit != NULL)
  {
    if (verbose) fprintf(stdout, "Monochromator_curved : %s : Transmission data (k, T)\n", NAME_CURRENT_COMP);
    Table_Read(&tTable, transmit, 1); 
    Table_Rebin(&tTable);         
    if (tTable.rows < 2) Table_Free(&tTable);
    if (verbose) Table_Info(tTable);
  } else tTable.data = NULL;

  if (width == 0) SlabWidth = zwidth;
  else SlabWidth = (width+gap)/NH - gap;
  if (height == 0) SlabHeight = yheight;
  else SlabHeight = (height+gap)/NV - gap;

"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """monochromator_curved"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = """"""
item1.default = item1_default
item1_description = """reflectivity file name of text file as 2 columns [k, R] [str]"""
item1.description = item1_description
item1_name = """reflect"""
item1.name = item1_name
item1_type = """char *"""
item1.type = item1_type
item2 = Parameter()
item2_default = """"""
item2.default = item2_default
item2_description = """transmission file name of text file as 2 columns [k, T] [str]"""
item2.description = item2_description
item2_name = """transmit"""
item2.name = item2_name
item2_type = """char *"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0.01
item3.default = item3_default
item3_description = """horizontal width of an individual slab (m)"""
item3.description = item3_description
item3_name = """zwidth"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0.01
item4.default = item4_default
item4_description = """vertical height of an individual slab (m)"""
item4.description = item4_description
item4_name = """yheight"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0.0005
item5.default = item5_default
item5_description = """typical gap  between adjacent slabs (m)"""
item5.description = item5_description
item5_name = """gap"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 11.0
item6.default = item6_default
item6_description = """number of slabs horizontal (columns)"""
item6.description = item6_description
item6_name = """NH"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 11.0
item7.default = item7_default
item7_description = """number of slabs vertical   (rows)"""
item7.description = item7_description
item7_name = """NV"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 30.0
item8.default = item8_default
item8_description = """Horisontal mosaic FWHM (arc minutes)"""
item8.description = item8_description
item8_name = """mosaich"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 30.0
item9.default = item9_default
item9_description = """Vertical mosaic FWHM (arc minutes)"""
item9.description = item9_description
item9_name = """mosaicv"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 0.7
item10.default = item10_default
item10_description = """Maximum reflectivity. O unactivates component (1)"""
item10.description = item10_description
item10_name = """r0"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = 1.0
item11.default = item11_default
item11_description = """transmission efficiency [1]"""
item11.description = item11_description
item11_name = """t0"""
item11.name = item11_name
item11_type = """double"""
item11.type = item11_type
item12 = Parameter()
item12_default = 1.8734
item12.default = item12_default
item12_description = """Scattering vector (AA-1)"""
item12.description = item12_description
item12_name = """Q"""
item12.name = item12_name
item12_type = """double"""
item12.type = item12_type
item13 = Parameter()
item13_default = 0.0
item13.default = item13_default
item13_description = """radius of vertical focussing. flat for 0 (m)"""
item13.description = item13_description
item13_name = """RV"""
item13.name = item13_name
item13_type = """double"""
item13.type = item13_type
item14 = Parameter()
item14_default = 0.0
item14.default = item14_default
item14_description = """radius of horizontal focussing. flat for 0 (m)"""
item14.description = item14_description
item14_name = """RH"""
item14.name = item14_name
item14_type = """double"""
item14.type = item14_type
item15 = Parameter()
item15_default = 0.0
item15.default = item15_default
item15_description = """monochromator d-spacing instead of Q=2*pi/DM (Angstrom)"""
item15.description = item15_description
item15_name = """DM"""
item15.name = item15_name
item15_type = """double"""
item15.type = item15_type
item16 = Parameter()
item16_default = 0.0
item16.default = item16_default
item16_description = """sets mosaich=mosaicv (arc minutes)"""
item16.description = item16_description
item16_name = """mosaic"""
item16.name = item16_name
item16_type = """double"""
item16.type = item16_type
item17 = Parameter()
item17_default = 0.0
item17.default = item17_default
item17_description = """total width of monochromator (m)"""
item17.description = item17_description
item17_name = """width"""
item17.name = item17_name
item17_type = """double"""
item17.type = item17_type
item18 = Parameter()
item18_default = 0.0
item18.default = item18_default
item18_description = """total height of monochromator (m)"""
item18.description = item18_description
item18_name = """height"""
item18.name = item18_name
item18_type = """double"""
item18.type = item18_type
item19 = Parameter()
item19_default = 0.0
item19.default = item19_default
item19_description = """verbosity level (0/1)"""
item19.description = item19_description
item19_name = """verbose"""
item19.name = item19_name
item19_type = """double"""
item19.type = item19_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11, item12, item13, item14, item15, item16, item17, item18, item19 ]
info.input_parameters = info_input_parameters
info_name = """Monochromator_curved"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """mos_rms_y"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """"""
item1.description = item1_description
item1_name = """mos_rms_z"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """"""
item2.description = item2_description
item2_name = """mos_rms_max"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """"""
item3.description = item3_description
item3_name = """mono_Q"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """"""
item4.description = item4_description
item4_name = """SlabWidth"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0
item5.default = item5_default
item5_description = """"""
item5.description = item5_description
item5_name = """SlabHeight"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0
item6.default = item6_default
item6_description = """"""
item6.description = item6_description
item6_name = """rTable"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0
item7.default = item7_default
item7_description = """"""
item7.description = item7_description
item7_name = """tTable"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0
item8.default = item8_default
item8_description = """row index for last hit blade (1"""
item8.description = item8_description
item8_name = """row"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0
item9.default = item9_default
item9_description = """column index for last hit blade (1"""
item9.description = item9_description
item9_name = """col"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
info_output_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """
// ----------- added by Jiao Lin --------
// ----------  header for SHARE ----------
// ----------- added by Jiao Lin --------
#ifndef GAUSS
  extern double Gauss_X[], Gauss_W[];

#define GAUSS(x,mean,rms) \
  (exp(-((x)-(mean))*((x)-(mean))/(2*(rms)*(rms)))/(sqrt(2*PI)*(rms)))
#endif

#include "mcstas2/share/read_table-lib.h"


// ----------- added by Jiao Lin --------"""
item1 = """// ----------  implementation for SHARE ----------
// ----------- added by Jiao Lin --------
  
  
  double Gauss_X[] = {-0.987992518020485, -0.937273392400706, -0.848206583410427,
                -0.724417731360170, -0.570972172608539, -0.394151347077563,
                -0.201194093997435, 0, 0.201194093997435,
                0.394151347077563, 0.570972172608539, 0.724417731360170,
                0.848206583410427, 0.937273392400706, 0.987992518020485};
  double Gauss_W[] = {0.030753241996117, 0.070366047488108, 0.107159220467172,
                0.139570677926154, 0.166269205816994, 0.186161000115562,
                0.198431485327111, 0.202578241925561, 0.198431485327111,
                0.186161000115562, 0.166269205816994, 0.139570677926154,
                0.107159220467172, 0.070366047488108, 0.030753241996117};

"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Double bent multiple crystal slabs with anisotropic gaussian mosaic."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
  double dt;
  // added by Jiao Lin. to replace the "RESTORE_NEUTRON" macro of mcstas
  // the following struct is used to save the neutron coming into this 
  // component
  struct {double x, y, z, vx, vy, vz, t, s1, s2, p;} saved = {
  x, y, z, vx, vy, vz, t, s1, s2, p};
#define RESTORE_NEUTRON {\
  x=saved.x; y=saved.y; z=saved.z; \
  vx=saved.vx; vy=saved.vy; vz=saved.vz; \
  t = saved.t; \
  s1 = saved.s1; s2 = saved.s2; \
  p = saved.p; \
  }

  if(vx != 0.0 && (dt = -x/vx) >= 0.0 && r0)
  {                             
    double zmin,zmax, ymin,ymax, zp,yp;

    
    PROP_DT(dt);    

    zmax = ((NH*(SlabWidth+gap))-gap)/2;
    zmin = -zmax;
    ymax = ((NV*(SlabHeight+gap))-gap)/2;
    ymin = -ymax;

    

    if (z>zmin && z<zmax && y>ymin && y<ymax) { 
      double tilth,tiltv;         
      double ratio, order, k, kux,kuy,kuz;
      double kix,kiy,kiz;
      int    do_transmit = 0;

      col = ceil ( (z-zmin)/(SlabWidth+gap));  
      row = ceil ( (y-ymin)/(SlabHeight+gap));
      if (RH != 0) tilth = asin((col-(NH+1)/2)*(SlabWidth+gap)/RH);
      else tilth=0;
      if (RV != 0) tiltv = -asin((row-(NV+1)/2)*(SlabHeight+gap)/RV);
      else tiltv=0;

      
      // modified by Jiao Lin
      // RESTORE_NEUTRON(INDEX_CURRENT_COMP, x, y, z, vx, vy, vz, t, sx, sy, sz, p);
      RESTORE_NEUTRON;

      
      double center_z=zmin+(col-0.5)*(SlabWidth+gap) -gap/2;
      double center_y=ymin+(row-0.5)*(SlabHeight+gap)-gap/2;
      Rotation T;
      rot_set_rotation(T, 0, tilth,    tiltv);
      
      mccoordschange_polarisation(T, &vx, &vy, &vz);
      y-=center_y;
      z-=center_z;
      coords_get(rot_apply(T,coords_set(x,y,z)),&x,&y,&z);

      
      

      
      PROP_X0;

      if (fabs(z) <= SlabWidth/2 && fabs(y) <= SlabHeight/2) { 
        kix = V2K*vx;             
        kiy = V2K*vy;
        kiz = V2K*vz;
        
        ratio = -2*kix/mono_Q;
        order = floor(ratio + .5);
        if(order == 0.0) order = ratio < 0 ? -1 : 1;
        
        if(order < 0) order = -order;
        
        k = sqrt(kix*kix + kiy*kiy + kiz*kiz);
        kux = kix/k;              
        kuy = kiy/k;
        kuz = kiz/k;
        if(order > 2*k/mono_Q) order--;
        if(order > 0) {           
          double q0, q0x, theta, delta, p_reflect, my_r0;

          q0 = order*mono_Q;
          q0x = ratio < 0 ? -q0 : q0;
          theta = asin(q0/(2*k)); 
          
          delta = asin(fabs(kux)) - theta;

          if (rTable.data != NULL)
          {
            my_r0 = r0*Table_Value(rTable, k, 1); 
          }
          else my_r0 = r0;
          if (my_r0 > 1)
          {
            if (my_r0 > 1.01 && verbose) fprintf(stdout, "Warning: Monochromator_curved : lowered reflectivity from %f to 1 (k=%f)\n", my_r0, k);
            my_r0=0.999;
          }
          if (my_r0 < 0)
          {
            if (verbose) fprintf(stdout, "Warning: Monochromator_curved : raised reflectivity from %f to 0 (k=%f)\n", my_r0, k);
            my_r0=0;
          }

          p_reflect = fabs(my_r0)*exp(-kiz*kiz/(kiy*kiy + kiz*kiz)*(delta*delta)/
                            (2*mos_rms_y*mos_rms_y))*
                        exp(-kiy*kiy/(kiy*kiy + kiz*kiz)*(delta*delta)/
                            (2*mos_rms_z*mos_rms_z));

          if(rand01() <= p_reflect) { 
            double bx,by,bz,ax,ay,az,phi;
            double cos_2theta,k_sin_2theta,cos_phi,sin_phi,kfx,kfy,kfz,q_x,q_y,q_z;
            double total,c1x,c1y,c1z,width,mos_sample;
            int i=0;

            cos_2theta   = cos(2*theta);
            k_sin_2theta = k*sin(2*theta);
            
            vec_prod(bx, by, bz, kix, kiy, kiz, q0x, 0, 0);
            NORM(bx,by,bz);
            bx *= k_sin_2theta;
            by *= k_sin_2theta;
            bz *= k_sin_2theta;
            
            vec_prod(ax, ay, az, bx, by, bz, kux, kuy, kuz);
            
            total = 0;
            
            mos_sample = mos_rms_max/cos(theta);
            c1x = kix*(cos_2theta-1);
            c1y = kiy*(cos_2theta-1);
            c1z = kiz*(cos_2theta-1);
            
            for(i=0; i<100; i++) {
              width = 5*mos_sample;
              cos_phi = cos(width);
              sin_phi = sin(width);
              q_x =  c1x + cos_phi*ax + sin_phi*bx;
              q_y = (c1y + cos_phi*ay + sin_phi*by)/mos_rms_z;
              q_z = (c1z + cos_phi*az + sin_phi*bz)/mos_rms_y;
              
              if(q_z*q_z + q_y*q_y < (25/(2.0/3.0))*(q_x*q_x))
                break;
              mos_sample *= (2.0/3.0);
            }
            
            for(i = 0; i < (sizeof(Gauss_X)/sizeof(double)); i++)
            {
              phi = width*Gauss_X[i];
              cos_phi = cos(phi);
              sin_phi = sin(phi);
              q_x = c1x + cos_phi*ax + sin_phi*bx;
              q_y = c1y + cos_phi*ay + sin_phi*by;
              q_z = c1z + cos_phi*az + sin_phi*bz;
              p_reflect = GAUSS((q_z/q_x),0,mos_rms_y)*
                          GAUSS((q_y/q_x),0,mos_rms_z);
              total += Gauss_W[i]*p_reflect;
            }
            total *= width;
            
            phi = mos_sample*randnorm();
            
            cos_phi = cos(phi);
            sin_phi = sin(phi);
            q_x = c1x + cos_phi*ax + sin_phi*bx;
            q_y = c1y + cos_phi*ay + sin_phi*by;
            q_z = c1z + cos_phi*az + sin_phi*bz;
            p_reflect = GAUSS((q_z/q_x),0,mos_rms_y)*
                        GAUSS((q_y/q_x),0,mos_rms_z);

            vx = K2V*(kix+q_x);
            vy = K2V*(kiy+q_y);
            vz = K2V*(kiz+q_z);
            p *= p_reflect/(total*GAUSS(phi,0,mos_sample));

            SCATTER;
          } 
          else do_transmit = 1;
            
        } 
        else do_transmit=1;
        if (do_transmit)
        {
          double my_t0;
          if (tTable.data != NULL)
          {
            my_t0 = t0*Table_Value(tTable, k, 1); 
          }
          else my_t0 = t0;
          
          if (my_t0 > 1)
          {
            if (my_t0 > 1.01 && verbose) fprintf(stdout, "Warning: Monochromator_curved : lowered transmission from %f to 1 (k=%f)\n", my_t0, k);
            my_t0=0.999;
          }
          if (my_t0 > 0) p*= my_t0;
          else ABSORB;
        }
      } 
      
      Rotation TT;
      rot_transpose(T, TT);
      
      mccoordschange_polarisation(TT, &vx, &vy, &vz);
      coords_get(rot_apply(TT,coords_set(x,y,z)),&x,&y,&z);
      y+=center_y;
      z+=center_z;

      
    } 
    else {
      
      // modified by Jiao Lin
      // RESTORE_NEUTRON(INDEX_CURRENT_COMP, x, y, z, vx, vy, vz, t, sx, sy, sz, p);
      RESTORE_NEUTRON;
    }
  } 
"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
