
def factory( name='monochromator_flat',zmin=0.0,zmax=0.0,ymin=0.0,ymax=0.0,width=0.0,height=0.0,mosaich=30.0,mosaicv=30.0,r0=0.7,Q=1.8734,DM=0.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from Monochromator_flatboostpython import Monochromator_flat as _factory_mcvine
    return component( _factory_mcvine( name,zmin,zmax,ymin,ymax,width,height,mosaich,mosaicv,r0,Q,DM ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Kristian Nielsen"""
info.copyright = info_copyright
info_declare = """
  double mos_rms_y; 
  double mos_rms_z;
  double mos_rms_max;
  double mono_Q;
"""
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """Flat, infinitely thin mosaic crystal, useful as a monochromator or analyzer.
For an unrotated monochromator component, the crystal surface lies in the Y-Z
plane (ie. parallel to the beam).
The mosaic is anisotropic gaussian, with different FWHMs in the Y and Z
directions. The scattering vector is perpendicular to the surface."""
info.full_description = info_full_description
info_initialize = """
  mos_rms_y = MIN2RAD*mosaich/sqrt(8*log(2));
  mos_rms_z = MIN2RAD*mosaicv/sqrt(8*log(2));
  mos_rms_max = mos_rms_y > mos_rms_z ? mos_rms_y : mos_rms_z;

  mono_Q = Q;
  if (DM != 0) mono_Q = 2*PI/DM;

  if (width>0)  { zmax = width/2;  zmin=-zmax; }
  if (height>0) { ymax = height/2; ymin=-ymax; }

  if (zmin==zmax || ymin==ymax)
    exit(fprintf(stderr, "Monochromator_flat: %s : Surface is null (zmin,zmax,ymin,ymax)\n", NAME_CURRENT_COMP));
"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """monochromator_flat"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0.0
item1.default = item1_default
item1_description = """Lower z-bound of crystal (m)"""
item1.description = item1_description
item1_name = """zmin"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0.0
item2.default = item2_default
item2_description = """Upper z-bound of crystal (m)"""
item2.description = item2_description
item2_name = """zmax"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0.0
item3.default = item3_default
item3_description = """Lower y-bound of crystal (m)"""
item3.description = item3_description
item3_name = """ymin"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0.0
item4.default = item4_default
item4_description = """Upper y-bound of crystal (m)"""
item4.description = item4_description
item4_name = """ymax"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0.0
item5.default = item5_default
item5_description = """"""
item5.description = item5_description
item5_name = """width"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0.0
item6.default = item6_default
item6_description = """"""
item6.description = item6_description
item6_name = """height"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 30.0
item7.default = item7_default
item7_description = """Horisontal mosaic (in Z direction) (FWHM) (arc minutes)"""
item7.description = item7_description
item7_name = """mosaich"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 30.0
item8.default = item8_default
item8_description = """Vertical mosaic (in Y direction) (FWHM) (arc minutes)"""
item8.description = item8_description
item8_name = """mosaicv"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0.7
item9.default = item9_default
item9_description = """Maximum reflectivity (1)"""
item9.description = item9_description
item9_name = """r0"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 1.8734
item10.default = item10_default
item10_description = """Magnitude of scattering vector (AA-1)"""
item10.description = item10_description
item10_name = """Q"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = 0.0
item11.default = item11_default
item11_description = """monochromator d-spacing instead of Q = 2*pi/DM (Angstrom)"""
item11.description = item11_description
item11_name = """DM"""
item11.name = item11_name
item11_type = """double"""
item11.type = item11_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11 ]
info.input_parameters = info_input_parameters
info_name = """Monochromator_flat"""
info.name = info_name
info_output_parameters = [  ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """
// ----------- added by Jiao Lin --------
// ----------  header for SHARE ----------
// ----------- added by Jiao Lin --------


#ifndef GAUSS

#define GAUSS(x,mean,rms) \
  (exp(-((x)-(mean))*((x)-(mean))/(2*(rms)*(rms)))/(sqrt(2*PI)*(rms)))

#endif


// ----------- added by Jiao Lin --------"""
item1 = """// ----------  implementation for SHARE ----------
// ----------- added by Jiao Lin --------

  
  
  double Gauss_X[] = {-0.987992518020485, -0.937273392400706, -0.848206583410427,
                -0.724417731360170, -0.570972172608539, -0.394151347077563,
                -0.201194093997435, 0, 0.201194093997435,
                0.394151347077563, 0.570972172608539, 0.724417731360170,
                0.848206583410427, 0.937273392400706, 0.987992518020485};
  double Gauss_W[] = {0.030753241996117, 0.070366047488108, 0.107159220467172,
                0.139570677926154, 0.166269205816994, 0.186161000115562,
                0.198431485327111, 0.202578241925561, 0.198431485327111,
                0.186161000115562, 0.166269205816994, 0.139570677926154,
                0.107159220467172, 0.070366047488108, 0.030753241996117};


"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Flat Monochromator crystal with anisotropic mosaic."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
  double y1,z1,t1,dt,kix,kiy,kiz,ratio,order,q0x,k,q0,theta;
  double bx,by,bz,kux,kuy,kuz,ax,ay,az,phi;
  double cos_2theta,k_sin_2theta,cos_phi,sin_phi,kfx,kfy,kfz,q_x,q_y,q_z;
  double delta,p_reflect,total,c1x,c1y,c1z,width,mos_sample;
  int i;

  if(vx != 0.0 && (dt = -x/vx) >= 0.0)
  {                             
    y1 = y + vy*dt;             
    z1 = z + vz*dt;
    t1 = t + dt;
    if (z1>zmin && z1<zmax && y1>ymin && y1<ymax)
    {                           
      kix = V2K*vx;             
      kiy = V2K*vy;
      kiz = V2K*vz;
      
      ratio = -2*kix/mono_Q;
      order = floor(ratio + .5);
      if(order == 0.0)
        order = ratio < 0 ? -1 : 1;
      
      if(order < 0)
        order = -order;
      
      k = sqrt(kix*kix + kiy*kiy + kiz*kiz);
      kux = kix/k;              
      kuy = kiy/k;
      kuz = kiz/k;
      if(order > 2*k/mono_Q)
        order--;
      if(order > 0)             
      {
        q0 = order*mono_Q;
        q0x = ratio < 0 ? -q0 : q0;
        theta = asin(q0/(2*k)); 
        
        delta = asin(fabs(kux)) - theta;
        p_reflect = r0*exp(-kiz*kiz/(kiy*kiy + kiz*kiz)*(delta*delta)/
                           (2*mos_rms_y*mos_rms_y))*
                       exp(-kiy*kiy/(kiy*kiy + kiz*kiz)*(delta*delta)/
                           (2*mos_rms_z*mos_rms_z));
        if(rand01() < p_reflect)
        {                       
          cos_2theta = cos(2*theta);
          k_sin_2theta = k*sin(2*theta);
          
          vec_prod(bx, by, bz, kix, kiy, kiz, q0x, 0, 0);
          NORM(bx,by,bz);
          bx *= k_sin_2theta;
          by *= k_sin_2theta;
          bz *= k_sin_2theta;
          
          vec_prod(ax, ay, az, bx, by, bz, kux, kuy, kuz);
          
          total = 0;
          
          mos_sample = mos_rms_max/cos(theta);
          c1x = kix*(cos_2theta-1);
          c1y = kiy*(cos_2theta-1);
          c1z = kiz*(cos_2theta-1);
          
          for(;;) {
            width = 5*mos_sample;
            cos_phi = cos(width);
            sin_phi = sin(width);
            q_x = c1x + cos_phi*ax + sin_phi*bx;
            q_y = (c1y + cos_phi*ay + sin_phi*by)/mos_rms_z;
            q_z = (c1z + cos_phi*az + sin_phi*bz)/mos_rms_y;
            
            if(q_z*q_z + q_y*q_y < (25/(2.0/3.0))*(q_x*q_x))
              break;
            mos_sample *= (2.0/3.0);
          }
          
          for(i = 0; i < (sizeof(Gauss_X)/sizeof(double)); i++)
          {
            phi = width*Gauss_X[i];
            cos_phi = cos(phi);
            sin_phi = sin(phi);
            q_x = c1x + cos_phi*ax + sin_phi*bx;
            q_y = c1y + cos_phi*ay + sin_phi*by;
            q_z = c1z + cos_phi*az + sin_phi*bz;
            p_reflect = GAUSS((q_z/q_x),0,mos_rms_y)*
                        GAUSS((q_y/q_x),0,mos_rms_z);
            total += Gauss_W[i]*p_reflect;
          }
          total *= width;
          
          phi = mos_sample*randnorm();
          
          cos_phi = cos(phi);
          sin_phi = sin(phi);
          q_x = c1x + cos_phi*ax + sin_phi*bx;
          q_y = c1y + cos_phi*ay + sin_phi*by;
          q_z = c1z + cos_phi*az + sin_phi*bz;
          p_reflect = GAUSS((q_z/q_x),0,mos_rms_y)*
                      GAUSS((q_y/q_x),0,mos_rms_z);
          x = 0;
          y = y1;
          z = z1;
          t = t1;
          vx = K2V*(kix+q_x);
          vy = K2V*(kiy+q_y);
          vz = K2V*(kiz+q_z);
          p *= p_reflect/(total*GAUSS(phi,0,mos_sample));
          SCATTER;
        } 
      } 
    } 
  } 
"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
