
def factory( name='selector',xmin=-0.015,xmax=0.015,ymin=-0.025,ymax=0.025,len=0.25,num=72.0,width=0.0004,radius=0.12,alfa=48.298,feq=500.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from Selectorboostpython import Selector as _factory_mcvine
    return component( _factory_mcvine( name,xmin,xmax,ymin,ymax,len,num,width,radius,alfa,feq ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Peter Link, <a href="mailto:Andreas.Ostermann@frm2.tum.de">Andreas Ostermann</a>"""
info.copyright = info_copyright
info_declare = """"""
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """escription
Velocity selector consisting of rotating Soller-like blades
defining a helically twisted passage.
Geometry is defined by two identical apertures at 12 o'clock position, 
The origin is at the ENTRANCE of the selector.

%VALIDATION
Jun 2005: extensive external test, one minor problem 
Jan 2006: problem solved (for McStas 1.11)
Validated by: K. Lieutenant

%BUGS
for transmission calculation, each neutron is supposed to be in the guide centre"""
info.full_description = info_full_description
info_initialize = """"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """selector"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = -0.015
item1.default = item1_default
item1_description = """Lower x bound of entry aperture (m)"""
item1.description = item1_description
item1_name = """xmin"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0.015
item2.default = item2_default
item2_description = """Upper x bound of entry aperture (m)"""
item2.description = item2_description
item2_name = """xmax"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = -0.025
item3.default = item3_default
item3_description = """Lower y bound of entry aperture (m)"""
item3.description = item3_description
item3_name = """ymin"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0.025
item4.default = item4_default
item4_description = """Upper y bound of entry aperture (m)"""
item4.description = item4_description
item4_name = """ymax"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0.25
item5.default = item5_default
item5_description = """rotor length  (m)"""
item5.description = item5_description
item5_name = """len"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 72.0
item6.default = item6_default
item6_description = """number of spokes/lamella (1)"""
item6.description = item6_description
item6_name = """num"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0.0004
item7.default = item7_default
item7_description = """width of spokes at beam-center (m)"""
item7.description = item7_description
item7_name = """width"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0.12
item8.default = item8_default
item8_description = """radius of beam-center (m)"""
item8.description = item8_description
item8_name = """radius"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 48.298
item9.default = item9_default
item9_description = """angle of torsion (deg)"""
item9.description = item9_description
item9_name = """alfa"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 500.0
item10.default = item10_default
item10_description = """frequency of rotation (1/s)"""
item10.description = item10_description
item10_name = """feq"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10 ]
info.input_parameters = info_input_parameters
info_name = """Selector"""
info.name = info_name
info_output_parameters = [  ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """"""
item1 = """"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """velocity selector (helical lamella type) such as <b>V_selector</b> component"""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
  double E;
  double dt;
  double open_angle,closed_angle,n_angle_in,n_angle_out;
  double sel_phase,
         act_radius;       

  PROP_Z0;
  E=VS2E*(vx*vx+vy*vy+vz*vz);
  if (x<xmin || x>xmax || y<ymin || y>ymax)
    ABSORB;                                  
  dt = len/vz;
  
  n_angle_in = atan2( x,y+radius)*RAD2DEG;
  act_radius = sqrt(x*x + pow(radius+y,2));
  closed_angle = width/act_radius*RAD2DEG;
  open_angle = 360/num-closed_angle;
  sel_phase = open_angle*rand01();
  p *= (open_angle/(closed_angle+open_angle));

  PROP_DT(dt);

  if (x<xmin || x>xmax || y<ymin || y>ymax)
    ABSORB;                                  
  

  n_angle_out = atan2(x,y+radius)*RAD2DEG;  

  sel_phase = sel_phase + feq*dt*360 - alfa;  

  if (n_angle_out<(n_angle_in-sel_phase) || n_angle_out>(n_angle_in-sel_phase+open_angle) )
    ABSORB;              
  else
    SCATTER;
"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
