
def factory( name='slit',xmin=0.0,xmax=0.0,ymin=0.0,ymax=0.0,radius=0.0,cut=0.0,width=0.0,height=0.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from Slitboostpython import Slit as _factory_mcvine
    return component( _factory_mcvine( name,xmin,xmax,ymin,ymax,radius,cut,width,height ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Kim Lefmann and Henrik M. Roennow"""
info.copyright = info_copyright
info_declare = """"""
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """A simple rectangular or circular slit. You may either
specify the radius (circular shape), or the rectangular bounds.
No transmission around the slit is allowed.
If cutting option is used, low-weight neutron rays are ABSORBED"""
info.full_description = info_full_description
info_initialize = """
  if (width > 0)  { xmax=width/2;  xmin=-xmax; }
  if (height > 0) { ymax=height/2; ymin=-ymax; }
  if (xmin == 0 && xmax == 0 && ymin == 0 && ymax == 0 && radius == 0)
    { fprintf(stderr,"Slit: %s: Error: give geometry\n", NAME_CURRENT_COMP); exit(-1); }

"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """slit"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0.0
item1.default = item1_default
item1_description = """Lower x bound (m)"""
item1.description = item1_description
item1_name = """xmin"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0.0
item2.default = item2_default
item2_description = """Upper x bound (m)"""
item2.description = item2_description
item2_name = """xmax"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0.0
item3.default = item3_default
item3_description = """Lower y bound (m)"""
item3.description = item3_description
item3_name = """ymin"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0.0
item4.default = item4_default
item4_description = """Upper y bound (m)"""
item4.description = item4_description
item4_name = """ymax"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0.0
item5.default = item5_default
item5_description = """Radius of slit in the z=0 plane, centered at Origo (m)"""
item5.description = item5_description
item5_name = """radius"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0.0
item6.default = item6_default
item6_description = """Lower limit for allowed weight (1)"""
item6.description = item6_description
item6_name = """cut"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0.0
item7.default = item7_default
item7_description = """"""
item7.description = item7_description
item7_name = """width"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0.0
item8.default = item8_default
item8_description = """"""
item8.description = item8_description
item8_name = """height"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8 ]
info.input_parameters = info_input_parameters
info_name = """Slit"""
info.name = info_name
info_output_parameters = [  ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """"""
item1 = """"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Rectangular/circular slit with optional insignificance cut"""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
    PROP_Z0;
    if (((radius == 0) && (x<xmin || x>xmax || y<ymin || y>ymax))
    || ((radius != 0) && (x*x + y*y > radius*radius)))
      ABSORB;
    else
      if (p < cut)
        ABSORB;
      else
        SCATTER;
"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
