
def factory( name='v_selector',width=0.03,height=0.05,l0=0.3,r0=0.12,phi=48.298,l1=0.25,tb=0.0004,rot=20000.0,nb=72.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from V_selectorboostpython import V_selector as _factory_mcvine
    return component( _factory_mcvine( name,width,height,l0,r0,phi,l1,tb,rot,nb ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Kim Lefmann"""
info.copyright = info_copyright
info_declare = """
  double RPM2OM, omega, phi_rad, dt0, dt1, r_i, r_f, r_mean, theta_i, theta_f, A, d_s_phi;
"""
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """Velocity selector consisting of rotating Soller-like blades
defining a helically twisted passage.
Geometry defined by two identical, centered apertures at 12 o'clock
position, Origo is at the centre of the selector (input is at -l0/2).
Transmission is analytical assuming a continuous source.

%VALIDATION
Jun 2005: extensive external test, no problems found 
Validated by: K. Lieutenant"""
info.full_description = info_full_description
info_initialize = """
    RPM2OM = 2*PI/60.0;
    omega=rot*RPM2OM;
    phi_rad = phi*DEG2RAD;
"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """v_selector"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0.03
item1.default = item1_default
item1_description = """(m)   Width of entry aperture"""
item1.description = item1_description
item1_name = """width"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0.05
item2.default = item2_default
item2_description = """(m)   Height of entry aperture"""
item2.description = item2_description
item2_name = """height"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0.3
item3.default = item3_default
item3_description = """(m)   Distance between apertures"""
item3.description = item3_description
item3_name = """l0"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0.12
item4.default = item4_default
item4_description = """(m)   Height from aperture centre to rotation axis"""
item4.description = item4_description
item4_name = """r0"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 48.298
item5.default = item5_default
item5_description = """(deg) Twist angle along the cylinder"""
item5.description = item5_description
item5_name = """phi"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0.25
item6.default = item6_default
item6_description = """(m)   Length of cylinder (less than l0)"""
item6.description = item6_description
item6_name = """l1"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0.0004
item7.default = item7_default
item7_description = """(m)   Thickness of blades"""
item7.description = item7_description
item7_name = """tb"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 20000.0
item8.default = item8_default
item8_description = """(rpm) Cylinder rotation speed, counter-clockwise"""
item8.description = item8_description
item8_name = """rot"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 72.0
item9.default = item9_default
item9_description = """(1)   Number of Soller blades"""
item9.description = item9_description
item9_name = """nb"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.input_parameters = info_input_parameters
info_name = """V_selector"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """RPM2OM"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """"""
item1.description = item1_description
item1_name = """omega"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """"""
item2.description = item2_description
item2_name = """phi_rad"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """"""
item3.description = item3_description
item3_name = """dt0"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """"""
item4.description = item4_description
item4_name = """dt1"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0
item5.default = item5_default
item5_description = """"""
item5.description = item5_description
item5_name = """r_i"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0
item6.default = item6_default
item6_description = """"""
item6.description = item6_description
item6_name = """r_f"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0
item7.default = item7_default
item7_description = """"""
item7.description = item7_description
item7_name = """r_mean"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0
item8.default = item8_default
item8_description = """"""
item8.description = item8_description
item8_name = """theta_i"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0
item9.default = item9_default
item9_description = """"""
item9.description = item9_description
item9_name = """theta_f"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 0
item10.default = item10_default
item10_description = """"""
item10.description = item10_description
item10_name = """A"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = 0
item11.default = item11_default
item11_description = """"""
item11.description = item11_description
item11_name = """d_s_phi"""
item11.name = item11_name
item11_type = """double"""
item11.type = item11_type
info_output_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11 ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """"""
item1 = """"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Velocity selector."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
    if (vz == 0)
      ABSORB;
    dt1= (-l0/2.0 - z)/vz;
    PROP_DT(dt1); 
    if (x<(-width/2.0) || x>(width/2.0) || y<(-height/2.0) || y>(height/2.0))
      ABSORB;

    dt0 = (l0-l1)/(2.0*vz);  
    PROP_DT(dt0);
    r_i = sqrt(x*x+(y+r0)*(y+r0));
    theta_i = atan2(x,y+r0);

    dt1 = l1/vz; 
    PROP_DT(dt1);
    r_f = sqrt(x*x+(y+r0)*(y+r0));
    theta_f = atan2(x,y+r0);

    dt0 = (l0-l1)/(2.0*vz);  
    PROP_DT(dt0);
    if (x<(-width/2.0) || x>(width/2.0) || y<(-height/2.0) || y>(height/2.0))
      ABSORB;

    
    
    r_mean = (r_i + r_f)/2.0;          
    d_s_phi = theta_f-theta_i;
    A = nb/(2*PI)*( tb/r_mean + fabs(phi_rad+d_s_phi-omega*l1/vz) ); 
    if (A >= 1)
      ABSORB;
    p*= (1-A);
    SCATTER;
"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
