
def factory( name='vertical_t0',len=0,w1=0,w2=0,nu=0,delta=0,tc=0,ymin=0,ymax=0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from Vertical_T0boostpython import Vertical_T0 as _factory_mcvine
    return component( _factory_mcvine( name,len,w1,w2,nu,delta,tc,ymin,ymax ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Garrett Granroth"""
info.copyright = info_copyright
info_declare = """

   double omega,off,splen,rad,sw;
  
"""
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """Models a vertical T0 chopper."""
info.full_description = info_full_description
info_initialize = """
   splen=len/2.0;
   omega=2.0*PI*nu;
   rad=sqrt(w2*w2/4.0+splen*splen); //radius of cylinder containing slit package.
"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """vertical_t0"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """length of slot (m)"""
item1.description = item1_description
item1_name = """len"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """center width (m)"""
item2.description = item2_description
item2_name = """w1"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """edgewidth"""
item3.description = item3_description
item3_name = """w2"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """frequency (Hz)"""
item4.description = item4_description
item4_name = """nu"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0
item5.default = item5_default
item5_description = """time from edge of chopper to center Phase angle (sec)"""
item5.description = item5_description
item5_name = """delta"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0
item6.default = item6_default
item6_description = """time when desired neutron is at the center of the chopper (sec)"""
item6.description = item6_description
item6_name = """tc"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0
item7.default = item7_default
item7_description = """Lower y bound (m)"""
item7.description = item7_description
item7_name = """ymin"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0
item8.default = item8_default
item8_description = """Upper y bound (m)"""
item8.description = item8_description
item8_name = """ymax"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8 ]
info.input_parameters = info_input_parameters
info_name = """Vertical_T0"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """omega"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """"""
item1.description = item1_description
item1_name = """r_blade"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """"""
item2.description = item2_description
item2_name = """x_width"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """"""
item3.description = item3_description
item3_name = """nn"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """"""
item4.description = item4_description
item4_name = """tdr"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
info_output_parameters = [ item0, item1, item2, item3, item4 ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """
// ----------- added by Jiao Lin --------
// ----------  header for SHARE ----------
// ----------- added by Jiao Lin --------
#ifndef FERMI_CHOP_DEFS
#define FERMI_CHOP_DEFS   
   
  double acos0_2pi(double x,double y);
  
  void neutxypos(double *x, double *y, double phi, double inrad,double c[]);
    
  void calccenter(double c[],double zr[], double xr[]);
  
  int t0checkabsorb(double phi, double inrad,double inw1, double inw2, double c[]);
#endif
// ----------- added by Jiao Lin --------"""
item1 = """// ----------  implementation for SHARE ----------
// ----------- added by Jiao Lin --------
     
    double acos0_2pi(double x,double y)
     {
       if (y>0.0){ 
         return acos(x);
       }
         return 2.0*PI-acos(x);
     }
   
   void neutxypos(double *x, double *y, double phi, double inrad,double c[])
    {      
        *x=c[0]+inrad*cos(phi);
        *y=c[1]+inrad*sin(phi);
    }
      
    void calccenter(double c[],double zr[], double xr[]){
      double denom, A,B,C,D,a,b;
      denom=2*(-zr[0]*xr[2] +zr[0]*xr[1]+ zr[1]*xr[2]+xr[0]*zr[2]-xr[0]*zr[1] - xr[1]*zr[2]);
       A=xr[1]-xr[2];B=xr[0]-xr[1];C=zr[2]-zr[1];D=zr[1]-zr[0];
       a=zr[0]*zr[0]-zr[1]*zr[1]+xr[0]*xr[0]-xr[1]*xr[1];
       b=zr[2]*zr[2]-zr[1]*zr[1]+xr[2]*xr[2]-xr[1]*xr[1];
       c[0]=1.0/denom*(A*a+B*b);
       c[1]=1.0/denom*(C*a+D*b);  
    }
    
    int t0checkabsorb(double phi, double inrad,double inw1, double inw2, double c[]){
     double xtmp,neuzr,neuxr;
     neutxypos(&neuzr,&neuxr,phi,inrad,c);
  // printf("xr:%g zr:%g phi: %g r: %g c[0]: %g c[1]: %g\n",neuxr,neuzr,phi,inrad,c[0],c[1]);
        if (fabs(neuxr)>inw1/2.0+(inw2-inw1)/(inrad/2.0)*fabs(neuzr)) // check if neutron x position is outside of channel 
         return 0;    
     return 1;
    }
"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Vertical T0 chopper"""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
 
  double t0,t1,dphi,dt2,tneuzr,tneuxr,nrad;
  double phivec[200],tpt[3],xpt[3],ypt[3],zpt[3],zr[3],xr[3],yr[3],theta[3],c[2];
  int chan_num,chan_num0,idx1,idx3;
  if (cylinder_intersect (&t0, &t1, x, y, z, vx, vy, vz, rad, ymax-ymin)){
    if (t0 < 0)			
        ABSORB;
    dt2=t1-t0;	
    PROP_DT(t0);                
    
    tpt[1]=t;
    tpt[2]=t+dt2;
    tpt[0]=t+dt2/2.0;
     //set local 0 in time as tc and calculate angle of rotation for each point
    for(idx3=0;idx3<3;idx3++){
      theta[idx3]=(tpt[idx3]-tc)*omega;
    }
    zpt[1]=-sqrt(rad*rad-x*x);  xpt[1]=x; ypt[1]=y;   
    zpt[2]=zpt[1]+vz*(dt2); xpt[2]=xpt[1]+vx*(dt2); ypt[2]=ypt[1]+vy*(dt2);  
    xpt[0]=xpt[1]+vx*(dt2/2.0); ypt[0]=ypt[1]+vy*(dt2/2.0); zpt[0]=zpt[1]+vz*(dt2/2.0); 
    
   for(idx3=0;idx3<3;idx3++){
       rotate(xr[idx3],yr[idx3],zr[idx3],xpt[idx3],ypt[idx3],zpt[idx3],theta[idx3],0,1,0);     
    }       
    calccenter(c,zr,xr); 
    nrad=sqrt((zr[0]-c[0])*(zr[0]-c[0])+(xr[0]-c[1])*(xr[0]-c[1])); 
   
   phivec[0]=acos0_2pi((zr[1]-c[0])/nrad,xr[1]-c[1]);phivec[1]=acos0_2pi((zr[2]-c[0])/nrad,xr[2]-c[1]);
   neutxypos(&tneuzr,&tneuxr,phivec[0],nrad,c);
 
   if(tneuzr<-splen){
       phivec[0]=acos0_2pi((-c[0]-splen)/nrad,-c[1]);
    }
   neutxypos(&tneuzr,&tneuxr,phivec[1],nrad,c);
   if(tneuzr>splen){
       phivec[1]=acos0_2pi((-c[0]+splen/2.0)/nrad,-c[1]);
    }
   dphi=phivec[1]-phivec[0];  
   idx1=2;
   phivec[idx1]=phivec[0]+dphi/2.0;  
   if (!t0checkabsorb(phivec[idx1],nrad,w1,w2,c))
      ABSORB;
   while (idx1<129){
     dphi=phivec[1]-phivec[idx1];
     idx1++;
     phivec[idx1]=phivec[0]+dphi/2.0;
     if (!t0checkabsorb(phivec[idx1],nrad,w1,w2,c))
      ABSORB;
     if (dphi>0){
       while ((phivec[idx1]<phivec[1])&&(idx1<129)){
        
         idx1++;
         phivec[idx1]=phivec[idx1-1]+dphi;
         if (!t0checkabsorb(phivec[idx1],nrad,w1,w2,c))
           ABSORB;
       }
     }
     else{
         while ((phivec[idx1]>phivec[1])&&(idx1<129)){
        
         idx1++;
         phivec[idx1]=phivec[idx1-1]+dphi;
         if (!t0checkabsorb(phivec[idx1],nrad,w1,w2,c))
           ABSORB;   
       }
     }
     if (idx1<129) idx1--;    
   }      
  }
  else				
    ABSORB;

"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
