
def factory( name='powdern',reflections='',d_phi=0.0,radius=0.0,radius_i=0.0,yheight=0.0,pack=1.0,Vc=0.0,sigma_abs=0.0,sigma_inc=0.0,Delta_d=0.0,frac=0.0,tfrac=0.1,xwidth=0.0,zthick=0.0,xwidth_i=0.0,yheight_i=0.0,zthick_i=0.0,h=0.0,DW=0.0,nb_atoms=1.0,concentric=0.0,density=0.0,weight=0.0,barns=1.0,format=0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from PowderNboostpython import PowderN as _factory_mcvine
    return component( _factory_mcvine( name,reflections,d_phi,radius,radius_i,yheight,pack,Vc,sigma_abs,sigma_inc,Delta_d,frac,tfrac,xwidth,zthick,xwidth_i,yheight_i,zthick_i,h,DW,nb_atoms,concentric,density,weight,barns,format ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """P. Willendrup, L. Chapon, K. Lefmann, A.B.Abrahamsen, N.B.Christensen, E.M.Lauridsen."""
info.copyright = info_copyright
info_declare = """
  struct line_info_struct line_info;
  int Nq;
  double my_s_v2_sum;
  double my_a_v, my_inc;
  double *w_v,*q_v, *my_s_v2;
  char   isrect;
  const int    *columns;
  double XsectionFactor;
"""
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """General powder sample with
many scattering vectors
possibility for intrinsic line broadening
incoherent backgorund ratio is specified by user.
No multiple scattering. No secondary extinction.

Based on Powder1/Powder2/Single_crystal.
Geometry is a powder filled cylinder or a box.
Incoherent scattering is only provided here to account for a background
The efficient is highly improved when restricting the vertical scattering
range on the Debye-Scherrer cone (with 'd_phi').
The unit cell volume Vc may also be computed when giving the density,
the atomic/molecular weight and the number of atoms per unit cell.

<b>Powder definition and file format</b>

Powder structure is specified with an ascii data file 'reflections'.
The powder data are free-text column based files.
Lines begining by '#' are read as comments (ignored) but they may contain
the following keywords (in the header):
#Vc           <value of unit cell volume Vc [Angs^3]>
#sigma_abs    <value of Absorption cross section [barns]>
#sigma_inc    <value of Incoherent cross section [barns]>
#Debye_Waller <value of Debye-Waller factor DW>
#Delta_d/d    <value of Detla_d/d width for all lines>
These values are not read if entered as component parameters (Vc=...)

The signification of the columns in the numerical block may be
set using the 'format' parameter. Built-in formats are:
format=Crystallographica
format=Fullprof
format=Lazy
and these specifications it is important NOT to use quotes, as shown.

An other possibility to define other formats is to set directly
the signification of the columns as a vector of indexes in the order
format={j,d,F2,DW,Delta_d/d,1/2d,q,F}
Signification of the symbols is given below. Indexes start at 1.
Indexes with zero means that the column is not present, so that:
Crystallographica={ 4,5,7,0,0,0,0,0 }
Fullprof         ={ 4,0,8,0,0,5,0,0 }
Lazy             ={17,6,0,0,0,0,0,13}
Here again, NO quotes should be around the 'format' value.

At last, the format may be overridden by direct definition of the
column indexes in the file itself by using the following keywords
in the header (e.g. '#column_j 4'):
#column_j     <index of the multiplicity 'j' column>
#column_d     <index of the d-spacing 'd' column [Angs]>
#column_F2    <index of the squared str. factor '|F|^2' column [b]>
#column_F     <index of the structure factor norm '|F|' column>
#column_DW    <index of the Debye-Waller factor 'DW' column>
#column_Dd    <index of the relative line width Delta_d/d 'Dd' column>
#column_inv2d <index of the 1/2d=sin(theta)/lambda 'inv2d' column>
#column_q     <index of the scattering wavevector 'q' column [Angs-1]>"""
info.full_description = info_full_description
info_initialize = """
  Nq = 0;
  isrect = 0;
  my_s_v2_sum = 0;
  columns = XtalData_formats[ format ];
  int i;
  struct line_data *L;
  line_info.Dd       = Delta_d;
  line_info.DWfactor = DW;
  line_info.V_0      = Vc;
  line_info.rho      = density;
  line_info.at_weight= weight;
  line_info.at_nb    = nb_atoms;
  line_info.sigma_a  = sigma_abs;
  line_info.sigma_i  = sigma_inc;
  for (i=0; i< 8; i++) line_info.column_order[i] = columns[i];
  strncpy(line_info.compname, NAME_CURRENT_COMP, 256);

  
  if (h && !yheight) {
    yheight = h;
  } else if (yheight && !h) {
    h = yheight;
  }

  if (!radius || !yheight) {
    if (!xwidth || !yheight || !zthick) exit(fprintf(stderr,"PowderN: %s: sample has no volume (zero dimensions)\n", NAME_CURRENT_COMP));
    else isrect=1; }
  
  if (!yheight_i) {
    yheight_i = yheight;
  }
  
  if ((isrect && ((zthick_i > zthick) || (xwidth_i > xwidth) || (yheight_i > yheight))) || (!isrect && (radius_i > radius) || (yheight_i > yheight))) {
    exit(fprintf(stderr,"PowderN: %s: Inner hollow exeeds sample volume\n", NAME_CURRENT_COMP));
  }
  
  if (frac + tfrac > 1) {
    exit(fprintf(stderr,"PowderN: %s: You have requested an unmeaningful choice of the 'frac' and 'tfrac' parameters (sum is %g, exeeding 1)\n", 
		 NAME_CURRENT_COMP, frac+tfrac));
  } else if (frac + tfrac == 1) {
    printf("PowderN %s WARNING: You have requested all neutrons be attenuated or incoherently scattered!\n", NAME_CURRENT_COMP);
  }

  if (concentric) {
    printf("PowderN %s WARNING: Concentric mode - remember to include the 'opposite' copy of this component!\n", NAME_CURRENT_COMP);
    if (tfrac == 0) {
      printf("PowderN %s WARNING: Concentric mode and tfrac==0 !? Don't you want any transmitted neutrons?\n", NAME_CURRENT_COMP);
    }
  }
  
  i = read_line_data(reflections, &line_info);

  if (!line_info.V_0 && line_info.at_nb > 0
    && line_info.at_weight > 0 && line_info.rho > 0) {
    
    
    line_info.V_0 = line_info.at_nb
      /(line_info.rho/line_info.at_weight/1e24*6.02214199e23);
  }

  if (line_info.V_0 <= 0)
    exit(fprintf(stderr,"PowderN: %s: density/unit cell volume is NULL (Vc)\n", NAME_CURRENT_COMP));

  if (barns) { 
    XsectionFactor = 100;
  } else {
    XsectionFactor = 1;
  }
  
  if (i) {
    L = line_info.list;

    Nq  = line_info.count;
    q_v = (double *)malloc(Nq*sizeof(double));
    w_v = (double *)malloc(Nq*sizeof(double));
    my_s_v2 = (double *)malloc(Nq*sizeof(double));
    if (!q_v || !w_v || !my_s_v2)
      exit(fprintf(stderr,"PowderN: %s: ERROR allocating memory (init)\n", NAME_CURRENT_COMP));
    for(i=0; i<Nq; i++)
    {
      my_s_v2[i] = 4*PI*PI*PI*pack*(L[i].DWfactor ? L[i].DWfactor : 1)
                 /(line_info.V_0*line_info.V_0*V2K*V2K)
                 *(L[i].j * L[i].F2 / L[i].q)*XsectionFactor;
      
      
      my_s_v2_sum+=my_s_v2[i];
      q_v[i] = L[i].q*K2V;
      w_v[i] = L[i].w;
#ifdef DEBUG
      printf("PowderN: %d: my_s_v2, q_v, w_v = %d: %g, %g, %g\n", 
             i, my_s_v2[i], q_v[i], w_v[i]);
#endif
    }
  }

  
  my_a_v = pack*line_info.sigma_a/line_info.V_0*2200*100;   // Factor 100 to convert from barns to fm^2
  my_inc = pack*line_info.sigma_i/line_info.V_0*100;   // Factor 100 to convert from barns to fm^2

  printf("PowderN: %s: Vc=%g [Angs] sigma_abs=%g [barn] sigma_inc=%g [barn]\n",
    NAME_CURRENT_COMP, line_info.V_0, line_info.sigma_a, line_info.sigma_i);

"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """powdern"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = """"""
item1.default = item1_default
item1_description = """Input file for reflections."""
item1.description = item1_description
item1_name = """reflections"""
item1.name = item1_name
item1_type = """char *"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0.0
item2.default = item2_default
item2_description = """Angle corresponding to the vertical angular range"""
item2.description = item2_description
item2_name = """d_phi"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0.0
item3.default = item3_default
item3_description = """Outer radius of sample in (x,z) plane [m]"""
item3.description = item3_description
item3_name = """radius"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0.0
item4.default = item4_default
item4_description = """Radius of optional inner hollow [m]"""
item4.description = item4_description
item4_name = """radius_i"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0.0
item5.default = item5_default
item5_description = """Height of sample y direction [m]"""
item5.description = item5_description
item5_name = """yheight"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 1.0
item6.default = item6_default
item6_description = """Packing factor [1]"""
item6.description = item6_description
item6_name = """pack"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0.0
item7.default = item7_default
item7_description = """Volume of unit cell=nb atoms per cell/density of atoms [AA^3]"""
item7.description = item7_description
item7_name = """Vc"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0.0
item8.default = item8_default
item8_description = """Absorption cross section per unit cell at 2200 m/s [barns]"""
item8.description = item8_description
item8_name = """sigma_abs"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0.0
item9.default = item9_default
item9_description = """Incoherent cross section per unit cell [barns]"""
item9.description = item9_description
item9_name = """sigma_inc"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 0.0
item10.default = item10_default
item10_description = """global relative Delta_d/d spreading when the 'w' column"""
item10.description = item10_description
item10_name = """Delta_d"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = 0.0
item11.default = item11_default
item11_description = """Fraction of incoherently scattered neutron rays [1]"""
item11.description = item11_description
item11_name = """frac"""
item11.name = item11_name
item11_type = """double"""
item11.type = item11_type
item12 = Parameter()
item12_default = 0.1
item12.default = item12_default
item12_description = """Fraction of transmitted (only attenuated) neutron rays [1]"""
item12.description = item12_description
item12_name = """tfrac"""
item12.name = item12_name
item12_type = """double"""
item12.type = item12_type
item13 = Parameter()
item13_default = 0.0
item13.default = item13_default
item13_description = """horiz. dimension of sample, as a width [m]"""
item13.description = item13_description
item13_name = """xwidth"""
item13.name = item13_name
item13_type = """double"""
item13.type = item13_type
item14 = Parameter()
item14_default = 0.0
item14.default = item14_default
item14_description = """thickness of sample [m]"""
item14.description = item14_description
item14_name = """zthick"""
item14.name = item14_name
item14_type = """double"""
item14.type = item14_type
item15 = Parameter()
item15_default = 0.0
item15.default = item15_default
item15_description = """horiz. dimension of optional inner hollow when in cubic geometry [m]"""
item15.description = item15_description
item15_name = """xwidth_i"""
item15.name = item15_name
item15_type = """double"""
item15.type = item15_type
item16 = Parameter()
item16_default = 0.0
item16.default = item16_default
item16_description = """vertical dimension of optional innher hollow in cubic and cyl geometry. If zero assumed == h. [m]"""
item16.description = item16_description
item16_name = """yheight_i"""
item16.name = item16_name
item16_type = """double"""
item16.type = item16_type
item17 = Parameter()
item17_default = 0.0
item17.default = item17_default
item17_description = """thickness of optional innner hollow when in cubic geometry [m]"""
item17.description = item17_description
item17_name = """zthick_i"""
item17.name = item17_name
item17_type = """double"""
item17.type = item17_type
item18 = Parameter()
item18_default = 0.0
item18.default = item18_default
item18_description = """the same as yheight [m]"""
item18.description = item18_description
item18_name = """h"""
item18.name = item18_name
item18_type = """double"""
item18.type = item18_type
item19 = Parameter()
item19_default = 0.0
item19.default = item19_default
item19_description = """global Debey-Waller factor when the 'DW' column"""
item19.description = item19_description
item19_name = """DW"""
item19.name = item19_name
item19_type = """double"""
item19.type = item19_type
item20 = Parameter()
item20_default = 1.0
item20.default = item20_default
item20_description = """number of atoms per unit cell [1]"""
item20.description = item20_description
item20_name = """nb_atoms"""
item20.name = item20_name
item20_type = """double"""
item20.type = item20_type
item21 = Parameter()
item21_default = 0.0
item21.default = item21_default
item21_description = """Flag to indicate if sample is 'concentric' [1]"""
item21.description = item21_description
item21_name = """concentric"""
item21.name = item21_name
item21_type = """double"""
item21.type = item21_type
item22 = Parameter()
item22_default = 0.0
item22.default = item22_default
item22_description = """density of material. V_rho=density/weight/1e24*N_A. [g/cm^3]"""
item22.description = item22_description
item22_name = """density"""
item22.name = item22_name
item22_type = """double"""
item22.type = item22_type
item23 = Parameter()
item23_default = 0.0
item23.default = item23_default
item23_description = """atomic/molecular weight of material [g/mol]"""
item23.description = item23_description
item23_name = """weight"""
item23.name = item23_name
item23_type = """double"""
item23.type = item23_type
item24 = Parameter()
item24_default = 1.0
item24.default = item24_default
item24_description = """Flag to indicate if |F|^2 from 'reflections' is in barns or fm^2 (barns=1 for laz, barns=0 for lau type files).[1]"""
item24.description = item24_description
item24_name = """barns"""
item24.name = item24_name
item24_type = """double"""
item24.type = item24_type
item25 = Parameter()
item25_default = 0
item25.default = item25_default
item25_description = """name of the format, or list of column indexes"""
item25.description = item25_description
item25_name = """format"""
item25.name = item25_name
item25_type = """int"""
item25.type = item25_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11, item12, item13, item14, item15, item16, item17, item18, item19, item20, item21, item22, item23, item24, item25 ]
info.input_parameters = info_input_parameters
info_name = """PowderN"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """line_info"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """"""
item1.description = item1_description
item1_name = """Nq"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """"""
item2.description = item2_description
item2_name = """my_s_v2"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """"""
item3.description = item3_description
item3_name = """my_s_v2_sum"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """"""
item4.description = item4_description
item4_name = """my_a_v"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0
item5.default = item5_default
item5_description = """"""
item5.description = item5_description
item5_name = """my_inc"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0
item6.default = item6_default
item6_description = """"""
item6.description = item6_description
item6_name = """q_v"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0
item7.default = item7_default
item7_description = """"""
item7.description = item7_description
item7_name = """w_v"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0
item8.default = item8_default
item8_description = """"""
item8.description = item8_description
item8_name = """isrect"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0
item9.default = item9_default
item9_description = """"""
item9.description = item9_description
item9_name = """columns"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
info_output_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """
// ----------- added by Jiao Lin --------
// ----------  header for SHARE ----------
// ----------- added by Jiao Lin --------

  
#include "mcstas2/share/read_table-lib.h"

#ifndef POWDERN_DECL
#define POWDERN_DECL

#ifndef XTALDATAFORMATS
#define XTALDATAFORMATS
     enum XtalDataFormat{
       Undefined  = 0,
       Crystallographica = 1,
       Fullprof = 2,
       Lazy = 3
     };

  const int XtalDataFormat_Undefined[8] =   { 0,0,0,0,0,0,0,0 };
  const int XtalDataFormat_Crystallographica[8] = { 4,5,7,0,0,0,0,0 };
  const int XtalDataFormat_Fullprof[8] =    { 4,0,8,0,0,5,0,0 };
  const int XtalDataFormat_Lazy[8] =        {17,6,0,0,0,0,0,13 };

  const int * const XtalData_formats[4] = { 
    XtalDataFormat_Undefined,
    XtalDataFormat_Crystallographica, 
    XtalDataFormat_Fullprof,
    XtalDataFormat_Lazy
  };
#endif

    struct line_data
    {
      double F2;                  
      double q;                   
      int j;                      
      double DWfactor;            
      double w;                   
    };

  struct line_info_struct
    {
      struct line_data *list;     
      int  count;                  
      double Dd;
      double DWfactor;
      double V_0;
      double rho;
      double at_weight;
      double at_nb;
      double sigma_a;
      double sigma_i;
      char   compname[256];
      int    column_order[8]; 
    };

#endif 

// ----------- added by Jiao Lin --------"""
item1 = """// ----------  implementation for SHARE ----------
// ----------- added by Jiao Lin --------

  int read_line_data(char *SC_file, struct line_info_struct *info)
  {
    struct line_data *list = NULL;
    int    size = 0;
    t_Table sTable; 
    int    i=0;
    int    mult_count  =0;
    char   flag=0;
    double q_count=0, j_count=0, F2_count=0;
    char **parsing;
    int    list_count=0;

    if (!SC_file || !strlen(SC_file) || !strcmp(SC_file, "NULL")) {
      printf("PowderN: %s: Using incoherent elastic scattering only\n",
          info->compname);
      info->count = 0;
      return(0);
    }
    Table_Read(&sTable, SC_file, 1); 

    
    parsing = Table_ParseHeader(sTable.header,
      "Vc","V_0",
      "sigma_abs","sigma_a ",
      "sigma_inc","sigma_i ",
      "column_j",
      "column_d",
      "column_F2",
      "column_DW",
      "column_Dd",
      "column_inv2d", "column_1/2d", "column_sintheta/lambda",
      "column_q", 
      "DW", "Debye_Waller",
      "Detla_d/d",
      "column_F ",
      "V_rho",
      "density",
      "weight",
      "nb_atoms",
      NULL);

    if (parsing) {
      if (parsing[0] && !info->V_0)     info->V_0    =atof(parsing[0]);
      if (parsing[1] && !info->V_0)     info->V_0    =atof(parsing[1]);
      if (parsing[2] && !info->sigma_a) info->sigma_a=atof(parsing[2]);
      if (parsing[3] && !info->sigma_a) info->sigma_a=atof(parsing[3]);
      if (parsing[4] && !info->sigma_i) info->sigma_i=atof(parsing[4]);
      if (parsing[5] && !info->sigma_i) info->sigma_i=atof(parsing[5]);
      if (parsing[6])                   info->column_order[0]=atoi(parsing[6]);
      if (parsing[7])                   info->column_order[1]=atoi(parsing[7]);
      if (parsing[8])                   info->column_order[2]=atoi(parsing[8]);
      if (parsing[9])                   info->column_order[3]=atoi(parsing[9]);
      if (parsing[10])                  info->column_order[4]=atoi(parsing[10]);
      if (parsing[11])                  info->column_order[5]=atoi(parsing[11]);
      if (parsing[12])                  info->column_order[5]=atoi(parsing[12]);
      if (parsing[13])                  info->column_order[5]=atoi(parsing[13]);
      if (parsing[14])                  info->column_order[6]=atoi(parsing[14]);
      if (parsing[15] && info->DWfactor<=0)    info->DWfactor=atof(parsing[15]);
      if (parsing[16] && info->DWfactor<=0)    info->DWfactor=atof(parsing[16]);
      if (parsing[17] && info->Dd <0)          info->Dd      =atof(parsing[17]);
      if (parsing[18])                  info->column_order[7]=atoi(parsing[18]);
      if (parsing[19] && !info->V_0)    info->V_0    =1/atof(parsing[19]);
      if (parsing[20] && !info->rho)    info->rho    =atof(parsing[20]);
      if (parsing[21] && !info->at_weight)     info->at_weight    =atof(parsing[21]);
      if (parsing[22] && info->at_nb <= 1)  info->at_nb    =atof(parsing[22]);
      for (i=0; i<=22; i++) if (parsing[i]) free(parsing[i]);
      free(parsing);
    }

    if (!sTable.rows)
      exit(fprintf(stderr, "PowderN: %s: Error: The number of rows in %s "
       "should be at least %d\n", info->compname, SC_file, 1));
    else size = sTable.rows;
    Table_Info(sTable);
    printf("PowderN: %s: Reading %d rows from %s\n",
          info->compname, size, SC_file);
    
    list = (struct line_data*)malloc(size*sizeof(struct line_data));

    for (i=0; i<size; i++)
    {
      
      double j=0, d=0, w=0, q=0, DWfactor=0, F2=0;
      int index;

      if (info->Dd >= 0)      w         = info->Dd;
      if (info->DWfactor > 0) DWfactor  = info->DWfactor;

      
      
      if (info->column_order[0] >0)
        j = Table_Index(sTable, i, info->column_order[0]-1);
      if (info->column_order[1] >0)
        d = Table_Index(sTable, i, info->column_order[1]-1);
      if (info->column_order[2] >0)
        F2 = Table_Index(sTable, i, info->column_order[2]-1);
      if (info->column_order[3] >0)
        DWfactor = Table_Index(sTable, i, info->column_order[3]-1);
      if (info->column_order[4] >0)
        w = Table_Index(sTable, i, info->column_order[4]-1);
      if (info->column_order[5] >0)
        { d = Table_Index(sTable, i, info->column_order[5]-1);
          d = (d > 0? 1/d/2 : 0); }
      if (info->column_order[6] >0)
        { q = Table_Index(sTable, i, info->column_order[6]-1);
          d = (q > 0 ? 2*PI/q : 0); }
      if (info->column_order[7] >0  && !F2)
        { F2 = Table_Index(sTable, i, info->column_order[7]-1); F2 *= F2; }

      
      j = (j > 0 ? j : 0);
      q = (d > 0 ? 2*PI/d : 0); 
      DWfactor = (DWfactor > 0 ? DWfactor : 1);
      w = (w>0 ? w : 0); 
      F2 = (F2 >= 0 ? F2 : 0);
      if (j == 0 || q == 0) {
        printf("PowderN: %s: line %i has invalid definition\n"
               "         (mult=0 or q=0 or d=0)\n", info->compname, i);
        continue;
      }
      list[list_count].j = j;
      list[list_count].q = q;
      list[list_count].DWfactor = DWfactor;
      list[list_count].w = w;
      list[list_count].F2= F2;

      
      
      if (!q_count)     q_count = q;
      if (!j_count)     j_count = j;
      if (!F2_count)     F2_count = F2;
      if (fabs(q_count-q) < 0.0001*fabs(q)
       && fabs(F2_count-F2) < 0.0001*fabs(F2) && j_count == j) {
       mult_count++; flag=0; }
      else flag=1;
      if (i == size-1) flag=1;
      
      if (flag) {
        if (i == size-1) list_count++;
      
      
        if (mult_count == list[list_count-1].j
        || (mult_count == list[list_count].j && i == size-1)) {
          printf("PowderN: %s: Set multiplicity to 1 for lines [%i:%i]\n"
                  "         (d-spacing %g is duplicated %i times)\n",
            info->compname, list_count-mult_count, list_count-1, list[list_count-1].q, mult_count);
          for (index=list_count-mult_count; index<list_count; list[index++].j = 1);
          mult_count   = 1;
          q_count = q;
          j_count = j;
          F2_count = F2;
        }
        if (i == size-1) list_count--;
        flag=0;
      }
      list_count++;
    } 
    printf("PowderN: %s: File %s done (%i valid lines).\n", info->compname, SC_file, list_count);
    Table_Free(&sTable);
    info->list  = list;
    info->count = list_count;

    return(list_count);
  }

"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """General powder sample (N lines, single scattering, incoherent scattering)"""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
  double t0, t1, t2, t3, v, v1,l_full, l, l_1, dt, alpha0, alpha, theta, my_s, my_s_n;
  double solid_angle, neutrontype;
  double arg, tmp_vx, tmp_vy, tmp_vz, vout_x, vout_y, vout_z,pmul=1;
  int    line;
  char   intersect=0;
  char   intersecti=0;

  if (isrect) {
    intersect = box_intersect(&t0, &t3, x, y, z, vx, vy, vz, xwidth, yheight, zthick);
    intersecti = box_intersect(&t1, &t2, x, y, z, vx, vy, vz, xwidth_i, yheight_i, zthick_i);
  } else {
    intersect = cylinder_intersect(&t0, &t3, x, y, z, vx, vy, vz, radius, yheight);
    intersecti = cylinder_intersect(&t1, &t2, x, y, z, vx, vy, vz, radius_i, yheight_i);
  }
  
  if(intersect && t3 >0)
    {
      
      if (concentric) {
	
	
	if (!intersecti) {
	  t1 = (t3 + t0) /2;
	}
	t2 = t1;
	t3 = t1;
	dt = -1.0*rand01(); 
      } else {
	if (!intersecti) {
	  t1 = (t3 + t0) /2;
	  t2 = t1;
	}
	dt = randpm1(); 
      }
      
      
      if(t0 < 0) t0=0; 
      if(t1 < 0) t1=0; 
      if(t2 < 0) t2=0; 
      v = sqrt(vx*vx + vy*vy + vz*vz);
      l_full = v * (t3 - t2 + t1 - t0);
  	
      if (t3 < 0) {
	t3=0; 
	printf("PowderN %s Warning: Neutron has already passed us? (Skipped)\n", NAME_CURRENT_COMP);
      } else {
	if (dt<0) { 
	  dt = fabs(dt)*(t1 - t0); 
	} else {
	  dt = dt * (t3 - t2) + (t2-t0) ; 
	}
	
	my_s = my_s_v2_sum/(v*v)+my_inc;
	
	
	neutrontype = rand01();
	
	if (neutrontype < tfrac) {
	  neutrontype = 1; 
	  l = l_full; 
	  PROP_DT(t3); 
	} else if (neutrontype >= tfrac && neutrontype < (tfrac + frac)) {
	  neutrontype = 2;
	  l = v*dt;       
	  PROP_DT(dt+t0); 
	  SCATTER;
	} else if (neutrontype >= tfrac + frac) {
	  neutrontype = 3;
	  l = v*dt;       
	  PROP_DT(dt+t0); 
	  SCATTER;
	} else {
	  exit(fprintf(stderr,"PowderN %s: DEAD - this shouldn't happen!\n", NAME_CURRENT_COMP));
	}
	
	if (neutrontype == 3)
	  { 
	    
	    if (line_info.count > 0) {
	      
	      if (Nq > 1) line=floor(Nq*rand01());  
	      else line = 0;
	      if (w_v[line])
		arg = q_v[line]*(1+w_v[line]*randnorm())/(2.0*v);
	      else
		arg = q_v[line]/(2.0*v);
	      my_s_n = my_s_v2[line]/(v*v);
	      if(fabs(arg) > 1)
		ABSORB;                   
	      theta = asin(arg);          
	      
	      
	      if (d_phi)
		{ 
		  arg = sin(d_phi*DEG2RAD/2)/sin(2*theta);
		  
		  if (arg < -1 || arg > 1) d_phi = 0;
		  
		  else alpha = 2*asin(arg);
		}
	      if (d_phi) {
		
		alpha = fabs(alpha);
		
		alpha0= 2*rand01()*alpha;
		if (alpha0 > alpha) {
		  alpha0=PI+(alpha0-1.5*alpha);
		} else {
		  alpha0=alpha0-0.5*alpha;
		}
	      }
	      else
		alpha0 = PI*randpm1();
	      
	      
	      vec_prod(tmp_vx,tmp_vy,tmp_vz, vx,vy,vz, 1,0,0);
	      
	      
	      if (!tmp_vx && !tmp_vy && !tmp_vz) { tmp_vx=tmp_vz=0; tmp_vy=1; }
	      
	      
	      rotate(vout_x,vout_y,vout_z, vx,vy,vz, 2*theta, tmp_vx,tmp_vy,tmp_vz);
	      
	      
	      rotate(tmp_vx,tmp_vy,tmp_vz, vout_x,vout_y,vout_z, alpha0, vx, vy, vz);
	      vx = tmp_vx;
	      vy = tmp_vy;
	      vz = tmp_vz;
	      
	      
	      if (isrect) {
		intersect = box_intersect(&t0, &t3, x, y, z, vx, vy, vz, xwidth, yheight, zthick);
		intersecti = box_intersect(&t1, &t2, x, y, z, vx, vy, vz, xwidth_i, yheight_i, zthick_i);
	      } else {
		intersect = cylinder_intersect(&t0, &t3, x, y, z, vx, vy, vz, radius, yheight);
		intersecti = cylinder_intersect(&t1, &t2, x, y, z, vx, vy, vz, radius_i, yheight_i);
	      }
	      
	      if (!intersect) {
		
		fprintf(stderr, "PowderN: FATAL ERROR: Did not hit sample from inside.\n");
		ABSORB;
	      }
	      
	      if (!intersecti) {
		t1 = (t3 + t0) /2;
		t2 = t1;
	      }
	      
	      if (concentric && intersecti) {
		
		t2 = t1;
		t3 = t1;
	      }
	      
	      if(t0 < 0) t0=0; 
	      if(t1 < 0) t1=0; 
	      if(t2 < 0) t2=0; 
	      

	      l_1 = v*(t3 - t2 + t1 - t0); 
#ifdef DEBUG
	      printf("my_s_n=%g, my_a_v=%g, v=%g, my_s=%g\n", 
		     my_s_n, my_a_v, v, my_s);
#endif
	      pmul  = Nq*l_full*my_s_n*exp(-(my_a_v/v+my_s)*(l+l_1))/(1-(frac+tfrac));
	      
	      if (d_phi) pmul *= alpha/PI;
	    } 
	  }  
	else if (neutrontype == 2)
	  {  
	    if(d_phi) {
	      randvec_target_rect_angular(&vx, &vy, &vz, &solid_angle,
					  0, 0, 1,
					  2*PI, d_phi*DEG2RAD, ROT_A_CURRENT_COMP);
	    } else {
	      randvec_target_circle(&vx, &vy, &vz,
				    &solid_angle, 0, 0, 1, 0);
	    }
	    v1 = sqrt(vx*vx+vy*vy+vz*vz);
	    vx *= v/v1;
	    vy *= v/v1;
	    vz *= v/v1;
	    
	    
	    if (isrect) {
	      intersect = box_intersect(&t0, &t3, x, y, z, vx, vy, vz, xwidth, yheight, zthick);
	      intersecti = box_intersect(&t1, &t2, x, y, z, vx, vy, vz, xwidth_i, yheight_i, zthick_i);
	    } else {
	      intersect = cylinder_intersect(&t0, &t3, x, y, z, vx, vy, vz, radius, yheight);
	      intersecti = cylinder_intersect(&t1, &t2, x, y, z, vx, vy, vz, radius_i, yheight_i);
	    }
	    
	    if (!intersect) {
	      
	      fprintf(stderr, "PowderN: FATAL ERROR: Did not hit sample from inside.\n");
	      ABSORB;
	    }  
	      
	    if (!intersecti) {
	      t1 = (t3 + t0) /2;
	      t2 = t1;
	    }
	      
	    if (concentric && intersecti) {
	      
	      t2 = t1;
	      t3 = t1;
	    }
	      
	    if(t0 < 0) t0=0; 
	    if(t1 < 0) t1=0; 
	    if(t2 < 0) t2=0; 
	      

	    l_1 = v*(t3 - t2 + t1 - t0); 
	    
	    pmul = l_full*my_inc*exp(-(my_a_v/v+my_s)*(l+l_1))/(frac);
	    pmul *= solid_angle/(4*PI);
	    
	  }  
	else if (neutrontype == 1) {
	  
	  
	  pmul = exp(-(my_a_v/v+my_s)*(l))/(tfrac);
        }
	p *= pmul;
      } 
    } 
  
"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
