
def factory( name='sans_spheres',R=100.0,Phi=0.001,Delta_rho=0.6,sigma_a=50.0,dist=6.0,Rdet=0.5,xwidth=0.01,yheight=0.01,zthick=0.005,qmax=0.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from Sans_spheresboostpython import Sans_spheres as _factory_mcvine
    return component( _factory_mcvine( name,R,Phi,Delta_rho,sigma_a,dist,Rdet,xwidth,yheight,zthick,qmax ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """P. Willendrup, K. Lefmann, L. Arleth"""
info.copyright = info_copyright
info_declare = """
  double my_s_pre, my_a_v, q_v, q;
"""
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """Sample for use in a SANS instrument, models hard, mono disperse spheres in thin solution.
The shape of the sample may be a cylinder of given radius or a box with dimensions
xwidth, yheight, zthick. qmax defines a cutoff in q.

%BUGS
This component does NOT simulate absolute intensities. This latter depends on the detector parameters."""
info.full_description = info_full_description
info_initialize = """
    if (!xwidth || !yheight || !zthick) {
  exit(fprintf(stderr,"Sans_spheres: %s:   sample has no volume (zero dimensions)\n", NAME_CURRENT_COMP));
  if (qmax) printf("Sans_spheres: %s: Parameter qmax is not used anymore. Ignoring.\n", NAME_CURRENT_COMP);
    }

  my_a_v = sigma_a*2200*100;           

  my_s_pre = Phi * 4*PI*R*R*R/3 * Delta_rho*Delta_rho;

"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """sans_spheres"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 100.0
item1.default = item1_default
item1_description = """"""
item1.description = item1_description
item1_name = """R"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0.001
item2.default = item2_default
item2_description = """"""
item2.description = item2_description
item2_name = """Phi"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0.6
item3.default = item3_default
item3_description = """"""
item3.description = item3_description
item3_name = """Delta_rho"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 50.0
item4.default = item4_default
item4_description = """"""
item4.description = item4_description
item4_name = """sigma_a"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 6.0
item5.default = item5_default
item5_description = """"""
item5.description = item5_description
item5_name = """dist"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0.5
item6.default = item6_default
item6_description = """"""
item6.description = item6_description
item6_name = """Rdet"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0.01
item7.default = item7_default
item7_description = """"""
item7.description = item7_description
item7_name = """xwidth"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0.01
item8.default = item8_default
item8_description = """"""
item8.description = item8_description
item8_name = """yheight"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0.005
item9.default = item9_default
item9_description = """"""
item9.description = item9_description
item9_name = """zthick"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 0.0
item10.default = item10_default
item10_description = """"""
item10.description = item10_description
item10_name = """qmax"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10 ]
info.input_parameters = info_input_parameters
info_name = """Sans_spheres"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """my_s_v2"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """"""
item1.description = item1_description
item1_name = """my_a_v"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
info_output_parameters = [ item0, item1 ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """"""
item1 = """"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Sample for Small Angle Neutron Scattering - hard spheres in thin solution, mono disperse."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
  double t0, t1, v, l_full, l, l_1, dt, d_phi, theta, my_s;
  double aim_x, aim_y, aim_z, axis_x, axis_y, axis_z;
  double arg, tmp_vx, tmp_vy, tmp_vz, vout_x, vout_y, vout_z;
  double f, solid_angle, vx_i, vy_i, vz_i, qx, qy, qz;
  char   intersect=0;

  intersect = box_intersect(&t0, &t1, x, y, z, vx, vy, vz, xwidth, yheight, zthick);
  if(intersect)
  {
    if(t0 < 0)
      ABSORB;
    
    v = sqrt(vx*vx + vy*vy + vz*vz);
    l_full = v * (t1 - t0);             
    dt = rand01()*(t1 - t0) + t0; 
    PROP_DT(dt);                  
    l = v*(dt-t0);                

    vx_i=vx;
    vy_i=vy;
    vz_i=vz;
    randvec_target_circle(&vx, &vy, &vz, &solid_angle, 0, 0, dist, Rdet);

    NORM(vx, vy, vz);
    vx *= v;
    vy *= v;
    vz *= v;
    qx = V2K*(vx_i-vx);
    qy = V2K*(vy_i-vy);
    qz = V2K*(vz_i-vz);
    q = sqrt(qx*qx+qy*qy+qz*qz);

    f = 3 * (sin(q*R) - q*R*cos(q*R))/(q*R*q*R*q*R);


    if(!box_intersect(&t0, &t1, x, y, z, vx, vy, vz, xwidth, yheight, zthick))    fprintf(stderr, "Sans_spheres: FATAL ERROR: Did not hit box from inside.\n");


    l_1 = v*t1;

    p *= l_full*solid_angle/(4*PI)*my_s_pre*f*f*exp(-my_a_v*(l+l_1)/v);
    SCATTER;
  }
"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
