
def factory( name='single_crystal',reflections='',xwidth=0,yheight=0,zthick=0,delta_d_d=0.0001,mosaic=-1.0,mosaic_h=-1.0,mosaic_v=-1.0,mosaic_n=-1.0,ax=0.0,ay=0.0,az=0.0,bx=0.0,by=0.0,bz=0.0,cx=0.0,cy=0.0,cz=0.0,p_transmit=-1.0,absorption=0.0,incoherent=0.0,aa=0.0,bb=0.0,cc=0.0,order=0.0,powder=0.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from Single_crystalboostpython import Single_crystal as _factory_mcvine
    return component( _factory_mcvine( name,reflections,xwidth,yheight,zthick,delta_d_d,mosaic,mosaic_h,mosaic_v,mosaic_n,ax,ay,az,bx,by,bz,cx,cy,cz,p_transmit,absorption,incoherent,aa,bb,cc,order,powder ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Kristian Nielsen"""
info.copyright = info_copyright
info_declare = """
  struct hkl_info_struct hkl_info;
"""
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """Single crystal with mosaic. Delta-D/D option for finite-size effects.
Rectangular geometry. Multiple scattering and secondary extinction included.
The mosaic may EITHER be specified isotropic by setting the mosaic input
parameter, OR anisotropic by setting the mosaic_h, mosaic_v, and mosaic_n
parameters. Geometry is a box.
Crystal structure is specified with an ascii data file. Each line contains
4 or more numbers, separated by white spaces:

h k l ... F2

The first three numbers are the (h,k,l) indices of the reciprocal lattice
point, and the 7-th number is the value of the structure factor |F|**2, in
barns. The rest of the numbers are not used; the file is in the format
output by the Crystallographica program.
Lines begining by '#' are read as comments (ignored). Most sample parameters
may be defined from the data file header, following the same mechanism as
PowderN.

Current data file header keywords include, for data format specification:
#column_h <index of the Bragg Qh column>
#column_k <index of the Bragg Qk column>
#column_l <index of the Bragg Ql column>
#column_F2 <index of the squared str. factor '|F|^2' column [b]>
#column_F  <index of the structure factor norm '|F|' column>
and for material specification:
#sigma_abs <value of Absorption cross section [barns]>
#sigma_inc <value of Incoherent cross section [barns]>
#Delta_d/d <value of Detla_d/d width for all lines>
#lattice_a <value of the a lattice parameter [Angs]>
#lattice_a <value of the b lattice parameter [Angs]>
#lattice_a <value of the c lattice parameter [Angs]>
#lattice_aa <value of the alpha lattice angle [deg]>
#lattice_bb <value of the beta  lattice angle [deg]>
#lattice_cc <value of the gamma lattice angle [deg]>

See the Component Manual for more defails.

Vanadium incoherent elastic scattering with multiple scattering
Single_crystal(name, xwidth=0.01, yheight=0.01, zthick=0.01,
reflections="", absorption=5.08, incoherent=4.935,
ax=3.0282, by=3.0282, cz=3.0282/2)

%BUGS:
The component is known not to work properly as a Bragg monochromator. The 
reason is likely to relate to the internal definition of reciprocal space 
orientation. Investigation undergoing. 

Also, always use a non-zero value of delta_d_d.

%VALIDATION:
This component is undergoing validation."""
info.full_description = info_full_description
info_initialize = """
  double as, bs, cs;

  
  hkl_info.m_delta_d_d = delta_d_d;
  hkl_info.m_a  = 0;
  hkl_info.m_b  = 0;
  hkl_info.m_c  = 0;
  hkl_info.m_aa = aa;
  hkl_info.m_bb = bb;
  hkl_info.m_cc = cc;
  hkl_info.m_ax = ax;
  hkl_info.m_ay = ay;
  hkl_info.m_az = az;
  hkl_info.m_bx = bx;
  hkl_info.m_by = by;
  hkl_info.m_bz = bz;
  hkl_info.m_cx = cx;
  hkl_info.m_cy = cy;
  hkl_info.m_cz = cz;
  hkl_info.sigma_a = absorption;
  hkl_info.sigma_i = incoherent;

  
  hkl_info.column_order[0]=1;
  hkl_info.column_order[1]=2;
  hkl_info.column_order[2]=3;
  hkl_info.column_order[3]=0;
  hkl_info.column_order[4]=7;

  
  if (!read_hkl_data(reflections, &hkl_info, mosaic, mosaic_h, mosaic_v, mosaic_n))
    exit(0);
  if (hkl_info.count)
    printf("Single_crystal: %s: Read %d reflections from file '%s'\n",
      NAME_CURRENT_COMP, hkl_info.count, reflections);
  else printf("Single_crystal: %s: Using incoherent elastic scattering only (s_inc=%g barn)\n",
      NAME_CURRENT_COMP, hkl_info.sigma_i);
"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """single_crystal"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = """"""
item1.default = item1_default
item1_description = """File name containing structure factors of reflections. Use"""
item1.description = item1_description
item1_name = """reflections"""
item1.name = item1_name
item1_type = """char *"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """Width of crystal [m]"""
item2.description = item2_description
item2_name = """xwidth"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """Height of crystal [m]"""
item3.description = item3_description
item3_name = """yheight"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """Thichness of crystal (no extinction simulated) [m]"""
item4.description = item4_description
item4_name = """zthick"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0.0001
item5.default = item5_default
item5_description = """Lattice spacing variance, gaussian RMS [1]"""
item5.description = item5_description
item5_name = """delta_d_d"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = -1.0
item6.default = item6_default
item6_description = """Crystal mosaic (isotropic), gaussian RMS [arc minutes]"""
item6.description = item6_description
item6_name = """mosaic"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = -1.0
item7.default = item7_default
item7_description = """Horizontal (rotation around Y) mosaic (anisotropic),"""
item7.description = item7_description
item7_name = """mosaic_h"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = -1.0
item8.default = item8_default
item8_description = """Vertical (rotation around Z) mosaic (anisotropic),"""
item8.description = item8_description
item8_name = """mosaic_v"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = -1.0
item9.default = item9_default
item9_description = """Out-of-plane (Rotation around X) mosaic (anisotropic),"""
item9.description = item9_description
item9_name = """mosaic_n"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 0.0
item10.default = item10_default
item10_description = """[AA]"""
item10.description = item10_description
item10_name = """ax"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = 0.0
item11.default = item11_default
item11_description = """Coordinates of first unit cell vector [AA]"""
item11.description = item11_description
item11_name = """ay"""
item11.name = item11_name
item11_type = """double"""
item11.type = item11_type
item12 = Parameter()
item12_default = 0.0
item12.default = item12_default
item12_description = """[AA]"""
item12.description = item12_description
item12_name = """az"""
item12.name = item12_name
item12_type = """double"""
item12.type = item12_type
item13 = Parameter()
item13_default = 0.0
item13.default = item13_default
item13_description = """[AA]"""
item13.description = item13_description
item13_name = """bx"""
item13.name = item13_name
item13_type = """double"""
item13.type = item13_type
item14 = Parameter()
item14_default = 0.0
item14.default = item14_default
item14_description = """Coordinates of second unit cell vector [AA]"""
item14.description = item14_description
item14_name = """by"""
item14.name = item14_name
item14_type = """double"""
item14.type = item14_type
item15 = Parameter()
item15_default = 0.0
item15.default = item15_default
item15_description = """[AA]"""
item15.description = item15_description
item15_name = """bz"""
item15.name = item15_name
item15_type = """double"""
item15.type = item15_type
item16 = Parameter()
item16_default = 0.0
item16.default = item16_default
item16_description = """[AA]"""
item16.description = item16_description
item16_name = """cx"""
item16.name = item16_name
item16_type = """double"""
item16.type = item16_type
item17 = Parameter()
item17_default = 0.0
item17.default = item17_default
item17_description = """Coordinates of third unit cell vector [AA]"""
item17.description = item17_description
item17_name = """cy"""
item17.name = item17_name
item17_type = """double"""
item17.type = item17_type
item18 = Parameter()
item18_default = 0.0
item18.default = item18_default
item18_description = """[AA]"""
item18.description = item18_description
item18_name = """cz"""
item18.name = item18_name
item18_type = """double"""
item18.type = item18_type
item19 = Parameter()
item19_default = -1.0
item19.default = item19_default
item19_description = """Monte Carlo probability for neutrons to be transmitted"""
item19.description = item19_description
item19_name = """p_transmit"""
item19.name = item19_name
item19_type = """double"""
item19.type = item19_type
item20 = Parameter()
item20_default = 0.0
item20.default = item20_default
item20_description = """Absorption cross-section per unit cell at 2200 m/s [barns]"""
item20.description = item20_description
item20_name = """absorption"""
item20.name = item20_name
item20_type = """double"""
item20.type = item20_type
item21 = Parameter()
item21_default = 0.0
item21.default = item21_default
item21_description = """Incoherent scattering cross-section per unit cell [barns]"""
item21.description = item21_description
item21_name = """incoherent"""
item21.name = item21_name
item21_type = """double"""
item21.type = item21_type
item22 = Parameter()
item22_default = 0.0
item22.default = item22_default
item22_description = """[deg]."""
item22.description = item22_description
item22_name = """aa"""
item22.name = item22_name
item22_type = """double"""
item22.type = item22_type
item23 = Parameter()
item23_default = 0.0
item23.default = item23_default
item23_description = """unit cell angles alpha, beta and gamma. Then uses norms of"""
item23.description = item23_description
item23_name = """bb"""
item23.name = item23_name
item23_type = """double"""
item23.type = item23_type
item24 = Parameter()
item24_default = 0.0
item24.default = item24_default
item24_description = """[deg]."""
item24.description = item24_description
item24_name = """cc"""
item24.name = item24_name
item24_type = """double"""
item24.type = item24_type
item25 = Parameter()
item25_default = 0.0
item25.default = item25_default
item25_description = """limit multiple scattering up to given order"""
item25.description = item25_description
item25_name = """order"""
item25.name = item25_name
item25_type = """double"""
item25.type = item25_type
item26 = Parameter()
item26_default = 0.0
item26.default = item26_default
item26_description = """"""
item26.description = item26_description
item26_name = """powder"""
item26.name = item26_name
item26_type = """double"""
item26.type = item26_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11, item12, item13, item14, item15, item16, item17, item18, item19, item20, item21, item22, item23, item24, item25, item26 ]
info.input_parameters = info_input_parameters
info_name = """Single_crystal"""
info.name = info_name
info_output_parameters = [  ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """
// ----------- added by Jiao Lin --------
// ----------  header for SHARE ----------
// ----------- added by Jiao Lin --------


  
#include "mcstas2/share/read_table-lib.h"

#ifndef SINGLE_CRYSTAL_DECL
#define SINGLE_CRYSTAL_DECL

    struct hkl_data
    {
      int h,k,l;                  
      double F2;                  
      double tau_x, tau_y, tau_z; 
      double tau;                 
      double u1x, u1y, u1z;       
      double u2x, u2y, u2z;       
      double u3x, u3y, u3z;       
      double sig1, sig2, sig3;    
      double sig123;              
      double m1, m2, m3;          
      double cutoff;              
    };

  struct hkl_info_struct
    {
      struct hkl_data *list;      
      int count;                  
      struct tau_data *tau_list;  
      double m_delta_d_d;         
      double m_ax,m_ay,m_az;      
      double m_bx,m_by,m_bz;      
      double m_cx,m_cy,m_cz;      
      double asx,asy,asz;         
      double bsx,bsy,bsz;         
      double csx,csy,csz;         
      double m_a, m_b, m_c;       
      double m_aa, m_bb, m_cc;    
      double sigma_a, sigma_i;    
      double rho;                 
      double at_weight;           
      double at_nb;               
      double V0;                  
      int    column_order[5];     
    };

  struct tau_data
    {
      int index;                  
      double refl;
      double xsect;
      double sigma_1, sigma_2;
      
      double kix, kiy, kiz;       
      double rho_x, rho_y, rho_z; 
      double rho;                 
      double ox, oy, oz;          
      double nx, ny, nz;          
      double b1x, b1y, b1z;       
      double b2x, b2y, b2z;
      double l11, l12, l22;       
      double det_L;               
      double y0x, y0y;            
    };
#endif 

// ----------- added by Jiao Lin --------"""
item1 = """// ----------  implementation for SHARE ----------
// ----------- added by Jiao Lin --------
  int
  read_hkl_data(char *SC_file, struct hkl_info_struct *info,
    double SC_mosaic, double SC_mosaic_h, double SC_mosaic_v, double SC_mosaic_n)
  {
    struct hkl_data *list = NULL;
    int size = 0;
    t_Table sTable; 
    int i=0;
    double tmp_x, tmp_y, tmp_z;
    char **parsing;
    char flag=0;
    double nb_atoms=1;

    if (!SC_file || !strlen(SC_file)) {
      info->count = 0;
      flag=1;
    }
    if (!flag) {
      Table_Read(&sTable, SC_file, 1); 
      if (sTable.columns < 4) {
        fprintf(stderr, "Single_crystal: Error: The number of columns in %s should be at least %d for [h,k,l,F2]\n", SC_file, 4);
        return(0);
      }
      if (!sTable.rows) {
        fprintf(stderr, "Single_crystal: Error: The number of rows in %s should be at least %d\n", SC_file, 1);
        return(0);
      } else size = sTable.rows;

      
      parsing = Table_ParseHeader(sTable.header,
        "sigma_abs","sigma_a ",
        "sigma_inc","sigma_i ",
        "column_h",
        "column_k",
        "column_l",
        "column_F ",
        "column_F2",
        "Detla_d/d",
        "lattice_a ",
        "lattice_b ",
        "lattice_c ",
        "lattice_aa",
        "lattice_bb",
        "lattice_cc",
        "nb_atoms",
        NULL);

      if (parsing) {
        if (parsing[0] && !info->sigma_a) info->sigma_a=atof(parsing[0]);
        if (parsing[1] && !info->sigma_a) info->sigma_a=atof(parsing[1]);
        if (parsing[2] && !info->sigma_i) info->sigma_i=atof(parsing[2]);
        if (parsing[3] && !info->sigma_i) info->sigma_i=atof(parsing[3]);
        if (parsing[4])                   info->column_order[0]=atoi(parsing[4]);
        if (parsing[5])                   info->column_order[1]=atoi(parsing[5]);
        if (parsing[6])                   info->column_order[2]=atoi(parsing[6]);
        if (parsing[7])                   info->column_order[3]=atoi(parsing[7]);
        if (parsing[8])                   info->column_order[4]=atoi(parsing[8]);
        if (parsing[9] && info->m_delta_d_d <0) info->m_delta_d_d=atof(parsing[9]);
        if (parsing[10] && !info->m_a)    info->m_a =atof(parsing[10]);
        if (parsing[11] && !info->m_b)    info->m_b =atof(parsing[11]);
        if (parsing[12] && !info->m_c)    info->m_c =atof(parsing[12]);
        if (parsing[13] && !info->m_aa)   info->m_aa=atof(parsing[13]);
        if (parsing[14] && !info->m_bb)   info->m_bb=atof(parsing[14]);
        if (parsing[15] && !info->m_cc)   info->m_cc=atof(parsing[15]);
        if (parsing[16])   nb_atoms=atof(parsing[16]);
        for (i=0; i<=16; i++) if (parsing[i]) free(parsing[i]);
        free(parsing);
      }
    }

    
    if (info->m_ax || info->m_ay || info->m_az) info->m_a=0;
    if (info->m_bx || info->m_by || info->m_bz) info->m_b=0;
    if (info->m_cx || info->m_cy || info->m_cz) info->m_c=0;

    if (info->m_ax || info->m_ay || info->m_az) {
      double as=sqrt(info->m_ax*info->m_ax+info->m_ay*info->m_ay+info->m_az*info->m_az);
      if (!info->m_bx && !info->m_by && !info->m_bz) info->m_a=info->m_b=as;
      if (!info->m_cx && !info->m_cy && !info->m_cz) info->m_a=info->m_c=as;
    }

    if (info->m_a && !info->m_b) info->m_b=info->m_a;
    if (info->m_b && !info->m_c) info->m_c=info->m_b;

    if (info->m_a && !info->m_aa) info->m_aa=90;

    if (info->m_aa && !info->m_bb) info->m_bb=info->m_aa;
    if (info->m_bb && !info->m_cc) info->m_cc=info->m_bb;

    if (!info->m_ax && !info->m_ay && !info->m_az && !info->m_a) {
      fprintf(stderr,
              "Single_crystal: Error:Wrong a lattice vector definition\n");
      return(0);
    }
    if (!info->m_bx && !info->m_by && !info->m_bz && !info->m_b) {
      fprintf(stderr,
              "Single_crystal: Error:Wrong b lattice vector definition\n");
      return(0);
    }
    if (!info->m_cx && !info->m_cy && !info->m_cz && !info->m_c) {
      fprintf(stderr,
              "Single_crystal: Error:Wrong c lattice vector definition\n");
      return(0);
    }

    
    if (info->m_aa && info->m_bb && info->m_cc)
    {
      double as,bs,cs;
      if (info->m_a) as = info->m_a;
      else as = sqrt(info->m_ax*info->m_ax+info->m_ay*info->m_ay+info->m_az*info->m_az);
      if (info->m_b) bs = info->m_b;
      else bs = sqrt(info->m_bx*info->m_bx+info->m_by*info->m_by+info->m_bz*info->m_bz);
      if (info->m_c) cs = info->m_c;
      else cs =  sqrt(info->m_cx*info->m_cx+info->m_cy*info->m_cy+info->m_cz*info->m_cz);

      info->m_ax = as; info->m_ay = 0; info->m_az = 0;
      info->m_bx = bs*cos(info->m_cc*DEG2RAD);
      info->m_by = bs*sin(info->m_cc*DEG2RAD);
      info->m_bz = 0;
      info->m_cx = cs*cos(info->m_bb*DEG2RAD);
      info->m_cy = cs*(cos(info->m_aa*DEG2RAD)-cos(info->m_cc*DEG2RAD)*cos(info->m_bb*DEG2RAD))
                     /sin(info->m_cc*DEG2RAD);
      info->m_cz = sqrt(cs*cs - info->m_cx*info->m_cx - info->m_cy*info->m_cy);

      printf("Single_crystal: %s structure a=%g b=%g c=%g aa=%g bb=%g cc=%g ",
        (flag ? "INC" : SC_file), as, bs, cs, info->m_aa, info->m_bb, info->m_cc);
    } else
      printf("Single_crystal: %s structure a=[%g,%g,%g] b=[%g,%g,%g] c=[%g,%g,%g] ",
        (flag ? "INC" : SC_file), info->m_ax ,info->m_ay ,info->m_az,
        info->m_bx ,info->m_by ,info->m_bz,
        info->m_cx ,info->m_cy ,info->m_cz);

    
    vec_prod(tmp_x, tmp_y, tmp_z,
             info->m_bx, info->m_by, info->m_bz,
             info->m_cx, info->m_cy, info->m_cz);
    info->V0 = fabs(
              scalar_prod(info->m_ax, info->m_ay, info->m_az, tmp_x, tmp_y, tmp_z))
              ;
    printf("V0=%g\n", info->V0);

    info->asx = 2*PI/info->V0*tmp_x;
    info->asy = 2*PI/info->V0*tmp_y;
    info->asz = 2*PI/info->V0*tmp_z;
    vec_prod(tmp_x, tmp_y, tmp_z, info->m_cx, info->m_cy, info->m_cz, info->m_ax, info->m_ay, info->m_az);
    info->bsx = 2*PI/info->V0*tmp_x;
    info->bsy = 2*PI/info->V0*tmp_y;
    info->bsz = 2*PI/info->V0*tmp_z;
    vec_prod(tmp_x, tmp_y, tmp_z, info->m_ax, info->m_ay, info->m_az, info->m_bx, info->m_by, info->m_bz);
    info->csx = 2*PI/info->V0*tmp_x;
    info->csy = 2*PI/info->V0*tmp_y;
    info->csz = 2*PI/info->V0*tmp_z;

    if (flag) return(-1);

    if (!info->column_order[0] || !info->column_order[1] || !info->column_order[2]) {
      fprintf(stderr,
              "Single_crystal: Error:Wrong h,k,l column definition\n");
      return(0);
    }
    if (!info->column_order[3] && !info->column_order[4]) {
      fprintf(stderr,
              "Single_crystal: Error:Wrong F,F2 column definition\n");
      return(0);
    }

    
    list = (struct hkl_data*)malloc(size*sizeof(struct hkl_data));

    for (i=0; i<size; i++)
    {
      double h=0, k=0, l=0, F2=0;
      double b1[3], b2[3];

      
      h = Table_Index(sTable, i, info->column_order[0]-1);
      k = Table_Index(sTable, i, info->column_order[1]-1);
      l = Table_Index(sTable, i, info->column_order[2]-1);
      if (info->column_order[3])
      { F2= Table_Index(sTable, i, info->column_order[3]-1); F2 *= F2; }
      else if (info->column_order[4])
        F2= Table_Index(sTable, i, info->column_order[4]-1);

      list[i].h = h;
      list[i].k = k;
      list[i].l = l;
      list[i].F2 = F2;
      
      list[i].tau_x = h*info->asx + k*info->bsx + l*info->csx;
      list[i].tau_y = h*info->asy + k*info->bsy + l*info->csy;
      list[i].tau_z = h*info->asz + k*info->bsz + l*info->csz;
      list[i].tau = sqrt(list[i].tau_x*list[i].tau_x +
                         list[i].tau_y*list[i].tau_y +
                         list[i].tau_z*list[i].tau_z);
      list[i].u1x = list[i].tau_x/list[i].tau;
      list[i].u1y = list[i].tau_y/list[i].tau;
      list[i].u1z = list[i].tau_z/list[i].tau;
      list[i].sig1 = FWHM2RMS*info->m_delta_d_d*list[i].tau;
      
      normal_vec(&b1[0], &b1[1], &b1[2],
                 list[i].u1x, list[i].u1y, list[i].u1z);
      vec_prod(b2[0], b2[1], b2[2],
               list[i].u1x, list[i].u1y, list[i].u1z,
               b1[0], b1[1], b1[2]);
      
      if(SC_mosaic > 0) {
        
        list[i].u2x = b1[0];
        list[i].u2y = b1[1];
        list[i].u2z = b1[2];
        list[i].sig2 = FWHM2RMS*list[i].tau*MIN2RAD*SC_mosaic;
        list[i].u3x = b2[0];
        list[i].u3y = b2[1];
        list[i].u3z = b2[2];
        list[i].sig3 = FWHM2RMS*list[i].tau*MIN2RAD*SC_mosaic;
      } else if(SC_mosaic_h > 0 && SC_mosaic_v > 0 && SC_mosaic_n > 0) {
        
        double mos_D[3][3];
        double mos_a, mos_b, mos_c, mos_det, mos_l1, mos_l2;
        double mos_e1_x, mos_e1_y, mos_e1_len, mos_e2_x, mos_e2_y, mos_e2_len;
        int j1, j2;
        
        mos_D[0][0] = 1/((FWHM2RMS*MIN2RAD*SC_mosaic_n)*(FWHM2RMS*MIN2RAD*SC_mosaic_n));
        mos_D[0][1] = 0;
        mos_D[0][2] = 0;
        mos_D[1][0] = 0;
        mos_D[1][1] = 1/((FWHM2RMS*MIN2RAD*SC_mosaic_h)*(FWHM2RMS*MIN2RAD*SC_mosaic_h));
        mos_D[1][2] = 0;
        mos_D[2][0] = 0;
        mos_D[2][1] = 0;
        mos_D[2][2] = 1/((FWHM2RMS*MIN2RAD*SC_mosaic_v)*(FWHM2RMS*MIN2RAD*SC_mosaic_v));
        
        
        mos_a = mos_b = mos_c = 0;
        for(j1 = 0; j1 < 3; j1++) {
          for(j2 = 0; j2 < 3; j2++) {
            mos_a += b1[j1]*b1[j2]*mos_D[j1][j2];
            mos_b += b1[j1]*b2[j2]*mos_D[j1][j2];
            mos_c += b2[j1]*b2[j2]*mos_D[j1][j2];
          }
        }
        
        if(mos_b*mos_b < 1e-20*(mos_a*mos_a + mos_c*mos_c)) {
          
          mos_l1 = mos_a;
          mos_e1_x = 1;
          mos_e1_y = 0;
          mos_l2 = mos_c;
          mos_e2_x = 0;
          mos_e2_y = 1;
        } else {                  
          mos_det = (mos_a - mos_c)*(mos_a - mos_c) + 4*mos_b*mos_b;
          mos_l1 = 0.5*(mos_a + mos_c - sqrt(mos_det));
          mos_l2 = 0.5*(mos_a + mos_c + sqrt(mos_det));
          
          mos_e1_x = -mos_b;
          mos_e1_y = mos_a - mos_l1;
          mos_e1_len = sqrt(mos_e1_x*mos_e1_x + mos_e1_y*mos_e1_y);
          mos_e1_x /= mos_e1_len;
          mos_e1_y /= mos_e1_len;
          mos_e2_x = mos_c - mos_l2;
          mos_e2_y = -mos_b;
          mos_e2_len = sqrt(mos_e2_x*mos_e2_x + mos_e2_y*mos_e2_y);
          mos_e2_x /= mos_e2_len;
          mos_e2_y /= mos_e2_len;
        }
        
        list[i].u2x = b1[0]*mos_e2_x + b2[0]*mos_e2_y;
        list[i].u2y = b1[1]*mos_e2_x + b2[1]*mos_e2_y;
        list[i].u2z = b1[2]*mos_e2_x + b2[2]*mos_e2_y;
        list[i].sig2 = list[i].tau/sqrt(mos_l1);
        list[i].u3x = b1[0]*mos_e1_x + b2[0]*mos_e1_y;
        list[i].u3y = b1[1]*mos_e1_x + b2[1]*mos_e1_y;
        list[i].u3z = b1[2]*mos_e1_x + b2[2]*mos_e1_y;
        list[i].sig3 = list[i].tau/sqrt(mos_l2);
      } else {
        fprintf(stderr,
                "Single_crystal: Error: EITHER mosaic OR (mosaic_h, mosaic_v, mosaic_n)\n"
                "must be given and be >0.\n");
        return(0);
      }
      list[i].sig123 = list[i].sig1*list[i].sig2*list[i].sig3;
      list[i].m1 = 1/(2*list[i].sig1*list[i].sig1);
      list[i].m2 = 1/(2*list[i].sig2*list[i].sig2);
      list[i].m3 = 1/(2*list[i].sig3*list[i].sig3);
      
      if(list[i].sig1 > list[i].sig2)
        if(list[i].sig1 > list[i].sig3)
          list[i].cutoff = 5*list[i].sig1;
        else
          list[i].cutoff = 5*list[i].sig3;
      else
        if(list[i].sig2 > list[i].sig3)
          list[i].cutoff = 5*list[i].sig2;
        else
          list[i].cutoff = 5*list[i].sig3;
    }
    Table_Free(&sTable);
    info->list = list;
    info->count = i;
// ----------- changed by Jiao Lin --------
    //info->tau_list = malloc(i*sizeof(*info->tau_list));
    info->tau_list = (struct tau_data*)malloc(i*sizeof(*info->tau_list));
// ----------- changed by Jiao Lin --------
    if(!info->tau_list)
    {
      fprintf(stderr, "Single_crystal: Error: Out of memory!\n");
      return(0);
    }
    return(info->count = i);
  }

"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Mosaic single crystal with multiple scattering vectors."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
  double t1, t2=0;                
  struct hkl_data *L;           
  int i;                        
  struct tau_data *T;           
  int j;                        
  int event_counter;            
  double kix, kiy, kiz, ki;     
  double kfx, kfy, kfz;         
  double v;                     
  double tau_max;               
  double rho_x, rho_y, rho_z;   
  double rho;
  double diff;                  
  double ox, oy, oz;            
  double b1x, b1y, b1z;         
  double b2x, b2y, b2z;         
  double n11, n12, n22;         
  double det_N;                 
  double inv_n11, inv_n12, inv_n22; 
  double l11, l12, l22;         
  double det_L;                 
  double Bt_D_O_x, Bt_D_O_y;    
  double y0x, y0y;              
  double alpha;                 
  int tau_count;                
  double V0;                    
  double l_full;                
  double l;                     
  double abs_xsect, abs_xlen;   
  double inc_xsect, inc_xlen;   
  double coh_xsect, coh_xlen;   
  double tot_xsect, tot_xlen;   
  double z1, z2, y1, y2;        
  double adjust, coh_refl;      
  double r, sum;                
  double xsect_factor;          
  double p_trans;               
  double mc_trans, mc_interact; 

  if(box_intersect(&t1, &t2, x, y, z, vx, vy, vz,
                   xwidth, yheight, zthick) && t2 > 0)
  {                             
    if(t1 > 0)
      PROP_DT(t1);                
    v  = sqrt(vx*vx + vy*vy + vz*vz);
    ki = V2K*v;
    event_counter = 0;
    abs_xsect = hkl_info.sigma_a*2200/v;
    inc_xsect = hkl_info.sigma_i;
    V0= hkl_info.V0;
    abs_xlen  = 1e2*abs_xsect/V0;
    inc_xlen  = 1e2*inc_xsect/V0;
    L = hkl_info.list;
    T = hkl_info.tau_list;
    for(;;)                       
    {
      
      if(!box_intersect(&t1, &t2, x, y, z, vx, vy, vz,
                        xwidth, yheight, zthick) || t2*v < -1e-9 || t1*v > 1e-9)
      {
        
        fprintf(stderr,
                "Single_crystal: %s: Warning: neutron has unexpectedly left the crystal!\n"
                "                t1=%g t2=%g x=%g y=%g z=%g vx=%g vy=%g vz=%g\n",
                NAME_CURRENT_COMP, t1, t2, x, y, z, vx, vy, vz);
        break;
      }
      l_full = t2*v;

      
      if (powder) { 
        double nvx, nvy, nvz;
        randvec_target_circle(&nvx, &nvy, &nvz, NULL, vx, vy, vz, 0);
        kix = V2K*nvx;
        kiy = V2K*nvy;
        kiz = V2K*nvz;
      } else {
        kix = V2K*vx;
        kiy = V2K*vy;
        kiz = V2K*vz;
      }

      

      
      tau_max = 2*ki/(1 - 5*hkl_info.m_delta_d_d);

      coh_xsect = 0;
      coh_refl  = 0;
      xsect_factor = pow(2*PI, 5.0/2.0)/(V0*ki*ki);
      for(i = j = 0; i < hkl_info.count; i++)
      {
        
        if(L[i].tau > tau_max)
          break;
        
        rho_x = kix - L[i].tau_x;
        rho_y = kiy - L[i].tau_y;
        rho_z = kiz - L[i].tau_z;
        rho = sqrt(rho_x*rho_x + rho_y*rho_y + rho_z*rho_z);
        diff = fabs(rho - ki);

        
        if(diff <= L[i].cutoff)
        {
          
          T[j].index = i;
          
          T[j].kix = kix*L[i].u1x + kiy*L[i].u1y + kiz*L[i].u1z;
          T[j].kiy = kix*L[i].u2x + kiy*L[i].u2y + kiz*L[i].u2z;
          T[j].kiz = kix*L[i].u3x + kiy*L[i].u3y + kiz*L[i].u3z;
          T[j].rho_x = T[j].kix - L[i].tau;
          T[j].rho_y = T[j].kiy;
          T[j].rho_z = T[j].kiz;
          T[j].rho = rho;
          
          T[j].nx = T[j].rho_x/T[j].rho;
          T[j].ny = T[j].rho_y/T[j].rho;
          T[j].nz = T[j].rho_z/T[j].rho;
          ox = (ki - T[j].rho)*T[j].nx;
          oy = (ki - T[j].rho)*T[j].ny;
          oz = (ki - T[j].rho)*T[j].nz;
          T[j].ox = ox;
          T[j].oy = oy;
          T[j].oz = oz;
          
          normal_vec(&b1x, &b1y, &b1z, T[j].nx, T[j].ny, T[j].nz);
          vec_prod(b2x, b2y, b2z, T[j].nx, T[j].ny, T[j].nz, b1x, b1y, b1z);
          T[j].b1x = b1x;
          T[j].b1y = b1y;
          T[j].b1z = b1z;
          T[j].b2x = b2x;
          T[j].b2y = b2y;
          T[j].b2z = b2z;
          
          
          n11 = L[i].m1*b1x*b1x + L[i].m2*b1y*b1y + L[i].m3*b1z*b1z;
          n12 = L[i].m1*b1x*b2x + L[i].m2*b1y*b2y + L[i].m3*b1z*b2z;
          n22 = L[i].m1*b2x*b2x + L[i].m2*b2y*b2y + L[i].m3*b2z*b2z;
          
          det_N = n11*n22 - n12*n12;
          inv_n11 = n22/det_N;
          inv_n12 = -n12/det_N;
          inv_n22 = n11/det_N;
          
          l11 = sqrt(inv_n11/2);
          l12 = inv_n12/(2*l11);
          l22 = sqrt(inv_n22/2 - l12*l12);
          T[j].l11 = l11;
          T[j].l12 = l12;
          T[j].l22 = l22;
          det_L = l11*l22;
          T[j].det_L = det_L;
          
          Bt_D_O_x = b1x*L[i].m1*ox + b1y*L[i].m2*oy + b1z*L[i].m3*oz;
          Bt_D_O_y = b2x*L[i].m1*ox + b2y*L[i].m2*oy + b2z*L[i].m3*oz;
          
          y0x = -(Bt_D_O_x*inv_n11 + Bt_D_O_y*inv_n12);
          y0y = -(Bt_D_O_x*inv_n12 + Bt_D_O_y*inv_n22);
          T[j].y0x = y0x;
          T[j].y0y = y0y;
          
          alpha = L[i].m1*ox*ox + L[i].m2*oy*oy + L[i].m3*oz*oz -
                       (y0x*y0x*n11 + y0y*y0y*n22 + 2*y0x*y0y*n12);
          T[j].refl = xsect_factor*det_L*exp(-alpha)/L[i].sig123; 
          coh_refl += T[j].refl;                                  
          T[j].xsect = T[j].refl*L[i].F2;
          coh_xsect += T[j].xsect;
          j++;
        }
      } 
      tau_count = j;

      
      tot_xsect = abs_xsect + inc_xsect + coh_xsect;
      
      coh_xlen = 1e2*coh_xsect/V0;
      tot_xlen = 1e2*tot_xsect/V0;

      
      p_trans = exp(-tot_xlen*l_full);
      if(!event_counter && p_transmit >= 0 && p_transmit <= 1) {
        mc_trans = p_transmit; 
      } else {
        mc_trans = p_trans;
      }
      mc_interact = 1 - mc_trans;
      if(mc_trans > 0 && (mc_trans >= 1 || rand01() < mc_trans))  
      {
        p *= p_trans/mc_trans;
        break;
      }
      if(tot_xlen <= 0)
        ABSORB;

      if(mc_interact <= 0)        
        break;
      if (!event_counter) p *= fabs(1 - p_trans)/mc_interact;
      
      
      if(tot_xlen*l_full < 1e-6)
        
        l = rand0max(l_full);
      else
        l = -log(1 - rand0max((1 - exp(-tot_xlen*l_full))))/tot_xlen;
      PROP_DT(l/v);
      event_counter++;

      
      p *= (coh_xlen + inc_xlen)/tot_xlen;
      
      if(coh_xlen == 0 || rand0max(coh_xlen + inc_xlen) <= inc_xlen)
      {
        
        randvec_target_circle(&kix, &kiy, &kiz, NULL, vx, vy, vz, 0);
        vx = kix; 
        vy = kiy;
        vz = kiz; 
      } else {
        
        if(coh_refl <= 0)
          ABSORB;
        r = rand0max(coh_refl);
        sum = 0;
        for(j = 0; j < tau_count; j++)
        {
          sum += T[j].refl;
          if(sum > r) break;
        }
        if(j >= tau_count)
        {
          fprintf(stderr, "Single_crystal: Error: Illegal tau search "
              "(r = %g, sum = %g).\n", r, sum);
          j = tau_count - 1;
        }
        i = T[j].index;
        
        z1 = randnorm();
        z2 = randnorm();
        y1 = T[j].l11*z1 + T[j].y0x;
        y2 = T[j].l12*z1 + T[j].l22*z2 + T[j].y0y;
        kfx = T[j].rho_x + T[j].ox + T[j].b1x*y1 + T[j].b2x*y2;
        kfy = T[j].rho_y + T[j].oy + T[j].b1y*y1 + T[j].b2y*y2;
        kfz = T[j].rho_z + T[j].oz + T[j].b1z*y1 + T[j].b2z*y2;
        
        adjust = ki/sqrt(kfx*kfx + kfy*kfy + kfz*kfz);
        kfx *= adjust;
        kfy *= adjust;
        kfz *= adjust;
        
        p *= T[j].xsect*coh_refl/(coh_xsect*T[j].refl);

        vx = K2V*(L[i].u1x*kfx + L[i].u2x*kfy + L[i].u3x*kfz);
        vy = K2V*(L[i].u1y*kfx + L[i].u2y*kfy + L[i].u3y*kfz);
        vz = K2V*(L[i].u1z*kfx + L[i].u2z*kfy + L[i].u3z*kfz);
      }
      SCATTER;
      
      if (order && event_counter >= order) break;
      
    } 
  } 
"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
