
def factory( name='v_sample',radius_i=0.0,radius_o=0.01,h=0.05,focus_r=0.0,pack=1.0,frac=1.0,f_QE=0.0,gamma=0.0,target_x=0.0,target_y=0.0,target_z=0.235,focus_xw=0.0,focus_yh=0.0,focus_aw=0.0,focus_ah=0.0,xwidth=0.0,yheight=0.0,zthick=0.0,sig_a=5.08,sig_i=4.935,V0=13.827,target_index=0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from V_sampleboostpython import V_sample as _factory_mcvine
    return component( _factory_mcvine( name,radius_i,radius_o,h,focus_r,pack,frac,f_QE,gamma,target_x,target_y,target_z,focus_xw,focus_yh,focus_aw,focus_ah,xwidth,yheight,zthick,sig_a,sig_i,V0,target_index ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Kim Lefmann and Kristian Nielsen"""
info.copyright = info_copyright
info_declare = """
    struct StructVarsV VarsV;
"""
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """A Double-cylinder shaped incoherent scatterer (a V-sample)
with both elastic and quasielastic (Lorentzian) components.
No multiple scattering. Absorbtion included.
The shape of the sample may be a box with dimensions xwidth, yheight, zthick.
The area to scatter to is a disk of radius 'focus_r' situated at the target.
This target area may also be rectangular if specified focus_xw and focus_yh
or focus_aw and focus_ah, respectively in meters and degrees.
The target itself is either situated according to given coordinates (x,y,z),
or defined with the relative target_index of the component to focus
to (next is +1).
This target position will be set to its AT position. When targeting to
centered components, such as spheres or cylinders, define an Arm component
where to focus to.

Peter Christiansen: Added outgoing polarization: P' = 1/3*P-2/3P = -1/3P
NB! As above multiple scattering is ignored ."""
info.full_description = info_full_description
info_initialize = """

  printf("V_sample initializing...\n");
  
  if (!xwidth || !yheight || !zthick) 
    if (!radius_o || !h)              
      exit(fprintf(stderr,"V_sample: %s: sample has no volume (zero dimensions)\n", NAME_CURRENT_COMP));
    else                              
      VarsV.isrect=0;
  else                                
    VarsV.isrect=1;



  VarsV.sigma_a=sig_a;
  VarsV.sigma_i=sig_i;
  VarsV.rho = (pack/V0);
  VarsV.my_s=(VarsV.rho * 100 * VarsV.sigma_i);
  VarsV.my_a_v=(VarsV.rho * 100 * VarsV.sigma_a);

  
  VarsV.tx= VarsV.ty=VarsV.tz=0;
  if (target_index)
  {
    
  }
  else
  { VarsV.tx = target_x; VarsV.ty = target_y; VarsV.tz = target_z; }

  if (!(VarsV.tx || VarsV.ty || VarsV.tz))
    printf("V_sample: %s: The target is not defined. Using direct beam (Z-axis).\n",
      NAME_CURRENT_COMP);

  VarsV.distance=sqrt(VarsV.tx*VarsV.tx+VarsV.ty*VarsV.ty+VarsV.tz*VarsV.tz);

  
  VarsV.aw  = VarsV.ah = 0;
  VarsV.xw  = VarsV.yh = 0;
  if (focus_xw) {
  VarsV.xw = focus_xw;
  }
  if (focus_yh) {
    VarsV.yh = focus_yh;
  }
  if (focus_aw) {
    VarsV.aw = DEG2RAD*focus_aw;
  }
  if (focus_ah) {
    VarsV.ah = DEG2RAD*focus_ah;
  }

  printf("V_sample initialized\n");
"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """v_sample"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0.0
item1.default = item1_default
item1_description = """Inner radius of sample in (x,z) plane [m]"""
item1.description = item1_description
item1_name = """radius_i"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0.01
item2.default = item2_default
item2_description = """Outer radius of sample in (x,z) plane [m]"""
item2.description = item2_description
item2_name = """radius_o"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0.05
item3.default = item3_default
item3_description = """Height of sample y direction [m]"""
item3.description = item3_description
item3_name = """h"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0.0
item4.default = item4_default
item4_description = """Radius of disk containing target. Use 0 for full space [m]"""
item4.description = item4_description
item4_name = """focus_r"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 1.0
item5.default = item5_default
item5_description = """Packing factor [1]"""
item5.description = item5_description
item5_name = """pack"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 1.0
item6.default = item6_default
item6_description = """MC Probability for scattering the ray; otherwise penetrate [1]"""
item6.description = item6_description
item6_name = """frac"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0.0
item7.default = item7_default
item7_description = """Fraction of quasielastic scattering (rest is elastic) [1]"""
item7.description = item7_description
item7_name = """f_QE"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0.0
item8.default = item8_default
item8_description = """Lorentzian width of quasielastic broadening (HWHM) [1]"""
item8.description = item8_description
item8_name = """gamma"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0.0
item9.default = item9_default
item9_description = """"""
item9.description = item9_description
item9_name = """target_x"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 0.0
item10.default = item10_default
item10_description = """position of target to focus at [m]"""
item10.description = item10_description
item10_name = """target_y"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = 0.235
item11.default = item11_default
item11_description = """"""
item11.description = item11_description
item11_name = """target_z"""
item11.name = item11_name
item11_type = """double"""
item11.type = item11_type
item12 = Parameter()
item12_default = 0.0
item12.default = item12_default
item12_description = """horiz. dimension of a rectangular area [m]"""
item12.description = item12_description
item12_name = """focus_xw"""
item12.name = item12_name
item12_type = """double"""
item12.type = item12_type
item13 = Parameter()
item13_default = 0.0
item13.default = item13_default
item13_description = """vert.  dimension of a rectangular area [m]"""
item13.description = item13_description
item13_name = """focus_yh"""
item13.name = item13_name
item13_type = """double"""
item13.type = item13_type
item14 = Parameter()
item14_default = 0.0
item14.default = item14_default
item14_description = """horiz. angular dimension of a rectangular area [deg]"""
item14.description = item14_description
item14_name = """focus_aw"""
item14.name = item14_name
item14_type = """double"""
item14.type = item14_type
item15 = Parameter()
item15_default = 0.0
item15.default = item15_default
item15_description = """vert.  angular dimension of a rectangular area [deg]"""
item15.description = item15_description
item15_name = """focus_ah"""
item15.name = item15_name
item15_type = """double"""
item15.type = item15_type
item16 = Parameter()
item16_default = 0.0
item16.default = item16_default
item16_description = """horiz. dimension of sample, as a width [m]"""
item16.description = item16_description
item16_name = """xwidth"""
item16.name = item16_name
item16_type = """double"""
item16.type = item16_type
item17 = Parameter()
item17_default = 0.0
item17.default = item17_default
item17_description = """vert.  dimension of sample, as a height [m]"""
item17.description = item17_description
item17_name = """yheight"""
item17.name = item17_name
item17_type = """double"""
item17.type = item17_type
item18 = Parameter()
item18_default = 0.0
item18.default = item18_default
item18_description = """thickness of sample [m]"""
item18.description = item18_description
item18_name = """zthick"""
item18.name = item18_name
item18_type = """double"""
item18.type = item18_type
item19 = Parameter()
item19_default = 5.08
item19.default = item19_default
item19_description = """Absorbtion cross section pr. unit cell [barns]"""
item19.description = item19_description
item19_name = """sig_a"""
item19.name = item19_name
item19_type = """double"""
item19.type = item19_type
item20 = Parameter()
item20_default = 4.935
item20.default = item20_default
item20_description = """Incoherent scattering cross section pr. unit cell [barns]"""
item20.description = item20_description
item20_name = """sig_i"""
item20.name = item20_name
item20_type = """double"""
item20.type = item20_type
item21 = Parameter()
item21_default = 13.827
item21.default = item21_default
item21_description = """Unit cell volume [AA^3]"""
item21.description = item21_description
item21_name = """V0"""
item21.name = item21_name
item21_type = """double"""
item21.type = item21_type
item22 = Parameter()
item22_default = 0
item22.default = item22_default
item22_description = """relative index of component to focus at, e.g. next is +1 [1]"""
item22.description = item22_description
item22_name = """target_index"""
item22.name = item22_name
item22_type = """int"""
item22.type = item22_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11, item12, item13, item14, item15, item16, item17, item18, item19, item20, item21, item22 ]
info.input_parameters = info_input_parameters
info_name = """V_sample"""
info.name = info_name
info_output_parameters = [  ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """
// ----------- added by Jiao Lin --------
// ----------  header for SHARE ----------
// ----------- added by Jiao Lin --------
  struct StructVarsV
  {
    double sigma_a; 
    double sigma_i; 
    double rho;     
    double my_s;
    double my_a_v;
    char   isrect;      
    double distance;    
    double aw,ah;       
    double xw,yh;       
    double tx,ty,tz;    
  };
// ----------- added by Jiao Lin --------"""
item1 = """// ----------  implementation for SHARE ----------
// ----------- added by Jiao Lin --------
"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Vanadium sample."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
  double t0, t3;                
  double t1, t2;                
  double v;                     
  double dt0, dt1, dt2, dt;     
  double l_full;                
  double l_i, l_o=0;            
  double my_a=0;                  
  double solid_angle=0;         
  double aim_x=0, aim_y=0, aim_z=1;   
  double v_i, v_f, E_i, E_f; 
  double dE;                 
  int    intersect=0;

  if (VarsV.isrect)
    intersect = box_intersect(&t0, &t3, x, y, z, vx, vy, vz, xwidth, yheight, zthick);
  else
    intersect = cylinder_intersect(&t0, &t3, x, y, z, vx, vy, vz, radius_o, h);
  if(intersect)
  {
    if(t0 < 0) ABSORB; 
    
    if(VarsV.isrect)
      t1 = t2 = t3;
    else
      if(!radius_i || !cylinder_intersect(&t1, &t2, x, y, z, vx, vy, vz, radius_i, h))
        t1 = t2 = t3;

    dt0 = t1-t0;                
    dt1 = t2-t1;                
    dt2 = t3-t2;                
    v = sqrt(vx*vx + vy*vy + vz*vz);
    l_full = v * (dt0 + dt2);   
    if (v) my_a = VarsV.my_a_v*(2200/v);

    if (frac >= 1 || rand01()<frac)          
    {
      dt = rand01()*(dt0+dt2);    
      l_i = v*dt;                 
      if (dt > dt0)
        dt += dt1;                

      PROP_DT(dt+t0);             

      if ((VarsV.tx || VarsV.ty || VarsV.tz)) {
        aim_x = VarsV.tx-x;       
        aim_y = VarsV.ty-y;
        aim_z = VarsV.tz-z;
      }
      if(VarsV.aw && VarsV.ah) {
        randvec_target_rect_angular(&vx, &vy, &vz, &solid_angle,
          aim_x, aim_y, aim_z, VarsV.aw, VarsV.ah, ROT_A_CURRENT_COMP);
      } else if(VarsV.xw && VarsV.yh) {
        randvec_target_rect(&vx, &vy, &vz, &solid_angle,
          aim_x, aim_y, aim_z, VarsV.xw, VarsV.yh, ROT_A_CURRENT_COMP);
      } else {
        randvec_target_circle(&vx, &vy, &vz, &solid_angle, aim_x, aim_y, aim_z, focus_r);
      }
      NORM(vx, vy, vz);

      v_i = v;          
      if (rand01()<f_QE)	
	{
          E_i = VS2E*v_i*v_i;
          dE = gamma*tan(PI/2*randpm1());
          E_f = E_i + dE;
          if (E_f <= 0)
            ABSORB;
	  v_f = SE2V*sqrt(E_f);
          v = v_f;
	  
	}

      vx *= v;
      vy *= v;
      vz *= v;

      if(!VarsV.isrect) {
        if(!cylinder_intersect(&t0, &t3, x, y, z, vx, vy, vz, radius_o, h))
        {
          
          printf("FATAL ERROR: Did not hit cylinder from inside.\n");
          exit(1);
        }
        dt = t3; 
        if(cylinder_intersect(&t1, &t2, x, y, z, vx, vy, vz, radius_i, h) &&
           t2 > 0)
          dt -= (t2-t1);            
      }
      else
      {
      if(!box_intersect(&t0, &t3, x, y, z, vx, vy, vz, xwidth, yheight, zthick))
        {
          
          printf("FATAL ERROR: Did not hit box from inside.\n");
          exit(1);
        }
        dt = t3;
      }
      l_o = v*dt; 

      p *= v/v_i*l_full*VarsV.my_s*exp(-my_a*(l_i+v_i/v*l_o)-VarsV.my_s*l_i);
      
      p /= 4*PI/solid_angle;
      p /= frac;

      
      

      SCATTER;
    }
  else 
    {
      p *= exp(-(my_a+VarsV.my_s)*l_full);
      p /= (1-frac);
    }
  }
"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
