
def factory( name='ess_moderator_long',size=0.12,l_low=0,l_high=0,dist=0,xw=0,yh=0,freq=16.67,T=50.0,tau=0.000287,tau1=0.0,tau2=2e-05,d=0.002,n=20.0,n2=5.0,chi2=0.9,I0=690000000000.0,I2=276000000000.0,branch1=1.0,branch2=0.5,branch_tail=0.1435,branchframe=0.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from ESS_moderator_longboostpython import ESS_moderator_long as _factory_mcvine
    return component( _factory_mcvine( name,size,l_low,l_high,dist,xw,yh,freq,T,tau,tau1,tau2,d,n,n2,chi2,I0,I2,branch1,branch2,branch_tail,branchframe ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """KL, February 2001"""
info.copyright = info_copyright
info_declare = """
  double l_range, w_mult;

 
"""
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """Produces a time-of-flight spectrum, from the ESS parameters
Chooses evenly in lambda, evenly/exponentially decaying in time
Adapted from Moderator by: KN, M.Hagen, August 1998

Units of flux: n/cm^2/s/AA/ster
(McStas units are in general neutrons/second)

Example general parameters (general):
size=0.12 l_low=0.1 l_high=10 dist=1.6 xw=0.19 yh=0.15 freq=16.67
branchframe=0.5

Example moderator specific parameters
(From F. Mezei, "ESS reference moderator characteristics for ...", 4/12/00:
Defining the normalised Maxwellian
M(lam,T) = 2 a^2 lam^-5 exp(-a/lam^2); a=949/T; lam in AA; T in K,
the "pulse integral" function
iexp(t,tau,d) = 0                              ; t<0
tau (1-exp(-t/tau))            ; 0<t<d
tau (exp(d/tau)-1) exp(-t/tau) ; t>d ,
and the long pulse shape function
I(t,tau,n,d) = (iexp(t,tau,d)-iexp(t,tau/n,d)) n/(n-1)/tau/d ,

the flux distribution is given as
Phi(t,lam) =  I0 M(lam,T) F(t,tau,n)
+ I2/(1+exp(chi2 lam-2.2))/lam*F(t,tau2*lam,n2)  )

c1: Ambient H20, long pulse, coupled
T=325 tau=80e-6 tau1=400e-6 tau2=12e-6 n=20 n2=5 d=2e-3 chi2=2.5
I0=13.5e11 I2=27.6e10    branch1=0.5 branch2=0.5

c2: Liquid H2, long pulse, coupled
T=50 tau=287e-6 tau1=0 tau2=20e-6 n=20 n2=5 d=2e-3 chi2=0.9
I0=6.9e11 I2=27.6e10    branch1=0 branch2=0.5

Debugged intensively against Mezei note (4/12 2000) and VitESS @ Rencurel 2006.
The output is now neutrons / second, not as previously neutrons / pulse.

%VALIDATION 
Validated against VitESS and Mezei note (4/12 2000) @ Rencurel 2006"""
info.full_description = info_full_description
info_initialize = """
  if (n == 1 || n2 == 1 || l_low<=0 || l_high <=0 || dist == 0
    || branch2 == 0 || branch_tail == 0 || tau == 0)
  {
    printf("ESS_moderator_long: %s: Check parameters (lead to Math Error).\n Avoid 0 value for {l_low l_high dist d tau branch1/2/tail} and 1 value for {n n2 branch1/2/tail}\n", NAME_CURRENT_COMP);
    exit(-1);
  }

  if (tau1==0 && !(branch1==1)) {
    branch1=1;
    printf("ESS_moderator_long: %s: WARNING: Setting tau1 to zero implies branch 1=1.\n", NAME_CURRENT_COMP);
  }

  l_range = l_high-l_low;
  w_mult = size*size*1.0e4;     
  w_mult *= l_range;            
  w_mult *= 1/mcget_ncount();   
  w_mult *= freq;               
"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """ess_moderator_long"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0.12
item1.default = item1_default
item1_description = """(m)    Edge of cube shaped source"""
item1.description = item1_description
item1_name = """size"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """(AA)   Lower edge of lambda distribution"""
item2.description = item2_description
item2_name = """l_low"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """(AA)   Upper edge of energy distribution"""
item3.description = item3_description
item3_name = """l_high"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """(m)    Distance from source to focusing rectangle; at (0,0,dist)"""
item4.description = item4_description
item4_name = """dist"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0
item5.default = item5_default
item5_description = """(m)    Width of focusing rectangle"""
item5.description = item5_description
item5_name = """xw"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0
item6.default = item6_default
item6_description = """(m)    Height of focusing rectangle"""
item6.description = item6_description
item6_name = """yh"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 16.67
item7.default = item7_default
item7_description = """(Hz)   Frequency of pulses"""
item7.description = item7_description
item7_name = """freq"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 50.0
item8.default = item8_default
item8_description = """(K)    Temperature of source"""
item8.description = item8_description
item8_name = """T"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0.000287
item9.default = item9_default
item9_description = """(s)    long time decay constant for pulse tail 1a"""
item9.description = item9_description
item9_name = """tau"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 0.0
item10.default = item10_default
item10_description = """(s)    long time decay constant for pulse tail 1b"""
item10.description = item10_description
item10_name = """tau1"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = 2e-05
item11.default = item11_default
item11_description = """(s)    long time decay constant for pulse, 2"""
item11.description = item11_description
item11_name = """tau2"""
item11.name = item11_name
item11_type = """double"""
item11.type = item11_type
item12 = Parameter()
item12_default = 0.002
item12.default = item12_default
item12_description = """(s)    pulse length"""
item12.description = item12_description
item12_name = """d"""
item12.name = item12_name
item12_type = """double"""
item12.type = item12_type
item13 = Parameter()
item13_default = 20.0
item13.default = item13_default
item13_description = """(1)    pulse shape parameter, 1"""
item13.description = item13_description
item13_name = """n"""
item13.name = item13_name
item13_type = """double"""
item13.type = item13_type
item14 = Parameter()
item14_default = 5.0
item14.default = item14_default
item14_description = """(1)    pulse shape parameter, 2"""
item14.description = item14_description
item14_name = """n2"""
item14.name = item14_name
item14_type = """double"""
item14.type = item14_type
item15 = Parameter()
item15_default = 0.9
item15.default = item15_default
item15_description = """(1/AA) lambda-distribution parameter in pulse 2"""
item15.description = item15_description
item15_name = """chi2"""
item15.name = item15_name
item15_type = """double"""
item15.type = item15_type
item16 = Parameter()
item16_default = 690000000000.0
item16.default = item16_default
item16_description = """(flux) integrated flux, 1 (in flux units, see above)"""
item16.description = item16_description
item16_name = """I0"""
item16.name = item16_name
item16_type = """double"""
item16.type = item16_type
item17 = Parameter()
item17_default = 276000000000.0
item17.default = item17_default
item17_description = """(flux) Flux, 2 (in flux units, see above)"""
item17.description = item17_description
item17_name = """I2"""
item17.name = item17_name
item17_type = """double"""
item17.type = item17_type
item18 = Parameter()
item18_default = 1.0
item18.default = item18_default
item18_description = """(1)   limit for switching between two time structures in"""
item18.description = item18_description
item18_name = """branch1"""
item18.name = item18_name
item18_type = """double"""
item18.type = item18_type
item19 = Parameter()
item19_default = 0.5
item19.default = item19_default
item19_description = """(1)   limit for switching between distribution 1 and 2."""
item19.description = item19_description
item19_name = """branch2"""
item19.name = item19_name
item19_type = """double"""
item19.type = item19_type
item20 = Parameter()
item20_default = 0.1435
item20.default = item20_default
item20_description = """(1)   limit for switching between pulse and tail"""
item20.description = item20_description
item20_name = """branch_tail"""
item20.name = item20_name
item20_type = """double"""
item20.type = item20_type
item21 = Parameter()
item21_default = 0.0
item21.default = item21_default
item21_description = """(1) limit for switching between 1st and 2nd pulse"""
item21.description = item21_description
item21_name = """branchframe"""
item21.name = item21_name
item21_type = """double"""
item21.type = item21_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11, item12, item13, item14, item15, item16, item17, item18, item19, item20, item21 ]
info.input_parameters = info_input_parameters
info_name = """ESS_moderator_long"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """l_range"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """"""
item1.description = item1_description
item1_name = """w_mult"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
info_output_parameters = [ item0, item1 ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """"""
item1 = """"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """A parametrised pulsed source for modelling ESS long pulses."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
  double v,tau_l,E,lambda,k,r,xf,yf,dx,dy,w_focus,tail_flag;

  z=0;

  x = 0.5*size*randpm1();
  y = 0.5*size*randpm1();         

  randvec_target_rect(&xf, &yf, &r, &w_focus,
        0, 0, dist, xw, yh, ROT_A_CURRENT_COMP);

  dx = xf-x;
  dy = yf-y;
  r = sqrt(dx*dx+dy*dy+dist*dist);

  lambda = l_low+l_range*rand01();    
  k = 2*PI/lambda;
  v = K2V*k;

  vz = v*dist/r;
  vy = v*dy/r;
  vx = v*dx/r;




  tail_flag = (rand01()<branch_tail);   
 if (tail_flag)
 {
  if (rand01() < branch2)
  {
    if (tau1>0)
      if (rand01() < branch1)     
      {  
        tau_l = tau;
        p = 1/(branch1*branch2*branch_tail); 
      }
      else
      {  
        tau_l = tau1;
        p = 1/((1-branch1)*branch2*branch_tail); 
      }
    else
      {
        tau_l = tau;
        p = 1/(branch2*branch_tail); 
      }
    t = -tau_l*log(1e-12+rand01());       
 
    p *= w_focus;                         
    p *= tau_l/d;                         
    
    double l = lambda;
    double a=949.0/T;
    double M= 2*a*a*exp(-a/(l*l))/(l*l*l*l*l);

    p *= I0*w_mult*M;           
  }
  else
  {
    
    tau_l = tau2*lambda;
    t = -tau_l*log(1e-12+rand01());       
    p = n2/(n2-1)*((1-exp(-d/tau_l))-(1-exp(-n2*d/tau_l))*exp(-(n2-1)*t/tau_l)/n);
                                          
    p /= (1-branch2)*branch_tail;          
    p *= tau_l/d;                         
    p *= w_focus;                         
    p *= I2*w_mult/(1+exp(chi2*lambda-2.2))/lambda;
                                          
  }
  t += d;                                 
 }
 else
 {
  t = d*rand01();                        
  if (rand01() < branch2)
  {
    if (rand01() < branch1)     
    {  
      tau_l = tau;
      p = 1/(branch1*branch2*(1-branch_tail)); 
    }
    else
    {  
      tau_l = tau1;
      p = 1/((1-branch1)*branch2*(1-branch_tail)); 
    }
    p *= 1-n/(n-1)*(exp(-t/tau_l)-exp(-n*t/tau_l)/n); 
    p *= w_focus;                         
    
    double l = lambda;
    double a=949.0/T;
    double M= 2*a*a*exp(-a/(l*l))/(l*l*l*l*l);

    p *= I0*w_mult*M;           
  }
  else
  {
    
    tau_l = tau2*lambda;
    p = 1-n2/(n2-1)*(exp(-t/tau_l)-exp(-n2*t/tau_l)/n2); 
    p /= (1-branch2)*(1-branch_tail);      
    p *= w_focus;                         
    p *= I2*w_mult/(1+exp(chi2*lambda-2.2))/lambda;
                                          
  }
 }
 if (rand01()<branchframe)
   {
     t+=1/freq;
     p/=(branchframe);
   }
 else
   p/=(1-branchframe);
"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
