
def factory( name='moderator',radius=0,E0=0,E1=0,dist=0,xw=0,yh=0,t0=37.15,Ec=9.0,gam=39.1 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from Moderatorboostpython import Moderator as _factory_mcvine
    return component( _factory_mcvine( name,radius,E0,E1,dist,xw,yh,t0,Ec,gam ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """KN, M.Hagen"""
info.copyright = info_copyright
info_declare = """
  double p_in;
"""
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """Produces a simple time-of-flight spectrum, with a flat energy distribution"""
info.full_description = info_full_description
info_initialize = """
  p_in = 1.0/(4*PI);
  if (radius < 0 || E0 < 0 || E1 < 0 || dist <= 0 || xw < 0 || yh < 0 || Ec < 0 || gam <= 0) {
      printf("Moderator: %s: Error in input parameter values!\n"
             "ERROR          Exiting\n",
           NAME_CURRENT_COMP);
      exit(0);
  }
"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """moderator"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """(m)   Radius of source"""
item1.description = item1_description
item1_name = """radius"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """(meV) Lower edge of energy distribution"""
item2.description = item2_description
item2_name = """E0"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """(meV) Upper edge of energy distribution"""
item3.description = item3_description
item3_name = """E1"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """(m)   Distance from source to the focusing rectangle"""
item4.description = item4_description
item4_name = """dist"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0
item5.default = item5_default
item5_description = """(m)   Width of focusing rectangle"""
item5.description = item5_description
item5_name = """xw"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0
item6.default = item6_default
item6_description = """(m)   Height of focusing rectangle"""
item6.description = item6_description
item6_name = """yh"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 37.15
item7.default = item7_default
item7_description = """(mus) decay constant for low-energy neutrons"""
item7.description = item7_description
item7_name = """t0"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 9.0
item8.default = item8_default
item8_description = """(meV) Critical energy, below which the flux decay is constant"""
item8.description = item8_description
item8_name = """Ec"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 39.1
item9.default = item9_default
item9_description = """(meV) energy dependence of decay time"""
item9.description = item9_description
item9_name = """gam"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.input_parameters = info_input_parameters
info_name = """Moderator"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """p_in"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
info_output_parameters = [ item0 ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """"""
item1 = """"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """A simple pulsed source for time-of-flight."""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
   double chi,v,r,tauv,E, xf, yf, rf, dx, dy, pdir;

   p=p_in;
   z=0;

   chi = 2*PI*rand01();          
   r = sqrt(rand01())*radius;    
   x = r*cos(chi);
   y = r*sin(chi);

   randvec_target_rect(&xf, &yf, &rf, &pdir,
        0, 0, dist, xw, yh, ROT_A_CURRENT_COMP);

   dx = xf-x;
   dy = yf-y;
   rf = sqrt(dx*dx+dy*dy+dist*dist);

   p = pdir*p_in;

   E = E0+(E1-E0)*rand01();      
   v = SE2V*sqrt(E);

   vz=v*dist/rf;
   vy=v*dy/rf;
   vx=v*dx/rf;

   if(E < Ec)
   {
     tauv = t0;
   }
   else
   {
     double tmp = ((E - Ec) / gam);
     tauv = t0 / (1 + (tmp*tmp));
   }
   t = -tauv*log(rand01())*1E-6;
   SCATTER;
"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
