
def factory( name='sns_source4',S_filename='',width=0,height=0,dist=0,xw=0,yh=0,Emin=0,Emax=0,tinmin=0,tinmax=0,sample_E=0,sample_t=0,proton_T=0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from SNS_source4boostpython import SNS_source4 as _factory_mcvine
    return component( _factory_mcvine( name,S_filename,width,height,dist,xw,yh,Emin,Emax,tinmin,tinmax,sample_E,sample_t,proton_T ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """F. Gallmeier"""
info.copyright = info_copyright
info_declare = """
  double hdiv, vdiv;   
  double p_in, CL, CU;
  double para_sp[14];
  double para_a[10];
  double para_b[10];
  double para_R[10];
  double para_to[10];
"""
info.declare = info_declare
info_finalize = """
  
"""
info.finalize = info_finalize
info_full_description = """Produces a time-of-flight spectrum from SNS moderator files
moderator files can be obtained from the author
The output units of this component are N/pulse
Notes:
(1) the raw moderator files are per Sr.  The focusing parameters provide the solid
angle accepted by the guide to remove the Sr dependence from the output. Therefore 
the best practice is to set xw and yh to the width and height of the next beam 
component, respectively. The dist parameter should then be set as the distance 
from the moderator to the first component.
(2) Be sure that Emin and Emax are within the limits of 1e-5 to 100 eV
(3) the proton pulse length T determines short- and long-pulse mode"""
info.full_description = info_full_description
info_initialize = """  
  char line[1000];
  int linelength=1000;
  int k, kk;

  FILE *fp;//, *fopen();

  fp = fopen(S_filename,"r");
  if (fp==NULL){
    printf("Error opening file\n");
  }
  else {
    printf("%s\n","File opened...");
 }
  
  while( (fgets(line,linelength,fp) != NULL) && (strchr(line,'#') != NULL) ){
    printf("%s",line);
  }
  kk=sscanf(line," %le %le %le %le %le %le %le %le %le %le %le %le %le %le", &para_sp[0], &para_sp[1], &para_sp[2], 
	    &para_sp[3], &para_sp[4], &para_sp[5], &para_sp[6], &para_sp[7], &para_sp[8], &para_sp[9], &para_sp[10],
            &para_sp[11], &para_sp[12], &para_sp[13]);

   
  while( (fgets(line,linelength,fp) != NULL) && (strchr(line,'#') != NULL) ){
    printf("%s",line);
  }
  kk=sscanf(line," %le %le %le %le %le %le %le %le %le %le", &para_a[0], &para_a[1], &para_a[2], 
	    &para_a[3], &para_a[4], &para_a[5], &para_a[6], &para_a[7], &para_a[8], &para_a[9]);
  if(kk!=10) {
    printf("para_a: insufficient number of data entries read\n");
    abort();
  }

   
  while((fgets(line,linelength,fp)!=NULL)&&(strchr(line,'#')!=NULL)){ 
    printf("%s",line);
  }
  kk=sscanf(line," %le %le %le %le %le %le %le %le %le %le", &para_b[0], &para_b[1], &para_b[2], 
	    &para_b[3], &para_b[4], &para_b[5], &para_b[6], &para_b[7], &para_b[8], &para_b[9]);
  if(kk!=10) {
    printf("para_b: insufficient number of data entries read\n");
    abort();
  }

   
  while((fgets(line,linelength,fp)!=NULL)&&(strchr(line,'#')!=NULL)){
    printf("%s",line);
  }
  kk=sscanf(line," %le %le %le %le %le %le %le %le %le %le", &para_R[0], &para_R[1], &para_R[2], 
	    &para_R[3], &para_R[4], &para_R[5], &para_R[6], &para_R[7], &para_R[8], &para_R[9]);
  if(kk!=10) {
    printf("para_R: kk=%d  insufficient number of data entries read\n"), kk;
    abort();
  }

   
  while((fgets(line,linelength,fp)!=NULL)&&(strchr(line,'#')!=NULL)){
    printf("%s",line);
  }
  kk=sscanf(line," %le %le %le %le %le %le %le %le %le %le", &para_to[0], &para_to[1], &para_to[2], 
	    &para_to[3], &para_to[4], &para_to[5], &para_to[6], &para_to[7], &para_to[8], &para_to[9]);
  if(kk!=10) {
    printf("para_R: insufficient number of data entries read\n");
    abort();
  }

  printf("\n");
  for(k=0; k<14; k++){
    printf("   para_sp(%d)=%e\n",k,para_sp[k]);
  }
  printf("\n");
  for(k=0; k<10; k++){
    printf("   para_a(%d)=%e\n",k,para_a[k]);
  }
  printf("\n");
  for(k=0; k<10; k++){
    printf("   para_b(%d)=%e\n",k,para_b[k]);
  }
  printf("\n");
  for(k=0; k<8; k++){
    printf("   para_R(%d)=%e\n",k,para_R[k]);
  }
  printf("\n");
  for(k=0; k<10; k++){
    printf("   para_to(%d)=%e\n",k,para_to[k]);
  }
  
  p_in = xw*yh/(dist*dist);
  CL = (1.0/sqrt(Emin*1e-3)-1.0/sqrt(Emax*1e-3))/(Emax-Emin)*1e3;
  CU = (log(Emax*1e-3)-log(Emin*1e-3))/(Emax-Emin)*1e3;

"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """sns_source4"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = """"""
item1.default = item1_default
item1_description = """Filename of source data"""
item1.description = item1_description
item1_name = """S_filename"""
item1.name = item1_name
item1_type = """char *"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """width of moderator (m)"""
item2.description = item2_description
item2_name = """width"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """height of moderator (m)"""
item3.description = item3_description
item3_name = """height"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """Distance from source to the focusing rectangle (m)"""
item4.description = item4_description
item4_name = """dist"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0
item5.default = item5_default
item5_description = """Width of focusing rectangle (m)"""
item5.description = item5_description
item5_name = """xw"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0
item6.default = item6_default
item6_description = """Height of focusing rectangle (m)"""
item6.description = item6_description
item6_name = """yh"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0
item7.default = item7_default
item7_description = """minimum energy of neutron to generate (meV)"""
item7.description = item7_description
item7_name = """Emin"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0
item8.default = item8_default
item8_description = """maximum energy of neutron to generate (meV)"""
item8.description = item8_description
item8_name = """Emax"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0
item9.default = item9_default
item9_description = """minimum time of neutron to generate (us)"""
item9.description = item9_description
item9_name = """tinmin"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 0
item10.default = item10_default
item10_description = """maximum time of neutron to generate (us)"""
item10.description = item10_description
item10_name = """tinmax"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = 0
item11.default = item11_default
item11_description = """sample energy uniformly over"""
item11.description = item11_description
item11_name = """sample_E"""
item11.name = item11_name
item11_type = """double"""
item11.type = item11_type
item12 = Parameter()
item12_default = 0
item12.default = item12_default
item12_description = """0="sample t uniformly over t-range;"""
item12.description = item12_description
item12_name = """sample_t"""
item12.name = item12_name
item12_type = """double"""
item12.type = item12_type
item13 = Parameter()
item13_default = 0
item13.default = item13_default
item13_description = """proton pulse length (us)"""
item13.description = item13_description
item13_name = """proton_T"""
item13.name = item13_name
item13_type = """double"""
item13.type = item13_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11, item12, item13 ]
info.input_parameters = info_input_parameters
info_name = """SNS_source4"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """hdiv"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """"""
item1.description = item1_description
item1_name = """vdiv"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """"""
item2.description = item2_description
item2_name = """p_in"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
info_output_parameters = [ item0, item1, item2 ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """
// ----------- added by Jiao Lin --------
// ----------  header for SHARE ----------
// ----------- added by Jiao Lin --------
#include <cstring>
double spectral_function( double para[14], double E );
double ikeda_carpenter_function_sp (double para[4], double t) ;
double ikeda_carpenter_function_sp_find_t (double para[4], double tmin, double tmax, double randd);
double ikeda_carpenter_function_lp (double para[4], double t, double T);
double ikeda_carpenter_function_lp_find_t (double para[4], double tmin, double tmax, double T, double randd);
double pade_function (double para[10], double E);

// ----------- added by Jiao Lin --------"""
item1 = """// ----------  implementation for SHARE ----------
// ----------- added by Jiao Lin --------
   
double spectral_function( double para[14], double E )
{
  double c, R1, T1, R2, T2, R3, T3, a, b, Ecut, s, delta, g, Io;
  double D, x, B, k, rho, arg1, arg2, arg3, arg4, arg5, arg6;
 
  
  k =  8.617e-5;
  B =  7.36e-3;

  
  c    = para[0];     ;
  R1   = para[1];   
  T1   = para[2];   
  R2   = para[3];   
  T2   = para[4];   
  R3   = para[5];   
  T3   = para[6];   
  a    = para[7];   
  b    = para[8];   
  Ecut = para[9];   
  s    = para[10];  
  delta= para[11];  
  g    = para[12];  
  Io   = para[13];  

  
  D  = 1.0/(1+pow(Ecut/E,s));
  x = 0.0;
  if(E>2.0*B) {x = g*(E-2.0*B);}
  rho = 1.0 + delta*exp(-x)*(1 + x +0.5*x*x);
  arg1 = Io*1.0e12 * exp(-c/sqrt(E));
  arg2 = R1*E/pow(k*T1,2) *exp(-E/(k*T1));
  arg3 = R2*E/pow(k*T2,2) *exp(-E/(k*T2));
  arg4 = R3*E/pow(k*T3,2) *exp(-pow(E/(k*T3),b));
  arg5 = D*rho/pow(E,1-a);
  arg6 =(arg1 * ( arg2 + arg3 + arg4 + arg5 ));
  return(arg6);
}


double ikeda_carpenter_function_sp (double para[4], double t) 
{
  double a, b, R, to;
  double tt, eat, ebt, arg1, arg2, arg3, arg4;
  a = para[0];
  b = para[1];
  R = para[2];
  to= para[3];
    
  tt = t - to*10.0;
  if(tt<0.0) {tt=0.0;}
  eat  = exp(-a*tt);
  ebt  = exp(-b*tt);
  arg1 = (1-R) *pow(a*tt,2) *eat;
  arg2 = 2*R*a*a *b/pow(a-b,3);
  arg3 = 1 + (a-b)*tt + 0.5*pow(a-b,2) *tt*tt;
  arg4 = 0.5*a *( arg1 + arg2 *( ebt - eat*arg3 ) );

  
  return(arg4);
}



double ikeda_carpenter_function_sp_find_t (double para[4], double tmin, double tmax, double randd)
{
  double a, b, R, to;
  double eat, ebt, arg1, arg2, arg3;
  double t, ttmin, ttmax, tm, tp, fm, fp, renorm, Imin, Imax;
  double delta, gt, ambda, ambda2;
  int n, flag;
  double diff, eps;

  eps = 1.0e-4;

  a = para[0];
  b = para[1];
  R = para[2];
  to= para[3];
    
  ttmin=tmin-to*10.0;
  if(ttmin<0.0) {ttmin=0.0;}
  ttmax=tmax-to*10.0;
  if(ttmax<0.0) {ttmax=0.0;}
  if(ttmin==ttmax) {
    printf("tmax-to and tmin-to are zero!\n");
    abort();
  }
   
  ambda = (a-b)/a;
  ambda2= ambda*ambda; 
  delta = pow(1/ambda,3);

  eat  = exp(-a*ttmax);
  ebt  = exp(-b*ttmax);
  gt   = 0.5*pow(a*ttmax,2) + a*ttmax + 1.0;        
  arg1  = 1.0-gt*eat;
  arg2  = 1.0 -delta*(ebt - eat*b/a*(1.0 + ambda*(a*ttmax+1.0) + ambda2*gt));
  Imax  = (1.0-R)*arg1 + R*arg2;
  
  eat  = exp(-a*ttmin);
  ebt  = exp(-b*ttmin);
  gt   = 0.5*pow(a*ttmin,2) + a*ttmin + 1.0;        
  arg1  = 1.0-gt*eat;
  arg2  = 1.0 -delta*(ebt - eat*b/a*(1.0 + ambda*(a*ttmin+1.0) + ambda2*gt));
  Imin  = (1.0-R)*arg1 + R*arg2;
  
  renorm = 1.0 /(Imax-Imin);
  tm = ttmin;
  fm = 0.0;
  tp = ttmax;
  fp = Imax;
  flag = 1;
  n = 0;

  diff = 1.0;
  while(fabs(diff)>eps) {
    n++;
    if(n<10) {
      t = 0.5*(tp+tm);
    }
    else {
      t   = tm +(tp-tm)/(fp-fm)*(randd-fm);
    }
    gt  = 0.5*pow(a*t,2) + a*t + 1.0;        
    eat = exp(-a*t);
    ebt = exp(-b*t);
    arg1  = 1.0-gt*eat;
    arg2  = 1.0 -delta*(ebt - eat*b/a*(1.0 + ambda*(a*t+1.0) + ambda2*gt));
    arg3  = (1.0-R)*arg1 + R*arg2; 
    arg3  = (arg3 -Imin)*renorm;
 
    diff = arg3 - randd;
    if(n<50){
    
   }
   else {
    printf(" ikeda_carpenter_function_find_t used 50 iterations!\n");
    abort();
   }
   if(arg3<randd) {
       tm = t;
       fm = arg3;
    }
    else if(arg3>randd) {
       tp = t;
       fp = arg3;
    }
  } 
  
  return(t+to*10.0);
}





double ikeda_carpenter_function_lp (double para[4], double t, double T)
{
  double a, b, R, to;
  double tt, eat, ebt, arg1, arg2, Imax, Imin, Itot, delta, ambda, ambda2, gt;
  a = para[0];
  b = para[1];
  R = para[2];
  to= para[3];
    
  ambda = (a-b)/a;
  ambda2= ambda*ambda; 
  delta = pow(ambda,-3);

  
  tt = t - to*10.0;
  if(tt<0.0) {tt=0.0;}
  eat  = exp(-a*tt);
  ebt  = exp(-b*tt);
  gt   = 0.5*pow(a*tt,2) + a*tt + 1.0;        
  arg1  = gt*eat;
  arg2  = delta*(ebt - eat*b/a*(1.0 + ambda*(a*tt+1.0) + ambda2*gt));
  Imax  = (1.0-R)*arg1 + R*arg2;

  
  tt = tt - T;
  if(tt<0.0) {tt=0.0;}
  eat  = exp(-a*tt);
  ebt  = exp(-b*tt);
  gt   = 0.5*pow(a*tt,2) + a*tt + 1.0;        
  arg1  = gt*eat;
  arg2  = delta*(ebt - eat*b/a*(1.0 + ambda*(a*tt+1.0) + ambda2*gt));
  Imin  = (1.0-R)*arg1 + R*arg2;

  Itot = (Imin - Imax)/T;

  return(Itot);
}



double intint_ikecar (double para[4], double t, double T)
{
  double a, b, R, to;
  double eat, ebt, arg1, arg2, arg3, arg4;
  double tt, delta, gamma, ambda, ambda2;

  a = para[0];
  b = para[1];
  R = para[2];
  to= para[3];
  tt = t - to*10.0;
  if(tt<0.0) {tt=0.0;}
  ambda  = (a-b)/a;
  ambda2 = ambda*ambda;
  delta = pow(ambda,-3);
 
  
  eat   = exp(-a*tt);
  ebt   = exp(-b*tt);
  gamma = 0.5*pow(a*tt,2) + 2.0*a*tt + 3.0;       
  arg1  = gamma/a*eat;
  arg2  = delta/b*ebt;
  arg3  = delta*b/a/a*(1.0+2.0*ambda+ambda*a*tt+ambda2*gamma)*eat;
  arg4  = ((1.0-R)*arg1 + R*arg2 +R*arg3)/T;
  
  return(arg4);
}




double ikeda_carpenter_function_lp_find_t (double para[4], double tmin, double tmax, double T, double randd)
{
  double a, b, R, to;
  double eat, ebt, arg1, arg2, arg3, arg4, arg5;
  double t, tt, ttmin, ttmax, tm, tp, fm, fp, renorm, CImin, CImax;
  double delta, gamma, ambda, ambda2;
  int n, flag;
  double diff, eps;

  eps = 1.0e-4;

  a = para[0];
  b = para[1];
  R = para[2];
  to= para[3];

  ambda  = (a-b)/a;
  ambda2 = ambda*ambda;
  delta = pow(ambda,-3);

  
  tt=tmax-to*10.;
  if(tt<0.0) {tt=0.0;} 
  CImax  = (1.0-R)/T*(tt - 3.0/a) 
            + R*delta/T*(tt -1.0/b - b/a*(1.0+ambda+ambda2)*tt + b/a/a*(1.0+2.0*ambda+3.0*ambda2)) 
            + intint_ikecar(para,tmax,T);
  if(tt>=T) { 
    CImax += (1.0-R)/T*((T-tt) + 3.0/a) 
             + R*delta/T*((T-tt) + 1.0/b - b/a*(1.0+ambda+ambda2)*(T-tt) + b/a/a*(1.0+2.0*ambda+3.0*ambda2)) 
             - intint_ikecar(para,tmax-T,T);  
  }
  

  tt=tmin-to*10.;
  if(tt<0.0) {tt=0.0;}
  CImin  = (1.0-R)/T*(tt - 3.0/a) 
           + R*delta/T*(tt -1.0/b - b/a*(1.0+ambda+ambda2)*tt + b/a/a*(1.0+2.0*ambda+3.0*ambda2)) 
           + intint_ikecar(para,tmin,T);
  if(tt>=T) {
     CImin += (1.0-R)/T*((T-tt) + 3.0/a) 
              + R*delta/T*((tt-T) + 1.0/b - b/a*(1.0+ambda+ambda2)*(T-tt) + b/a/a*(1.0+2.0*ambda+3.0*ambda2)) 
              - intint_ikecar(para,tmin-T,T);  
  }
  
  renorm = 1.0 /(CImax-CImin);
  tm = tmin;
  fm = 0.0;
  tp = tmax;
  fp = CImax;
  flag = 1;
  n = 0;

  diff = 1.0;
  while(fabs(diff)>eps) {
    n++;
    if(n<10) {
      t = 0.5*(tp+tm);
    }
    else {
      t = tm +(tp-tm)/(fp-fm)*(randd-fm);
    }
    tt=t-to*10.;
    if(tt<0.0) {tt=0.0;}
    arg4  = (1.0-R)/T*(tt - 3.0/a) 
            + R*delta/T*(tt -1.0/b - b/a*(1.0+ambda+ambda2)*tt + b/a/a*(1.0+2.0*ambda+3.0*ambda2)) 
            + intint_ikecar(para,t,T);
    if(tt>=T) {
      arg4 += (1.0-R)/T*((T-tt) + 3.0/a) 
              + R*delta/T*((T-tt) + 1.0/b - b/a*(1.0+ambda+ambda2)*(T-tt) + b/a/a*(1.0+2.0*ambda+3.0*ambda2)) 
              - intint_ikecar(para,t-T,T);  
    }
    
    arg4  = (arg4 -CImin)*renorm;
    diff = arg4 - randd;
    if(n<50){
    
   }
   else {
    printf(" ikeda_carpenter_function_find_t used 50 iterations!\n");
    abort();
   }
   if(arg4<randd) {
       tm = t;
       fm = arg4;
    }
    else if(arg4>randd) {
       tp = t;
       fp = arg4;
    }
  } 
  
  return(t+to*10.0);
}

double pade_function (double para[10], double E)
{
  double a, b, c, d, f, g, h, i, j, k;
 
    a = para[0];
    b = para[1];
    c = para[2];
    d = para[3];
    f = para[4];
    g = para[5];
    h = para[6];
    i = para[7];
    j = para[8];
    k = para[9];
   
    return(a*pow(E,b)*(1+c*E+d*E*E+pow(E/f,g))/(1+h*E+i*E*E+pow(E/j,k)));
}
"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """A source that produces a time and energy distribution from the SNS moderator files"""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """    
  double theta,phi,v,E,Eval,tval;
  double hdivmin,hdivmax,vdivmin,vdivmax;
  double p_E, p_t, para_t[4], p_sa, randd, ttmax;
  int k;
  p=p_in;
  z=0;
  x = (rand01()-0.5)*width; 
  y = (rand01()-0.5)*height;
  hdivmax=atan((xw/2.0-x)/dist);
  hdivmin=atan(-(xw/2.0+x)/dist);
  vdivmax=atan((yh/2.0-y)/dist);
  vdivmin=atan(-(yh/2.0+y)/dist);
  theta = hdivmin + (hdivmax-hdivmin)*rand01();  
  phi = vdivmin + (vdivmax-vdivmin)*rand01();
  hdiv=theta;
  vdiv=phi;
  
  if(sample_E==0){
    Eval=(Emin+(Emax-Emin)*rand01())*1e-3;
    p_sa = 1.0;   
    
  }
  else if(sample_E==1) {
    Eval = pow(1.0/(1.0/sqrt(Emax) +(1.0/sqrt(Emin)-1.0/sqrt(Emax))*rand01()),2)*1e-3;
    p_sa = CL*2.0*pow(Eval,1.5);
    
  }
  else if(sample_E==2) { 
    Eval = exp(log(Emin) +(log(Emax)-log(Emin))*rand01())*1e-3;
    p_sa = CU*Eval;
    
  }
  else {
    printf("sample_E allows only values 0, 1, or 2!\n");
  }  
  
  
  
  
  p_E = spectral_function(para_sp, Eval);
  para_t[0] = pade_function(para_a, Eval);
  para_t[1] = pade_function(para_b, Eval);
  para_t[2] = pade_function(para_R, Eval);
  para_t[3] = pade_function(para_to, Eval);
  if(sample_t==0) {
  
    randd = rand01();
    if(proton_T<=0.0) {
      ttmax = 6.0/para_t[1];
      if(ttmax>tinmax) {ttmax=tinmax;}
      tval = ikeda_carpenter_function_sp_find_t (para_t, tinmin, ttmax, randd);
    }
    else {
      ttmax = proton_T+6.0/para_t[1];
      if(ttmax>tinmax) {ttmax=tinmax;}
      tval = ikeda_carpenter_function_lp_find_t (para_t, tinmin, ttmax, proton_T, randd);
    }
    p_t = 1.0;
  }
  else if(sample_t==1) {
   
    if(proton_T<=0.0) {
      ttmax = 5.0/para_t[1];
      if(ttmax>tinmax) {ttmax=tinmax;}
      tval=tinmin+(ttmax-tinmin)*rand01();
      p_t = ikeda_carpenter_function_sp (para_t, tval);
    }
    else {
      ttmax = proton_T+5.0/para_t[1];
      if(ttmax>tinmax) {ttmax=tinmax;}
      tval=tinmin+(ttmax-tinmin)*rand01();
      p_t = ikeda_carpenter_function_lp (para_t, tval, proton_T);
    }
    p_t *= (ttmax-tinmin);
  }
  else {
    printf("sample_t allows only values 0 or 1!\n");
  }  

  E = Eval*1000.0;  
  t = tval*1e-6;        
  v = SE2V*sqrt(E);
  
  vz = v*cos(phi)*cos(theta);   
  vy = v*sin(phi);
  vx = v*cos(phi)*sin(theta);
  
  p*=p_E*p_sa*p_t/mcget_ncount();
  
"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
