
def factory( name='source_maxwell_3',size=0.0,height=0.0,width=0.0,l_low=0,l_high=0,dist=0,xw=0,yh=0,T1=0,T2=300.0,T3=300.0,I1=0,I2=0.0,I3=0.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from Source_Maxwell_3boostpython import Source_Maxwell_3 as _factory_mcvine
    return component( _factory_mcvine( name,size,height,width,l_low,l_high,dist,xw,yh,T1,T2,T3,I1,I2,I3 ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Kim Lefmann"""
info.copyright = info_copyright
info_declare = """
  double l_range, w_mult;
  double w_source, h_source; 
"""
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """A parametrised continuous source for modelling a (cubic) source
with (up to) 3 Maxwellian distributions.
The source produces a continuous spectrum.
The sampling of the neutrons is uniform in lambda.

Units of flux: neutrons/cm^2/second/ster
(McStas units are in general neutrons/second)"""
info.full_description = info_full_description
info_initialize = """
  if (size>0) {
    w_source = h_source = size;
  } else {
    w_source = width;
    h_source = height;
  }
  l_range = l_high-l_low;
  w_mult = w_source*h_source*1.0e4;     
  w_mult *= l_range;            
  w_mult *= 1/mcget_ncount();   

  if (w_source <0 || h_source < 0 || l_low <= 0 || l_high <= 0 || dist <= 0 || T1 <= 0 || T2 <= 0|| T3 <= 0) {
      printf("Source_Maxwell_3: %s: Error in input parameter values!\n"
             "ERROR          Exiting\n",
           NAME_CURRENT_COMP);
      exit(0);
  }

 """
info.initialize = info_initialize
item0 = Parameter()
item0_default = """source_maxwell_3"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0.0
item1.default = item1_default
item1_description = """(m)    Edge of cube shaped source (for backward compatibility)"""
item1.description = item1_description
item1_name = """size"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0.0
item2.default = item2_default
item2_description = """(m)    Height of rectangular source"""
item2.description = item2_description
item2_name = """height"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0.0
item3.default = item3_default
item3_description = """(m)    Width of rectangular source"""
item3.description = item3_description
item3_name = """width"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """(AA)   Lower edge of lambda distribution"""
item4.description = item4_description
item4_name = """l_low"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0
item5.default = item5_default
item5_description = """(AA)   Upper edge of lambda distribution"""
item5.description = item5_description
item5_name = """l_high"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0
item6.default = item6_default
item6_description = """(m)    Distance from source to focusing rectangle; at (0,0,dist)"""
item6.description = item6_description
item6_name = """dist"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0
item7.default = item7_default
item7_description = """(m)    Width of focusing rectangle"""
item7.description = item7_description
item7_name = """xw"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0
item8.default = item8_default
item8_description = """(m)    Height of focusing rectangle"""
item8.description = item8_description
item8_name = """yh"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0
item9.default = item9_default
item9_description = """(K)    1st temperature of thermal distribution"""
item9.description = item9_description
item9_name = """T1"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 300.0
item10.default = item10_default
item10_description = """(K)    2nd temperature of thermal distribution"""
item10.description = item10_description
item10_name = """T2"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = 300.0
item11.default = item11_default
item11_description = """(K)    3nd temperature of  - - -"""
item11.description = item11_description
item11_name = """T3"""
item11.name = item11_name
item11_type = """double"""
item11.type = item11_type
item12 = Parameter()
item12_default = 0
item12.default = item12_default
item12_description = """(*)    l-integrated flux, 1 (in flux units, see above)"""
item12.description = item12_description
item12_name = """I1"""
item12.name = item12_name
item12_type = """double"""
item12.type = item12_type
item13 = Parameter()
item13_default = 0.0
item13.default = item13_default
item13_description = """(*)    l-integrated flux, 2 (in flux units, see above)"""
item13.description = item13_description
item13_name = """I2"""
item13.name = item13_name
item13_type = """double"""
item13.type = item13_type
item14 = Parameter()
item14_default = 0.0
item14.default = item14_default
item14_description = """(*)    l-integrated flux, 3  - - -"""
item14.description = item14_description
item14_name = """I3"""
item14.name = item14_name
item14_type = """double"""
item14.type = item14_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11, item12, item13, item14 ]
info.input_parameters = info_input_parameters
info_name = """Source_Maxwell_3"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """M"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """"""
item1.description = item1_description
item1_name = """l_range"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """"""
item2.description = item2_description
item2_name = """w_mult"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """"""
item3.description = item3_description
item3_name = """w_source"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """"""
item4.description = item4_description
item4_name = """h_source"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
info_output_parameters = [ item0, item1, item2, item3, item4 ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """"""
item1 = """"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Source with up to three Maxwellian distributions"""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
  double v,tau_l,E,lambda,k,r,xf,yf,dx,dy,w_focus;

  t=0;
  z=0;
  x = 0.5*w_source*randpm1();
  y = 0.5*h_source*randpm1();         

  randvec_target_rect(&xf, &yf, &r, &w_focus,
        0, 0, dist, xw, yh, ROT_A_CURRENT_COMP);

  dx = xf-x;
  dy = yf-y;
  r = sqrt(dx*dx+dy*dy+dist*dist);

  lambda = l_low+l_range*rand01();    
  k = 2*PI/lambda;
  v = K2V*k;

  vz = v*dist/r;
  vy = v*dy/r;
  vx = v*dx/r;




    p *= w_mult*w_focus;                

    double l = lambda;    
    double a1=949.0/T1;
    double M1 = 2*a1*a1*exp(-a1/(l*l))/(l*l*l*l*l);
    double a2=949.0/T2;
    double M2 = 2*a2*a2*exp(-a2/(l*l))/(l*l*l*l*l);
    double a3=949.0/T3;
    double M3 = 2*a3*a3*exp(-a3/(l*l))/(l*l*l*l*l);

    p *= I1*M1+I2*M2+I3*M3;
                                        
"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
