
def factory( name='source_div',width=0,height=0,hdiv=0,vdiv=0,E0=0.0,dE=0.0,Lambda0=0.0,dLambda=0.0,gauss=1.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from Source_divboostpython import Source_div as _factory_mcvine
    return component( _factory_mcvine( name,width,height,hdiv,vdiv,E0,dE,Lambda0,dLambda,gauss ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """KL"""
info.copyright = info_copyright
info_declare = """
  double thetah, thetav, sigmah, sigmav, tan_h, tan_v, p_init;
"""
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """The routine is a rectangular neutron source, which has a gaussian or uniform
divergent output in the forward direction.
The neutron energy is uniformly distributed between E0-dE and E0+dE or
a wavelength is uniformly distributed between Lambda0-dLambda and Lambda0+dLambda.
If energy and wavelength are given, energy is used.
In the case of uniform distribution (gauss=0), angles are uniformly distributed
between -hdiv and +hdiv as well as -vdiv and +vdiv. For Gaussian distribution
(gauss=1), 'hdiv' and 'vdiv' define the FWHM of a Gauss'schen distribution,

%VALIDATION
Feb 2005: tested by Kim Lefmann    (o.k.)
Apr 2005: energy distribution used in external tests of Fermi choppers (o.k.)
Jun 2005: wavelength distribution used in external tests of velocity selectors (o.k.)
Validated by: K. Lieutenant

%BUGS
distribution is uniform in (hor. and vert.) angle (relative to moderator normal),
therefore not suited for large angles"""
info.full_description = info_full_description
info_initialize = """
  sigmah = DEG2RAD*hdiv/(2.0*sqrt(2.0*log(2.0)));
  sigmav = DEG2RAD*vdiv/(2.0*sqrt(2.0*log(2.0)));
  p_init = 1.0/mcget_ncount();

  if (width < 0 || height < 0 || hdiv < 0 || vdiv < 0
    || (!Lambda0 && !dLambda && (E0 <= 0 || dE < 0 || E0-dE <= 0))
    || (!E0 && !dE && (Lambda0 <= 0 || dLambda < 0 || Lambda0-dLambda <= 0))) {
      printf("Source_div: %s: Error in input parameter values!\n"
             "ERROR          Exiting\n",
           NAME_CURRENT_COMP);
      exit(0);
  }
"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """source_div"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """"""
item1.description = item1_description
item1_name = """width"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """"""
item2.description = item2_description
item2_name = """height"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """"""
item3.description = item3_description
item3_name = """hdiv"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """"""
item4.description = item4_description
item4_name = """vdiv"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0.0
item5.default = item5_default
item5_description = """"""
item5.description = item5_description
item5_name = """E0"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0.0
item6.default = item6_default
item6_description = """"""
item6.description = item6_description
item6_name = """dE"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0.0
item7.default = item7_default
item7_description = """"""
item7.description = item7_description
item7_name = """Lambda0"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0.0
item8.default = item8_default
item8_description = """"""
item8.description = item8_description
item8_name = """dLambda"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 1.0
item9.default = item9_default
item9_description = """"""
item9.description = item9_description
item9_name = """gauss"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.input_parameters = info_input_parameters
info_name = """Source_div"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """(rad) parameter 'sigma' of the Gaussian distribution for horizontal divergence"""
item0.description = item0_description
item0_name = """sigmah"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """(rad) parameter 'sigma' of the Gaussian distribution for vertical divergence"""
item1.description = item1_description
item1_name = """sigmav"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """(-)  normalisation factor 1/'neutron_count'"""
item2.description = item2_description
item2_name = """p_init"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
info_output_parameters = [ item0, item1, item2 ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """"""
item1 = """"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """Neutron source with Gaussian or uniform divergence"""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
 double E,lambda,v;

 p=p_init;
 z=0;
 t=0;

 x=randpm1()*width/2.0;
 y=randpm1()*height/2.0;

 if (E0 > 0.0 && E0 > fabs(dE))
 { E=E0+dE*randpm1();                    
   v=sqrt(E)*SE2V;
 }
 else if (Lambda0 > 0.0 && Lambda0 > fabs(dLambda))
 { lambda = Lambda0+dLambda*randpm1();   
   v      = 3956.036 / lambda;
 }
 else
 { printf ("wrong energy and wavelength parameters in 'Source_div'");
   exit(-1);
 }

 if (gauss==1)
 { thetah = randnorm()*sigmah;
   thetav = randnorm()*sigmav;
 }
 else
 { thetah = randpm1()*hdiv*DEG2RAD;
   thetav = randpm1()*vdiv*DEG2RAD;
 }

 tan_h = tan(thetah);
 tan_v = tan(thetav);

 
 vz = v / sqrt(1 + tan_v*tan_v + tan_h*tan_h);
 vy = tan_v * vz;
 vx = tan_h * vz;
"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
