
def factory( name='source_gen',flux_file='',xdiv_file='',ydiv_file='',radius=0.0,dist=0.0,xw=0.0,yh=0.0,E0=0.0,dE=0.0,Lambda0=0.0,dLambda=0.0,I1=0.0,h=0.0,w=0.0,verbose=0.0,T1=0.0,flux_file_perAA=0.0,flux_file_log=0.0,Lmin=0.0,Lmax=0.0,Emin=0.0,Emax=0.0,T2=0.0,I2=0.0,T3=0.0,I3=0.0,length=0.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from Source_genboostpython import Source_gen as _factory_mcvine
    return component( _factory_mcvine( name,flux_file,xdiv_file,ydiv_file,radius,dist,xw,yh,E0,dE,Lambda0,dLambda,I1,h,w,verbose,T1,flux_file_perAA,flux_file_log,Lmin,Lmax,Emin,Emax,T2,I2,T3,I3,length ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Emmanuel Farhi, Kim Lefmann"""
info.copyright = info_copyright
info_declare = """
 double p_in;
 double lambda0, lambda02, L2P;     
 double lambda0b, lambda02b, L2Pb;  
 double lambda0c, lambda02c, L2Pc;  
 t_Table pTable;
 t_Table pTable_x;
 t_Table pTable_y;
 double  pTable_xmin;
 double  pTable_xmax;
 double  pTable_xsum;
 double  pTable_ymin;
 double  pTable_ymax;
 double  pTable_ysum;
 double  pTable_dxmin;
 double  pTable_dxmax;
 double  pTable_dymin;
 double  pTable_dymax;

"""
info.declare = info_declare
info_finalize = """
  Table_Free(&pTable);
  Table_Free(&pTable_x);
  Table_Free(&pTable_y);
"""
info.finalize = info_finalize
info_full_description = """This routine is a neutron source (rectangular or circular), which aims at
a square target centered at the beam (in order to improve MC-acceptance
rate). The angular divergence is then given by the dimensions of the
target.
The neutron energy/wavelength is distributed between Emin=E0-dE and
Emax=E0+dE or Lmin=Lambda0-dLambda and Lmax=Lambda0+dLambda. The I1 may
be either arbitrary (I1=0), or specified in neutrons per steradian per
square cm per Angstrom. A Maxwellian spectra may be selected if you give
the source temperatures (up to 3). Finally, a file with the flux as a
function of the wavelength [lambda(AA) flux(n/s/cm^2/st/AA)] may be used
with the 'flux_file' parameter. Format is 2 columns free text.
Additionl distributions for the horizontal and vertical phase spaces
distributions (position-divergence) may be specified with the
'xdiv_file' and 'ydiv_file' parameters. Format is free text, requiring
a comment line '# xylimits: pos_min pos_max div_min div_max' to set
the axis of the distribution matrix. All these files may be generated using
standard monitors (better in McStas/PGPLOT format), e.g.:
Monitor_nD(options="auto lambda per cm2")
Monitor_nD(options="x hdiv, all auto")
Monitor_nD(options="y vdiv, all auto")
The source shape is defined by its radius, or can alternatively be squared
if you specify non-zero h and w parameters.
The beam is divergence uniform,.
The source may have a thickness, which will broaden the default zero time
distribution.
For the Maxwellian spectra, the generated intensity is dPhi/dLambda (n/s/AA)
For flat spectra, the generated intensity is Phi (n/s).

Usage example:
Source_gen(name, radius=0.1,Lambda0=2.36,dLambda=0.16,T1=20,I1=1e13,xw=0.01,yh=0.01)
Source_gen(name, h=0.1,w=0.1,Emin=1,Emax=3,I1=1e13,verbose=1,xw=0.01,yh=0.01)
EXTEND
%{
t = rand0max(1e-3); // set time from 0 to 1 ms for TOF instruments.
%}

<b>Some sources parameters:</b>
PSI cold source T1= 296.16, I1=8.5E11, T2=40.68, I2=5.2E11
ILL VCS cold source T1=216.8,I1=1.24e+13,T2=33.9,I2=1.02e+13
(H1)         T3=16.7 ,I3=3.0423e+12
ILL HCS cold source T1=413.5,I1=10.22e12,T2=145.8,I2=3.44e13
(H5)         T3=40.1 ,I3=2.78e13
ILL Thermal tube    T1=683.7,I1=0.5874e+13,T2=257.7,I2=2.5099e+13
(H12)        T3=16.7 ,I3=1.0343e+12
ILL Hot source      T1=1695,I1=1.74e13,T2=708,I2=3.9e12

%VALIDATION
Feb 2005: output cross-checked for 3 Maxwellians against VITESS source
I(lambda), I(hor_div), I(vert_div) identical in shape and absolute values
Validated by: K. Lieutenant"""
info.full_description = info_full_description
info_initialize = """
  double source_area, k;
  Lambda0 = 0;
  pTable_xsum = 0;
  pTable_ysum = 0;

  
  if (flux_file && strlen(flux_file) > 0) {
    if (Table_Read(&pTable, flux_file, 1) <= 0) 
      exit(fprintf(stderr, "Source_gen: %s: can not read file %s\n", NAME_CURRENT_COMP, flux_file));
    
    int i;
    if (pTable.columns < 2) exit(fprintf(stderr, "Source_gen: %s: Flux file %s should contain at least 2 columns\n", NAME_CURRENT_COMP, flux_file));
    double table_lmin=FLT_MAX, table_lmax=-FLT_MAX;
    double tmin=FLT_MAX, tmax=-FLT_MAX;
    for (i=0; i<pTable.rows; i++) {
      double val = Table_Index(pTable, i,1);
      val = Table_Index(pTable, i,0); 
      if (val > tmax) tmax=val;
      if (val < tmin) tmin=val;
    }
    for (i=0; i<pTable.rows; i++) {
      double val = Table_Index(pTable, i,1);
      if (val < 0) fprintf(stderr, "Source_gen: %s: File %s has negative flux at row %i\n", NAME_CURRENT_COMP, flux_file, i+1);
      if (flux_file_log)
        val = log(val > 0 ? val : tmin/10);
      Table_SetElement(&pTable, i, 1, val);
      val = Table_Index(pTable, i,0); 
      if (val > table_lmax) table_lmax=val;
      if (val < table_lmin) table_lmin=val;
    }
    if (!Lmin && !Lmax && !Lambda0 && !dLambda && !E0 && !dE && !Emin && !Emax) {
      Lmin = table_lmin; Lmax = table_lmax;
    }
    if (Lmax > table_lmax) {
      if (verbose) fprintf(stderr, "Source_gen: %s: Maximum wavelength %g is beyond table range upper limit %g. Constraining.\n", NAME_CURRENT_COMP, Lmax, table_lmax);
      Lmax = table_lmax;
    }
    if (Lmin < table_lmin) {
      if (verbose) fprintf(stderr, "Source_gen: %s: Minimum wavelength %g is below table range lower limit %g. Constraining.\n", NAME_CURRENT_COMP, Lmin, table_lmin);
      Lmin = table_lmin;
    }
  } else
  {
    k  = 1.38066e-23; 
    if (T1 > 0)
    {
      lambda0  = 1.0e10*sqrt(HBAR*HBAR*4.0*PI*PI/2.0/MNEUTRON/k/T1);
      lambda02 = lambda0*lambda0;
      L2P      = 2*lambda02*lambda02;
    }
    else
      { lambda0 = Lambda0; }

    if (T2 > 0)
    {
      lambda0b  = 1.0e10*sqrt(HBAR*HBAR*4.0*PI*PI/2.0/MNEUTRON/k/T2);
      lambda02b = lambda0b*lambda0b;
      L2Pb      = 2*lambda02b*lambda02b;
    }
    else
      { lambda0b = Lambda0; }

    if (T3 > 0)
    {
      lambda0c  = 1.0e10*sqrt(HBAR*HBAR*4.0*PI*PI/2.0/MNEUTRON/k/T3);
      lambda02c = lambda0c*lambda0c;
      L2Pc      = 2*lambda02c*lambda02c;
    }
    else
      { lambda0c = Lambda0; }
  }

  
  if (xdiv_file && strlen(xdiv_file) > 0) {
    int i,j;
    if (Table_Read(&pTable_x, xdiv_file, 1) <= 0) 
      exit(fprintf(stderr, "Source_gen: %s: can not read file %s\n", NAME_CURRENT_COMP, xdiv_file));
    pTable_xsum = 0;
    for (i=0; i<pTable_x.rows; i++)
      for (j=0; j<pTable_x.columns; j++)
        pTable_xsum += Table_Index(pTable_x, i,j);

    
    char **parsing;
    char xylimits[1024];
    strcpy(xylimits, "");
    parsing = Table_ParseHeader(pTable_x.header,
      "xlimits", "xylimits",
      NULL);

    if (parsing) {
      if (parsing[0])  strcpy(xylimits, str_dup_numeric(parsing[0]));
      if (parsing[1] && !strlen(xylimits))
                       strcpy(xylimits, str_dup_numeric(parsing[1]));
      for (i=0; i<=1; i++) {
        if (parsing[i]) free(parsing[i]);
      }
      free(parsing);
    }
    i = sscanf(xylimits, "%lg %lg %lg %lg",
      &(pTable_xmin),  &(pTable_xmax),
      &(pTable_dxmin), &(pTable_dxmax));
    if (i != 2 && i != 4 && verbose)
      fprintf(stderr, "Source_gen: %s: invalid xylimits '%s' from file %s. extracted %i values\n",
        NAME_CURRENT_COMP, xylimits, xdiv_file, i);

    if (!w) w=pTable_xmax-pTable_xmin;
    if (!xw && !dist) xw=fabs(pTable_dxmax-pTable_dxmin);
  }

  if (ydiv_file && strlen(ydiv_file) > 0) {
    int i,j;
    if (Table_Read(&pTable_y, ydiv_file, 1) <= 0) 
      exit(fprintf(stderr, "Source_gen: %s: can not read file %s\n", NAME_CURRENT_COMP, ydiv_file));
    pTable_ysum = 0;
    for (i=0; i<pTable_y.rows; i++)
      for (j=0; j<pTable_y.columns; j++)
        pTable_ysum += Table_Index(pTable_y, i,j);

    
    char **parsing;
    char xylimits[1024];
    strcpy(xylimits, "");
    parsing = Table_ParseHeader(pTable_y.header,
      "xlimits", "xylimits",
      NULL);

    if (parsing) {
      if (parsing[0])  strcpy(xylimits,str_dup_numeric(parsing[0]));
      if (parsing[1] && !strlen(xylimits))
                       strcpy(xylimits,str_dup_numeric(parsing[1]));
      for (i=0; i<=1; i++) {
        if (parsing[i]) free(parsing[i]);
      }
      free(parsing);
    }
    i = sscanf(xylimits, "%lg %lg %lg %lg",
      &(pTable_ymin),  &(pTable_ymax),
      &(pTable_dymin), &(pTable_dymax));
    if (i != 2 && i != 4 && verbose)
      fprintf(stderr, "Source_gen: %s: invalid xylimits '%s' from file %s. extracted %i values\n",
        NAME_CURRENT_COMP, xylimits, ydiv_file, i);
    if (!h)  h=pTable_ymax-pTable_ymin;
    if (!yh && !dist) yh=fabs(pTable_dymax-pTable_dymin);
  }

  
  if (Emin < 0 || Emax < 0 || Lmin < 0 || Lmax < 0 || E0 < 0 || dE < 0 || Lambda0 < 0 || dLambda < 0)
  {
    fprintf(stderr,"Source_gen: %s: Error: Negative average\n"
                   "            or range values for wavelength or energy encountered\n",
                   NAME_CURRENT_COMP);
    exit(-1);
  }
  if ((Emin == 0 && Emax > 0) || (dE > 0 && dE >= E0))
  {
    fprintf(stderr,"Source_gen: %s: Error: minimal energy cannot be less or equal zero\n",
      NAME_CURRENT_COMP);
    exit(-1);
  }
  if ((Emax >= Emin) && (Emin > 0))
  { E0 = (Emax+Emin)/2;
    dE = (Emax-Emin)/2;
  }
  if ((E0 > dE) && (dE >= 0))
  {
    Lmin = sqrt(81.81/(E0+dE)); 
    Lmax = sqrt(81.81/(E0-dE));
  }
  if (Lmax > 0)
  { Lambda0 = (Lmax+Lmin)/2;
    dLambda = (Lmax-Lmin)/2;
  }
  if ((Lambda0 < dLambda) || (dLambda < 0))
  { fprintf(stderr,"Source_gen: %s: Error: Wavelength range %.3f +/- %.3f AA calculated \n",
      NAME_CURRENT_COMP, Lambda0, dLambda);
    fprintf(stderr,"- whole wavelength range must be >= 0 \n");
    fprintf(stderr,"- range must be > 0; otherwise intensity gets zero, use other sources in this case \n\n");
    exit(-1);
  }

  radius = fabs(radius); w=fabs(w); h=fabs(h);  I1=fabs(I1);
  Lambda0=fabs(Lambda0); dLambda=fabs(dLambda);
  xw = fabs(xw); yh=fabs(yh); dist=fabs(dist);

  if (dist == 0)
  {
    fprintf(stderr,"Source_gen: %s: warning: focusing distance is null.\n"
                   "            xw and yh interpreted as full divergence in [deg]\n",
                   NAME_CURRENT_COMP);
  }
  Lmin = Lambda0 - dLambda; 
  Lmax = Lambda0 + dLambda;

  
  if ((I1 > 0  && T1 >= 0) || (flux_file && strlen(flux_file) > 0))
  { 
    if (radius)
      source_area = radius*radius*PI*1e4; 
    else
      source_area = h*w*1e4; 
    p_in  = source_area; 
    p_in *= (Lmax-Lmin); 
    if (flux_file && strlen(flux_file) && !flux_file_perAA)  p_in *= pTable.rows/(Lmax-Lmin);
  }
  else
    p_in = (I1 > 0? I1 : 1)/4/PI; 
  p_in /= mcget_ncount();
  if (!T1 && I1) p_in *= I1;

  if (radius == 0 && h == 0 && w == 0)
  {
    fprintf(stderr,"Source_gen: %s: Error: Please specify source geometry (radius, h, w)\n",
      NAME_CURRENT_COMP);
    exit(-1);
  }
  if (xw*yh == 0)
  {
    fprintf(stderr,"Source_gen: %s: Error: Please specify source target (xw, yh)\n",
      NAME_CURRENT_COMP);
    exit(-1);
  }

  if (verbose)
  {
    printf("Source_gen: component %s ", NAME_CURRENT_COMP);
    if ((h == 0) || (w == 0))
      printf("(disk, radius=%g)", radius);
    else
      printf("(square %g x %g)",h,w);
    printf("\n            spectra ");
    printf("%.3f to %.3f AA (%.3f to %.3f meV)", Lmin, Lmax, 81.81/Lmax/Lmax, 81.81/Lmin/Lmin);
    printf("\n");
    if (flux_file && strlen(flux_file) > 0)
    { printf("  File %s for flux distribution used. Flux is dPhi/dLambda in [n/s/AA]. \n", flux_file);
      Table_Info(pTable);
    }
    else if (T1>=0 && I1)
    { if (T1 != 0)
        printf("            T1=%.1f K (%.3f AA)", T1, lambda0);
      if (T2*I2 != 0)
        printf(", T2=%.1f K (%.3f AA)", T2, lambda0b);
      if (T3*I3 != 0)
        printf(", T3=%.1f K (%.3f AA)", T3, lambda0c);
      if (T1) printf("\n");
      printf("  Flux is dPhi/dLambda in [n/s/cm2].\n");
    }
    else
    { printf("  Flux is Phi in [n/s].\n");
    }
    if (xdiv_file && strlen(xdiv_file) > 0)
      printf("  File %s x=[%g:%g] [m] xdiv=[%g:%g] [deg] used as horizontal phase space distribution.\n", xdiv_file, pTable_xmin, pTable_xmax, pTable_dxmin, pTable_dxmax);
    if (ydiv_file && strlen(ydiv_file) > 0)
      printf("  File %s y=[%g:%g] [m] ydiv=[%g:%g] [deg] used as vertical phase space distribution.\n", ydiv_file, pTable_ymin, pTable_ymax, pTable_dymin, pTable_dymax);
  }
  else
    if (verbose == -1)
      printf("Source_gen: component %s unactivated", NAME_CURRENT_COMP);
"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """source_gen"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = """"""
item1.default = item1_default
item1_description = """"""
item1.description = item1_description
item1_name = """flux_file"""
item1.name = item1_name
item1_type = """char *"""
item1.type = item1_type
item2 = Parameter()
item2_default = """"""
item2.default = item2_default
item2_description = """"""
item2.description = item2_description
item2_name = """xdiv_file"""
item2.name = item2_name
item2_type = """char *"""
item2.type = item2_type
item3 = Parameter()
item3_default = """"""
item3.default = item3_default
item3_description = """"""
item3.description = item3_description
item3_name = """ydiv_file"""
item3.name = item3_name
item3_type = """char *"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0.0
item4.default = item4_default
item4_description = """"""
item4.description = item4_description
item4_name = """radius"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0.0
item5.default = item5_default
item5_description = """"""
item5.description = item5_description
item5_name = """dist"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0.0
item6.default = item6_default
item6_description = """"""
item6.description = item6_description
item6_name = """xw"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0.0
item7.default = item7_default
item7_description = """"""
item7.description = item7_description
item7_name = """yh"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0.0
item8.default = item8_default
item8_description = """"""
item8.description = item8_description
item8_name = """E0"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0.0
item9.default = item9_default
item9_description = """"""
item9.description = item9_description
item9_name = """dE"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 0.0
item10.default = item10_default
item10_description = """"""
item10.description = item10_description
item10_name = """Lambda0"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = 0.0
item11.default = item11_default
item11_description = """"""
item11.description = item11_description
item11_name = """dLambda"""
item11.name = item11_name
item11_type = """double"""
item11.type = item11_type
item12 = Parameter()
item12_default = 0.0
item12.default = item12_default
item12_description = """"""
item12.description = item12_description
item12_name = """I1"""
item12.name = item12_name
item12_type = """double"""
item12.type = item12_type
item13 = Parameter()
item13_default = 0.0
item13.default = item13_default
item13_description = """"""
item13.description = item13_description
item13_name = """h"""
item13.name = item13_name
item13_type = """double"""
item13.type = item13_type
item14 = Parameter()
item14_default = 0.0
item14.default = item14_default
item14_description = """"""
item14.description = item14_description
item14_name = """w"""
item14.name = item14_name
item14_type = """double"""
item14.type = item14_type
item15 = Parameter()
item15_default = 0.0
item15.default = item15_default
item15_description = """"""
item15.description = item15_description
item15_name = """verbose"""
item15.name = item15_name
item15_type = """double"""
item15.type = item15_type
item16 = Parameter()
item16_default = 0.0
item16.default = item16_default
item16_description = """"""
item16.description = item16_description
item16_name = """T1"""
item16.name = item16_name
item16_type = """double"""
item16.type = item16_type
item17 = Parameter()
item17_default = 0.0
item17.default = item17_default
item17_description = """"""
item17.description = item17_description
item17_name = """flux_file_perAA"""
item17.name = item17_name
item17_type = """double"""
item17.type = item17_type
item18 = Parameter()
item18_default = 0.0
item18.default = item18_default
item18_description = """"""
item18.description = item18_description
item18_name = """flux_file_log"""
item18.name = item18_name
item18_type = """double"""
item18.type = item18_type
item19 = Parameter()
item19_default = 0.0
item19.default = item19_default
item19_description = """"""
item19.description = item19_description
item19_name = """Lmin"""
item19.name = item19_name
item19_type = """double"""
item19.type = item19_type
item20 = Parameter()
item20_default = 0.0
item20.default = item20_default
item20_description = """"""
item20.description = item20_description
item20_name = """Lmax"""
item20.name = item20_name
item20_type = """double"""
item20.type = item20_type
item21 = Parameter()
item21_default = 0.0
item21.default = item21_default
item21_description = """"""
item21.description = item21_description
item21_name = """Emin"""
item21.name = item21_name
item21_type = """double"""
item21.type = item21_type
item22 = Parameter()
item22_default = 0.0
item22.default = item22_default
item22_description = """"""
item22.description = item22_description
item22_name = """Emax"""
item22.name = item22_name
item22_type = """double"""
item22.type = item22_type
item23 = Parameter()
item23_default = 0.0
item23.default = item23_default
item23_description = """"""
item23.description = item23_description
item23_name = """T2"""
item23.name = item23_name
item23_type = """double"""
item23.type = item23_type
item24 = Parameter()
item24_default = 0.0
item24.default = item24_default
item24_description = """"""
item24.description = item24_description
item24_name = """I2"""
item24.name = item24_name
item24_type = """double"""
item24.type = item24_type
item25 = Parameter()
item25_default = 0.0
item25.default = item25_default
item25_description = """"""
item25.description = item25_description
item25_name = """T3"""
item25.name = item25_name
item25_type = """double"""
item25.type = item25_type
item26 = Parameter()
item26_default = 0.0
item26.default = item26_default
item26_description = """"""
item26.description = item26_description
item26_name = """I3"""
item26.name = item26_name
item26_type = """double"""
item26.type = item26_type
item27 = Parameter()
item27_default = 0.0
item27.default = item27_default
item27_description = """"""
item27.description = item27_description
item27_name = """length"""
item27.name = item27_name
item27_type = """double"""
item27.type = item27_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11, item12, item13, item14, item15, item16, item17, item18, item19, item20, item21, item22, item23, item24, item25, item26, item27 ]
info.input_parameters = info_input_parameters
info_name = """Source_gen"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """p_in"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0
item1.default = item1_default
item1_description = """"""
item1.description = item1_description
item1_name = """lambda0"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0
item2.default = item2_default
item2_description = """"""
item2.description = item2_description
item2_name = """lambda02"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0
item3.default = item3_default
item3_description = """"""
item3.description = item3_description
item3_name = """L2P"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 0
item4.default = item4_default
item4_description = """"""
item4.description = item4_description
item4_name = """lambda0b"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0
item5.default = item5_default
item5_description = """"""
item5.description = item5_description
item5_name = """lambda02b"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0
item6.default = item6_default
item6_description = """"""
item6.description = item6_description
item6_name = """L2Pb"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 0
item7.default = item7_default
item7_description = """"""
item7.description = item7_description
item7_name = """lambda0c"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 0
item8.default = item8_default
item8_description = """"""
item8.description = item8_description
item8_name = """lambda02c"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0
item9.default = item9_default
item9_description = """"""
item9.description = item9_description
item9_name = """L2Pc"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 0
item10.default = item10_default
item10_description = """"""
item10.description = item10_description
item10_name = """pTable"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = 0
item11.default = item11_default
item11_description = """"""
item11.description = item11_description
item11_name = """pTable_x"""
item11.name = item11_name
item11_type = """double"""
item11.type = item11_type
item12 = Parameter()
item12_default = 0
item12.default = item12_default
item12_description = """"""
item12.description = item12_description
item12_name = """pTable_y"""
item12.name = item12_name
item12_type = """double"""
item12.type = item12_type
item13 = Parameter()
item13_default = 0
item13.default = item13_default
item13_description = """"""
item13.description = item13_description
item13_name = """pTable_xmin"""
item13.name = item13_name
item13_type = """double"""
item13.type = item13_type
item14 = Parameter()
item14_default = 0
item14.default = item14_default
item14_description = """"""
item14.description = item14_description
item14_name = """pTable_xmax"""
item14.name = item14_name
item14_type = """double"""
item14.type = item14_type
item15 = Parameter()
item15_default = 0
item15.default = item15_default
item15_description = """"""
item15.description = item15_description
item15_name = """pTable_xsum"""
item15.name = item15_name
item15_type = """double"""
item15.type = item15_type
item16 = Parameter()
item16_default = 0
item16.default = item16_default
item16_description = """"""
item16.description = item16_description
item16_name = """pTable_ymin"""
item16.name = item16_name
item16_type = """double"""
item16.type = item16_type
item17 = Parameter()
item17_default = 0
item17.default = item17_default
item17_description = """"""
item17.description = item17_description
item17_name = """pTable_ymax"""
item17.name = item17_name
item17_type = """double"""
item17.type = item17_type
item18 = Parameter()
item18_default = 0
item18.default = item18_default
item18_description = """"""
item18.description = item18_description
item18_name = """pTable_ysum"""
item18.name = item18_name
item18_type = """double"""
item18.type = item18_type
item19 = Parameter()
item19_default = 0
item19.default = item19_default
item19_description = """"""
item19.description = item19_description
item19_name = """pTable_dxmin"""
item19.name = item19_name
item19_type = """double"""
item19.type = item19_type
item20 = Parameter()
item20_default = 0
item20.default = item20_default
item20_description = """"""
item20.description = item20_description
item20_name = """pTable_dxmax"""
item20.name = item20_name
item20_type = """double"""
item20.type = item20_type
item21 = Parameter()
item21_default = 0
item21.default = item21_default
item21_description = """"""
item21.description = item21_description
item21_name = """pTable_dymin"""
item21.name = item21_name
item21_type = """double"""
item21.type = item21_type
item22 = Parameter()
item22_default = 0
item22.default = item22_default
item22_description = """"""
item22.description = item22_description
item22_name = """pTable_dymax"""
item22.name = item22_name
item22_type = """double"""
item22.type = item22_type
info_output_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11, item12, item13, item14, item15, item16, item17, item18, item19, item20, item21, item22 ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """
// ----------  header for SHARE ----------

#include "mcstas2/share/read_table-lib.h"
"""
item1 = """// ----------  implementation for SHARE ----------
#ifndef SOURCE_GEN_DEF
#define SOURCE_GEN_DEF

char *str_dup_numeric(char *orig)
{
  long i;
  char *valid;

  if (!orig || !strlen(orig)) return(NULL);

  for (i=0; i < strlen(orig); i++)
  {
    if ( (orig[i] > 122)
      || (orig[i] < 32)
      || (strchr("!\"#$%&'()*,:;<=>?@[\\]^`/ ", orig[i]) != NULL) )
    {
      orig[i] = ' ';
    }
  }
  orig[i] = '\0';
  
  for (i=0; i < strlen(orig); i++) {
    if (*orig == ' ') orig++;
    else break;
  }

  return(orig);
} 
#endif

"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """spectrum"""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
  double dx,dy,xf,yf,rf,pdir,chi,v,r, lambda;
  double Maxwell, lambda2, lambda5;
  double xwg, yhg;

  if (verbose >= 0)
  {

    z=0;

    if ((h == 0) || (w == 0))
    {
      chi=2*PI*rand01();                          
      r=sqrt(rand01())*radius;                    
      x=r*cos(chi);
      y=r*sin(chi);
    }
    else
    {
      x = w*randpm1()/2;   
      y = h*randpm1()/2;
    }
    if (length != 0)
      z = length*randpm1()/2;

    if (dist>0)
    randvec_target_rect(&xf, &yf, &rf, &pdir,
        0, 0, dist, xw, yh, ROT_A_CURRENT_COMP);
    else
    randvec_target_rect_angular(&xf, &yf, &rf, &pdir,
        0, 0, 1, xw*DEG2RAD, yh*DEG2RAD, ROT_A_CURRENT_COMP);

    
    lambda = Lambda0+dLambda*randpm1();
    if (lambda <= 0) ABSORB;
    v = K2V*(2*PI/lambda);
    p = p_in*pdir;

    dx = xf-x;
    dy = yf-y;
    rf = sqrt(dx*dx+dy*dy+dist*dist);

    vz=v*dist/rf;
    vy=v*dy/rf;
    vx=v*dx/rf;

    
    if (flux_file && strlen(flux_file) > 0)
    {
       double w=Table_Value(pTable, lambda, 1);
       if (flux_file_log) w=exp(w);
       p *= w;
    }
    else if (T1 > 0 && I1 > 0)
    {
      lambda2 = lambda*lambda;
      lambda5 = lambda2*lambda2*lambda;
      Maxwell = I1 * L2P/lambda5 * exp(-lambda02/lambda2);  

      if ((T2 > 0) && (I2 > 0))
      {
        Maxwell += I2 * L2Pb/lambda5 * exp(-lambda02b/lambda2);
      }
      if ((T3 > 0) && (I3 > 0))
      {
        Maxwell += I3 * L2Pc/lambda5 * exp(-lambda02c/lambda2);
      }
      p *= Maxwell;
    }

    
    if (xdiv_file && strlen(xdiv_file) > 0 && pTable_xsum > 0) {
      double i,j;
      j = (x-            pTable_xmin) /(pTable_xmax -pTable_xmin) *pTable_x.columns;
      i = (atan2(dx,rf)*RAD2DEG-pTable_dxmin)/(pTable_dxmax-pTable_dxmin)*pTable_x.rows;
      r = Table_Value(pTable_x, i,j); 
      p *= r/pTable_xsum;
    }
    if (ydiv_file && strlen(ydiv_file) > 0 && pTable_ysum > 0) {
      double i,j;
      j = (y-            pTable_ymin) /(pTable_ymax -pTable_ymin) *pTable_y.columns;
      i = (atan2(dy,rf)*RAD2DEG-  pTable_dymin)/(pTable_dymax-pTable_dymin)*pTable_y.rows;
      r = Table_Value(pTable_y, i,j);
      p *= r/pTable_ysum;
    }
    SCATTER;
  }
"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
