
def factory( name='source_simple',radius=0.05,height=0.0,width=0.0,dist=10.0,xw=0.1,yh=0.1,E0=60.0,dE=10.0,Lambda0=0.0,dLambda=0.0,flux=1.0,gauss=0.0 ):
    from mcstas2.bindings import boostpython
    from mcstas2.mcstas2bp import McStasComponentAsMcniComponent as component
    from Source_simpleboostpython import Source_simple as _factory_mcvine
    return component( _factory_mcvine( name,radius,height,width,dist,xw,yh,E0,dE,Lambda0,dLambda,flux,gauss ) )

from mcstas2.utils.parsers.ComponentInfo import ComponentInfo, Parameter
info = ComponentInfo()
info_copyright = """Kim Lefmann"""
info.copyright = info_copyright
info_declare = """
  double pmul, srcArea;
  int square;
"""
info.declare = info_declare
info_finalize = """"""
info.finalize = info_finalize
info_full_description = """The routine is a circular neutron source, which aims at a square target
centered at the beam (in order to improve MC-acceptance rate).  The angular
divergence is then given by the dimensions of the target.
The neutron energy is uniformly distributed between E0-dE and E0+dE.

This component replaces Source_flat, Source_flat_lambda,
Source_flux and Source_flux_lambda."""
info.full_description = info_full_description
info_initialize = """
  square = 0;
  
  if (!radius == 0 && height == 0 && width == 0) {
    square = 0;
    srcArea = PI*radius*radius;
  } else if(radius == 0 && !height ==0 && !width==0) {
    square = 1;
    srcArea = width * height;
  } else {
    printf("Source_simple: confused! Both radius and width/height are specified");
    exit(0);
  }
  
  pmul=flux*1e4*srcArea/mcget_ncount();
  if (srcArea <= 0 || dist < 0 || xw < 0 || yh < 0
    || (!Lambda0 && !dLambda && (E0 <= 0 || dE < 0 || E0-dE <= 0))
    || (!E0 && !dE && (Lambda0 <= 0 || dLambda < 0 || Lambda0-dLambda <= 0))) {
      printf("Source_simple: %s: Error in input parameter values!\n"
             "ERROR          Exiting\n",
           NAME_CURRENT_COMP);
      exit(0);
  }

  printf("Source_simple: initialized.\n");
"""
info.initialize = info_initialize
item0 = Parameter()
item0_default = """source_simple"""
item0.default = item0_default
item0_description = """component name"""
item0.description = item0_description
item0_name = """name"""
item0.name = item0_name
item0_type = """char *"""
item0.type = item0_type
item1 = Parameter()
item1_default = 0.05
item1.default = item1_default
item1_description = """[m]   Radius of circle in (x,y,0) plane where neutrons"""
item1.description = item1_description
item1_name = """radius"""
item1.name = item1_name
item1_type = """double"""
item1.type = item1_type
item2 = Parameter()
item2_default = 0.0
item2.default = item2_default
item2_description = """[m]   Height of rectangle in (x,y,0) plane where neutrons"""
item2.description = item2_description
item2_name = """height"""
item2.name = item2_name
item2_type = """double"""
item2.type = item2_type
item3 = Parameter()
item3_default = 0.0
item3.default = item3_default
item3_description = """[m]   Width of rectangle in (x,y,0) plane where neutrons"""
item3.description = item3_description
item3_name = """width"""
item3.name = item3_name
item3_type = """double"""
item3.type = item3_type
item4 = Parameter()
item4_default = 10.0
item4.default = item4_default
item4_description = """[m]   Distance to target along z axis."""
item4.description = item4_description
item4_name = """dist"""
item4.name = item4_name
item4_type = """double"""
item4.type = item4_type
item5 = Parameter()
item5_default = 0.1
item5.default = item5_default
item5_description = """[m]   Width(x) of target"""
item5.description = item5_description
item5_name = """xw"""
item5.name = item5_name
item5_type = """double"""
item5.type = item5_type
item6 = Parameter()
item6_default = 0.1
item6.default = item6_default
item6_description = """[m]   Height(y) of target"""
item6.description = item6_description
item6_name = """yh"""
item6.name = item6_name
item6_type = """double"""
item6.type = item6_type
item7 = Parameter()
item7_default = 60.0
item7.default = item7_default
item7_description = """[meV] Mean energy of neutrons."""
item7.description = item7_description
item7_name = """E0"""
item7.name = item7_name
item7_type = """double"""
item7.type = item7_type
item8 = Parameter()
item8_default = 10.0
item8.default = item8_default
item8_description = """[meV] Energy spread of neutrons (flat or gaussian sigma)."""
item8.description = item8_description
item8_name = """dE"""
item8.name = item8_name
item8_type = """double"""
item8.type = item8_type
item9 = Parameter()
item9_default = 0.0
item9.default = item9_default
item9_description = """[AA]  Mean wavelength of neutrons."""
item9.description = item9_description
item9_name = """Lambda0"""
item9.name = item9_name
item9_type = """double"""
item9.type = item9_type
item10 = Parameter()
item10_default = 0.0
item10.default = item10_default
item10_description = """[AA]  Wavelength spread of neutrons."""
item10.description = item10_description
item10_name = """dLambda"""
item10.name = item10_name
item10_type = """double"""
item10.type = item10_type
item11 = Parameter()
item11_default = 1.0
item11.default = item11_default
item11_description = """[1/(s*cm**2*st)] Energy integrated flux"""
item11.description = item11_description
item11_name = """flux"""
item11.name = item11_name
item11_type = """double"""
item11.type = item11_type
item12 = Parameter()
item12_default = 0.0
item12.default = item12_default
item12_description = """[1]   Gaussian (1) or Flat (0) energy/wavelength distribution"""
item12.description = item12_description
item12_name = """gauss"""
item12.name = item12_name
item12_type = """double"""
item12.type = item12_type
info_input_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9, item10, item11, item12 ]
info.input_parameters = info_input_parameters
info_name = """Source_simple"""
info.name = info_name
item0 = Parameter()
item0_default = 0
item0.default = item0_default
item0_description = """"""
item0.description = item0_description
item0_name = """pmul"""
item0.name = item0_name
item0_type = """double"""
item0.type = item0_type
info_output_parameters = [ item0 ]
info.output_parameters = info_output_parameters
info_save = """"""
info.save = info_save
item0 = """"""
item1 = """"""
info_share = [ item0, item1 ]
info.share = info_share
info_simple_description = """A circular neutron source with flat energy spectrum and arbitrary flux"""
info.simple_description = info_simple_description
item0 = """x"""
item1 = """y"""
item2 = """z"""
item3 = """vx"""
item4 = """vy"""
item5 = """vz"""
item6 = """t"""
item7 = """s1"""
item8 = """s2"""
item9 = """p"""
info_state_parameters = [ item0, item1, item2, item3, item4, item5, item6, item7, item8, item9 ]
info.state_parameters = info_state_parameters
info_trace = """
 double chi,E,Lambda,v,r, xf, yf, rf, dx, dy, pdir;

 t=0;
 z=0;
 
 if (square == 1) {
   x = width * (rand01() - 0.5);
   y = height * (rand01() - 0.5);
 } else {
   chi=2*PI*rand01();                          
   r=sqrt(rand01())*radius;                    
   x=r*cos(chi);
   y=r*sin(chi);
 }
 randvec_target_rect(&xf, &yf, &rf, &pdir,
        0, 0, dist, xw, yh, ROT_A_CURRENT_COMP);

 dx = xf-x;
 dy = yf-y;
 rf = sqrt(dx*dx+dy*dy+dist*dist);

 p = pdir*pmul;

 if(Lambda0==0) {
   if (!gauss) {
     E=E0+dE*randpm1();              
   } else {
     E=E0+randnorm()*dE;
   }
   v=sqrt(E)*SE2V;
 } else {
   if (!gauss) {
     Lambda=Lambda0+dLambda*randpm1();
   } else {
     Lambda=randnorm()*dLambda;
   }
   v = K2V*(2*PI/Lambda);
 }

 vz=v*dist/rf;
 vy=v*dy/rf;
 vx=v*dx/rf;

 //printf("Source_simple: Trace: %g,%g,%g, %g,%g,%g\n", x,y,z,vx,vy,vz);
"""
info.trace = info_trace

factory.info = info
factory.__doc__ = str(info)
